/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.configuration;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;

import java.io.Serializable;
import java.util.List;

/**
 * Add skeletons reference in configuration
 * @author A. Grassi, R. Scarciello
 *
 */
public class SkeletonsManager implements Serializable {
	private static final long serialVersionUID = -4607649217413073986L;

	public synchronized static boolean mergeConfiguration() {
		boolean ret = true ;
		SourceBean skeletonBean = (SourceBean)ConfigSingleton.getInstance().getAttribute("SKELETONS") ;
		if (skeletonBean == null) {
			System.out.println("SkeletonsManager: Skeletons configuration file not loaded in configuration") ;
			ret = false ;
		}
		
		if (ret == true) {
			List<SourceBeanAttribute> elementList = skeletonBean.getContainedAttributes() ;
			for (SourceBeanAttribute element : elementList) {
				List<SourceBeanAttribute> sourceBeanAttributeList = ConfigSingleton.getInstance().getContainedAttributes() ;
				for (SourceBeanAttribute sourceBeanAttribute : sourceBeanAttributeList) {
						try {
							searchAndMerge(element, sourceBeanAttribute) ;
						} catch (SourceBeanException e) {
							System.out.println("SkeletonManager: unattended exception while merging attributes\n") ;
							e.printStackTrace();
						}
				}
			}
		}
		
		return ret ;
	}
	
	public static SourceBean clearSkeletonBean(SourceBean beanToClear) {
		String skeletonId = (String) beanToClear.getAttribute("SKELETONID");
		if(skeletonId!=null) {
			beanToClear.clearBean();
			try {
				beanToClear.setAttribute("skeletonID",skeletonId);
			} catch (SourceBeanException e) {
				// TODO Blocco catch generato automaticamente
				e.printStackTrace();
			}
		}
		return beanToClear;
	}
	
	private static void searchAndMerge(SourceBeanAttribute skeleton, SourceBeanAttribute config) throws SourceBeanException {
		SourceBean skeletonElementBean = (SourceBean)skeleton.getValue() ;
		String skeletonId = (String)skeletonElementBean.getAttribute("skeletonId") ;
		String skeletonKey = skeleton.getKey() ; 

		if (config.getValue() instanceof SourceBean) {
			SourceBean targetElementBean = (SourceBean)config.getValue() ;
			// Test for key match; es: BUTTONS with BUTTONS key
			if (skeletonKey.equalsIgnoreCase(config.getKey())) {
				String targetSkeletonId = (String)targetElementBean.getAttribute("skeletonId") ;
				if (targetSkeletonId != null && targetSkeletonId.equalsIgnoreCase(skeletonId)) {
					merge(skeletonElementBean, targetElementBean) ;
					return ;
				}
			}
			List<SourceBeanAttribute> containedAttributes = targetElementBean.getContainedAttributes() ;
			for (SourceBeanAttribute sourceBeanAttribute : containedAttributes) {
				searchAndMerge(skeleton, sourceBeanAttribute) ;
			}
		}
	}
	
	private static void merge(SourceBean skeleton, SourceBean config) throws SourceBeanException {
		if (skeleton != config) {
			List<SourceBeanAttribute> containedAttributes = skeleton.getContainedSourceBeanAttributes() ;
			for (SourceBeanAttribute attribute : containedAttributes) {
				config.setAttribute(attribute) ;
			}
		}
	}
}
