/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dao;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.util.StatementProvider;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * @author scarciello
 *
 */
public abstract class AbstractPCMLDAO implements Serializable, IFaceQueryProvider {
	protected ArrayList _parameterList = null ;
	protected DefaultRequestContext _requestContext ;
	protected SourceBean _configQuery ;
	protected String _statementType ;
	protected Object _returnCode ;

	/**
	 * 
	 */
	private static final long serialVersionUID = 3709406493333955577L;
	
	public final String getQuery(StatementProvider statementProvider) {
		String ret = "";
		_configQuery = statementProvider.getConfigQuery() ;
		_requestContext = statementProvider.getRequestContext() ;
		_statementType = _configQuery.getName() ;
		
		try {
			ret = getQuery();
		} catch (Exception e) {
			// TODO Blocco catch generato automaticamente
			e.printStackTrace();
		}
		return "";
	}
	
	protected abstract String getQuery() throws Exception ;

	protected abstract String getListQuery() ;
	protected abstract String getDeleteQuery() ;
	protected abstract String getInsertQuery() ;
	protected abstract String getUpdateQuery() ;
	protected abstract String getSelectQuery() ;
	
	protected abstract String[] getParametersForUpdateQuery() ;
	protected abstract String[] getParametersForSelectQuery() ;
	

	protected abstract String getUserName();
	protected abstract String getProgramName();
	protected abstract String getCurrentDate();
	protected abstract String getCurrentTime();
	protected abstract int getVersion() throws Exception;

	/**
	 * Ricerca e ritorna lo statement nella configurazione
	 * @param statementName
	 * @return
	 */
	protected String getStatementFromConfiguration(String statementName) {
        return SQLStatements.getStatement(statementName);
	}
	
	/**
	 * Dal nome del bean di configurazione ricerca e ritorna lo statement associato
	 * @param configQuery
	 * @return
	 */
	protected String getStatement() {
		String statementName = "" ;
		if (_statementType.equalsIgnoreCase("LIST_QUERY"))
			statementName = getListQuery() ;
		else if (_statementType.equalsIgnoreCase("DELETE_QUERY"))
			statementName = getDeleteQuery() ;
		else if (_statementType.equalsIgnoreCase("UPDATE_QUERY"))
			statementName = getUpdateQuery() ;
		else if (_statementType.equalsIgnoreCase("INSERT_QUERY"))
			statementName = getInsertQuery() ;
		else if (_statementType.equalsIgnoreCase("SELECT_QUERY"))
			statementName = getSelectQuery() ;
		
		return getStatementFromConfiguration(statementName) ;
	}

	public Object getReturnCode() {
		return _returnCode;
	}
}
