/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.dispatching.action.list.smart.impl;

import java.lang.reflect.Method;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.action.list.smart.impl.DefaultSmartLookupAction;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.paginator.smart.IFaceListProvider;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ConfigRowHandler;

/**
 * @author grassi
 *
 */
public class ActLookupDaoField extends DefaultSmartLookupAction {
	private static final long serialVersionUID = 1L;
   @Override
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
	   	SourceBean request = handleTableName(serviceRequest, serviceResponse) ;
		super.service(request, serviceResponse);
	}
	 
    /**
     * Load in service request TableName Parameter
     * @param request
     * @param response
     * @throws Exception
     */
    private SourceBean handleTableName(SourceBean request, SourceBean response) throws Exception {
    	SourceBean ret = request ;
		SourceBean statement = (SourceBean) getConfig().getAttribute("ROW_PROVIDER.CONFIG.TABLE_NAME_QUERY");
		if (statement == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"DefaultDetailModule::select: select_query non impostata nel modulo");
			getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.ERROR, "TABLE_NAME_QUERY not found in lookup Action configuration"));
		} else {
			String classDaoName = (String)(new ConfigRowHandler()).getConfigRow(statement, getRequestContainer(), getResponseContainer());
			if (classDaoName == null) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
						"DefaultDetailModule::select: nessuna riga selezionata");
				getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.ERROR, "Module is not a Smart Type or doesn't have a dao CLASS"));
			} // if (rowObject == null)
			else {
				try {
					Class cls = Class.forName(classDaoName) ;
					String fileName =(String) cls.getMethod("getFileName").invoke(null, new Object[0]);
		            ret.setAttribute("FILENAME", fileName) ;
		            String environmentName =(String) cls.getMethod("getEnvironmentName").invoke(null, new Object[0]) ;
		            ret.setAttribute("ENVIRONMENTNAME", environmentName) ;
		         }
		         catch (Throwable e) {
						TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
						"DefaultDetailModule::Errore durante l'invocazione dei metodi per la class" + classDaoName);
						getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.ERROR, "Class " + classDaoName + " not found or problem in method getFileName(), getEnvironmentName()  invocation"));
		         }
	         }
		} // if (!(rowObject instanceof SourceBean))
		return ret ;
    }
}