/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.dispatching.module.detail.impl;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import it.eng.spago.base.Constants;
import it.eng.spago.base.NullValue;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.module.detail.AbstractDetailModule;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ConfigRowHandler;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.QueryExecutor;
import it.eng.spago.validation.EMFValidationError;

/**
 * @author grassi
 * 
 */
public class DefaultConfigDetailModule extends AbstractDetailModule {
	private static final long serialVersionUID = 1L;

	private String _templatePath = (String)ConfigSingleton.getInstance().getAttribute("DAOGEN.pathTemplate");

	public DefaultConfigDetailModule() {
		super();
	} // public DefaultConfigDetailModule()

    public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
    	if(! serviceRequest.containsAttribute("FILE_PATH")) {
    		serviceResponse.updAttribute("FILE_PATH", ConfigSingleton.getInstance().getAttribute("DAOGEN.pathXMLFile"));
			serviceRequest.updAttribute("FILE_PATH", ConfigSingleton.getInstance().getAttribute("DAOGEN.pathXMLFile"));
    	}
    	super.service(serviceRequest, serviceResponse) ;
    }
	
	public boolean select(SourceBean request, SourceBean response) {
		InitializerIFace serviceInitializer = (InitializerIFace) this;
		SourceBean statement = (SourceBean) serviceInitializer.getConfig()
				.getAttribute("QUERIES.SELECT_QUERY");
		if (statement == null) {
			TracerSingleton
					.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
							"DefaultDetailModule::select: select_query non impostata nel modulo");
			return false;
		}

		Object rowObject = (new ConfigRowHandler()).getConfigRow(statement,
				getRequestContainer(), getResponseContainer());
		if (rowObject == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"DefaultDetailModule::select: nessuna riga selezionata");
			getErrorHandler().addError(
					new EMFUserError(EMFErrorSeverity.WARNING, 10006));
			return false;
		} // if (rowObject == null)
		if (!(rowObject instanceof SourceBean)) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"DefaultDetailModule::select: pi� righe selezionate");
			getErrorHandler().addError(
					new EMFUserError(EMFErrorSeverity.WARNING, 10006));
			return false;
		} // if (!(rowObject instanceof SourceBean))
		try {
			response.setAttribute((SourceBean) rowObject);
		} // try
		catch (Exception ex) {
			TracerSingleton
					.log(
							Constants.NOME_MODULO,
							TracerSingleton.CRITICAL,
							"DefaultDetailModule::select: response.setAttribute((SourceBean)rowObject)",
							ex);
		} // catch (Exception ex)
		return true;
	} // public boolean select(SourceBean request, SourceBean response)

	public boolean update(SourceBean request, SourceBean response) {
		SourceBean statement = (SourceBean) getConfig().getAttribute(
				"QUERIES.UPDATE_QUERY");
		if (statement != null) {
			boolean ret = (new ConfigRowHandler()).updateConfigRow(statement,
					getRequestContainer(), getResponseContainer());
			try {
				SourceBean retBean = new SourceBean("RETURN_TO_LIST");
				retBean.setAttribute("RETURN_TO_LIST", "true");
				getServiceResponse().setAttribute(retBean);
			} catch (SourceBeanException ex) {
				TracerSingleton.log(Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"DefaultConfigDetailModule::insert:", ex);
			} // catch (Exception ex)
			return ret;
		} else {
			statement = (SourceBean) getConfig().getAttribute(
					"QUERIES.DELETE_QUERY");
			if (statement == null) {
				TracerSingleton
						.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
								"DefaultDetailModule::select: update_query non impostata nel modulo");
				return false;
			}
			if ((new ConfigRowHandler()).deleteConfigRow(statement,
					getRequestContainer(), getResponseContainer())) {

				// Pezza da finire con il default; attualmente elimina gli
				// attributi non valorizzati o a blank
				String val = "";

				try {
					String fieldSetNameNew = (String) request.getAttribute("FIELDSETNAMENEW"); 
					if ( fieldSetNameNew != null && fieldSetNameNew.length()>0) {
						request.delAttribute("FIELDSETNAME");
						SourceBeanAttribute attr = (SourceBeanAttribute) request
								.getAttributeItem("FIELDSETNAMENEW");
						attr.setKey("FIELDSETNAME");
					}
				} catch (SourceBeanException e1) {
					TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"DefaultConfigDetailModule::update:Error",e1);
					//e1.printStackTrace();
				}

				for (Object attr : request.getContainedSourceBeanAttributes()) {
					if (attr instanceof SourceBeanAttribute) {
						if (((SourceBeanAttribute) attr).getValue() instanceof String) {

							val = (String) ((SourceBeanAttribute) attr)
									.getValue();
							try {

								if (val != null || val.trim().length() == 0) {
									request
											.delAttribute(((SourceBeanAttribute) attr)
													.getKey());
								}
							} catch (SourceBeanException e) {
								TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"DefaultConfigDetailModule::sourceBeanFromNode::update",e);
								//e.printStackTrace();
							}
						}
					}
				}

				return insert(request, response);
			} else
				return false;
		}
	} // public boolean update(SourceBean request, SourceBean response)

	public void handleDetailNew(SourceBean request, SourceBean response) {
		// TODO Stub di metodo generato automaticamente
	}

	public boolean insert(SourceBean request, SourceBean response) {
		boolean ret = false;

		boolean alreadyExists = select(request, response);
		getErrorHandler().clear();
	
		if (alreadyExists) {
			//TODO Check message
			getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.ERROR,	"L'elemento esiste gi"));
		} else {
			try {
				String templateName = getTemplateName();
				if (templateName == null) {
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.ERROR,	"Template NOT FOUND"));
					return false;
				}
				templateName = templateName.trim();
				StringBuffer template = readTemplateFile(templateName);
				StringBuffer templateSaved = new StringBuffer(template);
				template = new StringBuffer(templateSaved);
				replaceToken(template, "<?ParametroAddizionale?>", "");
				for (Object par : getConfig().getAttributeAsList("QUERIES.INSERT_QUERY.PARAMETER")) {
					String value = ""+ ContextScooping.getScopedParameter(this,	(SourceBean) par);
					value = value.trim();
					String parName = "<?"
							+ ((SourceBean) par).getAttribute("value")
							+ "?>";
					replaceToken(template, parName, value);
				}
				SourceBean newBean = SourceBean.fromXMLString(template.toString());
				/*
				 * String key =
				 * (String)getConfig().getAttribute("QUERIES.INSERT_QUERY.KEY") ;
				 * SourceBean configurationKey =
				 * (SourceBean)ConfigSingleton.getInstance().getAttribute(key) ;
				 * configurationKey.setAttribute(newBean) ;
				 * ConfigSingleton.getInstance().getAttribute(key) ; //
				 * DEBUG
				 */
				SourceBean statement = (SourceBean) getConfig().getAttribute("QUERIES.INSERT_QUERY");
				(new ConfigRowHandler()).insertConfigRow(statement,
						newBean, getRequestContainer(),
						getResponseContainer());
				try {
					SourceBean retBean = new SourceBean("RETURN_TO_LIST");
					retBean.setAttribute("RETURN_TO_LIST", "true");
					getServiceResponse().setAttribute(retBean);
				} catch (SourceBeanException ex) {
					TracerSingleton.log(Constants.NOME_MODULO,
							TracerSingleton.CRITICAL,
							"DefaultConfigDetailModule::insert:", ex);
				} // catch (Exception ex)
				ret = autoSave(request, newBean) ;
			} catch (Exception ex) {
				TracerSingleton.log(Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"DefaultDetailModule::insert:", ex);
				getErrorHandler().addError(
						new EMFUserError(EMFErrorSeverity.ERROR,
								"Errore durante inserimento"));
			}
		}
		return ret;
	}
	
	protected boolean autoSave(SourceBean request, SourceBean beanToSave) {
		boolean ret = true ;
		String autoSaveEnable = (String)request.getAttribute("AUTOSAVE") ;
		if (autoSaveEnable != null && autoSaveEnable.trim().equalsIgnoreCase("S")) {
			String filePath = (String)request.getAttribute("FILE_PATH", ConfigSingleton.getInstance().getAttribute("DAOGEN.pathXMLFile")) ;
			String fileName = null;
			String name = (String)beanToSave.getAttribute("NAME");
			if(name.equalsIgnoreCase("pag"+request.getAttribute("NAME"))) 
				fileName = name+".xml";
			else {
				SourceBean validation = (SourceBean) beanToSave.getAttribute("VALIDATION");
				if(validation!=null)
					fileName = (String)request.getAttribute("NAME") + "Validation.xml" ;
				else
					fileName = (String)request.getAttribute("NAME") + ".xml" ;
			}
			String xmlOut = beanToSave.toString(true, true) ;
			xmlOut = xmlOut.substring(xmlOut.indexOf('\n')) ;
			String blockHead = (String)getConfig().getAttribute("QUERIES.SELECT_QUERY.KEY") ;
			blockHead = blockHead.substring(0, blockHead.indexOf('.')) ;
			try {
				new File(filePath).mkdirs() ;
		        BufferedWriter outFile = new BufferedWriter(new FileWriter(new File(filePath, fileName)));
		        outFile.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
				DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
		        Date date = new Date();
		        outFile.write("<!-- Creation date and time :"+dateFormat.format(date)+" -->\n");
		        outFile.write("<"+blockHead+">");
		        outFile.write(xmlOut);
		        outFile.write("</"+blockHead+">");
		        outFile.close();
		    } catch (IOException e) {
				TracerSingleton.log(Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"DefaultDetailModule::insert:", e);
				getErrorHandler().addError(
						new EMFUserError(EMFErrorSeverity.ERROR,
								"Errore durante la scrittura del file: " + fileName + " nella cartella: " + filePath));
		    }
		}
		
		return ret ;
	}

	private String getTemplateName() throws Exception {
		String pool = (String) getConfig().getAttribute("pool");
		SourceBean templateQuery = (SourceBean) getConfig().getAttribute(
				"QUERIES.TEMPLATE_QUERY");
		SourceBean selectedTemplateBean = (SourceBean) QueryExecutor
				.executeQuery(getRequestContainer(), getResponseContainer(),
						pool, templateQuery, "SELECT");

		return (String) selectedTemplateBean.getAttribute("ROW.refTemplate");
	}

	/**
	 * Sostiutisce "parti" all'interno dello StringBuffer passato
	 * 
	 * @param templateFile
	 *            StringBuffer contenente il template da modificare
	 * @param tplkey
	 *            String contiene la chiave da ricercare all'interno del
	 *            template
	 * @param newString
	 *            String con la nuova parte da sostituire alla tplkey trovata
	 * @return count Numero di blocchi sostituiti
	 * 
	 * @throws IOException
	 */
	protected int replaceToken(StringBuffer templateFile, String tplkey,
			String newString) throws IOException {
		int countReplace = 0, index = 0, tplKeySize = tplkey.length();
		while ((index = templateFile.toString().toUpperCase().indexOf(tplkey.toUpperCase(), index)) != -1) {
			countReplace++;
			templateFile.replace(index, index + tplKeySize, newString);
		}
		return countReplace;
	}

	/**
	 * Legge e carica nello StringBuffer ritornato il contenuto del template
	 * 
	 * @return
	 * @throws Exception
	 */
	protected StringBuffer readTemplateFile(String nomeTemplate)
			throws IOException {
		StringBuffer fileContent = new StringBuffer();
		File file = new File(ConfigSingleton.getRootPath() +_templatePath, nomeTemplate.trim());
		FileReader fr = null;
		try {
			fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			String riga;
			while ((riga = br.readLine()) != null)
				fileContent.append(riga + "\n");
			br.close();
		} catch (FileNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"DefaultConfigDetailModule::readTemplateFile::Error",e);
			//e.printStackTrace();
			throw e;
		} catch (IOException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"DefaultConfigDetailModule::readTemplateFile::Error",e);
			//e.printStackTrace(); // TODO: Da gestire log, messageHandler
			// e
			// redirect a PaginaStandard
			throw e;
		} finally {
			if (fr != null)
				fr.close();
		}

		return fileContent;
	}
}