/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.dispatching.module.list.config.impl;

import it.eng.spago.base.SourceBean;
import it.eng.spago.daogen.dispatching.module.detail.impl.DefaultConfigDetailModule;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.util.QueryExecutor;

import java.util.List;

/**
 * @author grassi
 *
 */
public class ConfigLookupModule extends DefaultConfigDetailModule {
	private static final long serialVersionUID = 1L;

	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		String pool = (String)getConfig().getAttribute("pool") ;
    	SourceBean fieldsQuery = (SourceBean)getConfig().getAttribute("QUERIES.FIELDS_QUERY") ;
    	SourceBean selectedFieldsBean = (SourceBean)QueryExecutor.executeQuery(getRequestContainer(),getResponseContainer(), pool, fieldsQuery, "SELECT") ;
    	
    	// Recover Module & Page Name (After validation)
    	String name = (String)serviceRequest.getAttribute("Name") ;
    	if (name.endsWith("Validation")) {
    		name = name.substring(0, name.indexOf("Validation")) ;
    	}
    	
    	// Lookups generations only for List Module
    	if (name.endsWith("List")) {
    		String nameWithoutList = name.replaceAll("List", "");
    		serviceRequest.updAttribute("Name", nameWithoutList + "Lookup") ;
    		nameWithoutList = nameWithoutList.replaceAll("mod","");
    		serviceRequest.updAttribute("LOOKUP_PAGE_NAME", "pag"+nameWithoutList) ;
    		serviceRequest.updAttribute("LOOKUP_MODULE_NAME", name) ;
    		
    		addLookupFieldElements(serviceRequest, serviceResponse) ;
    		super.service(serviceRequest, serviceResponse) ;
			getResponseContainer().getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "All Files are made")) ;
    	}
	}
	
	private void addLookupFieldElements(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		String eleXML = (String)serviceRequest.getAttribute("ElencoFields") ;
		SourceBean fieldsBean = SourceBean.fromXMLString("<FIELDS>" + eleXML + "</FIELDS>") ;
		List<SourceBean> fieldList = fieldsBean.getAttributeAsList("FIELD") ;

		StringBuffer lookupField = new StringBuffer() ;
		String destId="", showLens="TRUE", sourceId="" ;
		for (SourceBean field : fieldList) {
			destId = (String)field.getAttribute("Name") ;
			sourceId = (String)field.getAttribute("id") ;
			
			lookupField.append("<FIELD destid=\"" + destId +"\" showlens=\"" + showLens + "\" sourceid=\"" + sourceId +"\"/>\n") ;
			showLens = "FALSE" ;
		}
		serviceRequest.updAttribute("LookupFieldElements", lookupField.toString()) ;
	}
}