/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.dispatching.module.list.config.impl;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.dbaccess.sql.result.std.DefaultScrollableDataResult;
import it.eng.spago.dispatching.module.AbstractModule;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.services.ServiceCatalog;
import it.eng.spago.services.ServiceInterface;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.QueryExecutor;
import it.eng.spago.util.StatementProvider;
import it.eng.spago.util.sqlField.SqlFieldCreator;
import it.eng.spago.util.sqlField.SqlFieldType;
import it.eng.spago.validation.EMFValidationError;

/**
 * Configuration sample for this module :
 * 
 * 	<MODULE name="ModFilesAmbienteCreaCampi" class="it.eng.spago.dispatching.module.detail.impl.FieldCreatorModule">
 * 	 	<CONFIG title="Creazione campi DB via jdbc" pool="daopool">
 * 	 		<QUERIES>
 *				<QUERY_CAMPI_LOG statement="SELEZIONA_CAMPI_LOG">
 *				    <PARAMETER scope="SERVICE_REQUEST" type="RELATIVE" value="codAmbiente" />
 *			    </QUERY_CAMPI_LOG>
 * 	 			<QUERY_TIPO_CAMPI statements="ELENCO_TIPO_CAMPI" />
 *	 			<INSERT_CAMPI_TABELLA statement="INSERT_CAMPI_TABELLA" />
 *			  	<PARAMETER type="RELATIVE" name="codAmbiente"     scope="SERVICE_REQUEST" /> 
 *              <PARAMETER type="RELATIVE" name="libreriaRiferimento"     scope="SERVICE_REQUEST" /> 
 *			  	<PARAMETER type="RELATIVE" name="fileFisico"      scope="SERVICE_REQUEST" /> 
 *			  	<PARAMETER type="RELATIVE" name="prefissoCampi"   scope="SERVICE_REQUEST" /> 
 *				<PARAMETER type="RELATIVE" name="poolConnessione" scope="SERVICE_REQUEST" />
 * 			</QUERIES>
 * 	 	</CONFIG>
 *	</MODULE>
 *
 */
public class  FieldCreatorModule extends AbstractModule {
	private static final long serialVersionUID = 1L;
	
	private static final String _charsNotAllowed = ".,;:/!%&()?^=�[]+* ";
	private static final String ERROR_DICTIONARY_FIELD = "10125";
	
	@SuppressWarnings("unchecked")
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		try {
			String prefissoCampi  = (String)getParameter("prefissoCampi") ;
			int dimensionePrefisso = 0;
			if(prefissoCampi!=null) 
				dimensionePrefisso = prefissoCampi.length();
			String poolName  = (String)getParameter("poolConnessione") ;
			String fileFisico  = (String)getParameter("fileFisico") ;
			String codAmbiente  = (String)getParameter("codAmbiente") ;
			// String libreriaRiferimento  = (String)getParameter("libreriaRiferimento") ;
			String libreria  = (String)getParameter("LIBRERIA") ;
			String daoPool = (String) getConfig().getAttribute("pool") ;
			SourceBean queryCampiLog = (SourceBean)getConfig().getAttribute("QUERIES.QUERY_CAMPI_LOG") ;
			SourceBean campiLog = (SourceBean)QueryExecutor.executeQuery(getRequestContainer(), getResponseContainer(), daoPool, queryCampiLog, "SELECT") ;
			SourceBean queryTipoCampi = (SourceBean)getConfig().getAttribute("QUERIES.QUERY_TIPO_CAMPI") ;
			SourceBean typesList = getTypesList(queryTipoCampi) ;
			SourceBean queryDictionary = (SourceBean)getConfig().getAttribute("QUERIES.QUERY_DICTIONARY") ; 
			SourceBean campiDictionary = (SourceBean)QueryExecutor.executeQuery(getRequestContainer(), getResponseContainer(), daoPool, queryDictionary, "SELECT") ;
			SqlFieldCreator sqlFieldCreator = new SqlFieldCreator(poolName, fileFisico, libreria, campiLog, prefissoCampi, typesList) ;
			List<SqlFieldType>fieldTypeList = sqlFieldCreator.getFieldTypeList(serviceRequest) ;
			updateKey(serviceRequest, fieldTypeList) ;
			String elencoCampiNonCorretti = "";
			for (SqlFieldType field : fieldTypeList) {
				boolean trovato=false;
				for (SourceBean cd : (List<SourceBean>) campiDictionary.getAttributeAsList("ROW")) {
					String dictionaryField = (String) cd.getAttribute("DICTIONARYFIELD");
					
					if(field.getNomeCampo().substring(dimensionePrefisso).trim().equalsIgnoreCase(dictionaryField.trim())) {
						trovato=true;
						String labelField  = (String) cd.getAttribute("FIELDLANG01");
						String webField = (String) cd.getAttribute("WEBNAMEFIELD") ;
						int tipoCampo = (Integer) cd.getAttribute("TIPOFIELD");
						String useInInsert = (String) cd.getAttribute("USEININSERT");
						String useInUpdate = (String) cd.getAttribute("USEINUPDATE");
						field.setTipoCampo(typesList, tipoCampo);
						for(int i=0;i<_charsNotAllowed.length();i++) {
							char carattere = _charsNotAllowed.charAt(i);
							webField = webField.replace(carattere,'-');
						}
						webField = webField.replaceAll("-","");
						String templateWidget = (String) cd.getAttribute("TEMPLATEWIDGET");
						field.setDescrizioneCampo(labelField.trim());
						field.setCampoWeb(webField.trim());
						field.setWidgetCampo(templateWidget.trim());
						field.setUseInInsert(useInInsert);
						field.setUseInUpdate(useInUpdate);
					}
				}
				// Se non riesce a rintracciare il campo nel dictionary prova ad assegnargli le impostazioni del tipo di default
				if(!trovato) {
					int fldType = field.getSqlType();
					for(SourceBean singleType : (List<SourceBean>) typesList.getAttributeAsList("ROW")) {
						int sqlType = (Integer) singleType.getAttribute("CODSQLTYPE");
						String defType = (String) singleType.getAttribute("defaultType", "N");
						if(fldType==sqlType && defType.equalsIgnoreCase("Y")) {
							int tipoCampo = (Integer) singleType.getAttribute("CODTIPO");
							field.setTipoCampo(typesList, tipoCampo);
							String useInInsert = (String) singleType.getAttribute("USEININSERT");
							String useInUpdate = (String) singleType.getAttribute("USEINUPDATE");
							field.setUseInInsert(useInInsert);
							field.setUseInUpdate(useInUpdate);
							String templateWidget = (String) singleType.getAttribute("widgetTipo");
							field.setWidgetCampo(templateWidget.trim());
						}
					}
				}
				if(!trovato) {
					if(elencoCampiNonCorretti.length()>0) 
						elencoCampiNonCorretti+=", ";
					elencoCampiNonCorretti+=field.getNomeCampo();
				}
			}
			writeField(codAmbiente, fileFisico, fieldTypeList) ;
			if (elencoCampiNonCorretti.length()>0) {
				String fieldName = "NONAME";
				EMFErrorHandler errorHandler = this.getErrorHandler();
	            List errorParams = new ArrayList(1);
	            errorParams.add(elencoCampiNonCorretti);
				errorHandler.addError(new EMFValidationError(EMFErrorSeverity.WARNING,fieldName,ERROR_DICTIONARY_FIELD,errorParams));
			}

		} catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "FieldCreatorModule::service:", (Exception) ex);
			getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, "Error on FieldCreatorModule", (ex instanceof EMFInternalError) ? ((EMFInternalError)ex).getNativeException() : ex));
		} // catch (Exception ex) try
		
	}
	
	/**
	 * Richiama il servizio su AS per analizzare le chiavi sulla vista logica
	 * @param serviceRequest
	 * @param fieldTypeList
	 * @throws Exception
	 */
	protected void updateKey(SourceBean serviceRequest, List<SqlFieldType>fieldTypeList) throws Exception {
		String indiceAggiornamento = (String)serviceRequest.getAttribute("indiceAggiornamento") ;
		String fileFisico = (String)serviceRequest.getAttribute("fileFisico") ;
			if (indiceAggiornamento != null &&
					fileFisico != null && 
					! indiceAggiornamento.equalsIgnoreCase(fileFisico)) {
			ServiceInterface  serviceSOA_KEYFLD = ServiceCatalog.getInstance().getService(this, "SOA_KEYFLD") ;
			SourceBean risposta = (SourceBean)serviceSOA_KEYFLD.invoke(this) ;
			String outKey = (String)risposta.getAttribute("KEYOUT") ;
			if (outKey != null) {
				SourceBean keyBean = SourceBean.fromXMLString(outKey) ;
				String nomeCampo ;
				for (SqlFieldType field : fieldTypeList) {
					nomeCampo = field.getNomeCampo() ;
					if (keyBean.getFilteredSourceBeanAttribute("ROW", "FIELDNAME", nomeCampo) != null) {
						field.setCampoChiave(true) ;
						field.setCampoNull(false) ;
					}
				}		
			}
		}		
	}
	
	/**
	 * Loads all defined types
	 * @param poolName
	 * @param queryTipoCampi
	 * @return
	 * @throws Exception
	 */
	private SourceBean getTypesList(SourceBean queryTipoCampi)  throws Exception {
		SourceBean rows = null ;
		try {
			rows = (SourceBean)QueryExecutor.executeQuery(getRequestContainer(),
					                                      getResponseContainer(),
					                                      (String)getConfig().getAttribute("pool"),
                                                          queryTipoCampi, QueryExecutor.READ) ;
		} catch (Exception ex) {
			TracerSingleton.log("Spago", TracerSingleton.CRITICAL,
					"FieldCreatorModule::getTypesList - Errore durante il caricamento dei type.",  ex);
			throw ex;
		}
		return rows ;
	}
	
	/**
	 * Ritorna il valore di un PARAMETER in configurazione
	 * @param parName
	 * @return
	 * @throws Exception
	 */
	private Object getParameter(String parName)  throws Exception {
		return ContextScooping.getScopedParameter(this, (SourceBean)getConfig().getFilteredSourceBeanAttribute("QUERIES.PARAMETER", "NAME", parName)) ;
	}
	
	protected void writeField(String codAmbiente, String fileFisico, List<SqlFieldType> fieldTypeList) throws Exception {
		DataConnection dataConnection = null;
		try {
			dataConnection = QueryExecutor.openConnection((String)getConfig().getAttribute("pool"), _requestContainer);
			ArrayList<DataField>inputParameters = null;
			String statement = (String)getConfig().getAttribute("QUERIES.INSERT_CAMPI_TABELLA.statement") ;
			for (SqlFieldType sqlField : fieldTypeList) {
				if (sqlField != null) {
					String useInInsert = sqlField.getUseInInsert();
					if(useInInsert==null) 
						useInInsert="Y";
					String useInUpdate = sqlField.getUseInUpdate();
					if(useInUpdate==null) 
						useInUpdate="Y";
					inputParameters = new ArrayList<DataField>() ;
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, codAmbiente)) ;                    // AMB_CDE
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, fileFisico)) ;                     // FIL_PH
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, sqlField.getProgressivoCampo())) ; // FLD_PRO
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getDescrizioneCampo())) ; // FLD_DES
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getNomeCampo())) ;        // FLD_FLD
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getCampoWeb())) ;         // FLD_WEB
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getWidgetCampo()==null? "Text":sqlField.getWidgetCampo())) ;      // FLD_WIDGET 
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, sqlField.getLunghezzaCampo())) ;   // FLD_LENGTH 
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, (sqlField.getInternalLength() == 0 ? sqlField.getLunghezzaCampo() : sqlField.getInternalLength()))) ;   // FLD_ILEN
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, (sqlField.getExternalLength() == 0 ? sqlField.getLunghezzaCampo() : sqlField.getExternalLength()))) ;   // FLD_OLEN
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.isCampoChiave()?"Y":"N"));// FLD_KEY
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.isCampoNull()?"Y":"N")) ; // FLD_NUL
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, sqlField.getDecimaliCampo())) ;    // FLD_DEC
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, sqlField.getTipoCampo())) ;        // FLD_TYP
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.isCampoNull()?"N":"Y")) ; // FLD_MND
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, useInInsert)) ; // FLD_INS
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, useInUpdate)) ; // FLD_UPD
					QueryExecutor.executeStatement(dataConnection, SQLStatements.getStatement(statement), "INSERT", inputParameters) ;
				}
			}
		} // try
		catch (EMFInternalError ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "FieldCreatorModule::writeField:", (Exception) ex);
			throw ex;
		} // catch (Exception ex) try
		finally {
			Utils.releaseResources(dataConnection, null, null);
		} // finally try
	}
}