/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.dispatching.module.list.config.impl;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dispatching.module.AbstractModule;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.QueryExecutor;
import it.eng.spago.util.pcmlField.PcmlFieldCreator;
import it.eng.spago.util.sqlField.SqlFieldType;

/**
 * @author scarciello
 *
 */
public class PCMLFieldCreatorModule extends AbstractModule {

	/**
	 * 
	 */
	private static final long serialVersionUID = -922459427383570384L;


	/* (non Javadoc)
	 * @see it.eng.spago.dispatching.service.ServiceIFace#service(it.eng.spago.base.SourceBean, it.eng.spago.base.SourceBean)
	 */
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		try {
			String daoPool = (String) getConfig().getAttribute("pool") ;
			SourceBean queryCampiLog = (SourceBean)getConfig().getAttribute("QUERIES.QUERY_CAMPI_LOG") ;
			SourceBean campiLog = (SourceBean)QueryExecutor.executeQuery(getRequestContainer(), getResponseContainer(), daoPool, queryCampiLog, "SELECT") ;
			SourceBean queryTipoCampi = (SourceBean)getConfig().getAttribute("QUERIES.QUERY_TIPO_CAMPI") ;
			SourceBean typesList = getTypesList(queryTipoCampi) ;
			PcmlFieldCreator pcmlFieldCreator = new PcmlFieldCreator(campiLog, typesList);
			String codAmbiente = (String)getParameter("codAmbiente");
			String groupCode = (String) getParameter("groupCode");
			String pcmlCode = (String) getParameter("pcmlCode"); 
			String pcmlFileName = (String) getParameter("pcmlFile");
			StringBuffer pcmlFileData = readTemplateFile(pcmlFileName);
			if(pcmlFileData!=null) {
				SourceBean pcmlFile = SourceBean.fromXMLString(pcmlFileData.toString());
				List<SqlFieldType>fieldTypeList = pcmlFieldCreator.getFieldTypeList(pcmlFile) ;
				writeField(codAmbiente, groupCode, pcmlCode, fieldTypeList);
			}
		} catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "PCMLFieldCreatorModule::service:", (Exception) ex);
			getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, (ex instanceof EMFInternalError) ? ((EMFInternalError)ex).getNativeException() : ex));
		} // catch (Exception ex) try
	}

	private Object getParameter(String parName)  throws Exception {
		return ContextScooping.getScopedParameter(this, (SourceBean)getConfig().getFilteredSourceBeanAttribute("QUERIES.PARAMETER", "NAME", parName)) ;
	}
	
	protected StringBuffer readTemplateFile(String nomeTemplate) throws IOException {
		StringBuffer fileContent = new StringBuffer();
		File file = new File(nomeTemplate.trim());
		FileReader fr = null;
		try {
			fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			String riga = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
			fileContent.append(riga + "\n");
			while ((riga = br.readLine()) != null) {
				/*
				if(riga.trim().equalsIgnoreCase("<pcml version=\"4.0\">"))
					riga = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
				if(riga.trim().equalsIgnoreCase("</pcml>"))
					riga = "";
				*/
				fileContent.append(riga + "\n");
			}
			br.close();
		} catch (FileNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"PCMLFieldCreatorModule::readTemplateFile::Error",e);
			//e.printStackTrace(); // TODO: Da gestire log, messageHandler
			// e
			// redirect a PaginaStandard
			throw e;
		} catch (IOException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"PCMLFieldCreatorModule::readTemplateFile::Error",e);
			//e.printStackTrace(); // TODO: Da gestire log, messageHandler
			// e
			// redirect a PaginaStandard
			throw e;
		} finally {
			if (fr != null)
				fr.close();
		}
		
		return fileContent;
	}

	protected String stripFieldName(String originalFieldName) {
		String returnValue = originalFieldName.toUpperCase();
		if(returnValue.indexOf("FILTER_")==0) {
			returnValue = originalFieldName.substring(7);
		}
		if(returnValue.indexOf("SELECT_")==0) {
			returnValue = originalFieldName.substring(7);
		}
		return returnValue;
	}

	private SourceBean getTypesList(SourceBean queryTipoCampi)  throws Exception {
		SourceBean rows = null ;
		try {
			rows = (SourceBean)QueryExecutor.executeQuery(getRequestContainer(),
					                                      getResponseContainer(),
					                                      (String)getConfig().getAttribute("pool"),
                                                          queryTipoCampi, QueryExecutor.READ) ;
		} catch (Exception ex) {
			TracerSingleton.log("Spago", TracerSingleton.CRITICAL,
					"FieldCreatorModule::getTypesList - Errore durante il caricamento dei type.",  ex);
			throw ex;
		}
		return rows ;
	}


	private void writeField(String codAmbiente, String groupCode,String pcmlCode, List<SqlFieldType> fieldTypeList) throws Exception {
		DataConnection dataConnection = null;
		try {
			dataConnection = QueryExecutor.openConnection((String)getConfig().getAttribute("pool"), _requestContainer);
			ArrayList<DataField>inputParameters = null;
			String statement = (String)getConfig().getAttribute("QUERIES.INSERT_CAMPI_TABELLA.statement") ;
			for (SqlFieldType sqlField : fieldTypeList) {
				if (sqlField != null) {
					String useInInsert = sqlField.getUseInInsert();
					if(useInInsert==null) 
						useInInsert="Y";
					String useInUpdate = sqlField.getUseInUpdate();
					if(useInUpdate==null) 
						useInUpdate="Y";
					inputParameters = new ArrayList<DataField>() ;
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, codAmbiente)) ;                    // AMB_CDE
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, groupCode)) ;                      // GRP_CDE
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, pcmlCode)) ;                       // PCM_CDE
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, sqlField.getProgressivoCampo())) ; // FLD_PRO
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getUsage() )) ;           // FLD_USE
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getDescrizioneCampo())) ; // FLD_DES
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getNomeCampo())) ;        // FLD_FLD
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getCampoWeb())) ;         // FLD_WEB
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.getWidgetCampo()==null? "Text":sqlField.getWidgetCampo())) ;      // FLD_WIDGET 
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, sqlField.getLunghezzaCampo())) ;   // FLD_LENGTH 
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, (sqlField.getInternalLength() == 0 ? sqlField.getLunghezzaCampo() : sqlField.getInternalLength()))) ;   // FLD_ILEN
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, (sqlField.getExternalLength() == 0 ? sqlField.getLunghezzaCampo() : sqlField.getExternalLength()))) ;   // FLD_OLEN
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.isCampoChiave()?"Y":"N"));// FLD_KEY
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, sqlField.getDecimaliCampo())) ;    // FLD_DEC
					inputParameters.add(dataConnection.createDataField("", Types.INTEGER, sqlField.getTipoCampo())) ;        // FLD_TYP
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, sqlField.isCampoNull()?"N":"Y")) ; // FLD_MND
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, useInInsert)) ; // FLD_INS
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, useInUpdate)) ; // FLD_UPD
					QueryExecutor.executeStatement(dataConnection, SQLStatements.getStatement(statement), "INSERT", inputParameters) ;
				}
			}
		} // try
		catch (EMFInternalError ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "FieldCreatorModule::writeField:", (Exception) ex);
			throw ex;
		} // catch (Exception ex) try
		finally {
			Utils.releaseResources(dataConnection, null, null);
		} // finally try
	}

}
