/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.filler;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.ServletException;

/**
 * @author Cattiv
 * 
 */
public abstract class AbstractFiller implements Serializable {
	private static final long serialVersionUID = 1L;

	private String _templatePath = (String)ConfigSingleton.getInstance().getAttribute("DAOGEN.pathTemplate");

	protected HashMap<String,String> _variablesMap = null;
	protected RequestContainer _requestContainer = null;
	protected SourceBean _serviceRequest = null;
	protected SourceBean _serviceResponse = null;
	protected SourceBean _content = null;
	
	public AbstractFiller(HashMap startMap, RequestContainer requestContainer, ResponseContainer responseContainer) {
		String businessType = responseContainer.getBusinessType();
		
		_variablesMap = startMap;
		_requestContainer = requestContainer;
		_serviceRequest = requestContainer.getServiceRequest();
		_serviceResponse = responseContainer.getServiceResponse();
		if(businessType!=null) {
			if(businessType.equalsIgnoreCase("ACTION") || businessType.equalsIgnoreCase("ACTION_NAME")) {
				_content = responseContainer.getServiceResponse();
			}
			else if(businessType.equalsIgnoreCase("PAGE")) {
				String serviceName = (String) requestContainer.getSessionContainer().getPermanentContainer().getAttribute("LAST_MODULE_EXECUTED") ;
				_content = (SourceBean)responseContainer.getServiceResponse().getAttribute(serviceName) ;
			}
		}
	}
	
	public abstract StringBuffer service(String templateName,SourceBean step) throws ServletException,IOException ; 
	
	protected String replaceMatches(String source) {
		String value = ""+source;

		Set keys = _variablesMap.keySet();
		for(Iterator i = keys.iterator();i.hasNext();) {
			String key = (String)i.next();
			String item = _variablesMap.get(key);
			value=value.replace("?"+key+"?", item);
		}
		
		return value;
	}

	protected SourceBean getRowsAtIndex(int index) {
		return ((SourceBean) _content.getAttributeAsList("ROWS").get(index));
	}
	
	protected int replaceToken(StringBuffer templateFile, String tplkey, String newString) throws IOException {
		int countReplace = 0, index = 0, tplKeySize = tplkey.length();
		while ((index = templateFile.indexOf(tplkey, index)) != -1) {
			countReplace++;
			templateFile.replace(index, index + tplKeySize, newString);
		}
		return countReplace;
	}

	protected StringBuffer readTemplateFile(String nomeTemplate) throws IOException {
		StringBuffer fileContent = new StringBuffer();
		File file = new File(ConfigSingleton.getRootPath() +_templatePath, nomeTemplate.trim());
		FileReader fr = null;
		try {
			fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			String riga;
			while ((riga = br.readLine()) != null)
				fileContent.append(riga + "\n");
			br.close();
		} catch (FileNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"AbstractFiller::readTemplateFile::Error",e);
			//e.printStackTrace(); // TODO: Da gestire log, messageHandler e
			// redirect a PaginaStandard
			throw e;
		} catch (IOException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"AbstractFiller::readTemplateFile::Error",e);
			//e.printStackTrace(); // TODO: Da gestire log, messageHandler e
			// redirect a PaginaStandard
			throw e;
		} finally {
			if (fr != null)
				fr.close();
		}

		return fileContent;
	}

}
