/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/

package it.eng.spago.daogen.filler;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.util.HashMap;

/**
 * Factory per la generazione dei Decorator
 * @author A. Grassi
 *
 */
public class FillerFactory implements Serializable {
    private static final long serialVersionUID = 1L;
    private static FillerFactory _instance = null;

    private FillerFactory() {
    }
    
    public static FillerFactory getInstance() {
        if (_instance == null) {
            synchronized (FillerFactory.class) {
                if (_instance == null) {
                    try {
                        _instance = new FillerFactory();
                    } // try
                    catch (Exception ex) {
                		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "FillerFactory::getInstance: errore creazione");
                        //ex.printStackTrace();
                    } // catch(Exception ex) try
                } // if (_instance == null)
            } // synchronized(ConfigSingleton.class)
        } // if (_instance == null)
        return _instance;
    } // public static ConfigSingleton getInstance()

    public static void release() {
        if (_instance != null) {
            synchronized (ConfigSingleton.class) {
                _instance = null;
            } // synchronized (ConfigSingleton.class)
        } // if (_instance != null)
    } // public static void release()
    
    /**
     * Ritorna la classe del filler associata al nome richiesto
     * @param className : nome della classe del filler da cercare
     * @return classe di tipo AbstractFiller
     * @throws Exception
     */
    public AbstractFiller getFiller(String className, HashMap map, RequestContainer requestContainer, ResponseContainer responseContainer) throws Exception {
    	AbstractFiller decObj = null ;
    	if (className == null || className.trim().length() == 0) {
    		className = "Default" ;
    	}
        Class[] intArgsClass = new Class[] {HashMap.class, RequestContainer.class, ResponseContainer.class};
        Object[] intArgs = new Object[] {map, requestContainer, responseContainer};
        Constructor intArgsConstructor;
        Class fillerDefinition = Class.forName(className);
        intArgsConstructor = fillerDefinition.getConstructor(intArgsClass);
        decObj = (AbstractFiller)intArgsConstructor.newInstance(intArgs);
    	return decObj ;
    }
}