/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.filler;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import javax.servlet.ServletException;

/**
 * @author Cattiv
 *
 */
public class ValidationFiller extends AbstractFiller {

	public ValidationFiller(HashMap startMap, RequestContainer requestContainer, ResponseContainer responseContainer) {
		super(startMap, requestContainer, responseContainer);
		// TODO Stub di costruttore generato automaticamente
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = -3238207148980178432L;

	/* (non Javadoc)
	 * @see it.eng.spago.daogen.filler.AbstractFiller#service(java.lang.String, it.eng.spago.base.SourceBean)
	 */
	@Override
	public StringBuffer service(String templateName, SourceBean step)
			throws ServletException, IOException {
		String fileFisico = (String) _serviceRequest.getAttribute("FILEFISICO");
		StringBuffer moduleTemplate = readTemplateFile(templateName);
		SourceBean fileAmbiente = getRowsAtIndex(0);
		SourceBean rigaFile = (SourceBean) fileAmbiente.getAttribute("ROW");
		SourceBean elencoCampi = (SourceBean) rigaFile.getAttribute("ROWS");
		String campiLista = "";
		String campiDettaglio="";
		
		String prefissoCampi = ((String)rigaFile.getAttribute("PREFISSOCAMPI")).trim();
		if(prefissoCampi!=null && prefissoCampi.trim().length()>0)
			prefissoCampi=prefissoCampi.trim()+"_";
		else
			prefissoCampi="";
		
		for(SourceBean field : (List<SourceBean>)elencoCampi.getAttributeAsList("ROW")) {
			String webName = prefissoCampi+((String) field.getAttribute("CAMPOWEB")).trim();
			String chiave = (String) field.getAttribute("CAMPOCHIAVE");
			String mandatoryField = "FALSE";
			String validator =(String)field.getAttribute("VALIDATOR");
			if(validator!=null) 
				validator=validator.trim().toUpperCase() ;
			else
				validator="";
			String mandatorio = (String)field.getAttribute("MANDATORIO");
    		String lunghezzaCampo = "" + field.getAttribute("LUNGHEZZAOUTPUT");
			
			if(chiave!=null && chiave.equalsIgnoreCase("Y")) {
				mandatoryField="TRUE";
			}
			if(mandatorio!=null && mandatorio.equalsIgnoreCase("Y")) {
				mandatoryField="TRUE";
			}
			campiDettaglio+="<FIELD mandatory=\""+mandatoryField+"\" name=\""+webName+"\"";
			if (validator!=null && validator.length()>0) 
				campiDettaglio+=" type=\""+validator+"\"";
			campiDettaglio+=" maxlength=\""+lunghezzaCampo+"\" />\n";
			campiLista+="<FIELD name=\"FILTER_"+webName+"\" ";
			if (validator!=null && validator.length()>0) 
				campiLista+=" type=\""+validator+"\"";
			campiLista+="/>\n";
		}

		String validationType = (String) step.getAttribute("type");
		SourceBean mainRelations = getRowsAtIndex(4);
		String validatorName = "";
		if(!mainRelations.getAttributeAsList("ROW").isEmpty() && validationType.equalsIgnoreCase("DETAIL")) {
			validatorName=fileFisico;
		}
		_variablesMap.put("validators", validatorName);
		_variablesMap.put("parametroAddizionaleLista","");
		_variablesMap.put("parametroAddizionaleDettaglio","");
		_variablesMap.put("listFields",campiLista);
		_variablesMap.put("detailFields", campiDettaglio);
		
		for(SourceBean parametro : (List<SourceBean>) step.getAttributeAsList("PARAMETER")) {
			String tplParName = (String) parametro.getAttribute("TPLPARNAME");
			String value = (String) parametro.getAttribute("VALUE");
			
			value = replaceMatches(value);
			replaceToken(moduleTemplate,"<"+tplParName+">",value);
		}

		// Inserimento formattatore
		try {
			SourceBean xmlFmt = SourceBean.fromXMLString(moduleTemplate.toString());
			moduleTemplate = new StringBuffer(xmlFmt.toXML());
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ValidationFiller::service::Error",e);
			//e.printStackTrace();
		}

		moduleTemplate.insert(0,"<textarea name=\""+validationType+"validation\" cols=92 rows=15 wrap=\"off\">\n");
		moduleTemplate.append("</textarea><br>\n");
		String fileName=(String)ConfigSingleton.getInstance().getAttribute("DAOGEN.pathXMLFile");
		String relativePath=(String)_serviceRequest.getAttribute("XMLPATH");
		if(relativePath!=null && relativePath.length()>0  && !relativePath.equals("xmlPath")) 
			fileName+=File.separatorChar+relativePath;
		fileName+=File.separatorChar+"mod"+fileFisico+validationType+"Validation.xml";
		moduleTemplate.insert(0,"<input type=\"text\" name=\""+validationType+"validationfilename\" size=\"115\" value=\""+fileName+"\"><br>");
		
		return moduleTemplate;
	}

}
