/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.filler;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import javax.servlet.ServletException;

/**
 * @author scarciello
 *
 */
public class ValidatorFiller extends AbstractFiller {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5749742710331013988L;

	public ValidatorFiller(HashMap startMap, RequestContainer requestContainer, ResponseContainer responseContainer) {
		super(startMap, requestContainer, responseContainer);
		// TODO Stub di costruttore generato automaticamente
	}

	/* (non Javadoc)
	 * @see it.eng.spago.daogen.filler.AbstractFiller#service(java.lang.String, it.eng.spago.base.SourceBean)
	 */
	@Override
	public StringBuffer service(String templateName, SourceBean step)
			throws ServletException, IOException {

		String fileFisico = (String) _serviceRequest.getAttribute("FILEFISICO");
		StringBuffer validatorTemplate = readTemplateFile(templateName);
		String poolName = (String) _serviceRequest.getAttribute("POOLCONNESSIONE");
		String queryValidator = "";
		
		// Inserire qui la creazione degli statements SQL per la validazione
		SourceBean mainRelations = getRowsAtIndex(4);
		for(SourceBean singleRelation : (List<SourceBean>)mainRelations.getAttributeAsList("ROW")) {
			String fileFisicoRelazionato = (String)singleRelation.getAttribute("FILEFISICORELAZIONATO");
			String filePackage = (String)singleRelation.getAttribute("FILEPACKAGE");
			
			queryValidator += "<LIST_QUERY class=\""+filePackage+"."+fileFisicoRelazionato+"DAO\">\n";

			for(SourceBean singleSubRelation : (List<SourceBean>)singleRelation.getAttributeAsList("ROWS")) {
				String queryName = (String)singleSubRelation.getAttribute("QUERY_NAME");
				if(queryName.equalsIgnoreCase("SELEZIONA_CAMPI_RELAZIONATI")) {
					for(SourceBean relationField : (List<SourceBean>)singleSubRelation.getAttributeAsList("ROW")) {
						String nomeCampoRelazionato = (String)relationField.getAttribute("NOMECAMPORELAZIONATO");
						String nomeCampo = (String)relationField.getAttribute("NOMECAMPO");
						queryValidator+="<FILTER_PARAMETER scope=\"SERVICE_REQUEST\" sql=\"T1."+nomeCampoRelazionato.trim()+"\" type=\"RELATIVE\" value=\""+nomeCampo+"\"/>";
					}
				}
			}

			queryValidator += "</LIST_QUERY>\n";
		}
		
		_variablesMap.put("poolValidatorName",poolName);
		_variablesMap.put("queryValidator",queryValidator);

		for(SourceBean parametro : (List<SourceBean>) step.getAttributeAsList("PARAMETER")) {
			String tplParName = (String) parametro.getAttribute("TPLPARNAME");
			String value = (String) parametro.getAttribute("VALUE");
			
			value = replaceMatches(value);
			replaceToken(validatorTemplate,"<"+tplParName+">",value);
		}

		// Inserimento formattatore
		try {
			SourceBean xmlFmt = SourceBean.fromXMLString(validatorTemplate.toString());
			validatorTemplate = new StringBuffer(xmlFmt.toXML());
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ValidatorFiller::service::Error",e);
			//e.printStackTrace();
		}

		validatorTemplate.insert(0,"<textarea name=\"validator\" cols=92 rows=15 wrap=\"off\">\n");
		validatorTemplate.append("</textarea><br>\n");
		String fileName=(String)ConfigSingleton.getInstance().getAttribute("DAOGEN.pathXMLFile");
		String relativePath=(String)_serviceRequest.getAttribute("XMLPATH");
		if(relativePath!=null && relativePath.length()>0  && !relativePath.equals("xmlPath")) 
			fileName+=File.separatorChar+relativePath;
		fileName+=File.separatorChar+fileFisico+"Validator.xml";
		validatorTemplate.insert(0,"<input type=\"text\" name=\"validatorfilename\" size=\"115\" value=\""+fileName+"\"><br>");
		return validatorTemplate;
	}

}
