/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.servlet;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.daogen.filler.AbstractFiller;
import it.eng.spago.daogen.filler.FillerFactory;
import it.eng.spago.tracing.TracerSingleton;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author Cattiv
 *
 */
public class ServletCreateCRUD extends AbstractServletFileCreate {
	private static final long serialVersionUID = -5161391272967213552L;

	//private static final String regExpVar = "\\?[^<>\\?]+\\?";
	private HashMap<String,String> _variablesMap = new HashMap<String,String>();
	
	private String _templatePath = (String)ConfigSingleton.getInstance().getAttribute("DAOGEN.pathTemplate");
	protected RequestContainer _requestContainer = null;
	protected ResponseContainer _responseContainer = null;
	protected SourceBean _serviceRequest = null;
	protected SourceBean _serviceResponse = null;
	
	protected void init(HttpServletRequest httpRequest)  throws ServletException {
		super.init(httpRequest);

		_requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
		_serviceRequest = _requestContainer.getServiceRequest();
		_responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);
		_serviceResponse = _responseContainer.getServiceResponse();
		String fileFisico = (String) _serviceRequest.getAttribute("FILEFISICO");
		_variablesMap.put("fileFisico", fileFisico);
	}	

	/**
	 * Write a File
	 * @param httpRequest
	 * @param out
	 * @param sourceParName
	 * @param filenameParName
	 * @throws ServletException
	 * @throws IOException
	 */
	protected void saveFile(HttpServletRequest httpRequest, PrintWriter out, String sourceParName, String filenameParName) throws ServletException {
		String source = "" +httpRequest.getParameter(sourceParName) ;
		String fileName = "" +httpRequest.getParameter(filenameParName) ;
		if (fileName != null && source != null) {
			try {
				makeDir(fileName) ;
				writeFile(fileName, source) ;
				out.println("<h2>"+fileName+"</h2><h1>written successfully</h1>") ;
		    } catch (IOException e) {
				out.println("<h1>Error in writing  file </h1>\n<h2>"+fileName+"</h2>\n") ;
				e.printStackTrace(out) ;
		    }
		}
	}
	
	protected void service(HttpServletRequest httpRequest,HttpServletResponse httpResponse) throws ServletException, IOException {
		init(httpRequest);
		setHeader(httpResponse);
		PrintWriter out = httpResponse.getWriter();
		if (httpRequest.getParameter("WRITE") != null) {
			out.println("<html><body>\n") ;
			out.println("<input type=\"button\" name=\"Close\" value=\"Close\" onClick=\"window.close()\" />") ;
			saveFile(httpRequest, out, "page", "pagefilename") ;
			saveFile(httpRequest, out, "list", "listmodulefilename") ;
			saveFile(httpRequest, out, "detail", "detailmodulefilename") ;
			saveFile(httpRequest, out, "validator", "validatorfilename") ;
			saveFile(httpRequest, out, "Listvalidation", "Listvalidationfilename") ;
			saveFile(httpRequest, out, "Detailvalidation", "Detailvalidationfilename") ;
			saveFile(httpRequest, out, "lookup", "lookupfilename") ;
			saveFile(httpRequest, out, "master", "masterfilename") ;
			out.println("</body>\n</html>") ;
			out.flush();
			return ;
		}
		
		String action = "http://" + httpRequest.getServerName() + ":" + httpRequest.getServerPort()+ httpRequest.getContextPath()+"/servlet/"+getServletName() ;
		out.println("<html><body><form name=\"f1\" method=\"post\" action=\""+action+"\">");
		out.println("<input type=\"submit\" name=\"WRITE\" value=\"WRITE\"><br>") ;
		try {
			FillerFactory ff = FillerFactory.getInstance();
			String templateCRUD = (String) _serviceRequest.getAttribute("TMPL");
			if(templateCRUD==null || templateCRUD.length()==0 || templateCRUD.equalsIgnoreCase("FILETEMPLATECRUD"))
				templateCRUD = "TemplateAlberoGenerazione.xml"; 
			SourceBean treeStructure = SourceBean.fromXMLFile(_templatePath+"/"+templateCRUD);
			for(SourceBean step : (List<SourceBean>)treeStructure.getAttributeAsList("STEP")) {
				String templateName = (String) step.getAttribute("TEMPLATENAME");
				String function = (String) step.getAttribute("FUNCTION");
				AbstractFiller filler;
				try {
					filler = ff.getFiller(function, _variablesMap, _requestContainer, _responseContainer);
					if(filler!=null)
						out.println(filler.service(templateName, step).toString());
				} catch (Exception e) {
					TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ServletCreateCRUD::service::Error",e);
					//e.printStackTrace();
				}
			}
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ServletCreateCRUD::service::Error",e);
			//e.printStackTrace();
		}
		out.println("</form></body></html>") ;
		out.flush();
	}
	

	protected SourceBean getRowsAtIndex(int index) {
		return ((SourceBean) _content.getAttributeAsList("ROWS").get(index));
	}

	protected int replaceToken(StringBuffer templateFile, String tplkey, String newString) throws IOException {
		int countReplace = 0, index = 0, tplKeySize = tplkey.length();
		while ((index = templateFile.indexOf(tplkey, index)) != -1) {
			countReplace++;
			templateFile.replace(index, index + tplKeySize, newString);
		}
		return countReplace;
	}

	/**
	 * Imposta gli HeaderHttp per il browser
	 * 
	 * @param httpResponse
	 * @throws ServletException
	 */
	private void setHeader(HttpServletResponse httpResponse)
			throws ServletException {
		httpResponse.setHeader("Expires", "0");
		httpResponse.setHeader("Cache-Control",
				"must-revalidate, post-check=0, pre-check=0");
		httpResponse.setHeader("Pragma", "public");
		httpResponse.setContentType("text/html");
	}

}
