/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.daogen.servlet;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author scarciello
 *
 */
public class ServletCreateLookup extends AbstractServletFileCreate {

	private final static String TPL_NAME = "<?Name?>";
	private final static String TPL_PAGE = "<?Page?>";
	private final static String TPL_MODULE = "<?Module?>";
	private final static String TPL_SOURCEID = "<?SID?>";
	private final static String TPL_DESTID = "<?DID?>";
	private final static String TPL_SHOWLENS = "<?SL?>";
	private final static String TPL_FIELDS = "<?LookupFieldElements?>";
	
	// private String _templatePath = (String)ConfigSingleton.getInstance().getAttribute("DAOGEN.pathTemplate") ;
	// private StringBuffer _templateFile = null;

	protected void service(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException {
		PrintWriter out = httpResponse.getWriter() ;
		setHeader(httpResponse) ;
		if (httpRequest.getParameter("WRITE") != null) {
			saveFile(httpRequest, out, "source", "filename") ;
			out.flush() ;
			return ;
		}

		init(httpRequest);

		String nomeTemplate = (String) getRowsAtIndex(0).getAttribute("ROW.REFTEMPLATE");
		String nomeFieldTemplate = (String) getRowsAtIndex(1).getAttribute("ROW.REFTEMPLATE");
		
		String page = (String)httpRequest.getParameter("PAGE_NAME");
		String module = (String)httpRequest.getParameter("MODULENAME");
		String name = getLookupName(page) ;
		RequestContainer requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
		ResponseContainer responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);

		StringBuffer templateFile = createLookup(requestContainer, responseContainer, name, nomeTemplate, nomeFieldTemplate, page, module) ;

		String fileName = ConfigSingleton.getInstance().getAttribute("DAOGEN.pathXMLFile")+"/"+getLookupName(page)+".xml"; 
		String action = "http://" + httpRequest.getServerName() + ":" + httpRequest.getServerPort()+ httpRequest.getContextPath()+"/servlet/"+getServletName() ;
		out.println("<html><body><form name=\"f1\" method=\"post\" action=\""+action+"\">");
		out.println("<input type=\"submit\" name=\"WRITE\" value=\"WRITE\">") ;
		out.println("<input type=\"text\" name=\"filename\" size=\"105\" value=\""+fileName+"\"><br>");
		out.println("<textarea name=\"source\" cols=92 rows=30>");
		DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
        Date date = new Date();
		out.println(templateFile.toString()) ;
		out.println("</textarea>");
		out.println("</form></body></html>") ;
		out.flush() ;
		out.flush() ;
	}
	
	public String getLookupName(String pageName) {
		String name = new String(pageName);
		name = name.replaceFirst("pag", "");
		name = name + "Lookup";
		
		return name ;
	}

	/**
	 * Lookup generation method
	 * 
	 * @param requestContainer
	 * @param responseContainer
	 * @param name
	 * @param templateName
	 * @param templateFieldName
	 * @param pageName
	 * @param moduleName
	 * @return
	 * @throws IOException
	 */
	public static StringBuffer createLookup(RequestContainer requestContainer, ResponseContainer responseContainer,
			String name, String templateName, String templateFieldName, String pageName, String moduleName) throws IOException {
		String templatePath = (String)ConfigSingleton.getInstance().getAttribute("DAOGEN.pathTemplate") ;
		StringBuffer templateFile = readTemplateFile(templatePath, templateName);
		
		replaceToken(templateFile, TPL_PAGE, pageName);
		replaceToken(templateFile, TPL_MODULE, moduleName);
		replaceToken(templateFile, TPL_NAME, name);
		
		StringBuffer campi = new StringBuffer();
		int counter = 0;
		String referencedFile = (String) ((SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", moduleName)).getAttribute("REFERENCEDFILE");
		if(referencedFile==null || referencedFile.length()==0) {
			List<SourceBean> fieldsBean = ((SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", moduleName)).getAttributeAsList("CONFIG.COLUMNS.COLUMN");
			for(SourceBean field : fieldsBean) {
				++counter;
	
				StringBuffer templateField = readTemplateFile(templatePath, templateFieldName); 
				String columnName = (String)field.getAttribute("NAME");
				String showLens = (counter == 1? "TRUE" : "FALSE");
		
				replaceToken(templateField,TPL_SOURCEID,"" + counter);
				replaceToken(templateField,TPL_DESTID,columnName);
				replaceToken(templateField,TPL_SHOWLENS,showLens);
				
				campi.append(templateField);
			}
		} else {
			SourceBean serviceRequest = null;
			if (requestContainer != null) {
				serviceRequest = requestContainer.getServiceRequest();
			}
			try {
				serviceRequest.setAttribute("FILEFISICO",referencedFile);
				SourceBean action = (SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("ACTIONS.ACTION", "NAME", "ActCreazioneLookup");
		    	String pool = (String)action.getAttribute("CONFIG.POOL") ;
		    	SourceBean fieldsQuery = (SourceBean)action.getAttribute("CONFIG.QUERIES.FIELDS_QUERY") ;
		    	SourceBean selectedFieldsBean = (SourceBean)QueryExecutor.executeQuery(requestContainer,responseContainer, pool, fieldsQuery, "SELECT") ;
		    	boolean usedLens = false;
				List<SourceBean> fieldsBean = (List<SourceBean>) selectedFieldsBean.getAttributeAsList("ROW");
				for(SourceBean field : fieldsBean) {
					++counter;
		
					StringBuffer templateField = readTemplateFile(templatePath, templateFieldName); 
					String columnName = (String)field.getAttribute("campoWeb");
					String fieldVisible = (String)field.getAttribute("fieldVisible");
					String showLens = "FALSE";
					if(!usedLens && (fieldVisible==null || fieldVisible.equalsIgnoreCase("Y"))) {
						showLens="TRUE";
						usedLens=true;
					}
			
					replaceToken(templateField,TPL_SOURCEID,"" + counter);
					replaceToken(templateField,TPL_DESTID,columnName);
					replaceToken(templateField,TPL_SHOWLENS,showLens);
					
					campi.append(templateField);
				}
		    	
			}
			catch (SourceBeanException e) {
				//TODO: gestire l'eccezione
			}
		}
		replaceToken(templateFile,TPL_FIELDS,campi.toString());
		
		// Inserimento formattatore
		try {
			SourceBean xmlFmt = SourceBean.fromXMLString(templateFile.toString());
			templateFile = new StringBuffer(xmlFmt.toXML());
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ServletCreateLookup::createLookup::Error",e);
			//e.printStackTrace();
		}
		
		return templateFile ;
	}

	protected SourceBean getRowsAtIndex(int index) {
		return ((SourceBean)_content.getAttributeAsList("ROWS").get(index)) ;
	}

	/**
	 * Sostiutisce "parti" all'interno dello StringBufferd passato
	 * 
	 * @param templateFile StringBuffer contenente il template da modificare
	 * @param tplkey       String contiene la chiave da ricercare all'interno del template
	 * @param newString    String con la nuova parte da sostituire alla tplkey trovata
	 * @return count       Numero di blocchi sostituiti
	 * 
	 * @throws IOException
	 */
	protected static int replaceToken(StringBuffer templateFile, String tplkey, String newString) throws IOException {
		int countReplace = 0, index = 0, tplKeySize = tplkey.length() ;
		while ((index = templateFile.indexOf(tplkey, index)) != -1) {
			countReplace++ ;
			templateFile.replace(index, index+tplKeySize,newString) ;
		}
		return countReplace ;
	}

	/**
	 * Legge e carica nello StringBuffer ritornato il contenuto del template
	 * @return
	 * @throws Exception
	 */
	private static StringBuffer readTemplateFile(String templatePath, String templateName) throws IOException {
		StringBuffer fileContent = new StringBuffer() ;
		templateName = templateName.trim();
		File file = new File(ConfigSingleton.getRootPath() +templatePath, templateName);
		FileReader fr=null;
		try {
			fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			String riga;
			while ((riga = br.readLine()) != null )
				fileContent.append(riga+"\n") ;
			br.close() ;
		} catch (FileNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ServletCreateLookup::readTemplateFile::Error",e);
			//e.printStackTrace(); // TODO: Da gestire log, messageHandler e redirect a PaginaStandard
			throw e ;
		} catch (IOException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ServletCreateLookup::readTemplateFile::Error",e);
			//e.printStackTrace(); // TODO: Da gestire log, messageHandler e redirect a PaginaStandard
			throw e ;
		} finally {
			if (fr != null)
				fr.close() ;
		}
		
		return fileContent ;
	}
	
	/**
	 * Imposta gli HeaderHttp per il browser
	 * @param httpResponse
	 * @throws ServletException
	 */
	private void setHeader(HttpServletResponse httpResponse) throws ServletException{
		httpResponse.setHeader("Expires", "0");
		httpResponse.setHeader("Cache-Control",	"must-revalidate, post-check=0, pre-check=0");
		httpResponse.setHeader("Pragma", "public");
		httpResponse.setContentType("text/html");
	}

}
