/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dbaccess.sql.result.std;

import java.sql.ResultSet;
import java.util.ArrayList;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dbaccess.sql.mappers.SQLMapper;
import it.eng.spago.dbaccess.sql.result.DataResultInterface;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFInternalError;

/**
 * @author scarciello
 *
 */
public class PCMLScrollableDataResult implements ScrollableDataResult {
	private String[] _columnsNames = null;
	private int[] _columnsTypes = null;
	private int _currentRow = -1;
	private ArrayList<DataRow> _dataRows = null;
	private SQLMapper _sqlMapper = null;
	
	public PCMLScrollableDataResult (ArrayList<SourceBean> rs, String[] columnsNames, int[] columnsTypes, SQLMapper sqlMapper) {
		_columnsNames = columnsNames;
		_columnsTypes = columnsTypes;
		_sqlMapper = sqlMapper;
		if(rs.size()>0) { 
			_currentRow = 0;
			
			_dataRows = new ArrayList<DataRow>();
			// Alloca il vettore delle righe dati
			for(SourceBean riga : rs) {
				DataRow dataRow = new DataRow(_columnsNames.length);
				for(int i=0;i<_columnsNames.length;i++) {
					Object value = riga.getAttribute(_columnsNames[i]);
					DataField dataField = new DataField(_columnsNames[i],_columnsTypes[i],value,_sqlMapper);
					dataRow.addColumn(i, dataField);
				}
				_dataRows.add(dataRow);
			}
		}
	}
	
	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#close()
	 */
	public void close() throws EMFInternalError {
		// TODO Stub di metodo generato automaticamente

	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#getColumnCount()
	 */
	public int getColumnCount() {
		return _columnsNames.length;
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#getColumnNames()
	 */
	public String[] getColumnNames() {
		return _columnsNames;
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#getColumnTypes()
	 */
	public int[] getColumnTypes() {
		return _columnsTypes;
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#getDataResultType()
	 */
	public String getDataResultType() {
		return DataResultInterface.SCROLLABLE_DATA_RESULT;
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#getDataRow()
	 */
	public DataRow getDataRow() throws EMFInternalError {
		return getDataRow(_currentRow);
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#getDataRow(int)
	 */
	public DataRow getDataRow(int position) throws EMFInternalError {
		if(position>=0 && position<getRowsNumber())
			return _dataRows.get(position);
		else
			return null;
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#getResultSet()
	 */
	public ResultSet getResultSet() {
		// TODO Stub di metodo generato automaticamente
		return null;
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#getRowsNumber()
	 */
	public int getRowsNumber() throws EMFInternalError {
		return _dataRows.size();
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#hasRows()
	 */
	public boolean hasRows() throws EMFInternalError {
		boolean hasRows = (_currentRow<getRowsNumber())?true:false;
		return hasRows;
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#moveTo(int)
	 */
	public void moveTo(int position) throws EMFInternalError {
		if(position<getRowsNumber() && position>=0)
			_currentRow = position;
		else throw new EMFInternalError("SEVERE", "Index out of range");
	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.ScrollableDataResult#refresh()
	 */
	public void refresh() throws EMFInternalError {
		// TODO Stub di metodo generato automaticamente

	}

	/* (non Javadoc)
	 * @see it.eng.spago.dbaccess.sql.result.DataResultInterface#getSourceBean()
	 */
	public SourceBean getSourceBean() throws EMFInternalError {
        Monitor monitor =
            MonitorFactory.start(
                "model.data-access.pcml-default-scrollable-data-result.get-source-bean");
        try {
            int currentPosition = _currentRow;
            moveTo(0);
            SourceBean rowsSourceBean =
                new SourceBean(ScrollableDataResult.ROWS_TAG);
            DataRow dr = null;
            SourceBean dataRowSourceBean = null;
            int rowsNumber = this.getRowsNumber();
            for (int i = 0; i <= rowsNumber; i++) {
                dr = getDataRow(i);
                dataRowSourceBean = dr.getSourceBean();
                rowsSourceBean.setAttribute(dataRowSourceBean);
            } // for (int i = 1; i <= rowsNumber; i++)
            moveTo(currentPosition);
            return rowsSourceBean;
        } // try
        catch (Exception ex) {
            throw Utils.generateInternalError(
                ex,
                "DefaultScrollableDataResult::getSourceBean: ");
        } // catch (SourceBeanException sbe)
        finally {
            monitor.stop();
        } // finally
	}

}
