/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.decorators;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;

import java.io.Serializable;
import java.util.List;
import java.util.Locale;

/**
 * @author scarciello
 *
 */
public abstract class AbstractDecorator implements Serializable {
	private static final long serialVersionUID = 1L;
	protected Locale _locale ;
	protected SourceBean _config ;
	protected RequestContainer _requestContainer ;

    /**
	 * Costruttore della classe carica i dati locali per la rappresentazione numerica. 
	 * In particolare distingue tra i punti e le virgole come separatori
	 * 
	 * @author Scarciello Roberto
	 *
	 */
	public AbstractDecorator(SourceBean config, RequestContainer requestContainer) {
		_config = config ;
		_requestContainer = requestContainer ;
		
		// Recupera i default dalla configurazione
		String language = ((String)config.getAttribute("REGIONAL_SETTINGS.language", "it")).toLowerCase();
        String country = (String)config.getAttribute("REGIONAL_SETTINGS.country", "IT");

		// Sovrascrive i default con quelli eventualmente presenti nel permanent Container 
        //  inseriti in fase di Login
		SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer();
        language = (String)permanentContainer.getAttribute("AF_LANGUAGE", language);
		country = (String)permanentContainer.getAttribute("AF_COUNTRY", country);

        // Create all the objects locale dependent
        _locale = new Locale(language, country);
	}

	public String getCssClass4List() {
		return ""+_config.getAttribute("CSS_CLASS.listClass") ;
	}

	public String getCssClass4InputField() {
		return ""+_config.getAttribute("CSS_CLASS.inputClass") ;
	}
	
	public abstract String decorate(Object in) ;
	
	public List optionList() {
		return null;
	}
	
}