/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.decorators;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author grassi
 *
 */
public class DateDecorator extends AbstractDecorator {
	private static final long serialVersionUID = 1L;
	private SimpleDateFormat _loadData, _writeData ;
	public DateDecorator(SourceBean config, RequestContainer requestContainer) {
		super(config, requestContainer);
		String inPattern = (String)config.getAttribute("FORMAT.inPattern") ;
		String outPattern = (String)config.getAttribute("FORMAT.outPattern") ;
		_loadData = new SimpleDateFormat(inPattern) ;
		_writeData = new SimpleDateFormat(outPattern) ;
	}

	/**
	 * Formatta una stringa rappresentante un numero
	 * @param in Stringa da formattare
	 * @return Object formatta
	 */
	public String decorate(Object obj) {
		if (obj == null) {
    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, this.getClass().getName()+"::decorate: null input string");
			return "" ;
		}
		String in = ""+obj ;
		if (in!=null && in.equals("0")) {
			in = new String();
		}
		
		if(in.length()<6 && in.length()>0)
			in="000000".substring(in.length())+in;
		try {
			if (obj instanceof Date) {
 				in = _writeData.format((Date)obj) ;
			}else if (in.indexOf('-') < 0) { // Se trova il carattere '-' non effettua la conversione
 				in = _writeData.format(_loadData.parse(in)) ;
			}
		} catch (Exception ex) {
    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, this.getClass().getName()+"::decorate: invalid numeric parameter ["+ in+"]");
		}
		
		return in ;
	}
}