/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/

package it.eng.spago.decorators;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.util.HashMap;

/**
 * Factory per la generazione dei Decorator
 * @author A. Grassi
 *
 */
public class DecoratorFactory implements Serializable {
    private static final long serialVersionUID = 1L;
    private static DecoratorFactory _instance = null;

    private DecoratorFactory() {
    }
    
    public static DecoratorFactory getInstance() {
        if (_instance == null) {
            synchronized (DecoratorFactory.class) {
                if (_instance == null) {
                    try {
                        _instance = new DecoratorFactory();
                    } // try
                    catch (Exception ex) {
                		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DecoratorFactory::getInstance: errore creazione");
                        //ex.printStackTrace();
                    } // catch(Exception ex) try
                } // if (_instance == null)
            } // synchronized(ConfigSingleton.class)
        } // if (_instance == null)
        return _instance;
    } // public static ConfigSingleton getInstance()

    public static void release() {
        if (_instance != null) {
            synchronized (ConfigSingleton.class) {
                _instance = null;
            } // synchronized (ConfigSingleton.class)
        } // if (_instance != null)
    } // public static void release()
    
    /**
     * Ritorna la classe del decorator associata al nome richiesto
     * @param name : nome del decorator da cercare
     * @return classe di tipo AbstractDecorator
     * @throws Exception
     */
    public AbstractDecorator getDecorator(String name, RequestContainer requestContainer) throws Exception {
    	AbstractDecorator decObj = null ;
    	if (name == null || name.trim().length() == 0) {
    		name = "Default" ;
    	}
    	SourceBean decBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("DECORATORS.DECORATOR","name", name) ;
    	String decoratorClassName = (String)decBean.getAttribute("class")  ;
    	if (decoratorClassName == null || decoratorClassName.trim().length() == 0) {
    		throw new Exception("Decoratore ["+name+"] non trovato in configurazione") ;
    	}
        Class[] intArgsClass = new Class[] {SourceBean.class, RequestContainer.class};
        Object[] intArgs = new Object[] {decBean, requestContainer};
        Constructor intArgsConstructor;
        Class decoratorDefinition = Class.forName(decoratorClassName);
        intArgsConstructor = decoratorDefinition.getConstructor(intArgsClass);
        decObj = (AbstractDecorator)intArgsConstructor.newInstance(intArgs);
    	return decObj ;
    }
}