/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.decorators;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;

/**
 * @author grassi
 *
 */
public class NumberDecorator extends AbstractDecorator {
	private static final long serialVersionUID = 1L;
	private String _pattern ;
	private DecimalFormat _decimalFormat ;
	public NumberDecorator(SourceBean config, RequestContainer requestContainer) {
		super(config, requestContainer);
		_pattern = (String)config.getAttribute("FORMAT.pattern") ;
		NumberFormat nf = NumberFormat.getNumberInstance(_locale);
		_decimalFormat = (DecimalFormat) nf;
		_decimalFormat.applyPattern(_pattern);
	}

	/**
	 * Formatta una stringa rappresentante un numero
	 * @param in Stringa da formattare
	 * @return Object formatta
	 */
	public String decorate(Object in) {
		if (in == null) {
    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, this.getClass().getName()+"::decorate: null input string");
			return "" ;
		}
		String ret = ""+in ;
		try {
			ret = _decimalFormat.format(getDouble(in));
		} catch (Exception ex) {
    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, this.getClass().getName()+"::decorate: invalid numeric parameter ["+ in+"]");
		}
		
		// Toglie lo zero
		if (ret.equals("0"))
			ret = "" ;
		
		return ret ;
	}
	
	/**
	 * Converte un "Number" o la sua "String" rappresentation in "double"
	 * @param in input Object
	 * @return double convertito
	 * @throws Exception
	 */
	private double getDouble(Object in) throws Exception {
		if (in instanceof Number) {
			// Number  la superclasse di Integer, Float, BigDecimal, Double, ....
			return ((Number)in).doubleValue() ;
		}else if (in instanceof String) {
			String temp = (String) in;
			temp = temp.replace(',','.');
			return Double.parseDouble(temp) ;
		}
		throw new Exception("Impossibile convertire in double il valore ["+in+"] ("+in.getClass().getName()+")") ;
	}
}