/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.decorators;

import java.util.ArrayList;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;

/**
 * @author scarciello
 *
 */
public class ValueDecorator extends AbstractDecorator {
	/**
	 * 
	 */
	private static final long serialVersionUID = -205858870248092230L;
	
	private SourceBean _elements = null;

	/* (non Javadoc)
	 * @see it.eng.spago.decorators.AbstractDecorator#decorate(java.lang.Object)
	 */

	public ValueDecorator(SourceBean config, RequestContainer requestContainer) {
		super(config, requestContainer);
		_elements = (SourceBean)config.getAttribute("LIST_VALUES") ;
	}
	
	@Override
	public String decorate(Object in) {
		String decoratedValue="";
		String startValue = ""+in;
		if(startValue==null || startValue.equalsIgnoreCase("NULL")) 
			startValue="";
		for(SourceBean elemento : (List<SourceBean>)_elements.getAttributeAsList("VALUE")) {
			String item = (String)elemento.getAttribute("ITEM");
			String description = (String)elemento.getAttribute("DESCRIPTION");
			String defaultItemString = (String)elemento.getAttribute("DEFAULT","FALSE"); 
			boolean defaultItem =(defaultItemString.equalsIgnoreCase("TRUE") ? true : false);
			if(item.equalsIgnoreCase(startValue))
				decoratedValue = ""+description;
			if(decoratedValue.equalsIgnoreCase("") && defaultItem)
				decoratedValue = ""+description;
		}
		return decoratedValue;
	}

	public List optionList() {
		List options = new ArrayList<SourceBean>();
		for(SourceBean elemento : (List<SourceBean>)_elements.getAttributeAsList("VALUE")) {
			String item = (String)elemento.getAttribute("ITEM");
			String description = (String)elemento.getAttribute("DESCRIPTION");
			SourceBean option;
			try {
				option = new SourceBean("OPTION");
				option.setAttribute("VALUE", item);
				option.setAttribute("DISPLAY",description);
				options.add(option);
			} catch (SourceBeanException e) {
				TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ValueDecorator::optionList::Error",e);
				//e.printStackTrace();
			}
		}
		return options;
	}
}
