/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.action.impl;

import com.ibm.as400.access.AS400;
import com.ibm.as400.access.ConnectionDroppedException;
import com.ibm.as400.data.PcmlException;
import com.ibm.as400.data.ProgramCallDocument;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.io.os400.OS400AdapterResourcePool;
import it.eng.spago.tracing.TracerSingleton;

/**
 * @author grassi
 *
 */
public class ActGatewayRPG extends AbstractAction {
	private static final long serialVersionUID = 656782676100931850L;

	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		// Se esiste un attributo "message" o se esiste e NON vale "DATAIL_NEW 
		// viene eseguita l'azione !
		if (serviceRequest.getAttribute("MESSAGE") == null ||
				(!((String)serviceRequest.getAttribute("MESSAGE")).trim().equalsIgnoreCase("DETAIL_NEW"))) {
			String pcmlName = (String)getConfig().getAttribute("pcml") ;
			SourceBean pcmlBean = SourceBean.fromXMLFile("WEB-INF/pcml/"+pcmlName+".pcml", true, true) ;
			try {
				SourceBean response = invoke(this, pcmlName, pcmlBean) ;
				if (response != null)
					serviceResponse.setAttribute(response) ;
			} catch (PcmlException ex) {
				// Fix for ConnectionDroppedException
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "ServiceProxyRPG retry invoke: pcml("+pcmlName+")");
				SourceBean response = invoke(this, pcmlName, pcmlBean) ;
				if (response != null)
					serviceResponse.setAttribute(response) ;
			}
		} else {
			serviceResponse = new SourceBean("ROW");
		}
	}

	/**
	 * Execute remote program 
	 * @param par
	 * @return SourceBean 
	 * @throws Exception
	 */
	public static SourceBean invoke(RequestContextIFace requestContext, String pcmlName, SourceBean pcmlBean) throws Exception {
		AS400 serverConnection = null ;
		SourceBean serviceResponse = null ;
			try {
				serverConnection = OS400AdapterResourcePool.getInstance().getConnection() ;
				ProgramCallDocument	pcml = new ProgramCallDocument(serverConnection, pcmlName);
				String remoteProgramName = (String)pcmlBean.getAttribute("PROGRAM.NAME") ;
				handleParameter(requestContext, remoteProgramName, pcml, pcmlBean) ;
				serviceResponse = executeRemoteProgram(pcml, remoteProgramName, pcmlBean) ;
			} catch (PcmlException ex) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "ServiceProxyRPG::invoke: pcml("+pcmlName+")", ex);
				throw ex ;
			} catch (SourceBeanException ex) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "ServiceProxyRPG::invoke: pcml("+pcmlName+")", ex);
				throw ex ;
			} finally {
				if (serverConnection != null) {
					OS400AdapterResourcePool.getInstance().closeConnection( serverConnection);
				}
			}
		return serviceResponse;
	}
	
	/**
	 * Remote Program execution, errors and output extraction
	 * @param pcml
	 * @param programName
	 * @return
	 * @throws Exception
	 */
	private static SourceBean executeRemoteProgram(ProgramCallDocument pcml, String programName, SourceBean pcmlBean) throws Exception {
		SourceBean serviceResponse = null;

		if (! pcml.callProgram(programName)) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"ServiceProxyRPG::invoke: program ("+programName+") has returned error");
		} else {
			serviceResponse = handleResponse(pcml, programName, pcmlBean) ;
		}
		return serviceResponse ;
	}
	
	/**
	 * Adds parameters coming from SERVICE_REQUEST to ProgramCallDocument
	 * @param requestContext
	 * @param pcml
	 * @param par
	 * @throws Exception
	 */
	private static void handleParameter(RequestContextIFace requestContext, String remoteProgramName, ProgramCallDocument pcml, SourceBean pcmlBean) throws Exception {
		for (Object par : pcmlBean.getAttributeAsList("PROGRAM.DATA")) {
			if (par != null && ((String)((SourceBean)par).getAttribute("USAGE")).toUpperCase().indexOf("INPUT") != -1) {
				String parameterName = (String)((SourceBean)par).getAttribute("NAME") ;
				Object parameterValue = requestContext.getServiceRequest().getAttribute(parameterName);
				if (parameterValue != null) {
					pcml.setValue(remoteProgramName+"."+parameterName, parameterValue) ;
				}
			}
		}
	}
	
	/**
	 * Make response SourceBean
	 *   ex:  <ROW OYNERROR="1" OPROPROCESSO="abcde" /> 
	 * @param pcml
	 * @return
	 * @throws Exception
	 */
	private static SourceBean handleResponse(ProgramCallDocument pcml, String remoteProgramName, SourceBean pcmlBean) throws Exception {
		SourceBean row = new SourceBean("ROW");
		for (Object par : pcmlBean.getAttributeAsList("PROGRAM.DATA")) {
			if (par != null && ((String)((SourceBean)par).getAttribute("USAGE")).toUpperCase().indexOf("OUTPUT") != -1) {
				String parameterName = (String)((SourceBean)par).getAttribute("NAME") ;
				Object parameterValue = pcml.getValue(remoteProgramName+"."+parameterName) ;
				row.setAttribute(parameterName, parameterValue) ;
			}
		}
		for (Object par : pcmlBean.getAttributeAsList("PROGRAM.STRUCT")) {
			if (par != null && ((String)((SourceBean)par).getAttribute("USAGE")).toUpperCase().indexOf("OUTPUT") != -1) {
				String parameterName = (String)((SourceBean)par).getAttribute("NAME") ;
				Object parameterValue = pcml.getValue(remoteProgramName+"."+parameterName+".string") ;
				row.setAttribute(parameterName, parameterValue) ;
			}
		}		
		// TODO: Gestione ERRORS
		return row ;
	}
}
