/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.action.impl;

import java.util.ArrayList;
import java.util.List;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.dispatching.service.util.DelegateInnerQueryService;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;

/**
 * @author grassi
 * Classe per l'esecuzuione di query "nelle query"
 * 
 * Segue un esempio di configurazione:
 * 
 * 	<ACTION name="ActReportControlloProcessi" class="it.eng.spago.dispatching.action.impl.InnerQueriesAction" scope="REQUEST">
 *		<CONFIG pool="as400masterpool" title="Report Controllo Processi">
 *			<QUERIES>
 *              <!-- Query di primo livello -->
 *				<QUERY statement="SELEZIONA_CONTROLLOPROCESSI">
 *					<ORDER_BY sql="cp.AHPDDA DESC, cp.AHA1TM ASC" />
 *					<FILTER_PARAMETER scope="SERVICE_REQUEST" type="RELATIVE" value="regDataIniziale" sql="cp.AHPDDA &gt;= ?" />
 *					<FILTER_PARAMETER scope="SERVICE_REQUEST" type="RELATIVE" value="regDataFinale" sql="cp.AHPDDA &lt;= ?" />
 *
 *					<QUERY statement="SELEZIONA_DETT_CONTROLLOPROCESSI">
 *						<PARAMETER type="RELATIVE" value="ID_entita" scope="LOCAL" />
 *						<PARAMETER type="RELATIVE" value="ID_tipo_processo" scope="LOCAL" />
 *						<PARAMETER type="RELATIVE" value="progr_processo" scope="LOCAL" />
 *					</QUERY>
 *
 *					<QUERY statement="SELEZIONA_STATO_PROCESSI">
 *						<PARAMETER type="RELATIVE" value="ID_entita"        scope="LOCAL" />
 *						<PARAMETER type="RELATIVE" value="ID_tipo_processo" scope="LOCAL" />
 *						<PARAMETER type="RELATIVE" value="progr_processo"   scope="LOCAL" />
 *					</QUERY>
 *				</QUERY>

 *              <!-- Altra query di primo livello (verr ritornata con un altro <ROWS>  -->
 *				<QUERY statement="SELEZIONA_CONTROLLOPROCESSI2">
 *                  ...
 *			</QUERIES>
 *		</CONFIG>
 *	</ACTION>	
 */
public class InnerQueriesAction extends AbstractAction {
	private static final long serialVersionUID = 6925199851155670401L;

	/**
	 * 
	 * Esegue le query richieste ed inserisce il risultato nel SERVICE_RESPONSE
	 * Ritorna sempre almeno un <ROWS> con pi livelli di nidificazione.
	 * Se richiesto dalla configurazione pu eseguire anche pi query di primo livello e quindi
	 * queste vengono riportate con la numerazione ROWS, ROWS, ... ROWS
	 * 
	 *   <ROWS>
	 *     <ROW val1="1" val2="2"/>
	 *     <ROWS>
	 *       <ROW valk="A" valz="C"/>
	 *     </ROWS>
	 *   </ROWS>
	 *   <ROWS>
	 *     <ROW val1="1" val2="2"/>
	 *     <ROWS>
	 *       <ROW valk="A" valz="C"/>
	 *     </ROWS>
	 *   </ROWS>
	 *   ...
	 *   <ROWS>
	 *     <ROW val1="1" val2="2"/>
	 *     <ROWS>
	 *       <ROW valk="A" valz="C"/>
	 *     </ROWS>
	 *   </ROWS>
	 */
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		DelegateInnerQueryService.service(this, serviceRequest, serviceRequest) ;
	}
}