/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.action.list.basic.impl;

import java.util.List;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.action.list.basic.AbstractBasicListAction;
import it.eng.spago.dispatching.action.lookup.LookupIFace;
import it.eng.spago.paginator.basic.ListIFace;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.paginator.basic.impl.GenericList;
import it.eng.spago.paginator.basic.impl.GenericPaginator;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.sqlField.SqlTableList;
import it.eng.spago.util.sqlField.SqlTableType;

/**
 * @author grassi
 * 
 * Sample configuration: ...
 * 
 *	<ACTION name="ActLookupRicercaFileFisico" class="it.eng.spago.dispatching.action.list.basic.impl.SqlTableListLookupAction" scope="REQUEST" publisher_name="PubPaginaStandard">
 *		<CONFIG title="Lookup Ricerca File Fisici">
 *			<HIDDENS>
 *			  	<PARAMETER type="RELATIVE" name="codAmbiente"        scope="SERVICE_REQUEST" /> 
 *			  	<PARAMETER type="RELATIVE" name="poolConnessione"    scope="SERVICE_REQUEST" /> 
 *			  	<PARAMETER type="RELATIVE" name="FILTER_tableSchema" scope="SERVICE_REQUEST" />
 *		  		<PARAMETER type="RELATIVE" name="FILTER_tableName"   scope="SERVICE_REQUEST" />
 *			</HIDDENS>
 *			<COLUMNS>
 *				<COLUMN name="tableCatalog"           label="TABLE CATALOG"             type="field" nofilter="true"/> 
 *				<COLUMN name="tableSchema"            label="TABLE SCHEMA"              type="field" /> 
 *				<COLUMN name="tableName"              label="TABLE NAME"                type="field" /> 
 *				<COLUMN name="tableType"              label="TABLE TYPE"                type="field" nofilter="true" /> 
 *				<COLUMN name="explanatoryComment"     label="EXPLANATORY COMMENT"       type="field" nofilter="true" /> 
 *
 */
public class SqlTableListLookupAction extends DefaultBasicListAction  implements LookupIFace {
	private static final long serialVersionUID = 1L;
	
	public ListIFace getList(SourceBean request, SourceBean response) throws Exception {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "SqlTableListLookupAction::getList: invocato");
		String poolName = (String) request.getAttribute("FILTER_poolConnessione");
		String schemaPattern = null;
		String tableNamePattern = addPercentSign((String) request.getAttribute("FILTER_fileFisico"));
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "SqlTableListLookupAction::getList: " + "poolConnessione=" + poolName + ", codAmbiente=" + schemaPattern + ", fileFisico=" + tableNamePattern);
		List<SqlTableType> listTableType = SqlTableList.getDefinedTables(poolName, null, schemaPattern, tableNamePattern);
		return makeResponseBean(listTableType);
	}
	
	/**
	 * Convert a list o SqlTableType into ListIFace format
	 * @param listTableType
	 * @return
	 * @throws Exception
	 */
	private ListIFace makeResponseBean(List<SqlTableType> listTableType) throws Exception {
		ListIFace list = new GenericList();
		if (listTableType != null) {
			PaginatorIFace paginator = getNewPaginator();
			list.setPaginator(paginator);
			for (SqlTableType sqlTableType : listTableType) {
				if (sqlTableType != null) {
					paginator.addRow(sqlTableType.toSourceBean("ROW"));
				}
			}
		}
		return list;
	}
	
	/**
	 * Ritorna il paginatore definito in configurazione
	 * @return
	 * @throws Exception
	 */
	private PaginatorIFace getNewPaginator() throws Exception {
		PaginatorIFace paginator = new GenericPaginator();
		paginator.setPageSize(Integer.parseInt((String) getConfig().getAttribute("ROWS")));
		return paginator;
	}
	
	/**
	 * Ritorna il valore di un PARAMETER in configurazione
	 * @param parName
	 * @return
	 * @throws Exception
	 */
	private String getParameter(String parName) throws Exception {
		return (String) ContextScooping.getScopedParameter(this, (SourceBean) getConfig().getFilteredSourceBeanAttribute("HIDDENS.PARAMETER", "NAME", parName));
	}
	
	/**
	 * Inserisce % all'inizio e alla fine  a meno che non sia gi stato inserito dall'utente.
	 * @param tableNamePattern
	 * @return 
	 */
	private String addPercentSign(String tableNamePattern) {
		String returnParameter = (tableNamePattern == null) ? "%%" : tableNamePattern;
		if (returnParameter.indexOf("%") == -1) {
			returnParameter = new StringBuffer().append("%").append(returnParameter).append("%").toString();
		}
		return returnParameter;
	}
}
