/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.action.list.smart.impl;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.list.smart.impl.DelegatedSmartListService;
import it.eng.spago.paginator.smart.IFaceListProvider;
import it.eng.spago.util.ContextScooping;

public class QueryProviderSmartListAction extends DefaultSmartListAction {
	private static final long serialVersionUID = 1L;
	private final static String STATEMENT_KEY = "ROW_PROVIDER.CONFIG.LIST_QUERY.statement" ;
	private final static String POOL_KEY = "ROW_PROVIDER.CONFIG.pool" ;
	private final static String PAGE_SIZE_KEY = "PAGE_PROVIDER.CONFIG.page_size" ;
	
	private void restoreParameter(Object parValue, String key) throws SourceBeanException{
		if (parValue != null)
			getConfig().updAttribute(key, parValue) ;
	}

	private Object updateParameter(SourceBean configBean, String parName, String key) throws SourceBeanException{
		Object defaultValue = configBean.getAttribute(key) ;
		Object parValue = getParValue(parName) ;
		if (parValue != null)
			configBean.updAttribute(key, parValue) ;
		return defaultValue ;
	}
	
	private Object getParValue(String parName) {
		SourceBean parBean = (SourceBean)getConfig().getFilteredSourceBeanAttribute("PARAMETER", "name", parName) ;
		return ContextScooping.getScopedParameter(this, parBean) ;
	}


	/* (non Javadoc)
	 * @see it.eng.spago.dispatching.action.list.smart.AbstractSmartListAction#service(it.eng.spago.base.SourceBean, it.eng.spago.base.SourceBean)
	 */
	@Override
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		String defaultStatement = (String)updateParameter(getConfig(), "statement", STATEMENT_KEY) ;
		String defaultPool = (String)updateParameter(getConfig(),"pool", POOL_KEY) ;
		String defaultPageSize = (String)updateParameter(getConfig(),"page_size", PAGE_SIZE_KEY) ;
		IFaceListProvider list = null ;
		try {
			super.service(serviceRequest, serviceResponse);
		}finally {
			restoreParameter(defaultStatement, STATEMENT_KEY) ;
			restoreParameter(defaultPool, POOL_KEY) ;
			restoreParameter(defaultPageSize, PAGE_SIZE_KEY) ;
		}
	}
}
