/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.action.lookup;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;

import java.io.Serializable;
import java.util.List;

public class AbstractLookupImplementation implements Serializable, LookupIFace {
	private static final long serialVersionUID = 1L;

	/**
     * Gestione dei parametri per il filtering (in testa alle colonne)
     * Questo metodo aggiunge alla request un'eventuale valorizzazione implicita: valorizzo
     *   la colonna "xx" e di conseguenza aggiungo al request l'attributo "FILTER_xx" con il medesimo valore.
     * @param config
     * @param request
     * @return
     * @throws SourceBeanException
     */
    public static void handleFilterParameter(SourceBean config, SourceBean request, SourceBean excludeFilter) throws SourceBeanException {
    	for (Object columns : config.getAttributeAsList("COLUMNS.COLUMN")) {
    		String colName = (String)((SourceBean)columns).getAttribute("NAME") ;
    		if (! excludeFilter.containsAttribute(colName)) {
				if (request.getAttributeAsList(colName).size() == 1) {
					Object requestColValue = request.getAttribute(colName) ;
					if (request.getAttributeAsList("FILTER_"+colName).size() == 0) {
						request.setAttribute("FILTER_"+colName, requestColValue) ;
					}

					if (request.containsAttribute("TYPED_SERVICE_REQUEST."+colName) 
							&& !request.containsAttribute("TYPED_SERVICE_REQUEST.FILTER_"+colName)) {
						request.setAttribute("TYPED_SERVICE_REQUEST.FILTER_"+colName, requestColValue) ;
					}
				}
			}
    	}
    }
    
    /**
     * Questo metodo modifica la serviceRequest andando a modificare il nome dei parametri
     * che nel modulo chiamante hanno l'attributo "lookupAliasName"; in pratica
     * nel modulo l'attributo ha un suo nome, nella lookup ne ha un altro.
     * Alla lookup questo attributo passato serve per valorizzare gli eventuali FILTER_... e
     * per i parametri delle SELECT
     * @param serviceRequest
     * @return
     * @throws SourceBeanException
     */
    public static SourceBean handleAliasParameter(SourceBean serviceRequest, String currentLookupName) throws SourceBeanException {
    	String callModuleName = (String)serviceRequest.getAttribute("RET_MOD") ;
    	SourceBean excludeFilter = new SourceBean("EXCLUDE_FILTER") ;
    	try {
	    	SourceBean callModuleConfig = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "name", callModuleName) ;
	    	List<SourceBean> fieldsetList = callModuleConfig.getAttributeAsList("CONFIG.FIELDS.FIELDSET") ;
	    	String aliasName, aliasFieldName, aliasLookupName, fieldName, fieldValue ;
	    	for (SourceBean fieldset : fieldsetList) {
	    		List<SourceBean> fieldList = fieldset.getAttributeAsList("FIELD") ;
	    		for (SourceBean field : fieldList) {
	    			aliasName = (String)field.getAttribute("lookupAliasName") ;
    				fieldName = (String)field.getAttribute("name") ;
	    			fieldValue = (String)serviceRequest.getAttribute(fieldName) ;
	    			if (aliasName != null && aliasName != "" && fieldValue != null &&fieldValue.trim().length() > 0) {
	    				aliasLookupName = aliasName.substring(0, aliasName.indexOf('.'));
	    				aliasFieldName = aliasName.substring(aliasName.indexOf('.') + 1);
	    				if (aliasLookupName != null && aliasFieldName != null) {
	    					 if (aliasLookupName.equalsIgnoreCase(currentLookupName)) {
		    					// Effettuo la rename solo se i nomi sono diversi
		    					if (! aliasFieldName.trim().equalsIgnoreCase(fieldName.trim())) {
		    						updateAliasFieldBName(serviceRequest, fieldName, aliasFieldName) ;
		    						updateAliasFieldBName(serviceRequest, "TYPED_SERVICE_REQUEST."+fieldName, "TYPED_SERVICE_REQUEST."+aliasFieldName) ;
			    					// 13/12/2007 R.S. 
			    		    		if (excludeFilter.containsAttribute(aliasFieldName)) {
			    		    			excludeFilter.delAttribute(aliasFieldName);
			    		    		}
		    					}
	    					} else {
	    						// Segno il field da escludere nella generazione dei FILTER
	    						excludeFilter.setAttribute(fieldName, aliasFieldName) ;
	    					}
	    				}
	    			}
	    		}
	    	}
    	} catch (NullPointerException ex) {
    		// Catch intenzionalmente vuoto
    	}
    	
    	return excludeFilter ;
    }
    
    private static void updateAliasFieldBName(SourceBean request, String fieldName, String aliasFieldName) throws SourceBeanException {
    	if (request.containsAttribute(fieldName)) {
	    	request.updAttribute(aliasFieldName, request.getAttribute(fieldName));
			request.delAttribute(fieldName) ;
    	}
    }
}