/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.module.impl;

import org.apache.commons.fileupload.FileItem;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.module.AbstractModule;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.io.os400.OS400FileIO;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

/**
 * @author grassi
 *
 */
public class OS400FileIOModule extends AbstractModule {
	public final static String GET_MESSAGE	 = "GET" ;
	public final static String PUT_MESSAGE	 = "PUT" ;
	 
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		String message = (String)getConfig().getAttribute("MESSAGE") ;

		if (message != null && message.equalsIgnoreCase(GET_MESSAGE)) {
			service4Get(serviceRequest, serviceResponse) ;
		} else if (message != null && message.equalsIgnoreCase(PUT_MESSAGE)) {
				service4Put(serviceRequest, serviceResponse) ;
		} else {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
					"OS400FileIOModule::service: parametro 'message' non trovato in configurazione module");
			getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.WARNING, "Azione da eseguire su IFS non specificata (parametro message non trovato)"));
		}
	}
	
	/**
	 * Read a file from OS400 to serviceResponse
	 * 
	 * @param serviceRequest
	 *        MESSAGE specify (in this case) READ ACTION (GET)
	 *        path	specify path file name es: "/ARCAVAR/CSV"
	 *        name	specify file name es: "RICPOR_LOD.CSV"
	 *        
	 * @param serviceResponse
	 *        fileContent	byte array reding file
	 * @throws Exception
	 */
	protected void service4Get(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		String path = (String)serviceRequest.getAttribute("path") ;
		String name = (String)serviceRequest.getAttribute("name") ;
		
		serviceResponse.setAttribute("fileContent", OS400FileIO.readBinaryFile(OS400FileIO.getFullPathFileName(path, name))) ;
	}

	/**
	 * Write a file to OS400 from buffer in serviceRequest
	 *        MESSAGE        specify (in this case) READ ACTION (GET)
	 *        path           specify path file name es: "/ARCAVAR/CSV"
	 *        name           specify file name es: "RICPOR_LOD.CSV"
	 *		  fileContent    
	 *  
	 * @param serviceRequest
	 * @param serviceResponse
	 * @throws Exception
	 */
	protected void service4Put(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		String os400PathName = (String)ContextScooping.getScopedParameter((RequestContextIFace)this, (SourceBean)getConfig().getAttribute("OS400PATHNAME_PARAMETER")) ;
		String os400FileName =(String)ContextScooping.getScopedParameter((RequestContextIFace)this, (SourceBean)getConfig().getAttribute("OS400FILENAME_PARAMETER")) ;
		FileItem sndFile = (FileItem)ContextScooping.getScopedParameter((RequestContextIFace)this, (SourceBean)getConfig().getAttribute("FILEITEM_PARAMETER")) ;
		String sndFileName = sndFile.getName();
		if(os400PathName==null && sndFile!=null) {
			int ultimoSlash = sndFileName.lastIndexOf("/");
			if(ultimoSlash==-1) 
				ultimoSlash = sndFileName.lastIndexOf("\\");
			sndFileName=sndFileName.substring(ultimoSlash+1);
			sndFileName=sndFileName.replace(" ", "_");
			os400PathName = "/arcafile/"+sndFileName;
			serviceRequest.setAttribute("NOMEFILE", sndFileName);
			
		}
		if(os400PathName!=null && os400PathName.length()>0)
			os400FileName = os400PathName;
		FileItem fileItem = (FileItem)ContextScooping.getScopedParameter((RequestContextIFace)this, (SourceBean)getConfig().getAttribute("FILEITEM_PARAMETER")) ;
		
		OS400FileIO.writeAsciiFile(fileItem.get(), os400FileName) ;
	}
}