/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.module.list.config.impl;

import java.util.List;
import java.util.StringTokenizer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dispatching.module.list.basic.AbstractBasicListModule;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.paginator.basic.ListIFace;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.paginator.basic.impl.GenericList;
import it.eng.spago.paginator.basic.impl.GenericPaginator;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ConfigRowHandler;
import it.eng.spago.util.QueryExecutor;

/**
 * @author grassi
 *
 */
public class DefaultConfigListModule extends AbstractBasicListModule {
	private static final long serialVersionUID = 1L;

	/* (non Javadoc)
	 * @see it.eng.spago.dispatching.service.list.basic.IFaceBasicListService#getList(it.eng.spago.base.SourceBean, it.eng.spago.base.SourceBean)
	 */
	public ListIFace getList(SourceBean request, SourceBean response) throws Exception {
		PaginatorIFace paginator = new GenericPaginator();
		InitializerIFace serviceInitializer = (InitializerIFace) this;
		int pagedRows = Integer.parseInt((String) serviceInitializer.getConfig().getAttribute("ROWS"));
		paginator.setPageSize(pagedRows);
		SourceBean statement = (SourceBean) serviceInitializer.getConfig().getAttribute("QUERIES.SELECT_QUERY");
		RequestContextIFace serviceRequestContext = (RequestContextIFace) this;
		List rowsVector = (new ConfigRowHandler()).getConfigList(statement, getRequestContainer(), getResponseContainer());;
		if ((rowsVector == null) || (rowsVector.size() == 0)) {
			EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
			engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10001"));
		} // if ((rowsSourceBean == null) || (rowsVector.size() == 0))
		else {
			String reserved = getReservedWord() ;
			for (int i = 0; i < rowsVector.size(); i++) {
				if (! isReserved(rowsVector.get(i), reserved))
					paginator.addRow(rowsVector.get(i));
			}
		}
		ListIFace list = new GenericList();
		list.setPaginator(paginator);
		// list.addStaticData(firstData);
		return list;
	}
	
	private boolean isReserved(Object obj, String reserved) {
		boolean ret = false ;
		if (obj instanceof SourceBean) {
			if (((SourceBean)obj).getName().equalsIgnoreCase("PAGE") ||
					((SourceBean)obj).getName().equalsIgnoreCase("ACTION")) {
				StringTokenizer tok = new StringTokenizer(reserved, ",");
				String searchTok ;
				while (!ret && tok.hasMoreTokens()) {
					searchTok = tok.nextToken() ;
					if (searchTok.equalsIgnoreCase((String)((SourceBean)obj).getAttribute("NAME")))
						ret = true ;
				}
			}
		}
			
		return ret ;
	}
	
	private String getReservedWord() throws SourceBeanException {
		String reserved = "" ;
		reserved += getReservedWord("PAGE", "DAOGEN.configPageFile") ;
		reserved += getReservedWord("ACTION", "DAOGEN.configActionFile") ;
		reserved += getReservedWord("ACTION", "DAOGEN.configLookupFile") ;
		
		return reserved ;
	}
	
	private String getReservedWord(String searchKey, String configFileKey) throws SourceBeanException {
		String reserved = "" ;

		String demoKeys = ((String)ConfigSingleton.getInstance().getAttribute("DAOGEN.demoKeys", "")).toUpperCase() ;
		String configFile = (String)ConfigSingleton.getInstance().getAttribute(configFileKey) ;
		SourceBean pagConfigBean = SourceBean.fromXMLFile(configFile) ;
		List<SourceBean> listBean = pagConfigBean.getAttributeAsList(searchKey) ;
		String name = "" ;
		for (SourceBean bean : listBean) {
			name = ""+bean.getAttribute("NAME") ;
			if (demoKeys.indexOf(name.toUpperCase())== -1)
				reserved += "," + name ;
		}

		return reserved ;
	}

	/**
     * @see it.eng.spago.dispatching.service.list.basic.IFaceBasicListService#delete(SourceBean)
     */
    public boolean delete(SourceBean request, SourceBean response) {
    	SourceBean statement = (SourceBean)getConfig().getAttribute("QUERIES.DELETE_QUERY") ;
		if (statement == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"DefaultDetailModule::select: select_query non impostata nel modulo");
			return false;
		}
		return (new ConfigRowHandler()).deleteConfigRow(statement, getRequestContainer(), getResponseContainer());
    } // public boolean delete(SourceBean request, SourceBean response)
}