/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.service.util;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.dispatching.service.ServiceIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.io.os400.OS400FileIO;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import java.io.Serializable;
import java.util.List;

/**
 * Tipicamente utilizzata come modulo per la scrittura su file
 * 
 * E' solo un primo inizio, pu essere estesa alla lettura da fonti di svariata natura e soprattutto a scrive 
 * su pi tipologie di file system
 * @author grassi
 *
 */
public class DelegateIfsService extends AbstractDelegateService {
	private static final long serialVersionUID = 1L;
	
	/**
	 * Execution of I/O service into Integrate File System (AS/400 file system)
	 * Sample configuration:
	 *          <CONFIG ..>
	 *             <WRITE_TO filename="/arcafile/excel/FileComuni/FSFP_G_BPU_2007-03-07-13_23_38_161000.zip">
	 *                 <PARAMETER type="RELATIVE" value="ROWS" scope="SERVICE_REQUEST" />
	 *                 <FORMATTING type="csv" separator="|" />
	 *             </WRITE_TO>
	 *             ...
	 *          </CONFIG>
	 * @param service
	 * @param request
	 * @param response
	 * @throws Exception
	 */
	public static void service(ServiceIFace service, SourceBean request, SourceBean response) throws Exception {
		InitializerIFace serviceInitializer = (InitializerIFace) service;
		SourceBean writeTo = (SourceBean) serviceInitializer.getConfig().getAttribute("WRITE_TO");
		RequestContextIFace serviceRequestContext = (RequestContextIFace) service;
		writeTo(writeTo, serviceRequestContext, request, response) ;
/*
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "DelegateIfsService::insert: errore inserimento riga");
		EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
		engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, 10008));
		restoreFields(service, request, response);
*/
	}
	
	/**
	 * Read data following Config instruction then write into requested file of IFS
	 * @param writeToConfig
	 * @param serviceRequestContext
	 * @param request
	 * @param response
	 * @throws Exception
	 */
	private static void writeTo(SourceBean writeToConfig, RequestContextIFace serviceRequestContext, SourceBean request, SourceBean response) throws Exception {
		StringBuffer dataString = readFormattedData(writeToConfig, serviceRequestContext, request, response) ;
		String fileName = getParameter((SourceBean)writeToConfig.getAttribute("FILENAME"), "nomeFile", serviceRequestContext) ;
		OS400FileIO.writeAsciiFile(dataString.toString().getBytes(), fileName) ;
	}
	
	/**
	 * Read data from <PARAMETER> config keys
	 * @param writeToConfig
	 * @param serviceRequestContext
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	private static StringBuffer readFormattedData(SourceBean writeToConfig, RequestContextIFace serviceRequestContext, SourceBean request, SourceBean response) throws Exception {
		StringBuffer dataString = null ;
		List<SourceBean> parametersList = writeToConfig.getAttributeAsList("PARAMETER") ;
		if (parametersList != null) {
			for (SourceBean parameter : parametersList) {
				if (parameter != null) {
					Object dataValue = ContextScooping.getScopedParameter(serviceRequestContext.getRequestContainer(),
							serviceRequestContext.getResponseContainer(),
							(String)parameter.getAttribute("VALUE"),
							(String) parameter.getAttribute("SCOPE"),
							parameter) ;
					if (dataValue != null && dataString == null)
						dataString = new StringBuffer("") ;
					if (dataValue instanceof String)
						dataString.append(dataValue);
					else if (dataValue instanceof SourceBean)
						dataString.append(formatSourceBean((SourceBean)dataValue, writeToConfig, serviceRequestContext)) ;
					else {
						TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "DelegateIfsService::readFormattedData: Tipo " + dataValue.getClass().getName() +" non implementato");
						EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
						engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "Tipo " + dataValue.getClass().getName() +" non implementato"));
						dataString.append(dataValue);
					}
				}
			}
		}
		if (dataString == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "DelegateIfsService::readFormattedData: non ho trovato dati da leggere");
			EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
			engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "Non sono stati forniti dati da leggere: controllare la configurazione dei PARAMETER"));
		}
		
		return dataString ;
	}
	
	/**
	 * Formattig a SourceBean data using configuration parameter
	 * Sample config:
	 * <FORMATTING>
	 *    <PARAMETER type="RELATIVE" value="formatoRecord" scope="SERVICE_REQUEST" />
	 *    <PARAMETER type="RELATIVE" value="separator" scope="SERVICE_REQUEST" />
	 * </FORMATTING>
	 * 
	 * @param dataValue
	 * @param writeToConfig
	 * @param serviceRequestContext
	 * @return
	 * @throws SourceBeanException 
	 */
	private static StringBuffer formatSourceBean(SourceBean dataValue, SourceBean writeToConfig, RequestContextIFace serviceRequestContext) throws SourceBeanException {
		StringBuffer formattedValue = new StringBuffer("") ;
		String formatting = getParameter((SourceBean)writeToConfig.getAttribute("FORMATTING"), "formatoRecord", serviceRequestContext) ;

		int count=0 ;
		if (formatting.equalsIgnoreCase("CSV")) {
			String separator = getParameter((SourceBean)writeToConfig.getAttribute("FORMATTING"), "separator", serviceRequestContext) ;
			List<SourceBean> rowList = dataValue.getAttributeAsList("ROW") ;
			if (rowList != null) {
				for (SourceBean row : rowList) {
					if (row != null) {
						List<SourceBeanAttribute> attributes = row.getContainedAttributes() ;
						StringBuffer csvRow = new StringBuffer("") ;
						Object value ;
						for (SourceBeanAttribute attr : attributes) {
							value = attr.getValue() ;
							if (value instanceof String)
								value = ((String)value).trim() ;
							csvRow.append(value) ;
							csvRow.append(separator) ;
						}
						formattedValue.append(csvRow) ;
						count++ ;
						formattedValue.append("\n") ;
					} else {
						TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "DelegateIfsService::formatSourceBean: Attributo ROWS non contiene ROW");
						EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
						engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "Attributo ROWS non contiene ROW"));
					}
				}
				String rowCount = (String)serviceRequestContext.getServiceResponse().getAttribute("ROW_COUNT") ;
				if (rowCount != null) {
					count += Integer.parseInt(rowCount) ;
				}
				serviceRequestContext.getServiceResponse().updAttribute("ROW_COUNT", count) ;
			} else {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "DelegateIfsService::formatSourceBean: Attributo ROWS.ROW non trovato nel SourceBean di input");
				EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
				engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "Attributo ROWS.ROW non trovato nel SourceBean di input"));
			}
		}else {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "DelegateIfsService::formatSourceBean: FORMATTING.TYPE " + formatting +" non implementato");
			EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
			engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING, "FORMATTING.TYPE " + formatting +" non implementato"));
			formattedValue.append(dataValue.toString()) ;
		}
		return formattedValue ;
	}

	private static String getParameter(SourceBean bean, String key, RequestContextIFace serviceRequestContext) throws SourceBeanException {
		SourceBean paramBean = (SourceBean)bean.getFilteredSourceBeanAttribute("PARAMETER", "value", key) ;
		Object dataValue = ContextScooping.getScopedParameter(serviceRequestContext.getRequestContainer(),
				serviceRequestContext.getResponseContainer(),
				(String)paramBean.getAttribute("VALUE"),
				(String)paramBean.getAttribute("SCOPE"),
				paramBean) ;
		return dataValue.toString() ;
	}
}