/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/

package it.eng.spago.dispatching.service.util;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.ServiceIFace;
import it.eng.spago.util.QueryExecutor;
import java.util.List;

public class DelegateInnerQueryService extends AbstractDelegateService {
	private static final long serialVersionUID = 1L;

	@SuppressWarnings("unchecked")
	public static void service(ServiceIFace service, SourceBean request, SourceBean response) throws Exception {
		String poolName = (String)getConfig(service).getAttribute("pool") ;
		List<SourceBean> queryList = getConfig(service).getAttributeAsList("QUERIES.QUERY") ;
		
		int rowsIndex = 0 ;
		if (queryList != null) {
			for (SourceBean queryBean : queryList) {
				if (queryBean != null) {
					getServiceResponse(service).setAttribute(executeRecursiveQuery(service, poolName, queryBean, null)) ;
					rowsIndex++ ;
				}
			}
		}
		
		if (rowsIndex == 0) // Torno almeno un ROWS vuoto
			getServiceResponse(service).setAttribute(new SourceBean("ROWS")) ;
	}
	
	/**
	 * Esegue ricorsivamente delle queries in cascata e compone il risultato di tutte nella forma:
	 * 
	 *   <ROWS>
	 *     <ROW val1="1" val2="2">
	 *       <ROWS>
	 *         <ROW val3="3" val4="4>
	 *       </ROWS>
	 *     </ROW>
	 *   </ROWS>
	 *   
	 * @param poolName
	 * @param queryBean
	 * @param parentRow
	 * @return
	 * @throws SourceBeanException
	 */
	private static SourceBean executeRecursiveQuery(ServiceIFace service, String poolName, SourceBean queryBean, SourceBean parentRow) throws SourceBeanException {
		SourceBean result = null;
		boolean execute = true;
		
		SourceBean reqTo = (SourceBean)getServiceRequest(service).cloneObject();
		//XXX E' solo una pezza
		//Per correggere va modificato il meccanismo di gestione dei parametri 
		//delle query direttamente dal requestContainer
		
		try {
			if (queryBean.getAttribute("CONDITIONS")!=null) {
				for (Object row : queryBean.getAttributeAsList("CONDITIONS.PARAMETER")) {
					
					if (row != null && row instanceof SourceBean) {
						String scope = (String)((SourceBean)row).getAttribute("SCOPE") ;
						String name = (String)((SourceBean)row).getAttribute("NAME") ;
						Object valueToTest = ((SourceBean)row).getAttribute("VALUE");
						Object actualValue = null;
						
						if (scope != null && scope.equalsIgnoreCase("LOCAL")) {
							actualValue = parentRow.getAttribute(name);
						} else {
							actualValue = getServiceRequest(service).getAttribute(name);
						}
						
						if (actualValue!=null && !actualValue.equals(valueToTest)) {
							execute = false;
						}
					}
				}
			}
		} catch (RuntimeException e) {
			//XXX
		} 
		
		if (execute) {
		
			SourceBean changeQueryBean = handleLocalParameter(service, queryBean, parentRow) ;
			result = (SourceBean)QueryExecutor.executeQuery(getRequestContainer(service), getResponseContainer(service), poolName, changeQueryBean, QueryExecutor.READ);
			if (result != null) {
				result.setAttribute("QUERY_NAME", queryBean.getAttribute("statement")) ;
			}
			
			for (Object row : result.getAttributeAsList("ROW")) {
				if (row != null) {
					for (Object childQueryBean : queryBean.getAttributeAsList("QUERY")) {
						if (childQueryBean != null){
							SourceBean resultRecursive = executeRecursiveQuery(service, poolName, (SourceBean)childQueryBean, (SourceBean)row);
							if(resultRecursive!=null)
								((SourceBean)row).setAttribute(resultRecursive) ;
						}
					}
				}
			}
			
		}
		
		(( DefaultRequestContext )service).getRequestContainer().setServiceRequest(reqTo);
		
		return result;
	}
	
	/**
	 * Questo metodo serve per valorizzare la SERVICE_REQUEST con i parametri richiesti dalla query e
	 *   reperiti dalla riga appena estratta (row)
	 * Il metodo inoltre modifica il bean di configurazione della query con scope SERVICE_REQUEST
	 *   per poter far funzionare lo StatementProvider
	 * @param queryBean
	 * @param row
	 * @throws SourceBeanException 
	 */
	private static SourceBean handleLocalParameter(final ServiceIFace service, final SourceBean queryBean, final SourceBean row) throws SourceBeanException {
		SourceBean updateQueryBean = (SourceBean)queryBean.cloneObject() ;
		addParameterInServiceRequest(service, updateQueryBean, row, "PARAMETER") ;
		addParameterInServiceRequest(service, updateQueryBean, row, "FILTER_PARAMETER") ;
		return updateQueryBean ;
	}
	
	/**
	 * Questo metodo serve per valorizzare la SERVICE_REQUEST con i parametri richiesti dalla query e
	 *   reperiti dalla riga appena estratta (row)
	 * E' fatto per poter essere richiamato per i vari tipi di PARAMETER esistenti
	 * @param updateQueryBean
	 * @param row
	 * @param keyParameter
	 * @throws SourceBeanException
	 */
	private static void addParameterInServiceRequest(ServiceIFace service, SourceBean updateQueryBean, final SourceBean row, String keyParameter) throws SourceBeanException {
		for (Object par : updateQueryBean.getAttributeAsList(keyParameter)) {
			if (par != null && par instanceof SourceBean) {
				String scope = (String)((SourceBean)par).getAttribute("SCOPE") ;
				if (scope != null && scope.equalsIgnoreCase("LOCAL")) {
					// Estrae il nome del parametro (value nella configurazione di <PARAMETER>
					String parValue = (String)((SourceBean)par).getAttribute("VALUE") ;
					// Ricerca il valore tra i dati della riga
					Object value = row.getAttribute(parValue);
					// Inserisce nella SERVICE_REQUEST il nuovo parametro per l'esecuzione della Query
					getServiceRequest(service).updAttribute(parValue, value) ;
					// Modifica lo SCOPE del bean del <PARAMETER> da "LOCAL" a "SERVICE_REQUEST"
					((SourceBean)par).updAttribute("SCOPE", "SERVICE_REQUEST") ;
				}
			}
		}
	}
}