/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.io.os400;

import it.eng.spago.base.Constants;
import it.eng.spago.tracing.TracerSingleton;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import com.ibm.as400.access.AS400;
import com.ibm.as400.access.IFSFileInputStream;
import com.ibm.as400.access.IFSJavaFile;
import com.ibm.as400.access.IFSTextFileOutputStream;

public class OS400FileIO {
	public final static String FILE_PATH_SEPARATOR = "/"  ;
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			// byte[] fileByte = OS400FileIO.readBinaryFile("/arcafile/excel/REV_PORTAA_20061229.csv") ;
			byte[] fileByte = OS400FileIO.readBinaryFile("/arcafile/excel/FileComuni/FSFP_G_BPU_2007-03-07-13_23_38_161000.zip") ;
			System.out.println("Integer.MAX_VALUE= "+ Integer.MAX_VALUE) ;

			File outputFile = new File("c:\\test.txt");
			OutputStream out = new FileOutputStream(outputFile);
	        out.write(fileByte) ;
	        out.close() ;
/*
			char[] convFile = ByteToCharASCII.getDefault().convertAll(fileByte);
			for (int i=0 ; i < fileByte.length ; i++)
				System.out.print(convFile[i]) ;
*/
			System.out.println("\nLetti " + fileByte.length + " byte") ;
		} catch (Exception ex) {
			ex.printStackTrace() ;
		} finally {
			OS400AdapterResourcePool.release() ;
		}
	}
	
	/**
	 * Legge un file "binario" da AS400
	 * @param fName
	 * @return
	 * @throws Exception
	 */
	public static byte[] readBinaryFile(String fName) throws Exception {
		byte[] byteReading = null ;
		AS400 connection = null ;
		IFSFileInputStream inputStream = null ;
		try {
			connection = OS400AdapterResourcePool.getInstance().getConnection() ;
			IFSJavaFile file = new IFSJavaFile(connection, fName) ;
			inputStream = new IFSFileInputStream(file) ;

			// Get the size of the file
	        long length = file.length();

	        // You cannot create an array using a long type.
	        // It needs to be an int type.
	        // Before converting to an int type, check
	        // to ensure that file is not larger than Integer.MAX_VALUE.
	        if (length > Integer.MAX_VALUE) {
				TracerSingleton.log(
		                Constants.NOME_MODULO,
		                TracerSingleton.CRITICAL,
		                "OS400FileIO:readBynaryFile: File "+file.getName() + " too large for reading");
	            throw new IOException("File "+file.getName() + " too large for reading");
	        }
	    
	        // Create the byte array to hold the data
			byteReading = new byte[(int)length] ;
	    
	        // Read in the bytes
	        int offset = 0;
	        int numRead = 0;
	        while (offset < byteReading.length
	               && (numRead=inputStream.read(byteReading, offset, byteReading.length-offset)) >= 0) {
	            offset += numRead;
	        }
	    
	        // Ensure all the bytes have been read in
	        if (offset < byteReading.length) {
				TracerSingleton.log(
		                Constants.NOME_MODULO,
		                TracerSingleton.CRITICAL,
		                "OS400FileIO:readBynaryFile: Could not completely read file "+file.getName());
	            throw new IOException("Could not completely read file "+file.getName());
	        }
		} catch (Exception ex) {
			TracerSingleton.log(
	                Constants.NOME_MODULO,
	                TracerSingleton.CRITICAL,
	                "OS400FileIO:readBynaryFile errore in lettura " + ex);
			throw ex ;
		} finally {
			try {
				if (inputStream != null)
					inputStream.close() ;
			} catch (IOException ex) {
				TracerSingleton.log(
		                Constants.NOME_MODULO,
		                TracerSingleton.CRITICAL,
		                "OS400FileIO:readBynaryFile errore in chiusura InputStream " + ex);
			}
			if (connection != null)
				OS400AdapterResourcePool.getInstance().closeConnection(connection) ;
		}
		return byteReading ;
	}
	
	/**
	 * Write a string to a new file Into integrate File System
	 * @param bufIn
	 * @param fileName
	 * @throws Exception
	 */
	public static void writeAsciiFile(byte[] bufIn, String fileName) throws Exception {
		AS400 connection = null ;
		IFSTextFileOutputStream file = null ;
		//IFSFileOutputStream file = null ;
		try {
			connection = OS400AdapterResourcePool.getInstance().getConnection() ;
			file = new IFSTextFileOutputStream(connection, fileName,IFSTextFileOutputStream.SHARE_ALL,false,819);
			//file.setCCSID(280);
			//file = new IFSFileOutputStream(connection, fileName, -1,false,280);
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Valore del CCSID:"+file.getCCSID());
			file.write(bufIn);

		} catch (Exception ex) {
			TracerSingleton.log(
	                Constants.NOME_MODULO,
	                TracerSingleton.CRITICAL,
	                "OS400FileIO:writeAsciiFile errore in scrittura " + ex);
			throw ex ;
		} finally {
			try {
				if (file != null)
					file.close() ;
			} catch (IOException ex) {
				TracerSingleton.log(
		                Constants.NOME_MODULO,
		                TracerSingleton.CRITICAL,
		                "OS400FileIO:writeAsciiFile errore in chiusura TextFileOutputStream " + ex);
			}
			if (connection != null)
				OS400AdapterResourcePool.getInstance().closeConnection(connection) ;
		}
	}

	/**
	 * Make a full path name
	 * @param inPath   File path name
	 * @param name     File name
	 * @return
	 */
	public static String getFullPathFileName(final String inPath, final String name) {
		String path = inPath ;
		if (path == null || path.trim().length() == 0)
			path = FILE_PATH_SEPARATOR ;
		path = path.trim() ;
		if (! path.endsWith(FILE_PATH_SEPARATOR))
			path += FILE_PATH_SEPARATOR ;
		
		return path + name ;
	}
}