/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.listeners;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.cache.CacheResponseHandler;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.event.ActionEndEvent;
import it.eng.spago.event.ActionExceptionEvent;
import it.eng.spago.event.ISpagoEvent;
import it.eng.spago.event.ValidationFailedEvent;
import it.eng.spago.presentation.Publisher;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

import org.apache.commons.validator.GenericValidator;

public class CacheResponseListener implements ISpagoListener {

    public void onEvent(ISpagoEvent event, RequestContainer requestContainer,
            ResponseContainer responseContainer) {
        if ((event instanceof ValidationFailedEvent) ||
            (event instanceof ActionExceptionEvent) ||
            (event instanceof ActionEndEvent)) {
            
            EMFErrorHandler errorHandler = responseContainer.getErrorHandler();
            String businessName = (String)requestContainer.getAttribute(AbstractCoordinator.CURRENT_SERVICE_NAME);

            String lastPublisherName = getLastPublisherName(requestContainer);
            SourceBean cachedResponseForPublisher = getCachedResponseForPublisher(requestContainer, lastPublisherName);
			String categories = (String) ConfigSingleton.getInstance()
				.getAttribute(Constants.AUTOMATIC_REDIRECTION_ERRORS);
            if (errorHandler != null){
    			boolean hasErrors = (!GenericValidator.isBlankOrNull(categories))?
    					!errorHandler.isOKByCategory(categories) :
    					!errorHandler.isOK();
    			if (hasErrors) {
					// Si sono verificati errori nell'esecuzione del servizio
	                if ((isReachable(businessName)) && ( cachedResponseForPublisher != null)){
	    				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
			            	"CacheResponseListener::onEvent: Retrieving cached response");
	
	                    // Allow to recognize that this is a cached response
	    				try {
		    				cachedResponseForPublisher.updAttribute(CacheResponseHandler.CACHED_RESPONSE_ATTRIBUTE, "TRUE");
	    				} catch (SourceBeanException ex) {
	    					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
	    				            "CacheResponseListener::onEvent: ", ex);
	    				}
	    				
	    				// Set cached response as THE response
	    				responseContainer.setServiceResponse(cachedResponseForPublisher);
	                }
	            } else {
	                if ((isReachable(businessName)) && ( cachedResponseForPublisher != null)) {
	    				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
			            	"CacheResponseListener::onEvent: Removing cached response from cache");
	                    deleteCachedResponseForPublisher(requestContainer, lastPublisherName);
	                }
	            }
            }
        }
    }

    private boolean isReachable(String businessName){
        List isReachableList = (List)ApplicationContainer.getInstance().getAttribute("REACHABLE_LIST");
        return (isReachableList != null) ? isReachableList.contains(businessName) : false;
    }
    
    
    private String getLastPublisherName(RequestContainer requestContainer){
        return Publisher.getLastPublisherName();
    }
    
    private SourceBean getCachedResponseForPublisher(RequestContainer requestContainer, String publisherName){
    
        SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer(); 
        SourceBean cachedResponses =  (SourceBean)permanentContainer.getAttribute(CacheResponseHandler.CACHED_RESPONSE);
        
        if (cachedResponses != null){
            SourceBean sb = (SourceBean)cachedResponses.getAttribute(publisherName);
            if (sb != null) {
                return (SourceBean)sb.getAttribute(Constants.SERVICE_RESPONSE);
            } else {
                return null;
            }
        }else{  
            return null;
        }
    }
    
    private void deleteCachedResponseForPublisher(RequestContainer requestContainer, String publisherName){
        
        SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer(); 
        SourceBean cachedResponses =  (SourceBean)permanentContainer.getAttribute(CacheResponseHandler.CACHED_RESPONSE);
        
        if (cachedResponses != null){
            try{
                cachedResponses.delAttribute(publisherName);
            }catch (SourceBeanException ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "Error in deleting cached response for publisher ["+publisherName+"]", ex);
            }
        }   
    }
}
