/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.lookup;

import java.util.HashMap;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.navigation.NavigationException;
import it.eng.spago.navigation.Navigator;
import it.eng.spago.tracing.TracerSingleton;

/**
 * @author scarciello
 *
 */
public class LookupMaster {

	static void handleRequestForLookup(RequestContainer request) {
		SourceBean serviceRequest = (SourceBean) request.getServiceRequest();
		SourceBean fieldSet;
		String fieldSetName = (String)serviceRequest.getAttribute("FIELDSET");
		String afForm = (String)serviceRequest.getAttribute("AF_FORM");
		if (fieldSetName==null) return;
		String retAction = (String)serviceRequest.getAttribute("RET_ACTION");
		if(retAction!=null) {
			SourceBean action = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("ACTIONS.ACTION", "NAME", retAction);
			if (action==null) return;
			fieldSet = (SourceBean)action.getFilteredSourceBeanAttribute("FIELDS.FIELDSET", "NAME", fieldSetName);
			if(fieldSet==null)
				fieldSet = (SourceBean)action.getFilteredSourceBeanAttribute("CONFIG.FIELDS.FIELDSET", "NAME", fieldSetName);
		}
		else {
			//String retMod = (String)serviceRequest.getAttribute("RET_MOD");
			String retMod = (String)serviceRequest.getAttribute("CURRENT_MODULE");
			if (retMod==null) return;
			SourceBean module = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", retMod);
			if(module==null) return;
			fieldSet = (SourceBean)module.getFilteredSourceBeanAttribute("CONFIG.FIELDS.FIELDSET", "NAME", fieldSetName);
		}
		if(fieldSet==null) return;
		String lookupName = (String)fieldSet.getAttribute("LOOKUP");
		if(lookupName==null) return;
		// recupera il SourceBean della Lookup
		SourceBean lookup = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("LOOKUPS.LOOKUP", "NAME", lookupName);
		if(lookup==null) return ;
		// Creazione del nuovo RequestContainer e della nuova ServiceRequest
		// Crea una HashMap e vi valorizza i valori transitori provenienti dal modulo chiamante e destinati
		// alla lookup identificandone anche i nomi
		HashMap valoriParametri = new HashMap();
		// Cicla all'interno dei campi della lookup per recuperarne i riferimenti
		for(SourceBean field : (List<SourceBean>)lookup.getAttributeAsList("CONFIG.FIELD")) {
			String sourceID = (String)field.getAttribute("SOURCEID");
			String destinationID = (String)field.getAttribute("DESTID");
			SourceBean fieldID = (SourceBean)(fieldSet.getFilteredSourceBeanAttribute("FIELD","ID",sourceID));
			if(fieldID!=null) {
				String fieldName = (String)fieldID.getAttribute("NAME");
				if(fieldName!=null && fieldName.length()>0) {
					Object fieldValue = serviceRequest.getAttribute(fieldName);
					valoriParametri.put(destinationID, fieldValue);
					fieldValue = serviceRequest.getAttribute("READONLY_"+fieldName);
					if (fieldValue!=null)
						valoriParametri.put("READONLY_" + destinationID, fieldValue);
				}
			}
		}
		
		// Verificare se si pu� sovrascrivere semplicemente il requestContainer nella Session
		// o se bisogna appenderlo.
		// TODO verifica uso del request container
		RequestContainer newRequest = (RequestContainer) request.cloneObject();
		request.setParent(newRequest);
		
		// Salvataggio nella sessione del RequestContainer
		SessionContainer permanentContainer = request.getSessionContainer();
		permanentContainer.setAttribute("PARENT_REQUEST", request.cloneObject());
		
		try {
			
			// Prepara la request in caso di back
			SourceBean backRequest = ((RequestContainer)request.getParent()).getServiceRequest();
			backRequest.delAttribute("NAVIGATOR_DISABLED");
			backRequest.delAttribute("ACTION_NAME");
			backRequest.delAttribute("FIELDSET");
			//String retMod = (String)backRequest.getAttribute("RET_MOD");
			//backRequest.delAttribute("RET_MOD");
			//backRequest.setAttribute("MODULE",retMod);
			
			String retAfBut = (String)backRequest.getAttribute("RET_AF_BUTTON");
			backRequest.delAttribute("RET_AF_BUTTON");
			if(retAfBut!=null)
				backRequest.setAttribute("AF_BUTTON",retAfBut);
			
			
			String retPage = (String)backRequest.getAttribute("RET_PAGE");
			if(retPage!=null) {
				backRequest.delAttribute("RET_PAGE");
				backRequest.setAttribute("PAGE",retPage);
			}
			else {
				String retActionBack = (String)backRequest.getAttribute("RET_ACTION");
				backRequest.delAttribute("RET_ACTION");
				backRequest.setAttribute("ACTION_NAME",retActionBack);
			}

			// La sezione di codice seguente rimuove i campi con valori vuoti
			// in modo da non creare problemi con la validazione durante il rientro dalla lookup
			List<SourceBeanAttribute> attributi = (List<SourceBeanAttribute>)backRequest.getContainedAttributes();
			for(SourceBeanAttribute attributo : attributi) {
				String key = attributo.getKey();
				Object value = attributo.getValue();
				if (value instanceof String && value.equals("")) 
					backRequest.delAttribute(key);
			}
			
			String message = (String)backRequest.getAttribute("MESSAGE");
			if (message!=null) {
				if (message.equalsIgnoreCase("DETAIL_INSERT"))
					message="DETAIL_NEW";
				if (message.equalsIgnoreCase("DETAIL_UPDATE"))
					message="DETAIL_SELECT";
				backRequest.updAttribute("MESSAGE",message);
			}
			Navigator.signService(newRequest);
			Navigator.signService(newRequest, lookupName );
			
			// Ripulisce la service request e quindi setta gli attributi
			serviceRequest = request.getServiceRequest();
			serviceRequest.clearBean();

			String destPage = (String)lookup.getAttribute("CONFIG.DESTPAGE");
			if (destPage!=null)
				serviceRequest.setAttribute("PAGE", destPage);
			String destModule = (String)lookup.getAttribute("CONFIG.DESTMODULE");
			if (destModule!=null)
				serviceRequest.setAttribute("MODULE", destModule);
			String destAction = (String)lookup.getAttribute("CONFIG.DESTACTION");
			if (destAction!=null)
				serviceRequest.setAttribute("ACTION_NAME", destAction);
			for(SourceBean field : (List<SourceBean>)lookup.getAttributeAsList("CONFIG.FIELD")) {
				String destinationID = (String)field.getAttribute("DESTID");
				Object valPar = valoriParametri.get(destinationID);
				if(valPar!=null)
					serviceRequest.setAttribute("FILTER_"+destinationID, valPar);
				valPar = valoriParametri.get("READONLY_" + destinationID);
				if(valPar!=null)
					serviceRequest.setAttribute("READONLY_"+destinationID, valPar);
			}
			if(afForm!=null)
				serviceRequest.setAttribute("AF_FORM",afForm);
			serviceRequest.setAttribute("AF_BUTTON","LOOKUP");
			serviceRequest.setAttribute("SHOWLOOKUP","TRUE");
			serviceRequest.setAttribute("LOOKUPNAME",lookupName);
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"LookupMaster::handleRequestForLookup::Error",e);
			//e.printStackTrace();
		}
		
	}
	
	static void handleRequestFromLookup(RequestContainer request) {
		String module = "";
		String af_button = "";
		SessionContainer permanentContainer = request.getSessionContainer();
		RequestContainer parentRequest = (RequestContainer)permanentContainer.getAttribute("PARENT_REQUEST");
		if(parentRequest!=null)
			parentRequest = (RequestContainer)parentRequest.getParent();
			else parentRequest=request;
		try {
			SourceBean fieldset = null;
			SourceBean serviceRequest = parentRequest.getServiceRequest();
			serviceRequest.delAttribute("ACTION_NAME");
			serviceRequest.delAttribute("PAGE");
			serviceRequest.delAttribute("MODULE");
			String fieldsetName = (String)serviceRequest.getAttribute("FIELDSET");

			String actionName = (String)serviceRequest.getAttribute("RET_ACTION");
			if (actionName != null) {
				serviceRequest.delAttribute("RET_ACTION");
				serviceRequest.setAttribute("ACTION_NAME",actionName);
				SourceBean action = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("ACTIONS.ACTION", "NAME", actionName);
				fieldset = (SourceBean)action.getFilteredSourceBeanAttribute("FIELDS.FIELDSET", "NAME", fieldsetName);
				if(fieldset==null) 
					fieldset = (SourceBean)action.getFilteredSourceBeanAttribute("CONFIG.FIELDS.FIELDSET", "NAME", fieldsetName);
			} else {
				String pageName = (String)serviceRequest.getAttribute("RET_PAGE");
				
				module = (String)serviceRequest.getAttribute("CURRENT_MODULE");
				af_button = (String)serviceRequest.getAttribute("RET_AF_BUTTON");
				
				SourceBean moduleBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", module);
				fieldset = (SourceBean)moduleBean.getFilteredSourceBeanAttribute("CONFIG.FIELDS.FIELDSET", "NAME", fieldsetName);
				serviceRequest.delAttribute("RET_PAGE");
				serviceRequest.delAttribute("RET_AF_BUTTON");
				//serviceRequest.delAttribute("RET_MOD");
				serviceRequest.setAttribute("PAGE",pageName);
				//serviceRequest.setAttribute("MODULE",module);
				if (af_button==null || af_button.equalsIgnoreCase("null")) {
					
				} else {
					serviceRequest.setAttribute("AF_BUTTON",af_button);
				}
			}
			if(fieldset==null) return;
			String lookupName = (String)fieldset.getAttribute("LOOKUP");
			if(lookupName==null) return;
			// recupera il SourceBean della Lookup
			SourceBean lookup = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("LOOKUPS.LOOKUP", "NAME", lookupName);
			if(lookup==null) return ;
			SourceBean lookuprequest = request.getServiceRequest();
			for(SourceBean field : (List<SourceBean>)lookup.getAttributeAsList("CONFIG.FIELD")) {
				String sourceID = (String)field.getAttribute("SOURCEID");
				String destID = (String)field.getAttribute("DESTID");
				Object value = lookuprequest.getAttribute(destID);
				for(SourceBean fieldsetField : (List<SourceBean>)fieldset.getAttributeAsList("FIELD")) {
					String fieldID = (String)fieldsetField.getAttribute("ID");
					String fieldName = (String)fieldsetField.getAttribute("NAME");
					if (sourceID.equalsIgnoreCase(fieldID) && value!=null) {
						serviceRequest.updAttribute(fieldName,value);
					}
				}
			}
			
			// La sezione di codice seguente rimuove i campi con valori vuoti
			// in modo da non creare problemi con la validazione durante il rientro dalla lookup
			List<SourceBeanAttribute> attributi = (List<SourceBeanAttribute>)serviceRequest.getContainedAttributes();
			for(SourceBeanAttribute attributo : attributi) {
				String key = attributo.getKey();
				Object value = attributo.getValue();
				if (value instanceof String && value.equals("")) 
					serviceRequest.delAttribute(key);
			}
			
			// Corregge la forma del messaggio per evitare un inserimento illecito
			String message = (String)serviceRequest.getAttribute("MESSAGE");
			if (message!=null) {
				if (message.equalsIgnoreCase("DETAIL_INSERT"))
					message="DETAIL_NEW";
				if (message.equalsIgnoreCase("DETAIL_UPDATE"))
					message="DETAIL_SELECT";
			}
			else {
				if(((String)serviceRequest.getAttribute("AF_BUTTON","")).equalsIgnoreCase("SELECT"))
					message="DETAIL_SELECT";
				else message="DETAIL_NEW";
			}
			serviceRequest.setAttribute("PREV_MESSAGE",message);
			if (serviceRequest.containsAttribute("MESSAGE"))
					serviceRequest.delAttribute("MESSAGE") ;
			
			request.getServiceRequest().setAttribute("NAVIGATOR_BACK_TO_SERVICE_LABEL", lookupName);
			try {
				Navigator.checkNavigation(request);
				
				// Pulisce la request relativa al modulo dalle marchiature
				RequestContainer tmpRequest = request;
				String moduleName = (String) tmpRequest.getServiceRequest().getAttribute("CURRENT_MODULE");
				while (tmpRequest!=null) {
					moduleName = (String) tmpRequest.getServiceRequest().getAttribute("CURRENT_MODULE");
					if (moduleName!=null && moduleName.equalsIgnoreCase(module))
						Navigator.unsignService(tmpRequest);
					tmpRequest = (RequestContainer) tmpRequest.getParent();
				}
			} catch (NavigationException e) {
				TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"LookupMaster::handleRequestFromLookup::Error",e);
				//e.printStackTrace();
			}
			
			request.setAttribute("NAVIGATOR_MARK", "TRUE");
			request.setAttribute("IM_LOOKUP", "TRUE");
			
			request.setServiceRequest(serviceRequest);
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"LookupMaster::handleRequestFromLookup::Error",e);
			//e.printStackTrace();
		}
		return ;
	}
	
	public static void service(RequestContainer request) {
		SourceBean serviceRequest = (SourceBean) request.getServiceRequest();
		
		// Elabora la request per verificare se si sta chiamando una lookup
		String retPage = (String)serviceRequest.getAttribute("RET_PAGE");
		if (retPage != null) 
			handleRequestForLookup(request);
		else {
			String retAction = (String)serviceRequest.getAttribute("RET_ACTION");
			if (retAction != null) 
				handleRequestForLookup(request);
		}
		
		// Elabora la request per verificare se si sta tornando da una lookup
		String retLup = (String)serviceRequest.getAttribute("RET_LUP");
		if (retLup != null)
			handleRequestFromLookup(request);
	}

}
