/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.monitor;

import it.eng.spago.base.Constants;
import it.eng.spago.statistic.MonitorStringSerializer;
import it.eng.spago.tracing.TracerSingleton;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
 * ServletContextListener used to log Jamon statistics at the when the servlet
 * context is destroyed. The class log on <code>MONITOR_LOGGER</code> configured
 * logger.
 * A valid configuration for <code>tracing.xml</code> is:
 * 
 * <pre>
 *   &lt;LOGGER name="MONITOR_LOGGER" class="it.eng.spago.tracing.Log4JLogger">
 *     &lt;CONFIG trace_min_log_severity="0" debug="false"/>
 *   &lt;/LOGGER>
 * </pre>
 * 
 * @author buso
 *
 */
public class MonitorServletListener implements ServletContextListener {

	/**
	 * Log Jamon statistics.
	 */
	public void contextDestroyed(ServletContextEvent arg0) {
		Monitor[] monitors = MonitorFactory.getRootMonitor().getMonitors();
		
		try {
			for (int idx = 0; monitors != null && idx < monitors.length; idx++) {
				TracerSingleton.log("MONITOR_LOGGER", TracerSingleton.INFORMATION,
					MonitorStringSerializer.serialize(monitors[idx]));
			}
		} catch (Throwable th) {
			TracerSingleton.log("MONITOR_LOGGER",TracerSingleton.CRITICAL,"MonitorServletListener::contextDestroyes::Error");
			//th.printStackTrace();
		}
	}

	public void contextInitialized(ServletContextEvent arg0) {
	}

}
