/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.navigation;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;

import java.util.List;

public class NavigationHelper {
	
	/**
	 * Se la serviceResponse ha restituito il parametro RETURN_TO_LIST questo metodo forza il recupero e la riesecuzione 
	 * della precedente request salvata
	 * @param requestContainer
	 * @param serviceRequest
	 * @param serviceResponse
	 * @return
	 * @throws SourceBeanException
	 */
	public static boolean handleReturnToList(RequestContainer requestContainer, SourceBean serviceRequest, SourceBean serviceResponse) throws SourceBeanException {
		
		if (!"TRUE".equals(serviceRequest.getAttribute("DISABLE_RETURN_TO_LIST")) && serviceResponse.containsAttribute("RETURN_TO_LIST")) {
			String message = (String) serviceRequest.getAttribute("MESSAGE");

			serviceRequest = new SourceBean(Constants.SERVICE_REQUEST);
			serviceRequest.setAttribute("NAVIGATOR_BACK_TO_MARK", "1");
			requestContainer.setAttribute("RETURN_TO_LIST", "1");

			if (message.equalsIgnoreCase("DETAIL_INSERT")) {
				serviceRequest.setAttribute("CLEAN_FILTERS", "1");
			}
			requestContainer.setServiceRequest(serviceRequest);
			serviceResponse.delAttribute("RETURN_TO_LIST");
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Elimina dalla request di lista che deve essere ripristinata il valore dei
	 * filtri di ricerca SE  appena stato inserito un nuovo record ( che
	 * potrebbe risultare non presente tra i risultati del filtro)
	 * 
	 * @param serviceRequest
	 */
	public static void cleanListFilters(SourceBean oldRequest, SourceBean newRequest) throws SourceBeanException {

		if (oldRequest.getAttribute("CLEAN_FILTERS", "").equals("1")) {

			String module = "" + newRequest.getAttribute("AF_MODULE_NAME", "");

			SourceBean configure = ConfigSingleton.getInstance();
			SourceBean moduleBean = (SourceBean) configure.getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", module);

			if (moduleBean != null) {
				SourceBean moduleConfig = (SourceBean) moduleBean.getAttribute("CONFIG");

				List<SourceBean> filters = (List<SourceBean>) moduleConfig.getAttributeAsList("ROW_PROVIDER.CONFIG.LIST_QUERY.FILTER_PARAMETER");

				for (SourceBean bean : filters) {

					String parameterName = (String) bean.getAttribute("VALUE");

					if (("" + newRequest.getAttribute(parameterName, "")).trim().length() > 0) {

						for (Object columns : moduleConfig.getAttributeAsList("COLUMNS.COLUMN")) {
							String colName = (String) ((SourceBean) columns).getAttribute("NAME");

							if (parameterName.contains(colName)) {
								if (!((String) ((SourceBean) columns).getAttribute("READONLY", "")).equalsIgnoreCase("TRUE")) {
									newRequest.delAttribute(parameterName);
								}
							}
						}

					}

				}
			}
		}
	}

}
