/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.presentation;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;
import java.util.ArrayList;
import java.util.List;

/**
 * DATE CONTRIBUTOR/DEVELOPER NOTE 13-12-2004 Butano SourceBean ora restituisce
 * ArrayList
 * 
 */
public class Publisher {
	public static final String AF_PUBLISHER_NAME = "AF_PUBLISHER_NAME";

	public static PublisherConfiguration getPublisherConfiguration(final RequestContextIFace requestContext, Exception serviceException) {
		String publisherName = getPublisherName(requestContext, serviceException);
		PublisherConfiguration publisherConfig = getPublisher(requestContext, publisherName);
		// JAVA publisher management
		while ((publisherConfig != null) && (publisherConfig.getType() != null) && (publisherConfig.getType().equalsIgnoreCase("JAVA"))) {
			List resources = publisherConfig.getResources();
			if (resources.size() == 0) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: classe del publisher [" + publisherName + "] non specificata");
				return null;
			} // if (resources.size() == 0)
			// Modifica Monica 15/01/2004 - inizio
			// recupero il SourceBean della busta ITEM, contenente la busta
			// CONFIG
			String publisherDispatcherClass = (String) ((SourceBean) resources.get(0)).getAttribute("resource");
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Publisher::getPublisher: publisherDispatcherClass [" + publisherDispatcherClass + "]");
			PublisherDispatcherIFace publisherDispatcher = null;
			try {
				publisherDispatcher = (PublisherDispatcherIFace) Class.forName(publisherDispatcherClass).newInstance();
			} // try
			catch (Exception ex) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "Publisher::getPublisher: classe del publisher non valida", ex);
				return null;
			} // catch (Exception ex) try
			if (publisherDispatcher instanceof InitializerIFace) {
				SourceBean bustaConfig = (SourceBean) ((SourceBean) resources.get(0)).getAttribute("CONFIG");
				((InitializerIFace) publisherDispatcher).init(bustaConfig);
			}
			// Modifica Monica 15/01/2004 - fine
			publisherName = publisherDispatcher.getPublisherName(requestContext.getRequestContainer(), requestContext.getResponseContainer());
			publisherConfig = getPublisher(requestContext, publisherName);
		} // while ((publisher != null) && (publisher.getType() != null)
		// && (publisher.getType().equalsIgnoreCase("JAVA")))
		if (publisherConfig == null) {
			publisherConfig = new PublisherConfiguration();
			publisherConfig.setType(Constants.NOTHING_PUBLISHER_TYPE);
			publisherConfig.setName("UNDEFINED");
		} // if (publisher == null)
		else {
			// If the publisher is defined but no resources are defined, change publisher
			// type to NOTHING_PUBLISHER_TYPE
			// EXCEPTION: for the loop publisher the resources are not necessary because they are added
			// by the renderer
			String publisherType = publisherConfig.getType();
			if (!publisherType.equalsIgnoreCase(Constants.NOTHING_PUBLISHER_TYPE) && !publisherType.equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE) && (publisherConfig.getResources().size() == 0)) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: resources not defined for publisher [" + publisherName + "] ");
				publisherConfig.setType(Constants.NOTHING_PUBLISHER_TYPE);
			} // if (resources.size() == 0)
			// else
		} // if (publisher == null) else
		return publisherConfig;
	} // public static Publisher getPublisher(RequestContainer

	// requestContainer, ResponseContainer responseContainer, boolean
	// serviceError)
	private static String getPublisherName(final RequestContextIFace requestContext, Exception serviceException) {
		String publisherName = null;
		ConfigSingleton configSingleton = ConfigSingleton.getInstance();
		RequestContainer requestContainer = requestContext.getRequestContainer();
		ResponseContainer responseContainer = requestContext.getResponseContainer();
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Publisher::getPublisher: invocato");
		String businessType = responseContainer.getBusinessType();
		if (businessType == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: businessType nullo");
			return null;
		} // if (businessType == null)
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Publisher::getPublisher: businessType [" + businessType + "]");
		String businessName = responseContainer.getBusinessName();
		if (businessName == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: businessName nullo");
			return null;
		} // if (businessName == null)
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Publisher::getPublisher: businessName [" + businessName + "]");
		// "Automatic publisher redirection in case of errors" is a feature
		// available only
		// for HTTP channel
		if (requestContainer.getChannelType().equalsIgnoreCase(Constants.HTTP_CHANNEL)) {
			String autoPublisherRedirection = (String) configSingleton.getAttribute(Constants.AUTOMATIC_REDIRECTION);
			String errorManaged = (String) responseContainer.getAttribute(Constants.ERROR_MANAGED);
			// check if the error is already managed (by EMFExceptionHandler) 
			// and if the automatic redirection is configured
			if ((errorManaged == null || !errorManaged.equalsIgnoreCase("TRUE")) && (autoPublisherRedirection != null) && (autoPublisherRedirection.equalsIgnoreCase("TRUE"))) {
				String categories = (String) configSingleton.getAttribute(Constants.AUTOMATIC_REDIRECTION_ERRORS);
				boolean redirect = false;
				if (categories == null) {
					// Consider all errors
					redirect = !requestContext.getErrorHandler().isOK();
				} else {
					// Consider only configured errors
					redirect = !requestContext.getErrorHandler().isOKByCategory(categories);
				}
				if (redirect) {
					// Verifico se in request  presente l'attributo
					// corrispondente al publisher corrente
					String requestPublisherName = (String) requestContext.getServiceRequest().getAttribute(AF_PUBLISHER_NAME);
					// The publisher name in request has priority over the
					// stored publisher in session
					publisherName = (requestPublisherName != null) ? requestPublisherName : getLastPublisherName();
				}
			}
			// 26/01/07 A. Grassi: Modifica per intercettare publisher di Ajax
			// 15/11/07 A. Grassi: Se impostato il publisher AJAX deve essere sempre impostato 
			// anche in caso di errore !!
/*			
			String ajaxPublisher = (String) requestContainer.getServiceRequest().getAttribute("AJAX");
			if (ajaxPublisher != null && ajaxPublisher.trim().length() > 0) {
				publisherName = ajaxPublisher ;
			}
*/			
		}
		if (publisherName == null) {
			// Prova a reperire l'eventuale publisher configurato a livello di
			// servizio,
			// ossia nel file actions.xml/pages.xml
			publisherName = responseContainer.getPublisherName();
		}
		if (publisherName == null) {
			// Il publisher non  configurato a livello di servizio, ma in un
			// file di configurazione separato
			Object mappingsObject = configSingleton.getFilteredSourceBeanAttribute("PRESENTATION.MAPPING", "BUSINESS_NAME", businessName);
			if (mappingsObject == null) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: mapping non trovato");
				return null;
			} // if (mappingsObject == null)
			List mappings = null;
			if (mappingsObject instanceof SourceBean) {
				mappings = new ArrayList(1);
				mappings.add(mappingsObject);
			} // if (mappingsObject instanceof SourceBean)
			else
				mappings = (List) mappingsObject;
			// Take the first mapping
			for (int i = 0; i < mappings.size(); i++) {
				SourceBean mapping = (SourceBean) mappings.get(i);
				String localBusinessType = (String) mapping.getAttribute("BUSINESS_TYPE");
				if ((localBusinessType != null) && (localBusinessType.equalsIgnoreCase(businessType))) {
					publisherName = (String) mapping.getAttribute("PUBLISHER_NAME");
					break;
				} // if ((localBusinessName != null) &&
				// (localBusinessName.equalsIgnoreCase(businessName)))
			} // for (int i = 0; i < mappings.size(); i++)
		}
		return publisherName;
	}

	public static String getLogicalUserAgent(RequestContainer requestContainer) {
		String originalUserAgentString = null;
		if (requestContainer.getAttribute("user-agent") != null) {
			originalUserAgentString = (String) requestContainer.getAttribute("user-agent");
		}
		if (originalUserAgentString == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "Publisher::getLogicalUserAgent:  + originalUserAgentString nullo");
			return null;
		}
		SourceBean userAgents = (SourceBean) ConfigSingleton.getInstance().getAttribute("USER-AGENTS");
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "Publisher::getLogicalUserAgent: Cerco di mappare lo user agent [" + originalUserAgentString + "] su uno user agent logico");
		if (userAgents != null) {
			Object resultingUserAgent = userAgents.getFilteredSourceBeanAttribute("USER-AGENT", "identifierString", originalUserAgentString);
			if (resultingUserAgent != null) {
				if (resultingUserAgent instanceof List) {
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "Publisher::getLogicalUserAgent: Trovato piu di un user agent logico per user agent [" + originalUserAgentString + "]");
					return null;
				} else {
					return (String) ((SourceBean) resultingUserAgent).getAttribute("logicalName");
				}
			} else {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getLogicalUserAgent: NESSUN User agent logico trovato per  [" + originalUserAgentString + "]");
				return null;
			}
		} else {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getLogicalUserAgent: BUSTA USER-AGENTS non trovata nel Configuratore");
			return null;
		}
	}

	private static PublisherConfiguration getPublisher(final RequestContextIFace requestContext, String publisherName) {
		if (publisherName == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: mapping non trovato");
			return null;
		} // if (publisherName == null)
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Publisher::getPublisher: publisherName [" + publisherName + "]");
		ConfigSingleton configSingleton = ConfigSingleton.getInstance();
		SourceBean publisherConfig = (SourceBean) configSingleton.getFilteredSourceBeanAttribute("PUBLISHERS.PUBLISHER", "NAME", publisherName);
		if (publisherConfig == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: publisher non trovato");
			return null;
		} // if (publisherConfig == null)
		String channelType = requestContext.getRequestContainer().getChannelType();
		if (channelType == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: channelType nullo");
			return null;
		} // if (channelType == null)
		String logicalUserAgentOfRequest = getLogicalUserAgent(requestContext.getRequestContainer());
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Publisher::getPublisher: channelType [" + channelType + "]");
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Publisher::getPublisher: logicalUserAgent [" + logicalUserAgentOfRequest + "]");
		// 28-11-2005 Andrea Zoppello --> Handling user Agent
		Object renderingConfigObject = publisherConfig.getFilteredSourceBeanAttribute("RENDERING", "CHANNEL", channelType);
		SourceBean renderingConfig = null;
		if (renderingConfigObject instanceof List) {
			SourceBean tmpRenderConfig = null;
			String logicalUserAgentOfRenderConfig = null;
			if (logicalUserAgentOfRequest != null) {
				for (int i = 0; i < ((List) renderingConfigObject).size(); i++) {
					tmpRenderConfig = (SourceBean) ((List) renderingConfigObject).get(i);
					logicalUserAgentOfRenderConfig = (String) tmpRenderConfig.getAttribute("user-agent");
					if ((logicalUserAgentOfRenderConfig != null) && (logicalUserAgentOfRenderConfig.equalsIgnoreCase(logicalUserAgentOfRequest))) {
						renderingConfig = tmpRenderConfig;
						break;
					}// if ((logicalUserAgentOfRenderConfig != null) &&
					// (logicalUserAgentOfRenderConfig.equalsIgnoreCase(logicalUserAgentOfRequest))){
				}// for (int i=0; i < ((List)renderingConfigObject).size();
				// i++){
			}// if (logicalUserAgentOfRequest != null){
			// Listing all the rendering config section noone matches with the
			// user agent attribute
			if (renderingConfig == null) {
				renderingConfig = (SourceBean) ((List) renderingConfigObject).get(0);
			}
		} else {
			renderingConfig = (SourceBean) renderingConfigObject;
		}
		// 28-11-2005 Andrea Zoppello --> Fine Modifiche
		if (renderingConfig == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "Publisher::getPublisher: publisher non trovato per il canale [" + channelType + "]");
			return null;
		} // if (renderingConfig == null)
		PublisherConfiguration publisher = new PublisherConfiguration(publisherName, channelType, (String) renderingConfig.getAttribute("TYPE"), (String) renderingConfig.getAttribute("MODE"), renderingConfig.getAttributeAsList("RESOURCES.ITEM"),
				renderingConfig);
		return publisher;
	} // private static Publisher getPublisher(RequestContainer

	// requestContainer, ResponseContainer responseContainer, String
	// publisherName)
	public static String getLastPublisherName() {
		String publisherName = null;
		// Verifico se in request  presente l'attributo corrispondente al publisher corrente
		RequestContainer requestContainer = RequestContainer.getRequestContainer();
		SourceBean serviceRequest = requestContainer.getServiceRequest();
		if (serviceRequest != null) {
			// The publisher name in request, if present, has priority over the stored publisher in session
			publisherName = (String) serviceRequest.getAttribute(AF_PUBLISHER_NAME);
		}
		if (publisherName == null) {
			publisherName = (String) requestContainer.getSessionContainer().getPermanentContainer().getAttribute(Constants.LAST_PUBLISHER_NAME);
		}
		return publisherName;
	}

	public static void setLastPublisherName(final String publisherName) {
		RequestContainer requestContainer = RequestContainer.getRequestContainer();
		/* 2007-04-02 A. GRassi
		if (publisherName != null && requestContainer != null) {
			String pubAjax = (String) requestContainer.getServiceRequest().getAttribute("AJAX");
			if (pubAjax != null && pubAjax.equalsIgnoreCase(publisherName))
				return; // I Publisher AJAX non si salvano perch non  possibilre ripristinarli con la redirezione
		}
		*/
		requestContainer.getSessionContainer().getPermanentContainer().setAttribute(Constants.LAST_PUBLISHER_NAME, publisherName);
		// Rimuovo l'eventuale attributo dalla request
		SourceBean serviceRequest = requestContainer.getServiceRequest();
		if (serviceRequest != null) {
			try {
				serviceRequest.delAttribute(AF_PUBLISHER_NAME);
			} catch (SourceBeanException ex) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "Publisher::setLastPublisherName:", ex);
			}
		}
	}
} // public class Publisher
