/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.presentation.rendering;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.cache.CacheResponseHandler;
import it.eng.spago.dispatching.httpchannel.Router;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.presentation.PublisherConfiguration;
import it.eng.spago.presentation.rendering.AbstractRenderer;
import it.eng.spago.presentation.rendering.RenderIFace;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Manages the rendering phase based on servlet model: through a javax.servlet.RequestDispatcher
 * perform a forward or a redirect on another Web resource.
 * This implementation is specific for HTTP channel.
 */
public abstract class AbstractServletModelRenderer extends AbstractRenderer implements RenderIFace {

	private static final String FORWARD_PUBLISHING_MODE = "FORWARD";

	private static final String SENDREDIRECT_PUBLISHING_MODE = "SENDREDIRECT";

	/* (non-Javadoc)
	 * @see it.eng.spago.view.RenderIFace#prepareRender(it.eng.spago.dispatching.service.RequestContextIFace, it.eng.spago.presentation.PublisherConfiguration, java.lang.Object)
	 */
	public void prepareRender(RequestContextIFace requestContext, PublisherConfiguration publisher, Object additionalResources) throws Exception {
	}

	/* (non-Javadoc)
	 * @see it.eng.spago.view.RenderIFace#render(it.eng.spago.dispatching.service.RequestContextIFace, it.eng.spago.presentation.PublisherConfiguration, java.lang.Object)
	 */
	public void render(RequestContextIFace requestContext, PublisherConfiguration publisher,
			Object additionalResources) throws Exception {
		RequestContainer requestContainer = requestContext.getRequestContainer();
		ResponseContainer responseContainer = requestContext.getResponseContainer();
		HttpServletRequest request = (HttpServletRequest) requestContainer.getInternalRequest();
		HttpServletResponse response = (HttpServletResponse) requestContainer.getInternalResponse();
		HttpSession session = request.getSession();

		// Response caching management
		CacheResponseHandler.service(requestContainer, responseContainer, publisher.getName());

		// Retrieve forward or redirect mode
		String publisherMode = publisher.getMode();
		if ((publisherMode == null) || (publisherMode.length() == 0)) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
					"AdapterHTTP::service: publisherMode nullo, default [FORWARD]");
			publisherMode = FORWARD_PUBLISHING_MODE;
		} // if (publisherMode == null)
		
		// Save requestContainer and responseContainer in HTTP session
		session.setAttribute(Constants.REQUEST_CONTAINER, requestContainer);
		session.setAttribute(Constants.RESPONSE_CONTAINER, responseContainer);

		// Retrieve the name of the resource to forward/redirect
		List resources = publisher.getResources();
		
		// AG 04/Apr/2008 AJAX parameter specify different resource (es: differents JSP with the same Publisher)
		String prog = (String) requestContainer.getServiceRequest().getAttribute("AJAX", "0");
		String resource = null ;
		for (Iterator iter = resources.iterator(); iter.hasNext() && resource == null;) {
			SourceBean element = (SourceBean) iter.next();
			if (element.getAttribute("prog").equals(prog)) {
				resource = (String) element.getAttribute("resource");
				
				if (element.getAttribute("INCLUDE")!=null) {
					requestContainer.setAttribute(Constants.RESOURCES, element.getAttribute("INCLUDE"));
				}
				
			}
		}
		Router router = new Router(resource, publisherMode
				.equalsIgnoreCase(FORWARD_PUBLISHING_MODE));
		router.route(((ServletConfig) requestContainer.getAdapterConfig()).getServletContext(),
				request, response);
	}

}
