/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.presentation.rendering;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.dispatching.httpchannel.AdapterHTTP;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.presentation.PresentationRendering;
import it.eng.spago.presentation.PublisherConfiguration;
import it.eng.spago.presentation.rendering.AbstractRenderer;
import it.eng.spago.presentation.rendering.RenderIFace;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public abstract class AbstractTransformationRenderer extends AbstractRenderer implements RenderIFace {

	/* (non-Javadoc)
	 * @see it.eng.spago.view.RenderIFace#prepareRender(it.eng.spago.dispatching.service.RequestContextIFace, it.eng.spago.presentation.PublisherConfiguration, java.lang.Object)
	 */
	public void prepareRender(RequestContextIFace requestContext, PublisherConfiguration publisher, Object additionalResources) throws Exception {
	}

	/* (non-Javadoc)
	 * @see it.eng.spago.view.RenderIFace#render(it.eng.spago.dispatching.service.RequestContextIFace, it.eng.spago.presentation.PublisherConfiguration, java.lang.Object)
	 */
	public void render(RequestContextIFace requestContext, PublisherConfiguration publisher, Object additionalResources)
			throws Exception {
		RequestContainer requestContainer = requestContext.getRequestContainer();
		HttpServletResponse response = (HttpServletResponse)requestContainer.getInternalResponse();

        boolean isXMLRequest = false;

        // Check if the request is an AJAX request
        String requestedWith = (String)requestContainer.getAttribute(AdapterHTTP.HTTP_REQUESTED_WITH);
        if ((requestedWith != null) && (requestedWith.equalsIgnoreCase(AdapterHTTP.XMLHTTPREQUEST))) {
        	// This is probably an AJAX request
            isXMLRequest = true;
            requestContainer.setAttribute(Constants.HTTP_IS_XML_REQUEST, "TRUE");
        }

        if (requestContainer.getChannelType().equalsIgnoreCase(Constants.WAP_CHANNEL))
            response.setContentType(AdapterHTTP.WAP_CONTENT_TYPE);
        else if (isXMLRequest)
        	// For AJAX requests the response should be XML
            response.setContentType(AdapterHTTP.XML_CONTENT_TYPE);
        else
        	response.setContentType(AdapterHTTP.HTTP_CONTENT_TYPE);
        
        String publisherType = publisher.getType();
        String publisherName = (publisher != null) ? publisher.getName() : null;
        Monitor renderingMonitor = null;
        if (publisher == null) {
            renderingMonitor = MonitorFactory.start("view."
                    + requestContainer.getChannelType().toLowerCase() + "."
                    + publisherType.toLowerCase());
        }
        else {
            renderingMonitor = MonitorFactory
                    .start("view." + requestContainer.getChannelType().toLowerCase() + "."
                            + publisherType.toLowerCase() + "."
                            + publisherName.toLowerCase());
        }
        try {
    		List resources = publisher.getResources();
            response.getWriter().print(
                    PresentationRendering.render(requestContext, resources));
        } // try
        catch (Exception ex) {
            throw ex;
        } // catch (Exception ex)
        finally {
            renderingMonitor.stop();
        } // finally
        response.getWriter().flush();
        return;
    
	}

}
