/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.presentation.rendering;

import it.eng.spago.base.Constants;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.presentation.PublisherConfiguration;
import it.eng.spago.services.ServiceCatalog;
import it.eng.spago.services.fop.FOPService;
import it.eng.spago.tracing.TracerSingleton;

import java.io.ByteArrayInputStream;

import javax.servlet.http.HttpServletResponse;

/**
 * Renderer implementation that use the {@link FOPService} to implement
 * a publisher for FOP.
 * 
 * <pre>
 * 	&lt;PUBLISHER name="FOP_GLOSSARY_EXAMPLE_XSL">
 *	 &lt;RENDERING channel="HTTP" type="FOP" mode="FORWARD"
 *       <strong>xml_fop_path="XML_FOP.GLOSSARY"</strong>
 *       <strong>content_type="application/pdf"</strong> >
 *	  &lt;RESOURCES>
 *	   &lt;ITEM prog="0" resource="/WEB-INF/xsl/glossary.xsl" />
 *	  &lt;/RESOURCES>
 *	 &lt;/RENDERING>
 *  &lt;/PUBLISHER>
 * </pre> 
 * 
 * @author buso
 *
 */
public class HTTPFOPRenderer extends AbstractTransformationRenderer implements RenderIFace {
	/**
	 * Configuration to use in publisher to set the path used to obtain
	 * the <code>XML</code> contained the content of renderer transformation.
	 */
	public static final String RESPONSE_XMLFOP_PATH = "XML_FOP_PATH";
	/**
	 * Configuration to use to indicate che publisher output format. The value has to
	 * be a content type defined in {@link FOPService}.
	 */
	public static final String CONTENT_TYPE = "CONTENT_TYPE";
	
	/* (non-Javadoc)
	 * @see it.eng.spago.view.AbstractTransformationRenderer#render(it.eng.spago.dispatching.service.RequestContextIFace, it.eng.spago.presentation.PublisherConfiguration, java.lang.Object)
	 */
	public void render(RequestContextIFace requestContext, PublisherConfiguration publisher,
			Object additionalResources) throws Exception {
		
		ResponseContainer responseContainer = requestContext.getResponseContainer();
		String xmlFopPath = (String)publisher.getRenderingConfig()
			.getAttribute(RESPONSE_XMLFOP_PATH);
		String contentType = (String)publisher.getRenderingConfig()
			.getAttribute(CONTENT_TYPE);
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
			"Path to the xml in response: "+xmlFopPath);
    	SourceBean xmlFOP = (SourceBean)
    		responseContainer.getServiceResponse().getAttribute(xmlFopPath);
    	String xslName = (String) ((SourceBean) publisher.getResources().get(0))
			.getAttribute("resource");
    	FOPService fopS = (FOPService)ServiceCatalog.getInstance().getService(
    		requestContext, "FOP");
    	
    	HttpServletResponse response = (HttpServletResponse)requestContext
    		.getRequestContainer().getInternalResponse();
    	response.setContentType(contentType);
    	
    	fopS.process(
    		new ByteArrayInputStream(xmlFOP.toXML(true, true).getBytes()),
    		xslName, contentType, response.getOutputStream(),
    		requestContext);
	}

}
