/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.presentation.rendering;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.presentation.PublisherConfiguration;
import it.eng.spago.presentation.rendering.RenderIFace;
import it.eng.spago.tracing.TracerSingleton;

import java.util.HashMap;
import java.util.List;

public class RenderManager {
	private static RenderManager _instance = null;

	private HashMap _renderers = new HashMap();

	private RenderManager() {
		ConfigSingleton configure = ConfigSingleton.getInstance();
		List renderers = configure.getAttributeAsList("RENDERERS.RENDERER");
		for (int i = 0; i < renderers.size(); i++) {
			try {
				SourceBean rendererDefinition = (SourceBean) (renderers.get(i));
				String rendererClassName = (String) (rendererDefinition.getAttribute("CLASS"));
				String rendererType = (String) (rendererDefinition.getAttribute("TYPE"));
				String rendererConfigName = (String) (rendererDefinition.getAttribute("NAME"));
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
						"RenderManager::init: " + rendererClassName + " [" + rendererConfigName
								+ "]");
				SourceBean rendererConfig = (SourceBean) (rendererDefinition
						.getAttribute("CONFIG"));
				RenderIFace renderer = (RenderIFace) (Class.forName(rendererClassName)
						.newInstance());
				renderer.init(rendererType, rendererConfig);
				
				_renderers.put(rendererType, renderer);
			} // try
			catch (Exception ex) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
						"RenderManager::init:", ex);
			} // catch (Exception ex) try
		} // for (int i = 0; i < renderers.size(); i++)
	} // private RenderManager()

	public static RenderManager getInstance() {
		if (_instance == null) {
			synchronized (RenderManager.class) {
				if (_instance == null) {
					try {
						_instance = new RenderManager();
					} // try
					catch (Exception ex) {
						TracerSingleton.log("SPAGO", TracerSingleton.CRITICAL, "RenderManager::getInstance: errore creazione RenderManager", ex);
						//System.out
						//		.println("RenderManager::getInstance: errore creazione RenderManager");
						//ex.printStackTrace();
					} // catch(Exception ex) try
				} // if (_instance == null)
			} // synchronized(RenderManager.class)
		} // if (_instance == null)
		return _instance;
	} // public static RenderManager getInstance()

	/**
	 * Retrieve the renderer according to the publisher type and, potentially, the channel type
	 * @param publisherConfig Publisher Configuration
	 * @return The renderer used to produce the result
	 */
	public RenderIFace getRenderer(PublisherConfiguration publisherConfig) {
		String publisherType = publisherConfig.getType();
		
		RenderIFace renderer = (RenderIFace) _renderers.get(publisherType);

		if (renderer == null) {
			// Retrieve default renderer
			renderer = (RenderIFace)_renderers.get("UNDEFINED");
		}

		return renderer;
	} // public static CoordinatorIFace getRenderer(RequestContextIFace
	// requestContext)

} // public class RenderManager
