/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.publishers;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;

/**
 * @author scarciello
 *
 */
public abstract class AbstractServletPublisher extends HttpServlet {
	protected String _serviceName = null;
	protected SourceBean _config = null;
	protected SourceBean _content = null;

	/**
	 * Inizializzazione per recuperare il nome del servizio eseguito, la risposta e la relativa configurazione
	 * @param httpRequest
	 * @throws ServletException
	 */
	protected void init(HttpServletRequest httpRequest)  throws ServletException {
		ResponseContainer responseContainer = getResponseContainer(httpRequest) ;
		if (responseContainer.getBusinessType() != null) {
			try {
				String configPath = "ACTIONS.ACTION" ;
				if (responseContainer.getBusinessType().equalsIgnoreCase("ACTION")||
						responseContainer.getBusinessType().equalsIgnoreCase("ACTION_NAME")) {
					_serviceName = responseContainer.getBusinessName() ;
					_content = responseContainer.getServiceResponse() ;
				} else if (responseContainer.getBusinessType().equalsIgnoreCase("PAGE")) {
					_serviceName = (String)getRequestContainer(httpRequest).getSessionContainer().getPermanentContainer().getAttribute("LAST_MODULE_EXECUTED") ;
					configPath = "MODULES.MODULE" ;
					_content = (SourceBean)responseContainer.getServiceResponse().getAttribute(_serviceName) ;
				}
				_config = (SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute(configPath, "NAME", _serviceName);
				_config = (SourceBean)_config.getAttribute("CONFIG") ;
			}catch (Exception ex) {
				// Non viene bloccata intenzionalmente l'esecuzione del programma
				TracerSingleton.log(Constants.NOME_MODULO,
						TracerSingleton.WARNING, this.getClass().getName()
						+ "::init:: Impossibile recuperare informazioni sul servizio richiesto (ACTION, PAGE , ... ??)");
				throw new ServletException("Impossibile recuperare informazioni sul servizio richiesto", ex) ;
			}
		}
	}
	
	/**
	 * Estrae l'oggetto ResponseContainer tramite la request HTTP
	 * @param request
	 * @return
	 */
	protected ResponseContainer getResponseContainer(HttpServletRequest request) {
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(request);
        if (responseContainer == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractHttpJspPage::getResponseContainer: responseContainer nullo");
        return responseContainer;
    } // public ResponseContainer getResponseContainer(HttpServletRequest request)


	/**
	 * Estrae l'oggetto RequestContainer tramite la request HTTP 
	 * @param HttpServletRequest
	 * @return
	 */
	protected RequestContainer getRequestContainer(HttpServletRequest request) {
        RequestContainer requestContainer =
            RequestContainerAccess.getRequestContainer(request);
        if (requestContainer == null)
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "AbstractHttpJspPage::getRequestContainer: requestContainer nullo");
        return requestContainer;
    } // public RequestContainer getRequestContainer(HttpServletRequest request)

	/**
	 * Recupera il SourceBean della Service Request
	 * @param HttpServletRequest
	 * @return
	 */
	protected SourceBean getServiceRequest(HttpServletRequest request) {
		return getRequestContainer(request).getServiceRequest() ;
	}
	
	/**
	 * Recupera il SourceBean della Service Response
	 * @param HttpServletRequest
	 * @return
	 */
	protected SourceBean getServiceResponse(HttpServletRequest request) {
		return getResponseContainer(request).getServiceResponse() ;
	}
}
