/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.services.http;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.services.AbstractService;
import it.eng.spago.services.ServiceInterface;
import it.eng.spago.tracing.TracerSingleton;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.validator.GenericValidator;
 
public class HttpService
	extends AbstractService
	implements HttpServiceInterface {

	private static final long serialVersionUID = -6931561176794931666L;

	private String _host = "localHost";

	private int _port = 80;
	private String _proxyHost = null;
	private int _proxyPort;
	private String _path = null;
	private boolean _cache = false;
	private long _expiration = 10000;
	private int _timeout=-1;
	private int _connectionTimeout=-1;
	private Hashtable _hash = new Hashtable();
	private String _protocol = "http";

	/**
	 *  Constructor for the HttpService object
	 */
	public HttpService() {
	}//public HttpService()

	/**
	 * Configure http connection parameters. If some value are not found in
	 * <code>config</code> default values will be used.
	 * 
	 * @param config. Configuration <code>SourceBean</code>; example:
	 * 		<pre>
	 * 		&lt;CONFIG 
	 *			PROTOCOL="http"
	 *			HOST="localhost"
	 *			PORT="8080"
	 *			PROXY_HOST=""
	 *			PROXY_PORT=""
	 *			PATH="/TestPath"
	 *			CACHE="true"
	 *			EXPIRATION="10000"
	 *			TIMEOUT=""
	 *			CONNECTION_TIMEOUT=""
	 *			/&gt;
	 * 		</pre>
	 * 
	 * @see ServiceInterface#init(SourceBean)
	 */
	public void init(SourceBean config) throws Exception {
		if (config == null) { return; }
		if (!GenericValidator.isBlankOrNull((String)config.getAttribute("PROTOCOL"))) {
			this._protocol = (String)config.getAttribute("PROTOCOL");
			if (_protocol.equals("https")) {
				this._port = 443;
			}
		}
		if (!GenericValidator.isBlankOrNull((String)config.getAttribute("HOST"))) {
			this._host = (String)config.getAttribute("HOST");

		}
		String port = (String)config.getAttribute("PORT"); 
		if (!GenericValidator.isBlankOrNull(port)) {
			this._port = Integer.parseInt(port);
		}
		if (!GenericValidator.isBlankOrNull((String)config.getAttribute("PROXY_HOST"))) {
			this._proxyHost = (String)config.getAttribute("PROXY_HOST");
		}
		String proxyPort = (String)config.getAttribute("PROXY_PORT"); 
		if (!GenericValidator.isBlankOrNull(proxyPort)) {
			this._proxyPort = Integer.parseInt(proxyPort);
		}
		if (!GenericValidator.isBlankOrNull((String)config.getAttribute("PATH"))) {
			this._path = (String)config.getAttribute("PATH");
		}
		String cache = (String)config.getAttribute("CACHE"); 
		if (!GenericValidator.isBlankOrNull(cache)) {
			this._cache = Boolean.valueOf(cache).booleanValue();
		}
		String expiration = (String)config.getAttribute("EXPIRATION"); 
		if (!GenericValidator.isBlankOrNull(expiration)) {
			this._expiration = Long.parseLong(expiration);
		}
		String timeout = (String)config.getAttribute("TIMEOUT"); 
		if (!GenericValidator.isBlankOrNull(timeout)) {
			this._timeout = Integer.parseInt(timeout);
		}
		String connectionTimeout = (String)config.getAttribute("CONNECTION_TIMEOUT"); 
		if (!GenericValidator.isBlankOrNull(connectionTimeout)) {
			this._connectionTimeout = Integer.parseInt(connectionTimeout);
		}


//		if (_config.getProperty("log.level") != null) {
//			System.setProperty(
//				"org.apache.commons.logging.simplelog.defaultlog",
//				_config.getProperty("log.level"));
//		}
	}//public void init(SourceBean config) throws Exception

	/**
	 *  Description of the Method
	 *
	 *@param  surl               Description of the Parameter
	 *@param  prop               Description of the Parameter
	 *@return                    Description of the Return Value
	 *@exception  IOException    Description of the Exception
	 *@exception  HttpException  Description of the Exception
	 */
	public String doGet(String surl, Properties prop)
		throws Exception {

		return send(surl, prop, "GET");
	}

	/**
	 *  Description of the Method
	 *
	 *@param  surl               Description of the Parameter
	 *@return                    Description of the Return Value
	 *@exception  IOException    Description of the Exception
	 *@exception  HttpException  Description of the Exception
	 */
	public String doGet(String surl) throws Exception {
		return send(surl, null, "GET");
	}

	/**
	 *  Description of the Method
	 *
	 *@param  url                Description of the Parameter
	 *@return                    Description of the Return Value
	 *@exception  IOException    Description of the Exception
	 *@exception  HttpException  Description of the Exception
	 */
	public String doGet(URL url) throws Exception {
		int _port = url.getPort();

		if (_port == -1) {
			_port = 80;
		}
		String query = url.getPath();

		if (url.getQuery() != null) {
			query = query + "?" + url.getQuery();
		}

		return send(url.getHost(), _port, query, null, "GET");
	}

	/**
	 *  Description of the Method
	 *
	 *@param  surl               Description of the Parameter
	 *@param  prop               Description of the Parameter
	 *@return                    Description of the Return Value
	 *@exception  IOException    Description of the Exception
	 *@exception  HttpException  Description of the Exception
	 */
	public String doPost(String surl, Properties prop)
		throws Exception {
		return send(surl, prop, "POST");
	}

	/**
	 *  Description of the Method
	 *
	 *@param  surl               Description of the Parameter
	 *@return                    Description of the Return Value
	 *@exception  IOException    Description of the Exception
	 *@exception  HttpException  Description of the Exception
	 */
	public String doPost(String surl) throws Exception {
		return send(surl, null, "POST");
	}

	/**
	 *  Gets the client attribute of the HttpService object
	 *
	 *@param  _host  Description of the Parameter
	 *@param  _port  Description of the Parameter
	 *@return        The client value
	 */
	private HttpClient getClient(String _host, int _port) {

		HttpClient s = new HttpClient();
		HostConfiguration conf = new HostConfiguration();
		conf.setHost(_host, _port, _protocol);
		if (_proxyHost != null) {
			conf.setProxy(_proxyHost, _proxyPort);
		}
		s.setHostConfiguration(conf);
		if (_timeout!=-1){
			s.getHttpConnectionManager().getParams().setSoTimeout(_timeout);
		}
		if (_connectionTimeout!=-1){
			s.getHttpConnectionManager().getParams().setConnectionTimeout(_connectionTimeout);
		}
		return s;
	}

	public HttpClient getClient() {
		return getClient(_host, _port);
	}
	public String getPath() {
		return _path;
	}
	/**
	 *  Description of the Method
	 *
	 *@param  surl               Description of the Parameter
	 *@param  prop               Description of the Parameter
	 *@param  method             Description of the Parameter
	 *@return                    Description of the Return Value
	 *@exception  IOException    Description of the Exception
	 *@exception  HttpException  Description of the Exception
	 */
	private String send(String surl, Properties prop, String method)
		throws Exception {
		return send(_host, _port, surl, prop, method);
	}

	
	/**
	 * Description of the Method
	 *
	 * @param  surl. The url argument of the connection. If the service is
	 * 		configured with a path, the path is prepended to <code>surl</code>.
	 * @param  prop. List of properties to be used in the url connection.
	 *		Properties that have the key with prefix "header." are used as http
	 *		headers, the other one are used to compose the query string.
	 * @param  method             Description of the Parameter
	 * @param  host              Description of the Parameter
	 * @param  port              Description of the Parameter
	 * @return                    Description of the Return Value
	 *
	 * @exception  Exception.
	 */
	private String send(
		String host,
		int port,
		String surl,
		Properties prop,
		String method)
		throws Exception {
		long time = System.currentTimeMillis();

		if (prop != null) {
			int pos = surl.indexOf("?");
			if (pos < 0) {
				surl = surl + "?";
			}

			Enumeration keys = prop.keys();
			while (keys.hasMoreElements()) {

				String key = (String) keys.nextElement();
				if (!key.startsWith("header.")) {

					if (!surl.endsWith("&") && !surl.endsWith("?")) {
						surl = surl + "&";
					}
					surl = surl + key + "=" + prop.getProperty(key) + "&";
				}
			}
			if (surl.endsWith("&")) {
				surl = surl.substring(0, surl.length() - 1);
			}
		}
		if (_path != null) {
			surl = _path + surl;
		}
		String out = null;
		if (_cache) {
			HttpBean bean = (HttpBean) _hash.get(surl);
			if (bean != null) {
				if (!bean.isExpired()) {
					out = bean.getResponse();
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
						"HttpService::send: HTTP cached request=" + surl);
				} else {
					_hash.remove(surl);
				}
			}
		}
		if (out == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"HttpService::send: HTTP _host="
					+ host
					+ " _port="
					+ port
					+ " request="
					+ surl);

			HttpClient s = getClient(host, port);
			HttpMethod m = null;
			if (method.equalsIgnoreCase("GET")) {
				m = new GetMethod(surl);
			}
			if (method.equalsIgnoreCase("POST")) {
				m = new PostMethod(surl);
			}
			if (prop != null) {

				Enumeration keys = prop.keys();
				while (keys.hasMoreElements()) {

					String key = (String) keys.nextElement();
					if (key.startsWith("header.")) {
						TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.DEBUG,"HttpService:Send:" + key.substring("header.".length()) + "="	+ prop.getProperty(key));
						//System.out.println(
						//	key.substring("header.".length())
						//		+ "="
						//		+ prop.getProperty(key));

						m.addRequestHeader(
							key.substring("header.".length()),
							prop.getProperty(key));
					}
				}
			}
			int n = s.executeMethod(m);
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"HttpService::send: HTTP RC="
					+ n
					+ " _host="
					+ host
					+ " _port="
					+ port
					+ " request="
					+ surl
					+ " Time="
					+ (System.currentTimeMillis() - time)
					+ "ms.");

			if (n == 200) {
				out = m.getResponseBodyAsString();
				if (_cache) {
					HttpBean bean = new HttpBean(_expiration);
					bean.setResponse(out);
					_hash.put(surl, bean);
				}
				m.releaseConnection();
			} else {				
				m.releaseConnection();
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
					"HTTP error, code: "+n);
				throw new Exception("HTTP error, code: "+n);
			}

		}
		return out;
	}
}//public class HttpService
//extends AbstractService
//implements HttpServiceInterface
