/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.servlet;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.action.ActionException;
import it.eng.spago.dispatching.action.ActionFactory;
import it.eng.spago.dispatching.action.ActionIFace;
import it.eng.spago.dispatching.httpchannel.AdapterHTTP;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.dispatching.service.ServiceIFace;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;

public class AjaxServlet extends AdapterHTTP {

	@Override
	public void service(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException {
		// init(httpRequest);

		httpResponse.setHeader("Expires", "0");
		httpResponse.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
		httpResponse.setHeader("Pragma", "public");
		httpResponse.setContentType("text/html");

		RequestContainer requestContainer = new RequestContainer();
		RequestContainer.setRequestContainer(requestContainer);

		ResponseContainer responseContainer = new ResponseContainer();
		ResponseContainer.setResponseContainer(responseContainer);

		RequestContextIFace requestContext = new DefaultRequestContext(requestContainer, responseContainer);

		PrintWriter out;
		try {
			SourceBean serviceRequest = new SourceBean(Constants.SERVICE_REQUEST);
			requestContainer.setServiceRequest(serviceRequest);
			SourceBean serviceResponse = new SourceBean(Constants.SERVICE_RESPONSE);
			responseContainer.setServiceResponse(serviceResponse);
			
			setHttpRequestData(httpRequest, requestContainer);
			handleServiceName(serviceRequest, requestContainer);
			
			HttpSession session = httpRequest.getSession(true);
			checkSession(session, requestContext);
			
			boolean isMultipart = ServletFileUpload.isMultipartContent(new ServletRequestContext(httpRequest));
			if (isMultipart) {
				handleMultipartForm(httpRequest, requestContext);
			} else {
				handleSimpleForm(httpRequest, requestContext);
			}

			ActionIFace action = ActionFactory.getAction(requestContainer);
			((RequestContextIFace)action).setRequestContext(requestContext);
			action.service(serviceRequest, serviceResponse);
			String id = (String) serviceRequest.getAttribute("id");
			out = httpResponse.getWriter();

			// TODO handle different type of elements. For now just 'select'
			out.print(id + "|");
			// for (SourceBean bean : rows) {
			out.print(serviceResponse.toXML(false, false));

			out.flush();
		} catch (SourceBeanException e) {
			// TODO Blocco catch generato automaticamente
			e.printStackTrace();
		} catch (ActionException e) {
			// TODO Blocco catch generato automaticamente
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Blocco catch generato automaticamente
			e.printStackTrace();
		}
		// }

	}
}
