/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.statistic;

import it.eng.spago.base.Constants;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import com.jamonapi.MonKey;
import com.jamonapi.Monitor;
import com.jamonapi.Range;

/**
 * Utility class that serialize a <code>com.jamonapi.Monitor</code> instance into a
 * <code>String</code> and can deserialize the <code>String</code> in a java.util.Map.
 * 
 * @author buso
 *
 */
public class MonitorStringSerializer {
	/**
	 * Delimeter used to bound pair (key, value).
	 * TODO you use also the "=" delimeter to delimit key and value. Can this char be
	 * present in jamon label?
	 */
	private static final String STR_TKZ_DELIMETER = "";
	
	public static final String LABEL = "label";
	public static final String HITS = "hits";
	public static final String AVG = "avg";
	public static final String TOTAL = "total";
	public static final String UNITS = "units";
	public static final String STD_DEV = "stdDev";
	public static final String LASTVALUE = "lastValue";
	public static final String MIN = "min";
	public static final String MAX = "max";
	public static final String ACTIVE = "active";
	public static final String AVG_ACTIVE = "avgActive";
	public static final String MAX_ACTIVE = "maxActive";
	public static final String FIRST_ACCESS = "firstAccess";
	public static final String LAST_ACCESS = "lastAccess";
	public static final String ENABLED = "enabled";
	public static final String PRIMARY = "primary";

	/**
	 * Serialize the passed <code>Monitor</code> in a String.
	 * 
	 * @param mon
	 * @return a <code>Monitor</code> String rappresentation.
	 */
	public static String serialize(Monitor mon) {
		MonKey key = mon.getMonKey();
		StringBuffer buff = new StringBuffer()
			.append(LABEL+"=").append(key.getValue(MonKey.LABEL_HEADER)).append(STR_TKZ_DELIMETER)
			.append(UNITS+"=").append(key.getValue(MonKey.UNITS_HEADER)).append(STR_TKZ_DELIMETER)
			.append(HITS+"=").append(mon.getHits()).append(STR_TKZ_DELIMETER)
			.append(AVG+"=").append(mon.getAvg()).append(STR_TKZ_DELIMETER)
			.append(TOTAL+"=").append(mon.getTotal()).append(STR_TKZ_DELIMETER)
			
			.append(STD_DEV+"=").append(mon.getStdDev()).append(STR_TKZ_DELIMETER)
			.append(LASTVALUE+"=").append(mon.getLastValue()).append(STR_TKZ_DELIMETER)
			.append(MIN+"=").append(mon.getMin()).append(STR_TKZ_DELIMETER)
			.append(MAX+"=").append(mon.getMax()).append(STR_TKZ_DELIMETER)
			.append(ACTIVE+"=").append(mon.getActive()).append(STR_TKZ_DELIMETER)
			.append(AVG_ACTIVE+"=").append(mon.getAvgActive()).append(STR_TKZ_DELIMETER)
			.append(MAX_ACTIVE+"=").append(mon.getMaxActive()).append(STR_TKZ_DELIMETER)
			.append(FIRST_ACCESS+"=").append(mon.getFirstAccess().getTime()).append(STR_TKZ_DELIMETER)
			.append(LAST_ACCESS+"=").append(mon.getLastAccess().getTime()).append(STR_TKZ_DELIMETER)
			;
		
		return buff.toString();
	}
	
	/**
	 * Retrieve Monitor information serialized to a String into a Map.
	 * 
	 * @param strMon
	 * @return
	 */
	private static Map deserializeIntoMap(String strMon) {
		Map map = new HashMap();
		StringTokenizer strTkz = new StringTokenizer(strMon, STR_TKZ_DELIMETER);
		try {
			while (strTkz.hasMoreTokens()) {
				StringTokenizer pairTkz = new StringTokenizer(
					strTkz.nextToken(), "=");
				while (pairTkz.hasMoreTokens()) {
					String key = pairTkz.nextToken();
					String value = pairTkz.nextToken();
					map.put(key, value);
				}
			}
		} catch (RuntimeException ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"MonitorStringSerializer::deserializeIntoMap: Exception: strMon: "+strMon);
			throw ex;
		}
		
		return map;
	}
	
	/**
	 * Return a dummy Monitor instance parsing a string obtained with
	 * {@linkplain MonitorStringSerializer#serialize(Monitor)}
	 * 
	 * @param strMon
	 * @return
	 */
	public static Monitor deserialize(String strMon) {
		Map monInfo = deserializeIntoMap(strMon);
		return new MonitorMapAdapter(monInfo);
	} 
	
}

class MonitorMapAdapter extends Monitor {
	private Map monitorData;
	
	public MonitorMapAdapter(Map monData) {
		monitorData = monData;
	}

	public String getLabel() {
		return (String)monitorData.get(MonitorStringSerializer.LABEL);
	}

	public MonKey getMonKey() {
		return null;
	}

	public String getUnits() {
		return (String)monitorData.get(MonitorStringSerializer.UNITS);
	}

	public Monitor add(double value) {
		return null;
	}

	public void disable() {
	}

	public void enable() {
	}

	public double getActive() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.ACTIVE));
	}

	public double getAvgActive() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.AVG_ACTIVE));
	}

	public double getMaxActive() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.MAX_ACTIVE));
	}

	public Range getRange() {
		return null;
	}

	public boolean isEnabled() {
		return false;
	}

	public boolean isPrimary() {
		return false;
	}

	public void setActive(double value) {
	}

	public void setMaxActive(double value) {
	}

	public void setPrimary(boolean isPrimary) {
	}

	public void setTotalActive(double value) {
	}

	public Monitor start() {
		return null;
	}

	public Monitor stop() {
		return null;
	}

	public synchronized void addValue(double value) {
	}

	public synchronized double getAvg() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.AVG));
	}

	public synchronized Date getFirstAccess() {
		try {
			return new Date(Long.parseLong((String)monitorData
				.get(MonitorStringSerializer.FIRST_ACCESS)));
		} catch (Exception e) {
			// problem if the map does not contain the value and with
			// format errors
			return null;
		}
	}

	public synchronized double getHits() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.HITS));
	}

	public synchronized Date getLastAccess() {
		try {
			return new Date(Long.parseLong((String)monitorData
				.get(MonitorStringSerializer.LAST_ACCESS)));
		} catch (Exception e) {
			// problem if the map does not contain the value and with
			// format errors
			return null;
		}
	}

	public synchronized double getLastValue() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.LASTVALUE));
	}

	public synchronized double getMax() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.MAX));
	}

	public synchronized double getMin() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.MIN));
	}

	public synchronized double getStdDev() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.STD_DEV));
	}

	public synchronized double getTotal() {
		return Double.parseDouble((String)monitorData
			.get(MonitorStringSerializer.TOTAL));
	}

	public synchronized void reset() {
	}

	public synchronized void setAccessStats(long now) {
	}

	public synchronized void setFirstAccess(Date date) {
	}

	public synchronized void setHits(double value) {
	}

	public synchronized void setLastAccess(Date date) {
	}

	public synchronized void setLastValue(double value) {
	}

	public synchronized void setMax(double value) {
	}

	public synchronized void setMin(double value) {
	}

	public synchronized void setTotal(double value) {
	}
}

