/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/

package it.eng.spago.tags;

import it.eng.spago.base.BaseContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.decorators.AbstractDecorator;
import it.eng.spago.decorators.DecoratorFactory;
import it.eng.spago.dispatching.TokenHandler;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.detail.impl.DelegatedDetailService;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.JavaScript;
import it.eng.spago.util.QueryExecutor;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * @author grassi
 * 
 */
public abstract class AbstractStandardTag extends TagSupport {
	private static final long serialVersionUID = 1L;

	protected RequestContainer _requestContainer = null;

	protected SourceBean _serviceRequest = null;

	protected ResponseContainer _responseContainer = null;

	protected SourceBean _serviceResponse = null;

	protected ConfigSingleton configure = null;

	protected String _pageName = null;

	protected String _actionName = null;

	protected String _moduleName = null;

	protected SourceBean _content = null;

	protected SourceBean _layout = null;

	protected String _providerURL = null;

	protected String _serviceName = null;

	protected String _scope = "CONTENT";

	protected String _form = "f1";

	protected final static String MSG_CAPTION_SELECT_LABEL = "11011";

	protected final static String MSG_CAPTION_DELETE_LABEL = "11012";

	protected final static String MSG_CAPTION_GENERIC_LABEL = "11013";

	protected final static String MSG_ASK_SELECT = "11014";

	protected final static String MSG_ASK_DELETE = "11010";

	protected final static String MSG_ASK_GENERIC = "11015";

	protected final static String MSG_CAPTION_CHECKBOX_SEL = "11016";

	protected final static String DETAIL_UPDATE = "UPDATE";

	protected final static String DETAIL_INSERT = "INSERT";

	protected final static String DETAIL_SELECT = "SELECT";

	protected String _resourcePath = null;

	/**
	 * 
	 */
	public AbstractStandardTag() {
		super();
	}

	/**
	 * Metodo a disposizione per le classi di implementazione; oltre a contenere
	 * eventuali inizializzazioni specifiche, permette di "bloccare"
	 * l'esecuzione del Tag
	 * 
	 * @return 0 per confermare corretta inizializzazione e/o condizioni di
	 *         esecuzione
	 */
	protected int initTag() throws JspException {
		return 0;
	}

	public int doStartTag() throws JspException {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, this.getClass().getName() + "::doStartTag:: invocato");

		generalTagInit();
		if (initTag() != 0) {
			return SKIP_BODY;
		}

		try {
			StringBuffer htmlStream = getHtmlStream();
			if (htmlStream != null && htmlStream.length() > 0)
				pageContext.getOut().print(htmlStream);
		} // try

		catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, this.getClass().getName() + "::doStartTag:: Impossibile inviare lo stream. Errore: " + ex.getMessage());
			_responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, "Errore inaspettato nel rendering:" + ex.getMessage(),ex));

			// throw new JspException(ex);
		} // catch (Exception ex)
		return super.doStartTag();
	}

	/**
	 * Inizializzazione comune per tutti i tag Standard
	 * 
	 */
	protected void generalTagInit() {
		// Inizializza le variabili
		HttpServletRequest httpRequest = (HttpServletRequest) pageContext.getRequest();
		configure = ConfigSingleton.getInstance();

		_requestContainer = RequestContainerAccess.getRequestContainer(httpRequest);
		if (_requestContainer != null) {
			_serviceRequest = _requestContainer.getServiceRequest();
		}
		_responseContainer = ResponseContainerAccess.getResponseContainer(httpRequest);
		if (_responseContainer != null) {
			_serviceResponse = _responseContainer.getServiceResponse();
			smartInit();
		}
	}

	/**
	 * Ritorna l'url del server (es: "http://localhost:9080/Sied")
	 * 
	 * @return
	 */
	protected String getServerUrl() {
		BaseContainer reqc = _requestContainer;
		// WorkArround Gabri
		// La prima invocazione non contiene questi attributi che vanno reperiti
		// sul parent
		if (reqc.getAttribute("HTTP_REQUEST_SCHEME") == null)
			reqc = reqc.getParent();

		return reqc.getAttribute("HTTP_REQUEST_SCHEME") + "://" + reqc.getAttribute("HTTP_REQUEST_SERVER_NAME") + ":" + reqc.getAttribute("HTTP_REQUEST_SERVER_PORT") + reqc.getAttribute("HTTP_REQUEST_CONTEXT_PATH");
	}

	/**
	 * Questa funzione ricerca automaticamete il servizio in esecuzione ed
	 * inizializza le variabili di istanza
	 * 
	 */
	protected void smartInit() {
		String actionName = null, moduleName = null;
		if (_actionName == null && _moduleName == null) {
			if (_responseContainer.getBusinessType() != null) {
				try {
					if (_responseContainer.getBusinessType().equalsIgnoreCase("ACTION") || _responseContainer.getBusinessType().equalsIgnoreCase("ACTION_NAME")) {
						actionName = _responseContainer.getBusinessName();
						_actionName = actionName; // Per compatibilit
					} else if (_responseContainer.getBusinessType().equalsIgnoreCase("PAGE")) {
						_pageName = _responseContainer.getBusinessName();
						/*
						 * if (_serviceResponse != null &&
						 * _serviceResponse.getContainedAttributes().size() > 0) {
						 * int lastModuleExecuted =
						 * _serviceResponse.getContainedAttributes().size()-1 ;
						 * moduleName =
						 * ((it.eng.spago.base.SourceBeanAttribute)(_serviceResponse.getContainedAttributes().get(lastModuleExecuted))).getKey(); }
						 * else { moduleName = (String)
						 * _serviceRequest.getAttribute("MODULE"); }
						 */
						for (SourceBeanAttribute module : (List<SourceBeanAttribute>) _serviceResponse.getContainedSourceBeanAttributes()) {
							SourceBean bn = (SourceBean) module.getValue();
							if (bn.getAttribute("af_form") != null && bn.getAttribute("af_form").equals(_form)) {
								moduleName = _moduleName = bn.getName();
							}
						}

						if (moduleName == null) {
							moduleName = _moduleName = (String) _serviceRequest.getAttribute("CURRENT_MODULE");
						}
						// targetResponse.getServiceResponse().getFilteredSourceBeanAttribute("MODULE",
						// paramName, paramValue)
						// moduleName = _moduleName =
						// (String)_requestContainer.getSessionContainer().getPermanentContainer().getAttribute("LAST_MODULE_EXECUTED")
						// ;
					}
				} catch (Exception ex) {
					// Non viene bloccata intenzionalmente l'esecuzione del
					// programma
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, this.getClass().getName() + "::smartInit:: Impossibile recuperare informazioni sul servizio richiesto (ACTION, PAGE , ... ??)");
				}
			} else {
				// Faccio un'ultimo tentativo di recuperare le informazioni
				// dalla request
				actionName = (String) _serviceRequest.getAttribute("ACTION_NAME");
				moduleName = (String) _serviceRequest.getAttribute("CURRENT_MODULE");
			}
		} else {
			actionName = _actionName;
			moduleName = _moduleName;
		}

		if (actionName != null) {
			if (_actionName.equalsIgnoreCase("DYNAMIC_ACTION")) { // Undefined
				// action
				// used by
				// lookup
				// (see
				// Lookup.jsp)
				_actionName = _responseContainer.getBusinessName();
			}

			_serviceName = actionName;
			_content = _serviceResponse;
			SourceBean actionBean = (SourceBean) configure.getFilteredSourceBeanAttribute("ACTIONS.ACTION", "NAME", actionName);
			_layout = (SourceBean) actionBean.getAttribute("CONFIG");
			_providerURL = "ACTION_NAME=" + actionName + "&";
		} else if (moduleName != null) {
			_pageName = _responseContainer.getBusinessName();
			_serviceName = moduleName;
			_content = (SourceBean) _serviceResponse.getAttribute(moduleName);
			SourceBean moduleBean = (SourceBean) configure.getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", moduleName);
			_layout = (SourceBean) moduleBean.getAttribute("CONFIG");
			_providerURL = "PAGE=" + _pageName + "&MODULE=" + moduleName + "&";
		}
	}

	/*
	 * protected String findModuleName() { if (this instanceof DetailTag) { //
	 * Search for Detail Module } }
	 */

	/**
	 * Metodo a cura della classe di implementazione: deve ritornare la pagina
	 * HTML
	 * 
	 */
	protected abstract StringBuffer getHtmlStream() throws Exception;

	/**
	 * Verifica se la String in input  istanziata e valorizzata
	 * 
	 * @param in
	 * @return true/false
	 */
	protected boolean testString(Object in) {
		return (in != null && ((String) in).trim().length() > 0);
	}

	/**
	 * Rilascia le risorse allocate per velocizzare il garbage collector
	 */
	public int doEndTag() throws JspException {
		_requestContainer = null;
		_serviceRequest = null;
		_responseContainer = null;
		_serviceResponse = null;
		configure = null;

		return super.doEndTag();
	}

	/**
	 * Impostata nella JSP: Nome della Action
	 * 
	 * @param actionName
	 */
	public void setActionName(String actionName) {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, this.getClass().getName() + "::setActionName: actionName [" + actionName + "]");
		_actionName = actionName;
	} // public void setActionName(String actionName)

	/**
	 * Impostata nella JSP: Nome del Modulo
	 * 
	 * @param moduleName
	 */
	public void setModuleName(String moduleName) {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, this.getClass().getName() + "::setModuleName: moduleName [" + moduleName + "]");
		_moduleName = moduleName;
	} // public void setModuleName(String moduleName)

	/**
	 * Impostata nella JSP
	 * 
	 * @param scope
	 */
	public void setScope(String scope) {
		_scope = scope;
	}

	/**
	 * Ritorna il valore di uno specifico parametro es: <PARAMETER
	 * name="LISTA_VUOTA" type="RELATIVE", scope="SERVICE_RESPONSE" />
	 * 
	 * @param bean
	 * @return
	 */
	protected String getParameterValue(SourceBean bean) {
		String ret = "";
		if (bean != null) {
			String parameterType = (String) bean.getAttribute("TYPE");
			if (parameterType != null && parameterType.equalsIgnoreCase("ABSOLUTE")) {
				ret = (String) bean.getAttribute("VALUE");
			} else {
				String parName = (String) bean.getAttribute("NAME");
				String scope = (String) bean.getAttribute("SCOPE");
				if ((_moduleName != null))
					if (scope == null || !scope.equalsIgnoreCase("SERVICE_REQUEST"))
						parName = _moduleName.trim() + "." + parName;
				ret = (String) ContextScooping.getScopedParameter(_requestContainer, _responseContainer, parName, scope, null);
			}
			if (ret == null)
				ret = "";
		}

		return ret;
	}

	/**
	 * Valuta l'eventuale condizione impostata tramite: <CONDITION><PARAMETER
	 * name="xxx" value="xxx" scope="xxx" type="xxx" /></CONDITION>
	 * 
	 * Il value nel CONDITION.PARAMETER, pu contenere pi valori separati dalla
	 * virgola (es: value="Y,N")
	 * 
	 * @param bean
	 * @param row
	 * @return
	 */
	protected boolean testConditionParameter(SourceBean bean, SourceBean row) {
		if (bean.containsAttribute("CONDITION.PARAMETER")) {
			SourceBean condition = (SourceBean) bean.getAttribute("CONDITION.PARAMETER");
			String value = getParameterValue(condition, row);
			String conditionValues = (String) condition.getAttribute("VALUE");
			if (conditionValues != null && value != null) {
				if (conditionValues.equals("AF_NOT_DEFINED") && (value == null || value.trim().length() == 0)) {
					return true;
				}
				if (conditionValues.equals("AF_DEFINED") && (value != null && value.trim().length() > 0)) {
					return true;
				}
				value = value.toUpperCase();
				conditionValues = conditionValues.toUpperCase();
				StringTokenizer stk = new StringTokenizer(conditionValues, ",");
				while (stk.hasMoreTokens()) {
					String conditionValue = stk.nextToken().trim();
					if (value.indexOf(conditionValue) != -1)
						return true;
				}
			}
			return false;
		}
		return true; // Enanble x default
	}

	/**
	 * Ritorna il valore di uno specifico parametro es: <PARAMETER
	 * name="LISTA_VUOTA" type="RELATIVE", scope="SERVICE_RESPONSE" />
	 * 
	 * @param bean
	 *            SourceBean di configurazione
	 * @param row
	 *            SourceBean con la riga letta dalla base dati
	 * @return
	 */
	protected String getParameterValue(SourceBean bean, SourceBean row) {
		String ret = null, scope = null;
		if (bean != null && row != null) {
			scope = (String) bean.getAttribute("SCOPE");
			if (scope != null && scope.trim().equalsIgnoreCase("CURRENT_INDEX")) {
				String paramName = (String) bean.getAttribute("NAME");
				Object value = row.getAttribute(paramName);
				if (value != null)
					ret = value.toString();
			}
		}

		if (ret == null && (scope == null || (!scope.trim().equalsIgnoreCase("CURRENT_INDEX"))))
			ret = getParameterValue(bean);

		return ret;
	}

	/**
	 * Inserisce nello StringBuffer con l'HTML i parametri passati recuperandone
	 * il valore a seconda dello SCOPE
	 * 
	 * @param parameters
	 * @param row
	 * @return
	 * @throws JspException
	 */
	protected StringBuffer getParametersList(List parameters, SourceBean row) throws JspException {
		StringBuffer parametersList = new StringBuffer();
		for (int i = 0; i < parameters.size(); i++) {
			String name = (String) ((SourceBean) parameters.get(i)).getAttribute("NAME");
			String type = (String) ((SourceBean) parameters.get(i)).getAttribute("TYPE");
			String value = (String) ((SourceBean) parameters.get(i)).getAttribute("VALUE");
			String scope = (String) ((SourceBean) parameters.get(i)).getAttribute("SCOPE");
			if (name != null) {
				parametersList.append(JavaScript.escape(name.toUpperCase()) + "=");
				if ((type != null) && type.equalsIgnoreCase("RELATIVE")) {
					if ((scope != null) && scope.equalsIgnoreCase("LOCAL")) {
						if (row == null) {
							TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DefaultListTag::getParametersList: Non  possibile associare a questo bottone lo scope LOCAL");
							throw new JspException("Non  possibile associare a questo bottone lo scope LOCAL");
						} // if (row == null)
						Object valueObject = row.getAttribute(value);
						if (valueObject != null)
							value = valueObject.toString();
					} // if ((scope != null) &&
					// scope.equalsIgnoreCase("LOCAL"))
					else
						value = (String) ContextScooping.getScopedParameter(_requestContainer, _responseContainer, value, scope);
				} // if ((type != null) && type.equalsIgnoreCase("RELATIVE"))
				if (value == null)
					value = "";
				parametersList.append(JavaScript.escape(value.trim()) + "&"); // AG
				// 15
				// May
				// 2007:
				// added
				// trim()
			} // if (name != null)
		} // for (int i = 0; i < parameters.size(); i++)
		return parametersList;
	} // protected StringBuffer getParametersList(Vector parameters,

	/**
	 * Metodo principalmente utilizzato dal Tag di dettaglio e quello dei
	 * bottoni Ritorna true se la pagina di dettaglio e in modalit INSERT
	 * 
	 * @return
	 */
	protected boolean isInsertMode() {
		String serviceMode = (String) _serviceRequest.getAttribute("MESSAGE");
		// Fatto per rientro da lookup
		if (serviceMode == null)
			serviceMode = (String) _serviceResponse.getAttribute("MESSAGE");
		return ((serviceMode == null) || (serviceMode.equalsIgnoreCase(DelegatedDetailService.DETAIL_INSERT)) || (serviceMode.equalsIgnoreCase(DelegatedDetailService.DETAIL_NEW)));
	}

	/**
	 * Metodo che ritorna un semplice hidden field
	 * 
	 * @param name
	 * @param value
	 * @return
	 */
	protected String getHiddenField(String name, String value) {
		return getGenericHtmlField("hidden", name, value, 0, 0, 0, null);
	}

	/**
	 * Ritorna l'HTML base per i campi di Input
	 * 
	 * @param type
	 *            (es: FILE, TEXT, IMAGE, ...)
	 * @param name
	 *            nome ed id del campo
	 * @param value
	 *            valore da assegnare
	 * @param size
	 * @param maxLength
	 * @param jsEvents
	 * @return
	 */
	protected String getGenericHtmlField(String type, String name, String value, int size, int maxLength, int tabindex, String jsEvents) {
		String html = "<input type=\"" + type + "\"" + " name=\"" + name + "\"" + " id=\"" + name + "\"" + " value=\"" + value.trim() + "\"";
		if (size > 0) {
			// Il value pu contentere caratteri aggiuntivi inseriti dal
			// "decorator"
			if (value != null && value.length() > size)
				size = value.length() + 1;
			// Il size  aumentato del 20% per bilanciare il font non
			// proporzionale
			html += " size=\"" + (int) (size * 1.2) + "\"";
		} else {
			// html += " style=\"width:100%\" " ;
		}

		if (maxLength > 0)
			html += " maxlength=\"" + maxLength + "\"";

		html += " tabindex=\"" + tabindex + "\"";

		if (jsEvents != null && jsEvents.length() > 0)
			html += jsEvents;

		html += ">";

		if (tabindex == 1) {
			html += "\n<SCRIPT language=\"Javascript\" type=\"text/javascript\">\n";
			html += "\t document.f1." + name + ".focus();\n";
			html += "</SCRIPT>\n";
		}
		return html;
	}

	/**
	 * Ritorna l'html dinamico per una qualsiasi combo-box esempi di
	 * configurazione: a) <SELECT_QUERY statement="COMBO_ENTITA" />
	 * 
	 * b) <OPTION display="In errore" value="008" selected /> <OPTION
	 * display="In corso" value="000" />
	 * 
	 * @param name
	 *            Nome del campo di combo
	 * @param value
	 *            Valore corrente
	 * @param selectAction
	 *            eventuale javascript da eseguire
	 * @param optionList
	 *            lista SourceBean di tipo option (per le combo statiche)
	 * @param query
	 *            SourceBean con la query
	 * @param pool
	 *            Pool da utilizzare per l'esecuzione della query
	 * @return
	 */
	protected String getHtml4Combo(String name, String value, String selectAction, List optionList, SourceBean query, String pool, SourceBean module) {
		return getHtml4Combo(name, value, selectAction, optionList, query, pool, module, false, "", "", "");
	}

	/**
	 * Ritorna l'html dinamico per una qualsiasi combo-box esempi di
	 * configurazione: a) <SELECT_QUERY statement="COMBO_ENTITA" />
	 * 
	 * b) <OPTION display="In errore" value="008" selected /> <OPTION
	 * display="In corso" value="000" />
	 * 
	 * @param name
	 *            Nome del campo di combo
	 * @param value
	 *            Valore corrente
	 * @param selectAction
	 *            eventuale javascript da eseguire
	 * @param optionList
	 *            lista SourceBean di tipo option (per le combo statiche)
	 * @param query
	 *            SourceBean con la query
	 * @param pool
	 *            Pool da utilizzare per l'esecuzione della query
	 * @param readOnly
	 * @return
	 */
	protected String getHtml4Combo(FieldSupport field, String pool, String readOnly) {

		String name = field.getFieldName();
		String value = field.getFieldValue();
		String desc = field.getFieldDescription();
		String selectAction = field.getSelectAction();
		List optionList = field.getListValue();
		SourceBean query = field.getComboQuery();
		SourceBean module = field.getComboModule();
		boolean isMandatory = field.isMandatory();
		String decoratorName = field.getDecoratorName();

		return getHtml4Combo(name, value, selectAction, optionList, query, pool, module, isMandatory, decoratorName, readOnly, desc);
	}

	/**
	 * Ritorna l'html dinamico per una qualsiasi combo-box esempi di
	 * configurazione: a) <SELECT_QUERY statement="COMBO_ENTITA" />
	 * 
	 * b) <OPTION display="In errore" value="008" selected /> <OPTION
	 * display="In corso" value="000" />
	 * 
	 * @param name
	 *            Nome del campo di combo
	 * @param value
	 *            Valore corrente
	 * @param selectAction
	 *            eventuale javascript da eseguire
	 * @param optionList
	 *            lista SourceBean di tipo option (per le combo statiche)
	 * @param query
	 *            SourceBean con la query
	 * @param pool
	 *            Pool da utilizzare per l'esecuzione della query
	 * @param readOnly
	 * @param desc
	 * @return
	 */
	private String getHtml4Combo(String name, String value, String selectAction, List optionList, SourceBean query, String pool, SourceBean module, boolean isMandatory, String decoratorName, String readOnly, String descFromDecorator) {
		// Apertura della <select ....

		String html = "";
		optionList = loadOptionValuesList(optionList, pool, query, module, decoratorName);

		String optdspMia = "";
		String desc = "";

		if (optionList != null) {

			if (!(isMandatory && optionList.size() == 1)) {

				try {
					SourceBean blankSelection = new SourceBean("OPTION");
					blankSelection.setAttribute("DISPLAY", "");
					blankSelection.setAttribute("VALUE", "");
					optionList.add(0, blankSelection);
				} catch (SourceBeanException ex) {
					TracerSingleton.log("Spago", 4, ("AbstractStandardTag::loadOptionValuesList:: Errore nella costruzione del bean vuoto:" + ex.getMessage()));
				}

			}
		}

		if (readOnly.length() == 0) {
			html += "<select id=\"" + name + "\" class=\"combobox\" tabindex=\"0\" style=\"width:100%\" size=\"1\" name=\"" + name + "\"";
			if (selectAction == null || selectAction.trim().length() == 0)
				selectAction = "";
			// else
			// selectAction += " ;";
			// html += " onblur=\"" + "cambiaVisualizzazioneSelect('" + name +
			// "')" + "\">\n";
			html += " onchange=\"" + selectAction + ";" + /*
															 * "reimpostaInput('" +
															 * name + "')" +
															 */"\">";
			// Costruzione delle option / value
		}

		boolean trovato = false;
		for (int i = 0; optionList != null && i < optionList.size(); i++) {
			SourceBean option = (SourceBean) optionList.get(i);

			if (option.getAttribute("value").toString().trim().length() == 0) {
				if (!trovato) {
					trovato = true;
				} else {
					continue;
				}
			}

			if (readOnly.length() == 0) {
				html += "<option ";
			}
			String trimmedValue = value.trim();

			if ((isMandatory && optionList.size() == 1)) {

				optdspMia = option.getAttribute("display").toString();
				if (readOnly.length() == 0) {
					html += "selected=\"selected\" ";
				} else {
					desc = option.getAttribute("display").toString();
				}

			} else if (trimmedValue.equals(option.getAttribute("value").toString().trim()) || trimmedValue.equals(option.getAttribute("display").toString().trim())) {

				optdspMia = option.getAttribute("display").toString();
				if (readOnly.length() == 0) {
					html += "selected=\"selected\" ";
				} else {
					desc = option.getAttribute("display").toString();
				}

			}
			/*
			 * 
			 * Tolgo il trim sui value html += "value = \"" +
			 * option.getAttribute("value").toString().trim() + "\">" ;
			 */
			if (readOnly.length() == 0) {

				html += "value = \"" + option.getAttribute("value").toString() + "\">";
				html += option.getAttribute("display").toString().trim().toUpperCase();
				html += "</option>";
			}
		}

		// Chisura della select e del campo select finto per il javascript
		// che risolve il problema dei DIV bucati con le versioni inferiori
		// al 7 di I.E.
		if (readOnly.length() == 0) {
			html += "</select>";

			//html += "<input type=\"text\"  tabindex=\"0\" id=" + "\"" + name + "_I" + "\"" + " name=\"" + name + "_I" + "\"" + " class=" + "\"" + "selectfinto" + "\"" + " onmouseover=" + "\"" + "cambiaVisualizzazioneInput('" + name + "_I" + "')" + "\"" + " readonly " + " style=\"display: none; width:100%\"" + " value=\"" + optdspMia.trim() + "\">\n";

		} else {

			if (descFromDecorator != null) {
				desc = descFromDecorator;
			}

			html += "<input type=\"hidden\" name=\"" + name + "\" value=\"" + value + "\">";
			html += "<input readonly type=\"text\" name=\"" + name + "_descReadonly\" value=\"" + desc + "\">";
			// html += " readonly >";
		}

		return html;
	}

	/**
	 * Metodo interno per la generazione delle option/value nelle combo E'
	 * possibile richiamarlo con una lista di option (combo statica) o con una
	 * query / pool (combo dinamica)
	 * 
	 * @param optionList
	 *            List of SourceBean
	 * @param pool
	 *            Utilizzato per l'esecuzione della query
	 * @param query
	 *            SourceBean con LIST_QUERY da passare al QueryExecutor
	 * @return
	 */
	private List loadOptionValuesList(List optionList, String pool, SourceBean query, SourceBean module, String decoratorName) {
		List rowsList = null;

		if (decoratorName != null) {
			try {
				AbstractDecorator decorator = DecoratorFactory.getInstance().getDecorator(decoratorName, _requestContainer);
				rowsList = decorator.optionList();
			} catch (Exception e) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "AbstractStandardTag::loadOptionValueList::Error", e);
				// e.printStackTrace();
			}
		}
		if (query != null) {
			SourceBean rowsSourceBean = ((SourceBean) QueryExecutor.executeQuery(_requestContainer, _responseContainer, pool, query, "SELECT"));
			if (rowsSourceBean != null) {
				rowsList = rowsSourceBean.getAttributeAsList("ROW");
			}
		}
		if (module != null) {
			try {
				List<SourceBean> columnList = module.getAttributeAsList("CONFIG.COLUMNS.COLUMN");
				String Field1 = (String) columnList.get(0).getAttribute("NAME");
				String Field2 = (String) columnList.get(1).getAttribute("NAME");
				String moduleName = (String) module.getAttribute("NAME");

				List<SourceBean> daoRows = _responseContainer.getServiceResponse().getAttributeAsList(moduleName + ".PAGED_LIST.ROWS.ROW");
				for (SourceBean row : daoRows) {
					SourceBean newSelection = new SourceBean("OPTION");
					newSelection.setAttribute("DISPLAY", "" + row.getAttribute(Field1));
					newSelection.setAttribute("VALUE", "" + row.getAttribute(Field2));
					if (rowsList == null)
						rowsList = new ArrayList<SourceBean>();
					rowsList.add(0, newSelection);
				}
			} catch (SourceBeanException ex) {
				TracerSingleton.log("Spago", 4, ("AbstractStandardTag::loadOptionValuesList:: Errore nella costruzione del bean dal modulo:" + ex.getMessage()));
			}
		}

		return rowsList != null ? rowsList : optionList;
	}

	/**
	 * Con l'introduzione della classe LookupMaster un modulo pu essere
	 * chiamato come lookup se presenta il parametro ShowLookup nella
	 * ServiceRequest
	 * 
	 * @return true se  una action di lookup
	 */

	protected boolean isLookupAction() {
		boolean lookupAction = false;
		String showLookup = (String) _serviceRequest.getAttribute("SHOWLOOKUP");
		if (showLookup != null && showLookup.equalsIgnoreCase("TRUE"))
			lookupAction = true;

		return lookupAction;
	}

	/**
	 * Ritorna ACTION o MODULE a seconda del tipo di request
	 * 
	 * @return
	 */
	protected String getRequestType() {
		String requestType = "ACTION";
		if (_actionName == null)
			requestType = "MODULE";
		return requestType;
	}

	/**
	 * Genera un bean di tipo PARAMETER es: <PARAMETER name="nome"
	 * type="RELATIVE" scope="SERVICE_REQUEST", value="value" />
	 * 
	 * @param scope
	 * @param name
	 * @param value
	 * @param type
	 * @return
	 * @throws SourceBeanException
	 */
	protected SourceBean makeParameter(String scope, String name, String value, String type) throws SourceBeanException {
		SourceBean parameter = new SourceBean("PARAMETER");
		parameter.setAttribute("NAME", name);
		parameter.setAttribute("TYPE", type);
		parameter.setAttribute("SCOPE", (scope == null) ? "" : scope);
		parameter.setAttribute("VALUE", value);
		return parameter;
	}

	public void setForm(String _form) {
		if (_form != null) {
			this._form = _form;
		}
	}

	/**
	 * 
	 * @return
	 */
	public String getResourcePath() {
		if (_resourcePath == null) {
			SourceBean resources = (SourceBean) _requestContainer.getAttribute(Constants.RESOURCES);
			if (resources != null) {
				_resourcePath = (String) resources.getAttribute("rootPath");
			}
		}
		return _resourcePath;
	}

}