/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.tags;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.service.detail.impl.DelegatedDetailService;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.util.ConditionEngine;
import it.eng.spago.util.ContextScooping;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.jsp.JspException;

/**
 * @author salerno
 * Displays the BUTTONS section of an ACTION or MODULE
 */
public class ButtonPanel extends AbstractStandardTag {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 357940851448355443L;

	/**
	 * The DOM id  
	 */
	private String id;
	private String ajax ;
	
	/**
	 * The style class
	 */
	private String styleClass;

	@Override
	protected StringBuffer getHtmlStream() throws Exception {
		
		StringBuffer buffer = new StringBuffer();
		
		if (_layout != null) {
			
			SourceBean buttons = (SourceBean)_layout.getAttribute("BUTTONS");
			buffer.append("<div id=\""+id+"\" class=\""+styleClass+"\" >\n\r"); // onresize=\"resizeMe(this);\"

			if (buttons != null)  {
				
				
				buffer.append("\t\t\t\t<table name=\"tabellacomandi\" class=\"comandi\" id=\"tabellacomandi\">\n\r");
				
				buffer.append("\t\t\t\t\t<tr class=\"comandi\">\n\r");
		
				SourceBean element;
				for (Iterator iter = buttons.getAttributeAsList("BUTTON").iterator(); iter.hasNext();) {
					element = (SourceBean) iter.next();
					drawButton(element, buffer);
				}
					
				buffer.append("\t\t\t\t\t</tr>\n\r");
				buffer.append("\t\t\t\t</table>\n\r");	
				
			} 
			buffer.append("</div>\n\r");	
		}
		return buffer;
	}
	
	/**
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;
	}
	/**
	 * @param styleClass
	 */
	public void setStyleClass(String styleClass) {
		this.styleClass = styleClass;
	}

	/**
	 * 
	 * @param config
	 * @param bufferToWriteInto
	 * 
	 * @throws JspException
	 */
	public void drawButton(SourceBean config, StringBuffer bufferToWriteInto) throws JspException {
		String name = (String) config.getAttribute("name");
		String codeLabel = (String) config.getAttribute("label");
		String label = MessageBundle.getMessage(codeLabel).trim();
		List<SourceBean> sbParams = config.getAttributeAsList("PARAMETERS.PARAMETER");
		String paramName;
		SourceBean element;
		String scope;
		String value;
		
		
		ArrayList<String> params = new ArrayList<String>();
		boolean isRedirecting = false, existAjax = false;
		
		for (Iterator<SourceBean> iter = sbParams.iterator(); iter.hasNext();) {
			element = iter.next();
			paramName = (String)element.getAttribute("name");
			
			if (paramName!=null) {
				scope = (String)element.getAttribute("scope");
				value = (String)ContextScooping.getScopedParameter(_requestContainer,_responseContainer,paramName,scope,element);
				if (paramName.equalsIgnoreCase("ajax")) {
					existAjax = true ;
					// If it's set on jsp tag use it
					if (this.ajax != null && this.ajax.trim().length() > 0) {
						value = this.ajax ;
					}
				}
				params.add(paramName+"="+value);
			
				if (paramName.equals("PAGE") || paramName.equals("ACTION_NAME")) {
					isRedirecting = true;
				}
			}
		}
		
		if (this.ajax != null && this.ajax.trim().length() > 0 && (!existAjax)) {
			params.add("ajax=" + ajax);
		}
		
		//if there's no PAGE or ACTION_NAME parameter, defaults to same page as caller 
		if (!isRedirecting) {
			params.add("PAGE="+_pageName);
		}
		
		params.add("AF_BUTTON="+name);
		params.add("AF_PANEL_ID="+id); // A.G. 2008/09/23: Add for graph masterDetail (with more than one ButtonPanel)
		
		if (name.equals("SUBMIT"))
		{
			String message = DelegatedDetailService.DETAIL_UPDATE; // Default
			if (isInsertMode())
				message = DelegatedDetailService.DETAIL_INSERT;
			
			params.add("MESSAGE="+message);
			params.add("SUBMIT=TRUE");
			
		}
		
		
		String styleCss = (String)config.getAttribute("styleCss");
		
		
		bufferToWriteInto.append("<td");
		bufferToWriteInto.append(" id=\""+_form+name+"\"");
		
		if (!testConditions(config)) {
			
			bufferToWriteInto.append("class=\""+styleCss+"Disabled\"");
		
		} else {
			
			bufferToWriteInto.append("class=\""+styleCss+"\"");
			bufferToWriteInto.append(" ");
			bufferToWriteInto.append("alt=\""+label+"\"");
			bufferToWriteInto.append(" ");
			bufferToWriteInto.append("title=\""+label+"\"");
			bufferToWriteInto.append(" ");
			bufferToWriteInto.append("onmouseover=\"this.className=\'"+ styleCss+"Over\'\"");
			bufferToWriteInto.append(" ");
			bufferToWriteInto.append("onmouseout=\"this.className=\'"+ styleCss+ "\'\"");
			
			bufferToWriteInto.append(" ");
			
			String confirm = (String)config.getAttribute("CONFIRM");
			
			String js = "";
			//For now, confirm = true will display a YES/NO popup
			if (confirm!=null && confirm.equalsIgnoreCase("TRUE")) {
				String confirmMessage = (String)config.getAttribute("CONFIRM_MESSAGE");
				
				SourceBean onYes = ((SourceBean)config.getAttribute("ONYES"));
				SourceBean onNo  = ((SourceBean)config.getAttribute("ONNO"));
				
				if (confirmMessage==null || confirmMessage.length()==0) {
					confirmMessage = MessageBundle.getMessage(MSG_ASK_GENERIC);
				} 
				
				String onYesFunc="";
				if (onYes!=null) {
					onYesFunc = "javascript:"+(String)onYes.getAttribute("function");
					onYesFunc+="(%XX"+_form+"%XX";
					for (Iterator<String> iter = params.iterator(); iter.hasNext();) {
						String str = iter.next();
						onYesFunc+=",%XX"+str+"%XX";
					}
					onYesFunc+=")";
				}

				String onNoFunc="";
				if (onNo!=null) {
					onNoFunc = "javascript:"+(String)onNo.getAttribute("function")+"()";
				}
				//clicking on the button will display the popup ( via js )
				js += "impostaSpeciaSiNo('"+
				confirmMessage
				+"','" +
				onYesFunc
				+"','"+
				onNoFunc
				+"');";
				
			} else {
				
				SourceBean functionBean = (SourceBean)config.getAttribute("ONCLICK");
				if (functionBean!=null) {
					js = (String)functionBean.getAttribute("function");
					js+="('"+_form+"'";
					for (Iterator<String> iter = params.iterator(); iter.hasNext();) {
						String str = iter.next();
						js+=",'"+str+"'";
					}
					js+=");";
				}
			}
			
			if (js!=null && js.length()>0) {
				bufferToWriteInto.append("onclick=\""+js+"\"");
			}
		}

		bufferToWriteInto.append(" ");
		bufferToWriteInto.append(">");
		bufferToWriteInto.append("&nbsp;");
		bufferToWriteInto.append("</td>\n");
		
	}
	
	/**
	 * Valuta l'eventuale condizione impostata tramite: <CONDITION><PARAMETER
	 * name="xxx" value="xxx" scope="xxx" type="xxx" /></CONDITION>
	 * 
	 * Il value nel CONDITION.PARAMETER, pu contenere pi valori separati dalla
	 * virgola (es: value="Y,N")
	 * 
	 * @param bean
	 * @param row
	 * @return
	 */
	private boolean testConditions(SourceBean bean) {
		boolean isButtonEnabled = true;
		if (bean.containsAttribute("CONDITIONS.CONDITION")) {
			
			List<SourceBean> conditions = bean.getAttributeAsList("CONDITIONS.CONDITION");
			isButtonEnabled = false;
			
			for (Iterator<SourceBean> iter = conditions.iterator(); iter.hasNext();) {
				SourceBean condition = iter.next();
				
				boolean conditionVerified = ConditionEngine.evaluate(condition,_requestContainer,_responseContainer);
				
				if (conditionVerified) {
					isButtonEnabled = true;
				}
			}
		}
		
		return isButtonEnabled;

	}

	public String getAjax() {
		return ajax;
	}

	public void setAjax(String ajax) {
		this.ajax = ajax;
	}
		
}
