/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.tags;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.TokenHandler;
import it.eng.spago.dispatching.service.detail.impl.DelegatedDetailService;
import it.eng.spago.dispatching.service.list.basic.impl.DelegatedBasicListService;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryParameter;

import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
/**
 * This tag is called by the <code>ListTag</code> to draw a set of image buttons on the first column of the list.
 * Clicking on it may call an url ( via GET ) or submit the form via ajax ( ajax="TRUE" )  
 * @author salerno
 */
public class Caption extends TagComponent {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private String label;

	private String confirm;

	private String styleClass;

	private String disableParameter;

	private String popupMode;

	private String captionType;

	private String formName;

	public final static String SELECT_CAPTION = "SELECT_CAPTION";
	public final static String CHECKBOX_CAPTION = "CHECKBOX_CAPTION";
	public final static String GENERIC_CAPTION = "CAPTION";
	public final static String DELETE_CAPTION = "DELETE_CAPTION";
	
	@Override
	public void initFromConfig(SourceBean config) throws JspException  {
		this._config = config;

		captionType = config.getName();

		label = (String) _config.getAttribute("LABEL", getDefaultLabel(captionType));
		confirm = (String) _config.getAttribute("CONFIRM", getDefaultConfirm(captionType));
		
		//the image is set as a background
		styleClass = (String) _config.getAttribute("CLASS", getStyleClass(captionType));
		
		//PLAIN -> open url in the same window
		popupMode = (String) _config.getAttribute("MODE", "PLAIN");

	}

	/**
	 * 
	 * @param pageContext
	 * @param bean
	 */
	public Caption(PageContext pageContext, SourceBean bean, String formName) throws JspException  {
		super();
		this.setPageContext(pageContext);
		this.generalTagInit();
		this.initFromConfig(bean);
		this.formName = formName;
	}

	@Override
	protected StringBuffer getHtmlStream() throws Exception {
		// TODO Stub di metodo generato automaticamente
		return null;
	}

	/**
	 * @param bufferToWriteInto
	 * @param row
	 */
	public void drawCaption(StringBuffer bufferToWriteInto, SourceBean rowBean, int rowIndex) throws JspException {

		if (!isEnabled()) {
			disableParameter = "Disabled";
			styleClass += disableParameter; // Ex: editDisable
		} else {
			disableParameter = testConditionParameter(_config, rowBean) ? "" : "Disabled";
			styleClass += disableParameter; // Ex: editDisable
		}

		bufferToWriteInto.append("<a href=\"#\" class=\"" + styleClass + "\" onclick=\"");
		
		if (disableParameter == null || !disableParameter.equals("Disabled")) {

			List parameters = _config.getAttributeAsList("PARAMETER");
			StringBuffer parametersStr = getParametersList(parameters, rowBean);
			parametersStr.append(getMessage(captionType));
			
			if (_config.containsAttribute("QUERY_PARAMETER")) {
				try {
					SourceBean queryParameterBean = handleLocalParameter((SourceBean) _config.getAttribute("QUERY_PARAMETER"), rowBean);
					String retriveParameter = QueryParameter.getParametersAsQueryString(_requestContainer, _responseContainer, rowBean, queryParameterBean);
					// Se non recupero parametri per i QUER_PARAMETER disabilito
					// il check-box
					if (retriveParameter == null || retriveParameter.trim().length() == 0) {
						disableParameter = "Disabled";
						styleClass += disableParameter; // Ex: editDisable
						// TODO: Trace WARNING da inserire per segnalare
						// l'anomalia
					}
					parametersStr.append(retriveParameter);
				} catch (Exception e) {
					TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"Captions::drawCaption::Error",e);
					//e.printStackTrace();
				}
			}

			String preventResubmit = (String) _config.getAttribute("PREVENT_RESUBMIT");
			// Default = FALSE x preventResubmit
			if ((preventResubmit != null) && (preventResubmit.equalsIgnoreCase("TRUE")))
				parametersStr.append(TokenHandler.REQUEST_TOKEN_KEY + "=" + TokenHandler.getToken(_requestContainer) + "&");
			if (disableParameter.equalsIgnoreCase("Disabled")) {
				bufferToWriteInto.append("return");
			} else {
				/*
				 * Override del Publisher per le richieste AJAX. Esempio di
				 * configurazione:
				 * 
				 * <SELECT_CAPTION image="" label="Dettaglio Operazione"
				 * confirm="FALSE"> <AJAX publisher="PubPaginaDetailPopUpAjax" />
				 * </SELECT_CAPTION>
				 */
				String captionAjaxType = "";
				SourceBean serviceRequest = (SourceBean) _requestContainer.getServiceRequest();
				String beanName = (String)_config.getName();
				if(beanName.equalsIgnoreCase("DELETE_CAPTION")) 
					captionAjaxType = "ajax_delete";
				String ajaxCaption = (String) serviceRequest.getAttribute(captionAjaxType);
				//xString formName = "f1";
				String ajaxFlag = "FALSE";
				if(ajaxCaption!=null && ajaxCaption.length()>0) {
					ajaxFlag = "TRUE";
					parametersStr.append("ajax=" + ajaxCaption);
				}
				else {
						SourceBean ajaxBean = (SourceBean) _config.getAttribute("AJAX");
						if (ajaxBean != null) {
							formName = (String) ajaxBean.getAttribute("SOURCEFORMNAME", "f1");
							ajaxFlag = "TRUE";
							parametersStr.append("ajax=" + (String) ajaxBean.getAttribute("PUBLISHER", "PubPaginaAjax"));
						}
				}
				bufferToWriteInto.append("goConfirm('" + getQuestion(captionType) + "','" // Question
						+ parametersStr.toString() + "','" // url
						+ confirm + "','" // alertFlag
						+ ajaxFlag + "','" // ajaxFlag
						+ formName + "','" // formName
						+ popupMode + "'); return false;"); // usaPopUp
			}
		} else {
			bufferToWriteInto.append("return");
		}
		bufferToWriteInto.append("\"><img class=\"trasparente\" src=\""+getResourcePath()+"/spago/img/trasparente.gif\"" + " title=\"" + label + "\" " + "alt=\"" + label + "\"/></a>\n");
	} // if (_config != null)

	/**
	 * Sostitutisce i parametri LOCAL con dei tipi ABSOLUTE valorizzati dalla
	 * riga rowBean
	 * 
	 * @param parameterBean
	 * @param rowBean
	 * @return
	 * @throws Exception
	 */
	private SourceBean handleLocalParameter(SourceBean parameterBean, SourceBean rowBean) throws Exception {
		SourceBean handledBean = new SourceBean(parameterBean);
		handledBean.delAttribute("PARAMETER");

		for (Object par : parameterBean.getAttributeAsList("PARAMETER")) {
			if (((String) ((SourceBean) par).getAttribute("scope")).equalsIgnoreCase("LOCAL")) {
				SourceBean handledPar = new SourceBean("PARAMETER");
				handledPar.setAttribute("scope", "");
				handledPar.setAttribute("type", "ABSOLUTE");
				Object localValue = rowBean.getAttribute((String) ((SourceBean) par).getAttribute("name"));
				handledPar.setAttribute("value", "" + localValue);
				handledBean.setAttribute(handledPar);
			} else {
				handledBean.setAttribute((SourceBean) par);
			}
		}
		return handledBean;
	}

	/**
	 * Default labels for common captions
	 * @param captionType
	 * @return
	 */
	private String getDefaultLabel(String captionType) {
		if (captionType.equals(SELECT_CAPTION))
			return MessageBundle.getMessage(MSG_CAPTION_SELECT_LABEL);
		if (captionType.equals(DELETE_CAPTION))
			return MessageBundle.getMessage(MSG_CAPTION_DELETE_LABEL);
		if (captionType.equals(CHECKBOX_CAPTION))
			return MessageBundle.getMessage(MSG_CAPTION_CHECKBOX_SEL);

		return MessageBundle.getMessage(MSG_CAPTION_GENERIC_LABEL);
	}

	/**
	 * Default styles for common captions
	 * @param captionType
	 * @return
	 */
	private String getStyleClass(String captionType) {
		if (captionType.equals(SELECT_CAPTION))
			return "edit";
		if (captionType.equals(DELETE_CAPTION))
			return "delete";

		return "generic";
	}

	/**
	 * Default confirm message for common captions
	 * @param captionType
	 * @return
	 */
	private String getDefaultConfirm(String captionType) {
		if (captionType.equals(SELECT_CAPTION))
			return "false";
		if (captionType.equals(DELETE_CAPTION))
			return "true";
		if (captionType.equals(CHECKBOX_CAPTION))
			return "false";

		return "false";
	}

	/**
	 * Returns the MESSAGE for common captions
	 * 
	 * @param captionType
	 * @return
	 */
	private String getMessage(String captionType) {
		if (captionType.equals(SELECT_CAPTION))
			return "&MESSAGE=" + DelegatedDetailService.DETAIL_SELECT + "&AF_BUTTON=SELECT&";
		if (captionType.equals(DELETE_CAPTION)) {
			return "MESSAGE=" + DelegatedBasicListService.LIST_DELETE + "&NAVIGATOR_DISABLED=TRUE&AF_BUTTON=DELETE&";
			/*
			 * return _providerURL + "MESSAGE=" +
			 * DelegatedBasicListService.LIST_DELETE +
			 * "&NAVIGATOR_DISABLED=TRUE" ; + "&LIST_PAGE=" +
			 * _content.getAttribute("PAGED_LIST.PAGE_NUMBER") ;
			 */
		}

		return "";
	}

	/**
	 * Ritorna da domanda che viene visualizzata con la conferma
	 * 
	 * @param captionType
	 * @return
	 */
	private String getQuestion(String captionType) {
		if (captionType.equals(SELECT_CAPTION))
			return MessageBundle.getMessage(MSG_ASK_SELECT);
		if (captionType.equals(DELETE_CAPTION))
			return MessageBundle.getMessage(MSG_ASK_DELETE);

		return MessageBundle.getMessage(MSG_ASK_GENERIC);
	}

	/**
	 * Formatta i campi per la visualizzazione
	 * 
	 * @param fieldObject
	 * @param decoratorClassName
	 * @return
	 */
	protected String fieldFormat(Object fieldObject, String decoratorClassName) {
		String field = "&nbsp;";
		if (fieldObject != null) {
			if (fieldObject instanceof BigDecimal)
				field = ((BigDecimal) fieldObject).toPlainString();
			else
				field = fieldObject.toString();
			if (decoratorClassName != null && decoratorClassName.length() > 0) {
				try {
					Class thisClass = Class.forName(decoratorClassName);
					Object iClass = thisClass.newInstance();
					Class[] parameterTypes = new Class[] { String.class };
					Object[] arguments = new Object[] { field };
					Method thisMethod = thisClass.getDeclaredMethod("decorate", parameterTypes);
					field = (String) thisMethod.invoke(iClass, arguments);
				} catch (Exception e) {
					TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"Captions::fieldFormat::Error",e);
					//e.printStackTrace();
				}
			}
		}
		return field;
	}

	@Override
	public boolean isEnabled() {
		
		if (label.equalsIgnoreCase("ELIMINA")) {
			return super.isEnabled();
		} 
		
		return true;
		
	}
	

}
