/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.tags;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.decorators.AbstractDecorator;
import it.eng.spago.decorators.DecoratorFactory;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
/**
 * @author salerno
 * This class merges a COLUMN row from the configuration with the service request 
 * Prints out the column header, the filter box ( if any ), and the value of the cell in the table 
 */
public class Column extends TagComponent implements IConditionalTag {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	/**
	 * Value of the attribute <code>name</code> of the <code>COLUMN</code> element
	 * Name of the attribute of the row bean containing the value to be shown 
	 */
	private String name;
	
	/**
	 * Value of the attribute <code>label</code> of the <code>COLUMN</code> element
	 * Title of the column header
	 */
	private String label;	

	/**
	 * True if the attribute <code>noFilter</code> of the <code>COLUMN</code> is not null
	 * If true there's no filter field 
	 */
	private boolean noFilter;
	
	/**
	 * True if the attribute <code>readonly</code> of the <code>COLUMN</code> is not null
	 * If true the filter cannot be altered once it has been 
	 */
	private boolean readonly;

	private boolean colspanned;
	/**
	 * Value of the attribute <code>type</code> of the <code>COLUMN</code> element
	 */
	private String type;
	
	/**
	 * Value of the attribute <code>width</code> of the <code>COLUMN</code> element
	 * Define the width of the column cells
	 */
	private String width;
	
	/**
	 * By default, it's the value of the attribute <code>width</code> of the <code>COLUMN</code> element
	 * Define the width of the column header
	 */
	private String titleWidth;	
	
	/**
	 * Value of the attribute <code>widget</code> of the <code>FILTER</code> element that is child to the <code>COLUMN</code>
	 * Define the type of filter field
	 * Allowed values are TEXT,COMBO,CALENDAR
	 */
	private String widget;
	
	/**
	 * Value of the attribute <code>size</code> of the <code>FILTER</code> element that is child to the <code>COLUMN</code>
	 * Define the size of filter fields of type <code>text</code>
	 */
	private String filterSize;
	
	/**
	 * Value of the attribute <code>size</code> of the <code>FILTER</code> element that is child to the <code>COLUMN</code>
	 * Define the maxlength of filter fields of type <code>text</code>
	 */
	private String filterMaxLength;

	/**
	 * List of OPTION elements child of FILTER element ( widget = combo )
	 * List the values for a combo
	 */
	private List options;
	
	/**
	 * Value of the attribute <code>decorator</code> of the <code>COLUMN</code> element
	 * Decorate the value shown in the cells
	 */
	private AbstractDecorator decorator;
	
	/**
	 * 
	 */
	private SourceBean sourceQuery;
	
	/**
	 * 
	 */
	private String keyName;
	
	/**
	 * Unused
	 */
	private String disableImage;
	
	/**
	 * Used by the list tag to determine the colspan
	 */
	private int prevColspan = 0;
	/**
     * Used by the list tag to determine the colspan
	 */
	private int curColspan  = 0;
	
	public void setCurColspan(int curColspan) {
		this.curColspan = curColspan;
	}

	/**
	 * @param _parent
	 * @param column
	 */
	public Column(PageContext pageContext, SourceBean column) throws JspException {
		super();
		this.setPageContext(pageContext);
		this.generalTagInit();
		this.initFromConfig(column);
	}


	@Override
	/**
	 * 
	 */
	public void initFromConfig(SourceBean config) throws JspException {
		config.setDefaultPath("DEFAULT."+getRequestType()+".CONFIG.COLUMNS.COLUMN");
		
		name 	= (String) config.getAttribute("NAME");
		label = ((String) config.getAttribute("LABEL",MessageBundle.getMessage(name))).trim().toUpperCase();
		width = (String) config.getAttribute("WIDTH");
		
		titleWidth = width;
		

		keyName = ((String) config.getAttribute("KEYNAME"));
		disableImage =  (String) config.getAttribute("disable_image");

		noFilter =  Boolean.parseBoolean((String)config.getAttribute("noFilter", "false"));
		readonly = Boolean.parseBoolean((String)config.getAttribute("readonly", "false"));
			
		widget = (String)config.getAttributeWithDefault("FILTER.widget") ;
		
		// AG: 04 Apr 2008
		// Se la colonna ha un decorator di tipo DATE, viene forzato il CALENDAR come widget
		if (((String)config.getAttribute("DECORATOR", "TEXT")).equalsIgnoreCase("DATE"))
			widget = "CALENDAR" ;
		
		filterSize = (String) config.getAttributeWithDefault("FILTER.SIZE");
		filterMaxLength = (String) config.getAttributeWithDefault("FILTER.MAXLENGTH");
		
		type = (String) config.getAttribute("type");
		
        options = config.getAttributeAsList("FILTER.OPTION");
		sourceQuery = (SourceBean)config.getAttribute("FILTER.SELECT_QUERY");
			
		String decoratorName = ((String) config.getAttribute("decorator"));
		try {
			decorator = DecoratorFactory.getInstance().getDecorator(decoratorName, _requestContainer) ;
		} // try
		catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
					"DefaultListTag::makeRows: decorator ["+decoratorName+"] non valido: ", ex);
			throw new JspException(ex);
		} // catch (Exception ex)		
	}

	@Override
	protected StringBuffer getHtmlStream() throws Exception {
		return null;
	}
	
	/**
	 * Draw the title row
	 * @param bufferToWriteInto
	 * @param numColonne
	 * @param prevColspan
	 * @return
	 */
	public void drawHeader(StringBuffer bufferToWriteInto) {
		if (!colspanned) {
			String colspan = (curColspan>0) ? " colspan=\"" + curColspan + "\"" : "";
			bufferToWriteInto.append("\t<th class=\"ordina\"" + colspan + " width=\"" + titleWidth + "\">" + label + "</th>\n");
		}
	}
	
	/**
	 * Draw the filter cell
	 * @param bufferToWriteInto
	 * @return
	 * @throws JspException 
	 */
	public void drawFilterBox(StringBuffer bufferToWriteInto) throws JspException  {
		// Disabilitazione per i filtri  
		
		String larghezzaColonna = (width != null && width.trim().length() > 0) ?  "width=\"" + width + "\"" : "";
		
		bufferToWriteInto.append("\t<td "+larghezzaColonna+">\r\n");
		if (noFilter || !type.equalsIgnoreCase("field")) {
			bufferToWriteInto.append("\t</td>\r\n");
			return ;
		}
		
		// Only the columns of "field" type may have filters
		if (type.equalsIgnoreCase("field")) {
			
			String filterSizeHtml = (filterSize != null && filterSize.trim().length() > 0) ? "size=\"" + filterSize + "\"" : "";
			String filterMaxLengthHtml = (filterMaxLength != null && filterMaxLength.trim().length() > 0) ? "maxlength=\"" + filterMaxLength + "\"" : "" ;
			
			String filterName = "FILTER_" + name;
			String filterValue = loadFilterValue(filterName); // Get the filter value from the last service request
			String selectCriteria = (String) _serviceRequest.getAttribute("SELECT_" + label) ;
			
			if (filterValue==null || filterValue.length()==0)// If there's no value in the service request, set criteria to NONE
				selectCriteria="NONE";
			
			String ajaxValue = (String) _serviceRequest.getAttribute("AJAX") ;
			
			//Why? Cannot remember...
			if (ajaxValue==null && selectCriteria != null && !selectCriteria.equalsIgnoreCase("NONE")) {
				bufferToWriteInto.append(getHiddenField("SELECT_" + label, selectCriteria)) ;
			}
			
			//if we came from a lookup, the filter values cannot be changed 
			if (isLookupAction()&& !readonly) {
				readonly = _content.getAttribute("READONLY_" + name)!=null ;
				if (!readonly)  
					readonly = _serviceRequest.getAttribute("READONLY_" + name)!=null;
			}
			
			/*
			 * These prepare the criteria popup and the sort caption
			 */
			String _cheId="cell_select_"+filterName;
			String _ordId="cell_ordina_"+filterName;
			
			String _newId = _cheId.replace("cell_select_FILTER_","SELECT_");
			String _newIdforOrder = _cheId.replace("cell_select_FILTER_","ORDER_");
			
			bufferToWriteInto.append("<table width=\"100%\" class=\"forselect\">");
			bufferToWriteInto.append("<tr class=\"forselect\">");
			
			String newIdValue = (String)_serviceRequest.getAttribute(_newId,"");
			String newIdForOrderValue = (String)_serviceRequest.getAttribute(_newIdforOrder,"");
			
			String classId = "Select";
			String classIdOrder = "order";
			
			if (!newIdValue.equals("") && filterValue!=null && filterValue.length()>0)  {
				classId+="_"+newIdValue;
			} else {
				classId+="_none";
			}
			if (newIdForOrderValue.equals(""))  {
				classIdOrder+="";
			} else {
				classIdOrder+=newIdForOrderValue;
			}
			/*bufferToWriteInto.append("<td width=\"19\" id=\""+_cheId+"\" name=\""+_cheId+"\" class=\""+classId+"\" onclick=\"SelezioniArcaStart(event,'"+_cheId+"','site')\" onmouseover=\"javascript:this.className=this.className+'Over';\" onmouseout=\"javascript:this.className=this.className.replace('Over','');\">");
			bufferToWriteInto.append("<img border=\"0\" name=\"img_"+_cheId+"\" id=\"img_"+_cheId+"\" src=\""+getResourcePath()+"/spago/img/trasparente.gif\" width=\"19\" height=\"19\" style=\"position: relative; top: 0 left: 0\">");
			
			
			bufferToWriteInto.append("</td>");*/
			
			bufferToWriteInto.append("<td width=\"19px\" class=\"forselect\" >");
			bufferToWriteInto.append("<img border=\"0\" id=\""+_cheId+"\" name=\""+_cheId+"\" class=\"select\" src=\""+getResourcePath()+"/spago/img/1024/"+classId+".gif\" >");
			bufferToWriteInto.append("</td>");
			/*
			bufferToWriteInto.append("<td width=\"2px\" class=\"forselect\">");
			bufferToWriteInto.append("<img border=\"0\" src=\""+getResourcePath()+"/spago/img/trasparente.gif\" width=\"2\" height=\"19\">");
			bufferToWriteInto.append("</td>");*/
			bufferToWriteInto.append("<td width=\"100%\" class=\"forselect\">");
			
			
			//if a filter is readonly but its value is null you can still change it
			if (filterValue==null || filterValue.length()==0) {
				readonly = false;
			}
			
			//Draw the actual filter field
			if (widget.equalsIgnoreCase("TEXT")) {
				bufferToWriteInto.append("<input type=\"text\""
								   + " " + ((readonly) ? "readonly" : "")
								   + " style=\"width:100%;text-transform:uppercase\""
						           + " name=\"" + filterName + "\""
//						           + larghezzaColonna
						           + filterSizeHtml
						           + filterMaxLengthHtml
						           + " onkeyup=\"this.value=this.value.toUpperCase();\""  
						           + " value=\"" + filterValue + "\">") ;
			} else if (widget.equalsIgnoreCase("CALENDAR")) {
				
				try {
					AbstractDecorator decorator = DecoratorFactory.getInstance().getDecorator("DATE", _requestContainer) ;
					filterValue = decorator.decorate(filterValue) ;
				} catch (Exception ex) {
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
							"DefaultListTag::makeFilterField: Error formatting DATE for field " + filterName, ex);
					throw new JspException(ex);
				}

				String onClickJS = "";
				//When the date field is readonly, you can't call the calendar
				if (!readonly) {
					onClickJS =	" ondblclick=\""
			             + "CalendarioArcaStart(event,\'"+filterName+"\',\'site\',\'"
			             + "GMAA"
			             + "\',\'"
			             + "-"
			             + "\')\""
		                 + "" ;
				}
				
				bufferToWriteInto.append("<input type=\"text\""
						   + " " + ((readonly) ? "readonly" : "")
						   + " style=\"width:100%;text-transform:uppercase\""
				           + " name=\"" + filterName + "\""
//				           + larghezzaColonna
//				           + dimensioneFiltro
//				           + lunghezzaMaxFiltro
				           + onClickJS
				           + " onkeyup=\"this.value=this.value.toUpperCase();\""  
				           + " value=\"" + filterValue + "\">") ;
			} else if (widget.equalsIgnoreCase("COMBO")) {
				bufferToWriteInto.append(getHtml4Combo(filterName
					                       , filterValue
					                       , null
					                       , options
					                       , sourceQuery
					                       , ((String) _layout.getAttribute("POOL"))
					                       , null)) ;
			}
			
			
			//these hidden fields contains the criteria and the sort action chosen
			bufferToWriteInto.append("<input type=\"hidden\"  id=\""+_newId+"\" name=\""+_newId+"\" value=\""+newIdValue+"\" />");
			bufferToWriteInto.append("<input type=\"hidden\"  id=\""+_newIdforOrder+"\" name=\""+_newIdforOrder+"\" value=\""+newIdForOrderValue+"\"/>");
	
			bufferToWriteInto.append("</td>");
			/*
			bufferToWriteInto.append("<td width=\"1\" class=\"forselect\">");
			bufferToWriteInto.append("<img border=\"0\" src=\""+getResourcePath()+"/spago/img/trasparente.gif\" width=\"1\" height=\"19\">");
			bufferToWriteInto.append("</td>");
			*/
			/*bufferToWriteInto.append("<td id=\""+_ordId+"\" name=\""+_ordId+"\" class=\""+classIdOrder+"\" onclick=\"localCambiaOrdinamento('"+_ordId+"','"+_newIdforOrder+"')\" onmouseover=\"javascript:this.className=this.className+'Over';\" onmouseout=\"javascript:this.className=this.className.replace('Over','');\">");
		
			bufferToWriteInto.append("<img border=\"0\" name=\"img_"+_newIdforOrder+"\" id=\"img_"+_newIdforOrder+"\" src=\""+getResourcePath()+"/spago/img/trasparente.gif\" width=\"6\" height=\"19\" >");
			bufferToWriteInto.append("&nbsp;</td>");
			*/
			bufferToWriteInto.append("<td width=\"7px\" class=\"forselect\" >");
			bufferToWriteInto.append("<img class=\"order\" border=\"0\" name=\""+_ordId+"\" id=\""+_ordId+"\" src=\""+getResourcePath()+"/spago/img/1024/Order"+newIdForOrderValue+".gif\">");
			bufferToWriteInto.append("</td>");
			
			bufferToWriteInto.append("</tr>");
			bufferToWriteInto.append("</table>");
			
			bufferToWriteInto.append("\t</td>\r\n");
		}
	}
	
	/**
	 * Draw the cell
	 * @param bufferToWriteInto
	 * @param row
	 */
	public void drawRow(StringBuffer bufferToWriteInto , SourceBean row , int rowIndex) {
		
		Object fieldObject = row.getAttribute(name);
		String field = decorator.decorate(fieldObject);

		bufferToWriteInto.append("<td class=\"" + decorator.getCssClass4List()+ "\"> " + field + "</td>\n");
		
	}
	
	/**
	 * Recupera il valore di un campo di FILTER
	 * @param colName
	 * @return
	 */
	private String loadFilterValue(String colName) {
		Object val = ContextScooping.getScopedParameter(_requestContainer,
				 _responseContainer, colName, "SERVICE_REQUEST", null);
		return (val == null) ? "" : val.toString() ;
	}
	
	/**
	 * Called by the list tag,manage the colspan resizing the column header 
	 * @param widthToAdd
	 */
	public void resizeTitle(String widthToAdd) {
		if (widthToAdd!=null) {
			if (titleWidth!=null) {
				if (titleWidth.indexOf("%")>0) {
				
					if (widthToAdd.indexOf("%")>0) {
						titleWidth = Integer.parseInt(titleWidth.substring(0,titleWidth.indexOf("%")))+Integer.parseInt(widthToAdd.substring(0,widthToAdd.indexOf("%")))+"%";
					}
				} else {
					if (widthToAdd.indexOf("%")>0) {
						//You cannot use both px and %, ignored 
					} else {
						titleWidth = Integer.parseInt(titleWidth)+Integer.parseInt(widthToAdd)+"";
					}
				}
			} else {
			
				titleWidth = widthToAdd;
			}
		
		}
		
	}

	
	
	public void setPrevColspan(int prevColspan) {
		this.prevColspan = prevColspan;
	}

	public void setWidth(String columnWidth) {
		this.width = columnWidth;
	}
	
	public void setTitleWidth(String titleWidth) {
		this.titleWidth = titleWidth;
	}

	public String getType() {
		return type;
	}

	public static long getSerialVersionUID() {
		return serialVersionUID;
	}

	public String getWidth() {
		return width;
	}

	public int getCurColspan() {
		return curColspan;
	}

	public AbstractDecorator getDecorator() {
		return decorator;
	}

	public String getName() {
		return name;
	}

	public String getFilterMaxLength() {
		return filterMaxLength;
	}

	public String getFilterSize() {
		return filterSize;
	}

	public String getKeyName() {
		return keyName;
	}
	
	public List getOptions() {
		return options;
	}


	public int getPrevColspan() {
		return prevColspan;
	}

	public SourceBean getSourceQuery() {
		return sourceQuery;
	}

	public String getWidget() {
		return widget;
	}


	public String getDisableImage() {
		return disableImage;
	}

	public boolean isNoFilter() {
		return noFilter;
	}


	public boolean isReadonly() {
		return readonly;
	}

	public boolean isColspanned() {
		return colspanned;
	}

	public void setColspanned(boolean colspanned) {
		this.colspanned = colspanned;
	}
	
}
