/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.tags;

import it.eng.spago.base.Constants;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.detail.impl.DelegatedDetailService;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;

/**
 *  * Default implementation of the view of a Detail module/action
 */
public class DetailTag extends AbstractStandardTag {
	private static final long serialVersionUID = 1L;
	public static final String SCOPE_CONTENT = "CONTENT";
	public static final String SCOPE_SERVICE_REQUEST = "SERVICE_REQUEST";
	protected SourceBean _layerSet = null;
	private StringBuffer _hiddenField = null;
	private int _tabindex = 0;

	/**
	 * Metodo principale richiamato dalla doStartTag: ritorna un buffer con il
	 * codice HTML generato
	 */
	protected StringBuffer getHtmlStream() throws Exception {
		StringBuffer htmlStream = new StringBuffer();
		generateFields(htmlStream);
		// Add an hidden field for javascript elaboration (hide/show/resize div)
		htmlStream.append(getHiddenField("detailSize", ""+htmlStream.length()));

		if (_hiddenField != null && _hiddenField.length() > 0)
			htmlStream.append(_hiddenField);
		
		return htmlStream;
	}

	/**
	 * Inizializza e verifica che il modulo o l'azione siano attivi
	 * 
	 * @return 0 o -1 per inizializzazione fallita
	 */
	protected int initTag() throws JspException {
	//	if (_serviceName == null) {
	//		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DefaultDetailTag::doStartTag:: service name non specificato !");
	//		throw new JspException("Business name non specificato !");
	//	}
		if (_layout == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "DefaultDetailTag::doStartTag:: layout nullo");
			return -1;
		}
		if (_content == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "DefaultDetailTag::doStartTag:: content nullo");
			_scope = "SERVICE_REQUEST";
		} else if (_content.getAttribute("RETURN_TO_LIST") != null) {
			return -1; // Completata azione di dettaglio: ritorno alla lista
		}
		return 0;
	}

	/**
	 * Segue un esempio della struttura da generare
	 * 
	 * <fieldset style="float: left"> <legend>Allineato a sinistra</legend>
	 * <label for="input1">INPUT1</label> <input type="text" size="43%"
	 * id="input1" name="input1" value="INPUT1"> <input type="image"
	 * src="img/lookup.gif" onmouseover="this.src='img/lookup_over.gif'"
	 * onmouseout="this.src='img/lookup.gif'"> <br>
	 * <label for="input2" class="mandatory">INPUT2</label> <input type="text"
	 * size="10" id="input2" name="input2" value="INPUT2"> <input type="image"
	 * src="img/lookup.gif" onmouseover="this.src='img/lookup_over.gif'"
	 * onmouseout="this.src='img/lookup.gif'"> </fieldset> <fieldset
	 * style="float: right"> <legend>Allineato a sinistra</legend> ...
	 * </fieldset>
	 * 
	 * @param output
	 * @throws JspException
	 */
	protected void generateFields(StringBuffer output) throws JspException {
		SourceBean row = null;
		if (_content != null && _scope.equals("CONTENT")) {
			row = (SourceBean) _content.getAttribute("ROW");
		} else if (_scope.equals("SERVICE_REQUEST")) {
			row = _serviceRequest;
		}
		
		
		
		List fieldsets = _layout.getAttributeAsList("FIELDS.FIELDSET");
		String[] colPos = { "top", "left", "mid", "right" };
		String[] fieldSetName = { "fieldsettp", "fieldsetsx", "fieldsetmd", "fieldsetdx" };
		_tabindex = 1;
		for (int iCol = 0; iCol < colPos.length; iCol++) {
			String col = colPos[iCol];
			for (int i = 0; fieldsets != null && i < fieldsets.size(); i++) {
				SourceBean aFieldset = (SourceBean) fieldsets.get(i);
				String position = (String) aFieldset.getAttribute("position");
				if (position.equalsIgnoreCase(col)) {
					String name = (String) aFieldset.getAttribute("name");
					String legend = (String) aFieldset.getAttribute("legend", "");
					String width = (String) aFieldset.getAttribute("WIDTH", "100%");
					output.append("\t<div id=\"" + name + "\" class=\"" + fieldSetName[iCol] + "\" style=\"width :"+width+";\" >\n");
					output.append("\t<table><tr><td class=\"fieldsettd\">" + legend.trim() + "</td></tr></table>\n");
					output.append("\t<table>\n");
					List fields = aFieldset.getAttributeAsList("FIELD");
					// Gestione della lentina sul fieldset
					String lookup = (String)aFieldset.getAttribute("LOOKUP");
					String fieldsetName = (String)aFieldset.getAttribute("NAME");
					generateFieldsetField(output, row, fields, lookup, fieldsetName);
					output.append("\t</table>\n\t</div>\n");
				}
			}
		}
		

	}


	/**
	 * Genera i campi configurati per un singolo fieldset
	 * 
	 * @param output
	 * @param row
	 * @param fields
	 * @throws JspException
	 */
	protected void generateFieldsetField(StringBuffer output, SourceBean row, List fields, String lookup, String fieldsetName) throws JspException {
		String mode = isInsertMode() ? DETAIL_INSERT : DETAIL_UPDATE;
		String lastLookupActionName = (String) _serviceRequest.getAttribute("FROM_LOOKUP");
		boolean isFromLookup = (lastLookupActionName != null && lastLookupActionName.trim().length() > 0)  ;
		String riga = "dispari";
		String newValue = "" ;
		for (int i = 0; i < fields.size(); i++) {
			FieldSupport field = new FieldSupport((SourceBean) fields.get(i), row, mode, _requestContainer);
			// Se non  un ritorno di lookup provo a valorizzare dalla request
			if (field.getFieldValue().equals("") && !isFromLookup) {
				newValue = FieldSupport.getStringValue(_serviceRequest.getAttribute(field.getFieldName()));
				if (newValue != null && newValue.trim().length() >0)
					field.setFieldValue(newValue);
			}
			if (writeInputField(field, output, riga, lookup, fieldsetName) != 0)
				riga = riga.equals("dispari") ? "pari" : "dispari";
		}
	}
	
	/**
	 * Writes an html input field complete with its label 
	 * @param field
	 * @param output
	 * @return 0 if the field is hidden, 1 otherwise
	 */
	private int writeInputField(FieldSupport field, StringBuffer output, String riga, String lookup, String fieldsetName) throws JspException {
		String widget = getFieldWidget(field).trim().toUpperCase();
		if (!field.isVisible()) {
			// 29/10/2008 AG: Text_area not visible are made by css properties
			if (! widget.equals(FieldSupport.WIDGET_TEXT_AREA)) {
				addHiddenField(field.getFieldName(), field.getFieldValue());
				return 0;
			}
		}
		if (field.isReadonly()) {
			addHiddenField("READONLY_" + field.getFieldName(), "readonly");
		}
		// Table "pari" o "dispari"

		if (field.isVisible()) {
			output.append("\t<tr class=\"" + riga + "\">\n");
		} else {
			output.append("\t<tr style=\"display: none\">\n");
		}
		// Label
		output.append("\t\t<td class=\"label\">" + field.getFieldLabel() + "</td>\n");
		// Mandatory or not
		String mandatory = field.isMandatory() ? " class=\"mandatory\"" : " class=\"notmandatory\"";
		output.append("\t\t<td " + mandatory + "><img class=\"imgcell\" src=\""+getResourcePath()+"/spago/img/trasparente.gif\"></td>\n");
		// Daniele: la gestione della lookup avviene qui
		output.append("\t\t<td class=\"lookup\">");
		if (!field.isReadonly())
			writeLookup(field, output, lookup, fieldsetName);
		output.append("</td>\n\t\t<td class=\"field\">\n");
		String readOnly = field.isReadonly() ? " readonly " : "";
		if (widget.equals(FieldSupport.WIDGET_TEXT)) {
			addTextField(field, output, readOnly);
		} else if (widget.equals(FieldSupport.WIDGET_PASSWORD)) {
			addPasswordField(field, output, readOnly);
		} else if (widget.equals(FieldSupport.WIDGET_COMBO)) {
			addComboField(field, output, readOnly);
		} else if (widget.equals(FieldSupport.WIDGET_TEXT_AREA)) {
			addTextAreaField(field, output);
		} else if (widget.equals(FieldSupport.WIDGET_RADIO)) {
			addRadioField(field, output);
		} else if (widget.equals(FieldSupport.WIDGET_CHECK_BOX)) {
			addCheckBoxField(field, output);
		} else if (widget.equals(FieldSupport.WIDGET_CALENDAR)) {
			addCalendarField(field, output, readOnly);
		} else if (widget.equals(FieldSupport.WIDGET_FILE)) {
			addFileField(field, output, readOnly);
		} else {
			output.append("\t\t<font color=\"RED\">WIDGET \"" + field.getWidgetName() + "\" NON SUPPORTATA</font>");
		}
		output.append("\t\t</td>\n\t</tr>\n");
		return 1;
	}

	/**
	 * Valuta e ritorna il tipo di widget definito per il campo richiesto
	 * 
	 * @param field
	 * @return
	 */
	private String getFieldWidget(FieldSupport field) {
		if (field.getWidgetName().equalsIgnoreCase("COMBO") && field.getComboQuery() == null && field.getListValue().size() == 0 && field.getComboModule() == null && field.getDecoratorName() == null) {
			field.setWidgetName(FieldSupport.WIDGET_TEXT);
		}
		return field.getWidgetName();
	}

	/**
	 * Aggiunge all'HTML da ritornare un campo di tipo "text"
	 * 
	 * @param field
	 * @param output
	 * @param readOnly
	 */
	private void addTextField(FieldSupport field, StringBuffer output, String readOnly) {
		String html = getGenericHtmlField("text", field.getFieldName(), field.getFieldValue(), field.getFieldSize(), field.getMaxLength(), _tabindex++, field.getOnKeyPress() + field.getOnKeyuP() + field.getOnChange() + readOnly);
		output.append("\t\t" + html + "\n");
	}

	/**
	 * Aggiunge all'HTML da ritornare un campo di tipo "password"
	 * 
	 * @param field
	 * @param output
	 * @param readOnly
	 */
	private void addPasswordField(FieldSupport field, StringBuffer output, String readOnly) {
		String html = getGenericHtmlField("password", field.getFieldName(), field.getFieldValue(), field.getFieldSize(), field.getMaxLength(), _tabindex++, field.getOnKeyPress() + field.getOnKeyuP() + field.getOnChange() + readOnly);
		output.append("\t\t" + html + "\n");
	}

	/**
	 * Aggiunge all'HTML da ritornare un campo di tipo "file"
	 * 
	 * @param field
	 * @param output
	 * @param readOnly
	 */
	private void addFileField(FieldSupport field, StringBuffer output, String readOnly) {
		String html = getGenericHtmlField("file", field.getFieldName(), field.getFieldValue(), field.getFieldSize(), field.getMaxLength(), _tabindex++, field.getOnKeyPress() + field.getOnKeyuP() + field.getOnChange() + readOnly);
		output.append("\t\t" + html + "\n");
	}

	/**
	 * Aggiunge all'HTML da ritornare un campo di tipo "checkbox"
	 * 
	 * @param field
	 * @param output
	 */
	private void addCheckBoxField(FieldSupport field, StringBuffer output) {
		String checked = field.getFieldValue().equalsIgnoreCase("S") ? " checked=\"checked\" " : "";
		String onClickJS = field.isReadonly() ? " onclick=\"return false\" " : null;
		if (onClickJS == null) {
			onClickJS = " onclick=\"(this.checked == true) ? f1." + field.getFieldName() + ".value='S' : f1." + field.getFieldName() + ".value='N'\"";
		}
		// output.append("\t\t" +
		// getHiddenField(field.getFieldName(),field.getFieldValue()) + "\n") ;
		String html = getGenericHtmlField("checkbox", "check_" + field.getFieldName(), field.getFieldValue(), 0, 0, 0, checked + onClickJS);
		output.append("\t\t" + html + "\n");
		addHiddenField(field.getFieldName(), field.getFieldValue());
	}

	/**
	 * Aggiunge all'HTML da ritornare un campo di tipo "textarea"
	 * 
	 * @param field
	 * @param output
	 */
	private void addTextAreaField(FieldSupport field, StringBuffer output) {
		if (field.getTextAreaCols() == 0 || field.getTextAreaRows() == 0)
			output.append("\t\t<font color=\"RED\">DIMENSIONE TEXTAREA NON CORRETTA</font>");
		else {
			output.append("\t\t<textarea name = \"" + field.getFieldName() + "\"");
			output.append("cols = \"" + field.getTextAreaCols() + "\"");
			output.append("rows = \"" + field.getTextAreaRows() + "\"");
			output.append(">" + field.getFieldValue().trim() + "</textarea>");
		}
	}

	/**
	 * Realizza un campo di input protetto con due tasti per la valorizzazione
	 * tramite calendario pop-pp
	 * 
	 * @param field
	 * @param output
	 */
	private void addCalendarField(FieldSupport field, StringBuffer output, String readOnly) {
		if (readOnly.length() == 0) {
			String onClickJS = " ondblclick=\"CalendarioArcaStart(event,\'" + field.getFieldName() + "\',\'site\',\'" + field.getCalendarFormat() + "\',\'" + field.getCalendarSeparator() + "\')\""
					+ " readonly ";
			String html = getGenericHtmlField("text", field.getFieldName(), field.getFieldValue(), field.getFieldSize(), field.getMaxLength(), 0, onClickJS);
/*
			output.append("\t\t<table width=\"100%\"><tr><td>" + html + "</td>\n");
			// 
			// Sostituzione del tasto con un "A" che ha per sfondo un'immagine
			// (per poter attivare "ALT")
			//
			output.append("<td>\t\t<a href=\"#\"  tabindex=\"" + _tabindex++ + "\" class=\"calendario\" onclick=\"javascript:BD_proteggitutto.style.display=\'inline\';CalendarioArcaStart(\'" + field.getFieldName() + "\',\'site\',\'"
					+ field.getCalendarFormat() + "\',\'" + field.getCalendarSeparator() + "\')\"" + " id=\"" + field.getFieldName() + "\">&nbsp;</a>");
			output.append("\t\t<a href=\"#\" class=\"calendariodel\"" + " onclick=\"" + field.getFieldName() + ".value=''\" >&nbsp;</a></td></tr></table>\n");
*/
			output.append(html+"&nbsp;&nbsp;<a href=\"#\"  tabindex=\"" + _tabindex++ + "\" class=\"calendario\" onclick=\"CalendarioArcaStart(event,\'" + field.getFieldName() + "\',\'site\',\'"
					+ field.getCalendarFormat() + "\',\'" + field.getCalendarSeparator() + "\')\"" + " id=\"" + field.getFieldName() + "\"><img class=\"calendar\" src=\""+getResourcePath()+"/spago/img/trasparente.gif\"></a>");
			output.append("\t\t<a href=\"#\" class=\"calendariodel\"" + " onclick=\"" + field.getFieldName() + ".value=''\" ><img class=\"calendar\" src=\""+getResourcePath()+"/spago/img/trasparente.gif\"></a>\n");
		
		} else {
			addTextField(field, output, readOnly);
		}
	}

	/**
	 * Aggiunge all'HTML da ritornare un campo di tipo "radio button"
	 * 
	 * @param field
	 * @param output
	 */
	private void addRadioField(FieldSupport field, StringBuffer output) {
		output.append("\t\t<font color=\"RED\">WIDGET RADIO NON SUPPORTATA</font>");
	}

	/**
	 * Aggiunge all'HTML da ritornare un campo di tipo "combo box"
	 * 
	 * @param field
	 * @param output
	 */
	private void addComboField(FieldSupport field, StringBuffer output, String readOnly) {
		//if (readOnly.length() == 0) {
			output.append(getHtml4Combo(field,((String) _layout.getAttribute("POOL")),readOnly));
		//} else {
		//	addTextField(field, output, readOnly);
		//}
	}

	/**
	 * Genera l'html per il tasto lookup
	 * 
	 * @param field
	 * @param output
	 * @throws JspException
	 */
	private void writeLookup(FieldSupport field, StringBuffer output, String lookup, String fieldsetName) throws JspException {
		if (lookup == null) return;
		boolean ok = false;
		String onClick = null;
		
		// Verifica se il campo corrente deve presentare il collegamento per la lookup		
		SourceBean lookupBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("LOOKUPS.LOOKUP", "NAME", lookup);
		if (lookupBean==null)
			return;
		String fieldID = field.getFieldID();
		for(SourceBean lookupField : (List<SourceBean>)lookupBean.getAttributeAsList("CONFIG.FIELD")) {
			String sourceID = (String)lookupField.getAttribute("SOURCEID");
			if(sourceID.equalsIgnoreCase(fieldID)) {
				String showLens = (String)lookupField.getAttribute("SHOWLENS");
				ok = showLens.equalsIgnoreCase("TRUE");
				onClick = (String) lookupField.getAttribute("ONCLICK");
				if(onClick==null || onClick.length()==0)
					onClick="";
			}
		}

		if(ok) {
			String lookupAction = field.getLookupAction();
			if(lookupAction!=null)
				lookupAction="ACTION_NAME="+lookupAction+"&";
			else lookupAction="";
			output.append("\t\t<input tabindex=\"0\"onClick=\""+onClick+/*"removeElement('PAGE');removeElement('MODULE');" +*/ field.getLookupOnClick() + "\" name=\"QUERY_STRING{" + lookupAction
					+ "NAVIGATOR_DISABLED=TRUE&FIELDSET=" + fieldsetName + getReturnUrl() + "}\""
					+ "	src=\""+getResourcePath()+"/spago/img/1024/Lookup.gif\"" +
				      "onmouseover=\"this.src=\'"+getResourcePath()+"/spago/img/1024/LookupOver.gif\'\"" +
				      "onmouseout=\"this.src=\'"+getResourcePath()+"/spago/img/1024/Lookup.gif\'\"" +
					  " class=\"lookup\"" +
					  " type=\"image\" />&nbsp;");




		}
	}

	/**
	 * Prepara i parametri che verranno utilizzati dalla lookup per ritornare al
	 * chiamante
	 * 
	 * @return
	 */
	private String getReturnUrl() {
		String retUrl = "";
		if (_actionName != null) {
			retUrl = "&RET_ACTION=" + _actionName;
		} else {
			retUrl = "&RET_PAGE=" + _pageName;
			
		}	
		retUrl += "&RET_AF_BUTTON=";
			if (_serviceRequest.getAttribute("AF_BUTTON")==null || _serviceRequest.getAttribute("AF_BUTTON").equals("")) {
				
			} else {
				if (isInsertMode()) {
					retUrl+="NEW";
				} else {
					retUrl+="SELECT";
				}
			}
			
			if (_serviceRequest.getAttribute("MESSAGE")==null || _serviceRequest.getAttribute("MESSAGE").equals("")) {
				
			} else {
				if (isInsertMode()) {
					retUrl+="&MESSAGE=DETAIL_NEW";
				} else {
					retUrl+="&MESSAGE=DETAIL_SELECT";
				}
			}
		
		
		return retUrl;	}

	/**
	 * Metodo interno per aggiungere campi nascosti
	 * 
	 * @param name
	 * @param value
	 */
	private void addHiddenField(String name, String value) {
		if (_hiddenField == null)
			_hiddenField = new StringBuffer();
		if (_hiddenField.indexOf("name=\"" + name) == -1)
			_hiddenField.append(getHiddenField(name, value));
	}
}