/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.tags;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.error.EMFAbstractError;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.EMFValidationError;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;
/**
 * This tag iterates into the errorHandler and show a popup div if encounter any errors
 * @author salerno
 */
public class ErrorTag extends TagSupport {

	private static final long serialVersionUID = 1L;

	private HttpServletRequest _httpRequest = null;
	private ResponseContainer _responseContainer = null;
	private RequestContainer _requestContainer = null;
	EMFErrorHandler _engErrorHandler = null ;
	StringBuffer _output = null;
	private boolean _ajaxRequest = false ;

	public int doStartTag() throws JspException {
		TracerSingleton.log("Spago", 5, "DefaultErrorTag::doStartTag:: invocato");

		try {
			initTag() ;
			makeJavascript() ;
			pageContext.getOut().print(_output.toString());
		} catch (Exception ex) {
			TracerSingleton.log("Spago", 4, "DefaultErrorTag::doStartTag::", ex);
			throw new JspException(ex);
		}
		return super.doStartTag() ;
	}
	
	/**
	 * Inizializzazione per il Tag
	 * @throws JspException
	 */
	protected void initTag() throws JspException {
		if (pageContext == null) {
			TracerSingleton.log("Spago", 4, "DefaultErrorTag::doStartTag:: pageContext nullo");
			throw new JspException("pageContext nullo");
		}
		try {
			_httpRequest = (HttpServletRequest) pageContext.getRequest();
		} catch (Exception ex) {
			TracerSingleton.log("Spago", 4, "DefaultErrorTag::doStartTag::", ex);
			throw new JspException(ex.getMessage());
		}
		if (_httpRequest == null) {
			TracerSingleton.log("Spago", 4,	"DefaultErrorTag::doStartTag:: httpRequest nullo");
			throw new JspException("httpRequest nullo");
		}
		_requestContainer = RequestContainerAccess.getRequestContainer(_httpRequest) ;
		_responseContainer = ResponseContainerAccess.getResponseContainer(_httpRequest);
		if (_responseContainer == null) {
			TracerSingleton.log("Spago", 4,	"DefaultErrorTag::doStartTag:: responseContainer nullo");
			throw new JspException("responseContainer nullo");
		}
		_engErrorHandler = _responseContainer.getErrorHandler();
		
		_output = new StringBuffer();
		
		// Analizza le richieste in modalit AJAX
		if (_requestContainer != null) {
	    	String publisherName = (String) _requestContainer.getServiceRequest().getAttribute("AJAX");
	    	if (publisherName != null && publisherName.trim().length() > 0) {
	    		set_ajaxRequest(true) ;
	    	}
    	}
	}
	
	/**
	 * Create the javascript showing the popup 
	 * @throws UnsupportedEncodingException 
	 */
	private void makeJavascript() throws UnsupportedEncodingException {
		// Per le richieste AJAX, non viene aggiunto il javascript ma solo il tag "error"

		if (is_ajaxRequest() && showPopUp()) {
			
			String message = getErrorList();
			
			if (message==null || message.trim().length()==0) {
				message = getWarningList();
			}
			_output.append("\n|error|"+ escape(message)) ;
			return ;
		}
		
		_output.append("<SCRIPT language=\"Javascript\" type=\"text/javascript\">\n");
		
		String message =  getMessage(true, false, _engErrorHandler);
		String warnings = "";
		String errors = "";
		String infos = "";
		if (_engErrorHandler != null) {
			List<String> campiDaEvidenziare = getFields(true, false, _engErrorHandler);
			if (campiDaEvidenziare!=null && campiDaEvidenziare.size()>0) {
				_output.append("fields = new Array();");
				int i = 0;
				for (Iterator<String> iter = campiDaEvidenziare.iterator(); iter.hasNext();) {
					String element = iter.next();
					if(element!=null)  {
						_output.append("fields["+i+"] = '"+element+"';\n");
						i++;
					}
				}
			}
		}
		
		if (showPopUp()) {
			
			String errorList = getErrorList();
			
			if (errorList==null || errorList.trim().length()==0) {
				warnings =  escape(getWarningList());				
			} else {
				errors =  escape(errorList);	
			}
			
		} else {
			if (message!=null && message.trim().length()>0) {
				infos = escape(message);	
			}
		}
		_output.append("</SCRIPT>\n");
		
		_output.append("<input type=\"hidden\" id =\"warnings\" value=\""+warnings+"\">");
		_output.append("<input type=\"hidden\" id =\"errors\" value=\""+errors+"\">");
		_output.append("<input type=\"hidden\" id =\"infos\" value=\""+infos+"\">");
	}
	
	/**
	 * Gets all EMFInternalErrors from the errorHandler and concats their messages 
	 * @return
	 */
	private String getErrorList() {
		return getMessage(false, true, _engErrorHandler) ;
	}
	/**
	 * Gets all EMFUserErrors from the errorHandler and concats their messages 
	 * @return
	 */
	private String getWarningList() {
		return getMessage(true, true, _engErrorHandler) ;
	}
	
	public static String escape(String in) throws UnsupportedEncodingException {
		String out = "" ;
		String temp = URLEncoder.encode(in, "ISO-8859-1") ;
		for (int i=0 ; i < temp.length() ; i++) {
			if (temp.charAt(i) == '+')  {
				out += " " ;
			} else if (temp.charAt(i) == '/')  {
				out += "%2F" ;
			} else {
				out += temp.charAt(i) ;
			}
		}
		
		return out ;
	}
	

	private boolean showPopUp() {
		return 	(_engErrorHandler != null
				&& (!_engErrorHandler.isOKBySeverityList("WARNING,ERROR,BLOCKING")));
	}
	
	/**
	 * Concat the error messages
	 * @param infoFlag
	 * @param listFlag
	 * @param engErrorHandler
	 * @return
	 */
	protected String getMessage(boolean infoFlag, boolean listFlag,	EMFErrorHandler engErrorHandler) {
		String returnMessage = new String();
		ArrayList _errorStack = new ArrayList(engErrorHandler.getErrors());
		for (int i = 0; i < _errorStack.size(); i++) {
			String message = getMessageDescription(infoFlag,(EMFAbstractError) _errorStack.get(i));
			if (message != null && message.length() > 0)
				returnMessage += returnMessage.concat((listFlag ? "<li>" : "")+ message + (listFlag ? "</li>" : " - "));
		}
		return returnMessage;
	}

	/**
	 * Returns a list containing the names of the input fields with invalid content
	 * @param infoFlag
	 * @param listFlag
	 * @param engErrorHandler
	 * @return
	 */
	protected List<String> getFields(boolean infoFlag, boolean listFlag,	EMFErrorHandler engErrorHandler) {
		ArrayList<String> returnFields = new ArrayList<String>();
		ArrayList _errorStack = new ArrayList(engErrorHandler.getErrors());
		for (int i = 0; i < _errorStack.size(); i++) {
			if ((Object)(EMFAbstractError) _errorStack.get(i) instanceof EMFValidationError && infoFlag) {
				returnFields.add(((EMFValidationError) _errorStack.get(i)).getFieldName());
			}
		}
		return returnFields;
	}	
	
	public static String getInformationMessage(EMFErrorHandler engErrorHandler) {
		String message = "" ;
		for (Object errorItem : engErrorHandler.getErrors()) {
			if (((EMFAbstractError)errorItem).getSeverity().equalsIgnoreCase(EMFErrorSeverity.INFORMATION))
				message += ((EMFAbstractError)errorItem).getDescription() ;
		}
		return message ;
	}
	
	/**
	 * Gets an error description
	 * @param info
	 * @param ex
	 * @return
	 */
	private String getMessageDescription(boolean info, EMFAbstractError ex) {
		if ((Object)ex instanceof EMFInternalError && !info) {
			return ex.getDescription();
		}
		if ((Object)ex instanceof EMFUserError && info)
			return ex.getDescription();
		return "";
	}

	public boolean is_ajaxRequest() {
		return _ajaxRequest;
	}

	public ResponseContainer get_responseContainer() {
		return _responseContainer;
	}

	public void set_ajaxRequest(boolean request) {
		_ajaxRequest = request;
	}

	public void set_responseContainer(ResponseContainer container) {
		_responseContainer = container;
	}
}