/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.tags;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.List;
import org.apache.commons.fileupload.FileItem;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.decorators.AbstractDecorator;
import it.eng.spago.decorators.DecoratorFactory;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.validation.impl.ValidationImpl;

/**
 * Represents an html input tag
 * @author salerno
 */
public class FieldSupport implements Serializable {
	private static final long serialVersionUID = -1;
	public static final String WIDGET_COMBO = "COMBO";
	public static final String WIDGET_TEXT = "TEXT";
	public static final String WIDGET_TEXT_AREA = "TEXT_AREA";
	public static final String WIDGET_RADIO = "RADIO";
	public static final String WIDGET_CALENDAR = "CALENDAR";
	public static final String WIDGET_PASSWORD = "PASSWORD";
	public static final String WIDGET_CHECK_BOX = "CHECK_BOX";
	public static final String WIDGET_FILE = "FILE";
	public static final String FIELD_LAYER = "LAYER";
	public static final String FIELD_NAME = "NAME";
	public static final String FIELD_LABEL = "LABEL";
	public static final String FIELD_SIZE = "SIZE";
	public static final String FIELD_WIDGET = "WIDGET";
	public static final String FIELD_COLUMN = "COLUMN";
	public static final String FIELD_PARAMETER = "PARAMETER";
	public static final String PARAMETER_OPTION = "OPTION";
	public static final String PARAMETER_TEXTAREA = "TEXT_AREA";
	public static final String PARAMETER_TEXTAREA_ROWS = "ROWS";
	public static final String PARAMETER_TEXTAREA_COLS = "COLS";
	public static final String PARAMETER_IS_READONLY = "IS_READONLY";
	public static final String PARAMETER_IS_MANDATORY = "IS_MANDATORY";
	public static final String PARAMETER_IS_VISIBLE = "IS_VISIBLE";
	public static final String PARAMETER_DEFAULT_VALUE = "DEFAULT_VALUE";
	public static final String PARAMETER_OPTION_VALUE = "VALUE";
	public static final String PARAMETER_OPTION_DISPLAY = "DISPLAY";
	public static final String PARAMETER_QUERY = "SELECT_QUERY";
	public static final String PARAMETER_QUERY_STATEMENT = "statement";
	public static final String PARAMETER_LOOKUP_ONCLICK = "LOOKUPONCLICK";
/*
	public static final String MODE_INSERT = "INSERT" ;
	public static final String MODE_UPDATE = "UPDATE" ;
	public static final String MODE_SELECT = "SELECT" ;
*/
	public static final String TYPE_DATE = "DATE" ;
	public static final String TYPE_TIMESTAMP = "TIMESTAMP" ;
	public static final String TYPE_TEXT = "TEXT" ;

	private String fieldID;
	private String fieldName;
	private String fieldLabel;
	private String fieldValue;
	private String defaultValue;
	private String widgetName;
	private int fieldSize;
	private int maxLength;
	private boolean is_readonly;
	private boolean is_mandatory;
	private boolean is_visible;
	private List listValue;
	private SourceBean comboQuery;
	private SourceBean comboModule;
	private String selectAction;
	private String onKeyPress;
	private String onKeyUp;
	private String onChange;
	private String decoratorName;
	private String lookupAction ;
	private int textAreaRows;
	private int textAreaCols;
	private String calendarFormat;
	private String calendarSeparator;
	private String lookupOnClick;
	private String lookupAliasName;
	private RequestContainer _requestContainer;
	private AbstractDecorator _decorator = null;
	protected DefaultRequestContext _requestContextIface ;

	/**
	 * FieldSupport
	 * @param xmlField
	 * @param row
	 */
	public FieldSupport(SourceBean xmlField, SourceBean row, String mode, RequestContainer requestContainer) {
		_requestContainer = requestContainer ;
		String pageName = (String) requestContainer.getServiceRequest().getAttribute("PAGE");
		xmlField.setDefaultPath("DEFAULT.MODULE.CONFIG.FIELDS.FIELDSET.FIELD");
		fieldID = (String) xmlField.getAttribute("ID");
		fieldName = (String) xmlField.getAttribute("NAME");
		lookupOnClick = (String)xmlField.getAttribute(PARAMETER_LOOKUP_ONCLICK, "");
		fieldValue = "" + xmlField.getAttribute("VALUE") ;
		fieldLabel = ((String) xmlField.getAttribute("LABEL",MessageBundle.getMessage(fieldName))).trim().toUpperCase();
		setFieldSize((String) xmlField.getAttribute("SIZE"));
		setMaxLength((String) xmlField.getAttribute("MAXLENGTH", (String)xmlField.getAttribute("SIZE")));
		setWidgetName((String) xmlField.getAttribute("WIDGET", "TEXT"));
		calendarFormat = (String) xmlField.getAttributeWithDefault("CALFMT");
		calendarSeparator = (String) xmlField.getAttributeWithDefault("CALSEP");
		lookupAction = (String) xmlField.getAttribute("LOOKUPACTION");
		lookupAliasName = (String) xmlField.getAttribute("LOOKUPALIASNAME");
		_requestContextIface = new DefaultRequestContext(_requestContainer, null) ;
		loadParameter((SourceBean) xmlField.getAttribute(mode),mode, fieldName);
		onKeyPress = (String) xmlField.getAttribute("ONKEYPRESS");
		onKeyUp = (String) xmlField.getAttribute("ONKEYUP");
		onChange = (String) xmlField.getAttribute("ONCHANGE");
		decoratorName = (String) xmlField.getAttribute("DECORATOR");
		setFieldValue(row) ;
		if (widgetName.equalsIgnoreCase(WIDGET_CHECK_BOX)) {
			fixValue() ;
		}
	}

	/**
	 * Fixes checkboxes values 
	 */
	private void fixValue() {
		if (getFieldValue() == null || 
			getFieldValue().length() == 0 ||
			getFieldValue().equalsIgnoreCase("0") || 
			getFieldValue().equalsIgnoreCase("F") || 
			getFieldValue().equalsIgnoreCase(" ")) { 
			setFieldValue("N") ;
		} else if (getFieldValue().equalsIgnoreCase("T") || 
		   getFieldValue().equalsIgnoreCase("V") || 
		   getFieldValue().equalsIgnoreCase("1")) { 
			setFieldValue("S") ;
		}
	}
	
	/**
	 * loadParameter
	 * @param parameter
	 */
	private void loadParameter(SourceBean parameter, String mode, String fieldName) {
		String selectModule = "";
		
		if (parameter == null) {
			setIsReadonly((String) null);
			setIsMandatory((String) null);
			setIsVisible((String) null);
		} else {
			parameter.setDefaultPath("DEFAULT.MODULE.CONFIG.FIELDS.FIELDSET.FIELD."+mode);
			
			String defaultValue = null;
			Object defObj = parameter.getAttribute("DEFAULT_VALUE");
			if (defObj!=null) {
				if (defObj instanceof String) {
					defaultValue = (String) parameter.getAttribute("DEFAULT_VALUE");
					setIsReadonly((String) parameter.getAttributeWithDefault("IS_READONLY"));
				} else if (defObj instanceof SourceBean) {
					defObj = ((SourceBean)defObj).getAttribute("PARAMETER");

					if (defObj != null) {
						String parameterType = (String) ((SourceBean)defObj).getAttribute("TYPE");
						if (parameterType != null && parameterType.equalsIgnoreCase("ABSOLUTE")) {
							defaultValue = (String) ((SourceBean)defObj).getAttribute("VALUE");
						} else {
							String parName = (String) ((SourceBean)defObj).getAttribute("NAME");
							String scope = (String) ((SourceBean)defObj).getAttribute("SCOPE");
							defaultValue = (String) ContextScooping.getScopedParameter(_requestContextIface.getRequestContainer(), _requestContextIface.getResponseContainer(), parName, scope, (SourceBean)defObj);
						}
						if (defaultValue == null) {
							defaultValue = "";
						}
					}
					
					setIsReadonly("true");
					
				}
			} else {
				setIsReadonly((String) parameter.getAttributeWithDefault("IS_READONLY"));	
			}
			
			
			setDefaultValue(defaultValue);
			
			
			setIsMandatory((String) parameter.getAttribute("IS_MANDATORY"));
			
			// 20-12-2007 R.S.
			// Gestione degli attributi mandatory tramite validation.
			String mandatory = ValidationImpl.isMandatory(_requestContainer.getServiceRequest(),_requestContextIface, fieldName);
			setIsMandatory(mandatory);
			// Fine modifica R.S.
			
			setIsVisible((String) parameter.getAttributeWithDefault("IS_VISIBLE"));
			// Opzionale: il campo lookup pu essere utilizzato anche a livello dei Parameter
			String lookup = (String)parameter.getAttribute("LOOKUPACTION");
			if (lookup != null && lookup.trim().length() > 0)
				lookupAction = lookup ;
			
			if (widgetName.equals("COMBO")) {
				listValue = parameter.getAttributeAsList("OPTION");
				comboQuery = (SourceBean) parameter.getAttribute("SELECT_QUERY");
				selectAction = (String) parameter.getAttribute("SELECT_ACTION");
				selectModule = (String) parameter.getAttribute("SELECT_MODULE");
				if (selectModule != null) comboModule = (SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", selectModule) ;
			}
			if (widgetName.equalsIgnoreCase(WIDGET_TEXT_AREA)) {
				SourceBean textAreaBean = (SourceBean) parameter.getAttribute("TEXT_AREA");
				if (textAreaBean == null) {
					textAreaCols = 0;
					textAreaRows = 0;
				} else {
					setTextAreaRows(textAreaBean.getAttribute("ROWS"));
					setTextAreaCols(textAreaBean.getAttribute("COLS"));
				}
			}
		}
	}

	public String getFieldLabel() {
		return fieldLabel;
	}

	public String getFieldID() {
		return fieldID;
	}
	
	public String getFieldName() {
		return fieldName;
	}

	public int getFieldSize() {
		return fieldSize;
	}

	public int getMaxLength() {
		return maxLength;
	}

	public String getFieldValue() {
		if (_decorator == null && decoratorName != null && decoratorName.length() > 0 && (widgetName==null || !widgetName.equals("COMBO"))) {
			try {
				_decorator = DecoratorFactory.getInstance().getDecorator(decoratorName, _requestContainer) ;
				fieldValue = _decorator.decorate(fieldValue) ;
			} catch (Exception e) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "FieldSupport::getFieldValue:: decorator ["+decoratorName+"] errore", e);
			}
		}
		return fieldValue;
	}

	public String getFieldDescription() {
		String desc = null;
		if (((_decorator != null) || ( decoratorName != null && decoratorName.length() > 0)) && (widgetName!=null && widgetName.equals("COMBO"))) {
			try {
				_decorator = DecoratorFactory.getInstance().getDecorator(decoratorName, _requestContainer) ;
				desc = _decorator.decorate(fieldValue) ;
			} catch (Exception e) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "FieldSupport::getFieldValue:: decorator ["+decoratorName+"] errore", e);
			}
		}
		return desc;
	}
	
	public String getWidgetName() {
		return widgetName;
	}

	public String getDefaultValue() {
		return defaultValue;
	}

	public void setDefaultValue(String defaultValue) {
		this.defaultValue = (defaultValue == null) ? "" : defaultValue;
	}

	public boolean isMandatory() {
		return is_mandatory;
	}

	public void setIsMandatory(String mandAttr) {
		is_mandatory = (mandAttr == null ? false : mandAttr.trim().equalsIgnoreCase("true"));
	}

	public boolean isReadonly() {
		return is_readonly;
	}

	public void setIsReadonly(String readAttr) {
		is_readonly = (readAttr == null ? false : readAttr.trim().equalsIgnoreCase("true"));
	}

	public boolean isVisible() {
		return is_visible;
	}

	public void setIsVisible(String visAttr) {
		is_visible = visAttr == null ? true : visAttr.trim().equalsIgnoreCase("true");
	}

	public void setFieldLabel(String fieldLabel) {
		this.fieldLabel = fieldLabel;
	}

	public void setFieldName(String fieldName) {
		this.fieldName = fieldName;
	}

	public void setFieldSize(int fieldSize) {
		this.fieldSize = fieldSize;
	}

	public void setFieldSize(String sizAttr) {
		this.fieldSize = (sizAttr != null && sizAttr.trim().length() > 0) ? Integer.parseInt(sizAttr) : 0;
	}

	public void setMaxLength(int maxLength) {
		this.maxLength = maxLength;
	}

	public void setMaxLength(String maxLength) {
		this.maxLength = (maxLength != null && maxLength.trim().length() > 0) ? Integer.parseInt(maxLength) : 0;
	}

	public void setFieldValue(SourceBean row) {
		if (row != null && row.getAttribute(fieldValue) != null) {
			fieldValue = getStringValue(row.getAttribute(fieldValue)).trim() ;
		} else if (row != null && row.getAttribute(fieldName) != null) {
			fieldValue = getStringValue(row.getAttribute(fieldName)).trim() ;
		} else if (fieldValue == null || fieldValue.trim().length() == 0 || fieldValue.equalsIgnoreCase("null")){
			fieldValue = (defaultValue != null) ? defaultValue : "" ;
		}
	}

	/**
	 * COnverte un field generico in String
	 * @param in
	 * @return
	 */
	public static String getStringValue(Object in) {
		if (in == null)
			return (String)in ;
		
		String out = "" ;
		if (in instanceof FileItem) {
			out = ((FileItem)in).getName() ;
		} else if (in instanceof String) {
			out = ((String)in).trim() ;
		}else {
			out = in.toString() ;
		}
		
		if (in instanceof BigDecimal && out.equals("0")) {
			out = "" ;
		}
		
		return out ;
	}
	
	public void setWidgetName(String widgetName) {
		this.widgetName = (widgetName == null
				|| widgetName.trim().length() == 0 ? "TEXT" : widgetName.trim()
				.toUpperCase());
	}

	public void setFieldValue(String fieldValue) {
		this.fieldValue = fieldValue;
		_decorator = null ; // Permette di rieseguire il decorator a fronte di un cambio di valore
	}

	public List getListValue() {
		return listValue;
	}

	public int getTextAreaCols() {
		return textAreaCols;
	}

	public void setTextAreaCols(int textAreaCols) {
		this.textAreaCols = textAreaCols;
	}

	public void setTextAreaCols(Object colAttr) {
		try {
			textAreaCols = Integer.parseInt(colAttr.toString());
		} catch (Exception ex) {
			textAreaCols = 0;
		}
	}

	public int getTextAreaRows() {
		return textAreaRows;
	}

	public SourceBean getComboQuery() {
		return comboQuery;
	}

	public SourceBean getComboModule() {
		return comboModule;
	}
	
	public void setComboModule(SourceBean comboModule) {
		this.comboModule = comboModule;
	}
	
	public void setComboQuery(SourceBean comboQuery) {
		this.comboQuery = comboQuery;
	}

	public void setTextAreaRows(int textAreaRows) {
		this.textAreaRows = textAreaRows;
	}

	public void setTextAreaRows(Object rowAttr) {
		try {
			textAreaRows = Integer.parseInt(rowAttr.toString());
		} catch (Exception ex) {
			textAreaRows = 0;
		}
	}

	public String getSelectAction() {
		return selectAction;
	}
	
	public String getOnKeyPress() {
		if (onKeyPress == null || onKeyPress.trim().length() == 0)
			return " ";
		else 
			return " onKeyPress=\""+onKeyPress + "\" " ;
	}
	public String getOnKeyuP() {
		if (onKeyUp == null || onKeyUp.trim().length() == 0)
			return " ";
		else 
			return " onKeyUp=\""+onKeyUp + "\" " ;
	}

	public String getOnChange() {
		if (onChange == null || onChange.trim().length() == 0)
			return " ";
		else 
			return " onChange=\""+onChange + "\" " ;
	}
	public String getLookupAction() {
		return lookupAction;
	}

	public String getCalendarFormat() {
		return calendarFormat;
	}

	public String getCalendarSeparator() {
		return calendarSeparator;
	}

	/**
	 * @return Restituisce il valore lookupOnClick.
	 */
	public String getLookupOnClick() {
		return lookupOnClick;
	}

	public String getLookupAliasName() {
		return lookupAliasName;
	}
	
	/**
	 * Ritorna il nome dell'Action di lookup prelevandolo dal prefisso impostato in lookupAliasName
	 * es:
	 *    lookupAliasName = "ActLookupAnaOrigineDati.codEntita"
	 *    lookupAliasNamePrefixActionName = "ActLookupAnaOrigineDati"
	 * @return
	 */
	public String getLookupAliasNamePrefixActionName() {
		String actionLookupAliasName = null ;
		if (lookupAliasName != null && lookupAliasName.trim().length() > 0) {
			actionLookupAliasName = lookupAliasName.substring(0, lookupAliasName.indexOf('.'));
		}
		return actionLookupAliasName ;
	}
	
	/**
	 * Ritorna il nome dell'Alias utilizzato nella lookup
	 * es:
	 *    lookupAliasName = "ActLookupAnaOrigineDati.codEntita"
	 *    lookupAliasFieldName = "codEntita"
	 * @return
	 */
	public String getLookupAliasFieldName() {
		String lookupAliasFieldName = null ;
		if (lookupAliasName != null && lookupAliasName.trim().length() > 0) {
			lookupAliasFieldName = lookupAliasName.substring(lookupAliasName.indexOf('.') + 1);
		}
		return lookupAliasFieldName ;
	}

	public String getDecoratorName() {
		return decoratorName;
	}
}