/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.tags;

import it.eng.spago.base.SourceBean;

import java.util.Iterator;

/**
 * Prints a drop-down(right) menu
 */
public class MenuTag extends AbstractStandardTag {
	
	private static final long serialVersionUID = 1L;
	
	protected String application = null;

	protected StringBuffer getHtmlStream() throws Exception {
    	SourceBean menuBean = (SourceBean)configure.getFilteredSourceBeanAttribute("MENUS.APPLICATION",
                "NAME",
                application);

    	return  makeMenu(menuBean) ;
	}

	/**
	 * @param menuBean
	 * @return
	 */
	protected StringBuffer makeMenu(SourceBean menuBean) throws Exception {
		StringBuffer outStream = new StringBuffer();
		Iterator itListMenu = menuBean.getAttributeAsList("MENU").iterator();
		int menuIndex = 0 ;
		outStream.append("<ul id=\"menu\">\n");
		while (itListMenu.hasNext()) {
			menuIndex+=100 ; // Max 100 items for submenu
			outStream.append(makeItem((SourceBean) itListMenu.next(), 0, menuIndex)) ;
		}
		outStream.append("</ul>\n");
		
		return outStream ;
	}

	/**
	 * Prints a menu item ( a row )
	 * @param menuItem
	 * @param level
	 * @return
	 * @throws Exception
	 */
	protected StringBuffer makeItem(SourceBean menuItem, int level, int menuIndex) throws Exception {
		StringBuffer outStream = new StringBuffer() ;
		if (! isUserInRole(menuItem))
			return outStream ;
		
		menuItem.setDefaultPath("DEFAULT."+getRequestType()+".MENUS.APPLICATION.MENU.SUBMENU");
		outStream.append(getHREF((String) menuItem.getAttribute("NAME"),
				getUrl((String) menuItem.getAttribute("URL"), menuIndex),
				                 (String) menuItem.getAttributeWithDefault("ALT"))) ;
		if (level == 0) {
			outStream.append("<ol class=\"active\">" +"\n");
		}
		Iterator itListMenu = menuItem.getAttributeAsList("SUBMENU").iterator() ;
		while (itListMenu.hasNext()) {
			outStream.append(makeItem((SourceBean) itListMenu.next(), level+1, menuIndex++)) ;
		}

		if (level == 0) {
			outStream.append("</ol>\n");
		}
		return outStream ;
	}
	
	/**
	 * 
	 * @param baseUrl
	 * @param levelIndex
	 * @return
	 */
	private String getUrl(String baseUrl, int levelIndex) {
		String url = baseUrl ;
		if (baseUrl != null) {
//			String businessName = "SUSPEND_RESUME_ID=" + levelIndex ; AG: Old lookup parameter
			String businessName = "AF_STPROC=1" ;
			if (baseUrl.indexOf('?') > 0) 
				url += "&" +businessName ;
			else 
				url += "?" +businessName ;
			}
		return url ;
	}

	/**
	 * Authorization check ( stub )
	 * @param menuItem
	 * @return
	 * @throws Exception
	 */
	protected boolean isUserInRole(SourceBean menuItem) throws Exception {
		return true;
	}

	/**
	 * Generate the anchor 
	 * @param menuName
	 * @param menuUrl
	 * @param menuTitle
	 * @return
	 */
	protected String getHREF(String menuName, String menuUrl, String menuTitle) {
		String html = "<li>" ;
		if (menuUrl == null) {
			html += menuName ;
		} else {
			html += "- <a href=\"" + menuUrl + "\""
							+ " title=\"" + menuTitle + "\""
							+ " onclick=\"impostaSpecia(this)\""
							+ " alt=\""+ menuName + "\">"
							+ menuName + "</a>" ;
		}
		html += "</li>\n" ;
		
		return html ;
	}

	/**
	 * setApplication: nome dell'applicazione come chiave per il Menu
	 * @param application
	 */
	public void setApplication(String application) {
		this.application = application;
	}
}