/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.test;

import it.eng.spago.tracing.TracerSingleton;

import java.lang.*;
import java.sql.*;

class DtInfo {
	public static class DataFormat {
		public String format(String val, int len) {
			final String padding = "                                                        ";
			if (val == null)
				val = "";
			if (val.length() < len)
				val += padding.substring(0, len - val.length());
			return val;
		}

		public String format(int val, int len) {
			return format("" + val, len);
		}

		public String format(Integer val, int len) {
			return format(val.toString(), len);
		}
	}

	public static void main(String argv[]) {
		try {
			System.out.println();
			System.out.println("THIS SAMPLE SHOWS HOW TO GET INFO ABOUT DATA TYPES.");
			// connect to the database
			Connection conn = DriverManager.getConnection("jdbc:mysql://sql.useractive.com/USERNAME?user=USERNAME&password=PASSWORD");
			// Get information about the Data type
			infoGet(conn);
			conn.close();
		} catch (SQLException s) {
			TracerSingleton.log("Spago",TracerSingleton.CRITICAL,"SQL Exception",s);
			//System.err.println("SQLException: " + s.getMessage());
			//System.err.println("SQLState:     " + s.getSQLState());
			//System.err.println("VendorError:  " + s.getErrorCode());
		}
	} // main

	static void infoGet(Connection con) throws SQLException {
		DataFormat Data = new DataFormat();
		
		System.out.println();
		System.out.println("----------------------------------------------------------\n" + "USE THE JAVA APIs:\n" + "  Connection.getMetaData()\n" + "  ResultSet.getTypeInfo()\n" + "  ResultSetMetaData.getMetaData()\n"
				+ "TO GET INFO ABOUT DATA TYPES AND\n" + "TO RETRIEVE THE AVAILABLE INFO IN THE RESULT SET.");
		DatabaseMetaData dbMetaData = con.getMetaData();
		// Get a description of all the standard SQL types supported by
		// this database
		ResultSet rs = dbMetaData.getTypeInfo();
		// Retrieve the number, type and properties of the resultset's columns
		ResultSetMetaData rsMetaData = rs.getMetaData();
		// Get the number of columns in the ResultSet
		int colCount = rsMetaData.getColumnCount();
		System.out.println();
		System.out.println("  Number of columns in the ResultSet = " + colCount);
		// Retrieve and display the column's name along with its type
		// and precision in the ResultSet
		System.out.println();
		System.out.println("  A LIST OF ALL COLUMNS IN THE RESULT SET:\n" + "    Column Name         Column Type\n" + "    ------------------- -----------");
		String colName, colType;
		for (int i = 1; i <= colCount; i++) {
        	// 08/07/2008 - RS
        	// Sostituito getColumnName con getColumnLabel
        	// per compatibilit JDBC
			colName = rsMetaData.getColumnLabel(i);
			colType = rsMetaData.getColumnTypeName(i);
			System.out.println("    " + Data.format(colName, 19) + " " + Data.format(colType, 13) + " ");
		}
		System.out.println();
		System.out.println("  HERE ARE SOME OF THE COLUMNS' INFO IN THE TABLE ABOVE:\n" + "           TYPE_NAME          DATA_  COLUMN    NULL-   CASE_\n" + "                              TYPE   _SIZE     ABLE  SENSITIVE\n"
				+ "                              (int)                          \n" + "    ------------------------- ----- ---------- ----- ---------");
		String typeName;
		int dataType;
		Integer columnSize;
		boolean nullable;
		boolean caseSensitive;
		// Retrieve and display the columns' information in the table
		while (rs.next()) {
			typeName = rs.getString(1);
			dataType = rs.getInt(2);
			if (rs.getInt(7) == 1) {
				nullable = true;
			} else {
				nullable = false;
			}
			if (rs.getInt(8) == 1) {
				caseSensitive = true;
			} else {
				caseSensitive = false;
			}
			if (rs.getString(3) != null) {
				columnSize = Integer.valueOf(rs.getString(3));
				System.out.println("    " + Data.format(typeName, 25) + " " + Data.format(dataType, 5) + " " + Data.format(columnSize, 10) + " " + Data.format(String.valueOf(nullable), 5) + " " + Data.format(String.valueOf(caseSensitive), 10));
			} else
				// for the distinct data type, column size does not apply
			{
				System.out.println("    " + Data.format(typeName, 25) + " " + Data.format(dataType, 5) + "        n/a" + " " + Data.format(String.valueOf(nullable), 5) + " " + Data.format(String.valueOf(caseSensitive), 10));
			}
		}
		// close the result set
		rs.close();
	}
	
	
} // DtInfo
