/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.testing.validators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;

import java.util.ArrayList;
import java.util.List;

/**
 * La classe <code>AbstractListValidator</code> contiene i metodi comuni
 * utilizzati dai validatori di PAGE di tipo List
 * <code>AbstractListValidator</code>
 * <p>
 * 
 * @version 1.0, 01/01/2008
 * @author Gabriele Rigamonti
 * @see AbstractListValidator
 */
public abstract class AbstractListValidator {
	protected SourceBean _serviceResponse;

	protected SourceBean _testConfiguration;

	/**
	 * Valuta se  stata eseguita una ACTION od una PAGE e prepara di
	 * conseguenza il _serviceResponse
	 * 
	 * @param inputBean
	 */
	protected void setServiceResponse(SourceBean inputBean) {
		SourceBean parBean = (SourceBean) _testConfiguration.getFilteredSourceBeanAttribute("PARAMETERS.PARAMETER", "NAME", "PAGE");
		_serviceResponse = (SourceBean) inputBean.getAttribute("SERVICE_RESPONSE");
		if (parBean != null) {
			// E' stata eseguita una pagina; verifico se nei parametri ho
			// specificato il
			// nome del modulo
			parBean = (SourceBean) _testConfiguration.getFilteredSourceBeanAttribute("PARAMETERS.PARAMETER", "NAME", "MODULE");
			if (parBean != null) {
				String moduleName = (String) parBean.getAttribute("VALUE");
				_serviceResponse = (SourceBean) _serviceResponse.getAttribute(moduleName);
			} else {
				// Non  stato specificato il nome del modulo
				// prendo il primo nella risposta
				_serviceResponse = (SourceBean) ((SourceBeanAttribute) _serviceResponse.getContainedAttributes().get(0)).getValue();
			}
		}
	}

	/**
	 * Estrae la lista di righe lette
	 * 
	 * @return
	 */
	protected List<SourceBean> getRowList() {
		if (_serviceResponse != null)
			return (List<SourceBean>) _serviceResponse.getAttributeAsList("PAGED_LIST.ROWS.ROW");
		else
			return new ArrayList<SourceBean>();
	}

	/**
	 * Ritorna il nome della colonna prefissata: es FILTER_codLista oppure
	 * ORDER_codLista oppure SELECT_codLista , ...
	 * 
	 * @param parameterName
	 * @param parameterPrefix
	 * @return
	 */
	protected String getColumnName(String parameterName, String parameterPrefix) {
		String out = parameterName;
		int l = parameterPrefix.length();
		if (parameterName != null && parameterName.substring(0, l).equals(parameterPrefix)) {
			out = parameterName.substring(l, parameterName.length());
		}
		return out;
	}

	/**
	 * Esegue la validazione specifica per il tipo di parametro passato
	 * 
	 * @param filterParameter
	 * @return
	 */
	protected abstract boolean validateParameter(SourceBean parameter);

	/**
	 * Estrae tutti parameter con il nome passato e per ognuno esegue lo specifico di validazione test
	 * @param testParameter parameter's search name
	 * @return true for SUCCESSEFULL otherwise false
	 */
	protected boolean handleListParameter(String testParameter) {
		boolean test = true;
		List<SourceBean> parameterList = _testConfiguration.getAttributeAsList("PARAMETERS.PARAMETER");
		String parameterName;
		for (SourceBean parameter : parameterList) {
			if (parameter != null) {
				parameterName = (String) parameter.getAttribute("NAME");
				if (parameterName.indexOf(testParameter) == 0) {
					if (! validateParameter(parameter)) {
						test = false;
						break;
					}
				}
			}
		}

		return test;
	}
}