/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/

package it.eng.spago.testing.validators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;

import java.io.Serializable;
import java.util.List;

/**
 * La classe <code>ListFilterValidator</code> esegue le validazioni dei FILTER_ per 
 * le PAGE di tipo List
 * <code>ListFilterValidator</code>
 * <p>
 * 
 * @version 1.0, 01/01/2008
 * @author Gabriele Rigamonti
 * @see AbstractListValidator
 */
public class ListFilterValidator extends AbstractListValidator implements Serializable, TestValidatorIFace {
	private static final long serialVersionUID = 1L;

	private final static int CRITERIA_LIKE = 0;
	private final static int CRITERIA_NOT_LIKE = 1;
	private final static int CRITERIA_GT = 2; // >
	private final static int CRITERIA_GE = 3; // >=
	private final static int CRITERIA_EQ = 4; // ==
	private final static int CRITERIA_LE = 5; // <=
	private final static int CRITERIA_LT = 6; // <
	private final static int CRITERIA_NE = 7;
	private final static int CRITERIA_START_WITH = 8;
	private final static int CRITERIA_END_WITH = 9;
	private final static int CRITERIA_CONTAINS_IN_MIDDLE = 10;

	/**
	 * Verifica la configurazione del Test
	 */
	public boolean validateParameters(SourceBean config) {
		// TODO Stub di metodo generato automaticamente
		return true;
	}

	public boolean validator(SourceBean validatorConfiguration, SourceBean inputBean, SourceBean testConfiguration) {
		boolean ret = true;
		_testConfiguration = testConfiguration;
		setServiceResponse(inputBean);
		ret = handleListParameter("FILTER_");
		return ret;
	}

	/**
	 * Verifica che il filtro passato sia stato effettivamente eseguito es:
	 * <PARAMETER NAME="FILTER_STATO_PROCESSO" TYPE="ABSOLUTE" VALUE="RRO" />
	 * deve verificare che la colonna STATO_PROCESSO contenga sempre il pattern
	 * "RRO"
	 * 
	 * @param filterParameter
	 * @return
	 */
	protected boolean validateParameter(SourceBean filterParameter) {
		String filterValue = filterValue(filterParameter);
		String columnName = getColumnName((String) filterParameter.getAttribute("NAME"), "FILTER_");
		String searchCriteria = getSearchCriteriaForColumnName(columnName);
		List rowList = getRowList();
		String rowValue;
		boolean result = true;
		for (Object rowBean : rowList) {
			if (rowBean != null) {
				rowValue = (String) ((SourceBean) rowBean).getAttribute(columnName);
				boolean res = compare(filterValue, rowValue.toUpperCase(), searchCriteria);
				if (!res) {
					TracerSingleton.log("Spago",TracerSingleton.WARNING,"ListValidators:validateFilter filterName:[" + "FILTER_" + columnName + "] column content:[" + rowValue + "]");
					//System.err.println("ListValidators:validateFilter filterName:[" + "FILTER_" + columnName + "] "
					//		+ "column content:[" + rowValue + "]");
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * Ritorna il valore del filtro
	 * 
	 * @param filterParameter
	 * @return
	 */
	private String filterValue(SourceBean filterParameter) {
		return (String) filterParameter.getAttribute("VALUE", "");
	}

	/**
	 * Ritorna i criteri di ricerca lt, le, eq, ge, gt, ne, like, startwith,
	 * notlike
	 * 
	 * @param filterParameter
	 * @return
	 */
	private String getSearchCriteriaForColumnName(String columnName) {
		String searchCriteria = "like";
		if (columnName != null) {

			SourceBean sb = (SourceBean) _testConfiguration.getFilteredSourceBeanAttribute("PARAMETERS.PARAMETER", "name",
					"SELECT_" + columnName);
			if (sb != null) {
				searchCriteria = (String) sb.getAttribute("VALUE");
				if (searchCriteria == null || searchCriteria.trim().length() == 0 || searchCriteria.equalsIgnoreCase("none"))
					searchCriteria = "like";
			}
		}
		return searchCriteria;
	}

	private boolean compare(String parameterString, String responseString, String operator) {
		boolean result = false;
		int comparisionOperator = trasformOperator(operator);

		switch (comparisionOperator) {
		case CRITERIA_LIKE:
			if (responseString.indexOf(parameterString) != -1)
				return true;
			break;

		case CRITERIA_NOT_LIKE:
			if (responseString.indexOf(parameterString) == -1)
				return true;
			break;

		case CRITERIA_EQ:
			if (parameterString.compareTo(responseString) == 0)
				return true;
			break;

		case CRITERIA_NE:
			if (!(parameterString.compareTo(responseString) == 0))
				return true;
			break;

		case CRITERIA_GT:
			if (parameterString.compareTo(responseString) < 0)
				return true;
			break;

		case CRITERIA_LT:
			if (parameterString.compareTo(responseString) > 0)
				return true;
			break;

		case CRITERIA_GE:
			if (parameterString.compareTo(responseString) < 0 | parameterString.compareTo(responseString) == 0)
				return true;
			break;

		case CRITERIA_LE:
			if (parameterString.compareTo(responseString) > 0 | parameterString.compareTo(responseString) == 0)
				return true;
			break;

		case CRITERIA_START_WITH:
			if (parameterString.substring(0, responseString.length()).compareTo(responseString) == 0)
				return true;
			break;

		case CRITERIA_END_WITH:
			if (parameterString.substring((parameterString.length() - responseString.length()), parameterString.length())
					.compareTo(responseString) == 0)
				return true;

		case CRITERIA_CONTAINS_IN_MIDDLE:
			// contiene ma non deve iniziare/terminare con il parametro passato
			if ((responseString.indexOf(parameterString) != -1)
					& !(parameterString.substring(0, responseString.length()).compareTo(responseString) == 0)
					& !(parameterString.substring((parameterString.length() - responseString.length()), parameterString.length())
							.compareTo(responseString) == 0))
				return true;
			break;

		default:
			if (parameterString.indexOf(responseString) != -1)
				return true;
		}

		return result;
	}

	private int trasformOperator(String operator) {

		int result = CRITERIA_LIKE;
		if (operator != null) {
			if (operator.equals("lt"))
				result = CRITERIA_EQ;
			if (operator.equals("gt"))
				result = CRITERIA_GT;
			if (operator.equals("lt"))
				result = CRITERIA_LT;
			if (operator.equals("ge"))
				result = CRITERIA_GE;
			if (operator.equals("le"))
				result = CRITERIA_LE;
			if (operator.equals("ne"))
				result = CRITERIA_NE;
			if (operator.equals("like"))
				result = CRITERIA_LIKE;
			if (operator.equals("nlike"))
				result = CRITERIA_NOT_LIKE;
			if (operator.equals("startwith"))
				result = CRITERIA_START_WITH;
			if (operator.equals("end"))
				result = CRITERIA_END_WITH;
			if (operator.equals("mid"))
				result = CRITERIA_CONTAINS_IN_MIDDLE;
		}
		return result;
	}
}