/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.testing.validators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.SourceBeanComparator;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;

/**
 * La classe <code>SortValidators</code> gestisce le validazioni di tipo SORT_
 * per le PAGE di tipo List Utilizza il file di configurazione
 * <code>SortValidators</code>
 * <p>
 * 
 * @version 1.0, 01/01/2008
 * @author Gabriele Rigamonti
 * @see SortValidators
 */
public class ListSortValidator extends AbstractListValidator implements Serializable, TestValidatorIFace {
	private static final long serialVersionUID = 1L;

	/**
	 * Verifica la configurazione del Test
	 */
	public boolean validateParameters(SourceBean config) {
		// TODO Stub di metodo generato automaticamente
		return true;
	}

	public boolean validator(SourceBean validatorConfiguration, SourceBean inputBean, SourceBean testConfiguration) {
		boolean ret = true;
		_testConfiguration = testConfiguration;
		setServiceResponse(inputBean);

		ret = handleListParameter("ORDER_");
		return ret;
	}

	/**
	 * Estrae i criteri di ordinamento e richiama la funzione per la verifica dei dati 
	 * @return
	 */
	protected boolean validateParameter(SourceBean parameter) {
		String orderColumnName = getColumnName((String)parameter.getAttribute("NAME"), "ORDER_") ;
		String sortMethod = (String)parameter.getAttribute("VALUE", "asc") ;
		return validateSort(orderColumnName, sortMethod);
	}

	/**
	 * Verifica il corretto ordinamento per la colonna
	 * @param orderColumnName
	 * @param sortMethod
	 * @return
	 */
	private boolean validateSort(String orderColumnName, String sortMethod) {
		List rowList = getRowList();
		SourceBeanComparator comparator = new SourceBeanComparator(orderColumnName, sortMethod);
		// aggiungo i progressivi sequenziali
		try {
			addSequenceNumber(rowList);
			Collections.sort(rowList, comparator);
		} catch (Exception ex) {
			TracerSingleton.log("SPAGO",TracerSingleton.CRITICAL,"ListSortValidator::validateSort::Error", ex);
			//ex.printStackTrace() ;
			return false ;
		}
		return validateOrderSequence(rowList);

	}

	/**
	 * Controlla se dopo l'ordinamento la sequenza iniziale ' preservata
	 * 
	 * @param rowList
	 * @return
	 */
	private boolean validateOrderSequence(List rowList) {

		Integer counter = null;

		boolean testResult = true;
		int current = 0;
		int previous = 0;
		boolean firstTime = true;

		for (Object rowBean : rowList) {
			if (rowBean != null) {
				previous = current;
				counter = (Integer) ((SourceBean) rowBean).getAttribute("PROGR_4_TEST");
				current = counter.intValue();

				if (firstTime) {
					firstTime = false;
					previous = current;
				}

				if ((current - previous) != 1 && (current != previous)) {
					testResult = false; // sequenza errata, test fallito
					break;
				}
			}
		}

		return testResult;
	}

	/**
	 * Aggiunge un attributo fisso con un valore sequenziale
	 * 
	 * @param rowList
	 * @throws Exception
	 */
	private void addSequenceNumber(List rowList) throws Exception {
		SourceBean rowSourceBean = null;
		int count = 1;
		for (Object rowBean : rowList) {
			if (rowBean != null) {
				rowSourceBean = (SourceBean) rowBean;
				rowSourceBean.setAttribute("PROGR_4_TEST", count++);
			}
		}
	}
}