/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.tracing;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.logging.ConsoleHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.MemoryHandler;
import java.util.logging.SimpleFormatter;
import java.util.logging.XMLFormatter;
import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;
import it.eng.spago.base.SourceBean;
import it.eng.spago.init.InitializerIFace;

/**
 * Interfaccia verso lo standard Java Logger
 * 
 * @author Andrea Grassi
 *
 */
public class JavaLogger implements IFaceLogger, InitializerIFace {
    private String _name = null;
    private SourceBean _config = null;
    private Logger _logger = null;
    private boolean _isEnabled = false;
    private int _minLogSeverity = 0;
    private boolean _debug = false;

    public JavaLogger() {
        super();
        _name = null;
        _config = null;
        _logger = null;
        _isEnabled = false;
        _minLogSeverity = 0;
        _debug = true;
    } // public JavaLogger()

    public String getName() {
		return _name ;
	}

	public void setName(String loggelName) {
		_name = loggelName ;
	}

    public synchronized void init(SourceBean config) {
        _config = config;
    } //public synchronized void init(SourceBean config)

    public synchronized SourceBean getConfig() {
        return _config;
    } //public synchronized SourceBean getConfig()

	public synchronized void enable() {
        if (_isEnabled)
            return;
        if (isDisabledServer()) // Appartiene alla lista dei server in cui il logger  disabilitati
        	return ;
        //non deve essere tenuto in considerazione il valore definito nel file .properties
        //bens il file tracing.xml
        _logger = Logger.getLogger(_name);
        Handler[] handlers = _logger.getHandlers() ;
        for (int i=0 ; handlers != null && i < handlers.length ; i++) {
        	_logger.removeHandler(handlers[i]) ;
        }
        
        _logger.addHandler(getHandler((SourceBean)getConfig())) ;
        _logger.setUseParentHandlers(false);
        // _logger.

        String minLogSeverity = (String) ((SourceBean) getConfig()).getAttribute( "TRACE_MIN_LOG_SEVERITY");
        _minLogSeverity = 0;
        try {
            _minLogSeverity = Integer.parseInt(minLogSeverity);
            if (_minLogSeverity < 0)
                _minLogSeverity = 0;
        } // try
        catch (Exception ex) {
            System.out.println(
                "JavaLogger::enable: parametro TRACE_MIN_LOG_SEVERITY non valido");
            ex.printStackTrace();
        } // catch (Exception ex)
        _logger.setLevel(getLevel(_minLogSeverity));
        String debug = (String) ((SourceBean) getConfig()).getAttribute("DEBUG");
        _debug =
            ((debug != null) && (debug.equalsIgnoreCase("TRUE")))
                ? true
                : false;
        if (_debug)
            _logger.setLevel(Level.ALL);
        _isEnabled = true;
	}

	private Handler getHandler(SourceBean config) {
		ConsoleHandler handler = new ConsoleHandler();
		ConsoleFormatter formatter = new ConsoleFormatter() ;
	
		handler.setFormatter(formatter) ;
		return handler ;
/*		
		XMLFormatter defaultFormatter = new XMLFormatter() ;
		MemoryHandler defaultHandler = new MemoryHandler(handler, 2, Level.ALL) ;
		defaultHandler.setFormatter(defaultFormatter) ;
		
		return defaultHandler ;
*/
	}
	
	public class ConsoleFormatter extends SimpleFormatter {

		public synchronized String format(LogRecord record) {
			Date date =new Date(record.getMillis()) ;
			SimpleDateFormat formatter = new SimpleDateFormat("hh:mm:ss"); 
			String out= formatter.format(date) + "("+ record.getLevel().getName() + ") " + record.getMessage()  ;
			if (out.indexOf("\n") == -1)
				out += "\n" ;
			return out ;
		}
		public String getHead(Handler h) {
			return "";
		}
		public String getTail(Handler h) {
			return "";
		}
		
	}
	
    /**
     * Disabilita il tracciamento
     */
	public synchronized void disable() {
        if (!_isEnabled)
            return;
        _logger = null;
        _isEnabled = false;
	}

    /**
     * Verifica se il tracer &egrave; abilitato a tracciare.
     * @return <code>boolean</code>
     */
	public boolean isEnabled() {
		return _isEnabled ;
	}

    /**
     * Questo metodo verifica se il parametro severity e ritorna:
     * <em>false</em> se il severity non  <em>DEBUG</em> e inferiore della severity minima.
     * <em>true</em>se il severity   <em>DEBUG</em> o maggiore della severity minima.
     * @param severity codice della severity
     * @return il risultato della comparazione.
     */
    public boolean testToTrace(int severity) {
        //se viene richiesto il tracciamento di un log con severity = DEBUG
        if (!_isEnabled)
            return false;
        if (severity == TracerSingleton.DEBUG)
            return _debug;
        else if (severity < _minLogSeverity)
            return false;
        else
            return true;
    } // public synchronized boolean testToTrace(int severity)

    public synchronized void trace(
            String application,
            int severity,
            String message) {
            if (!testToTrace(severity))
                return;
            _logger.log(getLevel(severity), message);
    } // public synchronized void trace(String application, int severity, String message)

    private Level getLevel(int severity) {
        switch (severity) {
            case TracerSingleton.INFORMATION :
                return Level.INFO ;
            case TracerSingleton.WARNING :
                return Level.WARNING ;
            case TracerSingleton.MINOR :
                return Level.FINER ;
            case TracerSingleton.MAJOR :
                return Level.FINEST ;
            case TracerSingleton.CRITICAL :
                return Level.SEVERE ;
            case TracerSingleton.DEBUG :
                return Level.ALL ;
            default :
                return Level.ALL ;
        } // switch (severity)
    } // private Level getLevel(int severity)

    private boolean isDisabledServer() {
		try {
			String disableServerList = (String)getConfig().getAttribute("disable_server_list") ;
			String hostAddress =java.net.InetAddress.getLocalHost().getHostAddress() ;
	        StringTokenizer stk = new StringTokenizer(disableServerList, ";");
	        while (stk.hasMoreTokens())  {
	        	if (hostAddress.equals(stk.nextToken())) {
	        		_isEnabled = false ;
	        		break ;
	        	}
	        }
			
		}catch(Exception ex){
			System.err.println("JavaLogger:isDisabledServer(): Errore in recupero indirizzo host.") ;
		}
		return _isEnabled ;
	}
}