/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/package it.eng.spago.util;

import it.eng.spago.base.Constants;
import it.eng.spago.base.NullValue;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.base.XMLSerializer;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tags.FieldSupport;
import it.eng.spago.tracing.TracerSingleton;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.xerces.dom.DocumentImpl;
import org.apache.xpath.NodeSet;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * This utility class containes some methods for handling Configuration (ConfigSingleton)
 * @author grassi
 *
 */
public class ConfigRowHandler implements Serializable {
	private static final long serialVersionUID = 1L;

    /**
     * Returns row from configuration
     * @param statement
     * @return
     */
    public Object getConfigRow(SourceBean statement, final RequestContainer requestContainer, final ResponseContainer responseContainer) {

    	Object dataRow = null ;
    	SourceBean genericBean;
		try {
			
			if (statement.getAttribute("xpath")!=null) {
				
				String xquery = (String)statement.getAttribute("xpath");
				//String xparameters = (String)statement.getAttribute("parameters");
				
				List<SourceBean> xparameters = statement.getAttributeAsList("PARAMETER");
		    
				int count = 0;
				for (int i = 0; i < xquery.length(); i++) {
					if (xquery.charAt(i) == '?') {
						count++;
					}
				}
				
				if (count!=xparameters.size()) {
					responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, "Numero di parametri errato"));
				}
				
				String elab = xquery;
				
				SourceBean bean = ConfigSingleton.getInstance();
				
	
				bean.toXML();
				
				XPathFactory factory = XPathFactory.newInstance();
				XPath xpath = factory.newXPath();
				
				Iterator<SourceBean> parameters = xparameters.iterator();
				
				StringTokenizer tokenizer = new StringTokenizer(elab,"?");
				Iterator<SourceBean> paramsIterator = xparameters.iterator();
				StringBuffer sbuff = new StringBuffer();
				
				while (tokenizer.hasMoreTokens() && paramsIterator.hasNext()) {
					String element = tokenizer.nextToken();
					sbuff.append(element);
					
			    	SourceBean parameter = paramsIterator.next();
			    	String parName = (String)parameter.getAttribute("NAME");
			    	String parValue = (String)getParameter(parameter, requestContainer, responseContainer);
			    	sbuff.append(parValue);
			    	
				}
				
				if (tokenizer.hasMoreTokens()) {
					sbuff.append(tokenizer.nextToken());
				}
				
				try {
					
					
					XPathExpression expr = xpath.compile(sbuff.toString());
					
					Document doc = bean.toDocument();
					Node node = (Node) expr.evaluate(doc,XPathConstants.NODE);
					//doc.removeChild(node);
					
					dataRow = sourceBeanFromNode(node);
					
				} catch (XPathExpressionException e) {
					TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::getConfigRow::Error",e);
					//e.printStackTrace();
				}

				
				
				
//				while (elab.indexOf("[?]")>=0) {
//					
//			    	SourceBean parameter = parameters.next();
//			    	String parName = (String)parameter.getAttribute("NAME");
//			    	String parValue = (String)getParameter(parameter, requestContainer, responseContainer);
//			    	
//			    	if (elab.charAt(0)=='.') {
//			    		elab = elab.substring(1);
//			    	}
//			    	bean = (SourceBean)bean.getFilteredSourceBeanAttribute(elab.substring(0,elab.indexOf("[?]")), parName, parValue) ;
//					elab = elab.substring(elab.indexOf("[?]")+3);
//				}
				//SourceBean sb = new SourceBean();	
				

				//dataRow.
					
			} else {
			
				genericBean = getBeanForSpecialAttributes(statement, requestContainer, responseContainer);
		    	String key        = (String)statement.getAttribute("key") ;
		    	SourceBean filterParameterName = (SourceBean)statement.getAttribute("FILTER_PARAMETER_NAME") ;
		    	SourceBean filterParameterValue = (SourceBean)statement.getAttribute("FILTER_PARAMETER_VALUE") ;
		    	String paramName = null, paramValue = null ;
		    	if (filterParameterName != null && filterParameterValue != null) {
		    		paramName  = (String)getParameter(filterParameterName, requestContainer, responseContainer) ;
		    		paramValue = (String)getParameter(filterParameterValue, requestContainer, responseContainer) ;
		    	}
				if ((paramName != null && paramName.trim().length() > 0) &&
					(paramValue != null && paramValue.trim().length() > 0)) {
					dataRow = genericBean.getFilteredSourceBeanAttribute(key, paramName, paramValue) ;
				}else
					dataRow = genericBean.getAttribute(key) ;
	
				setShortMessage(statement, requestContainer, responseContainer) ;
				
				
			}		
			
			String defaultKey = (String)statement.getAttribute("default") ;
			SourceBean defaultBean = null ;

			if (defaultKey != null && defaultKey.trim().length() > 0) {
				defaultBean = (SourceBean)ConfigSingleton.getInstance().getAttribute(defaultKey) ;
			}
				if (dataRow != null && dataRow instanceof SourceBean) {
					if (defaultBean != null) {
						((SourceBean)dataRow).mergeWidth(defaultBean) ;
					}
					dataRow = ((SourceBean)dataRow).cloneObject() ;
					((SourceBean)dataRow).setName("ROW") ;
				}
				if (dataRow != null && dataRow instanceof List) {
					for (Object row : ((List)dataRow)) {
						if (row != null && row instanceof SourceBean) {
							row = ((SourceBean)row).cloneObject() ;
							((SourceBean)row).setName("ROW") ;
							if (defaultBean != null) {
								((SourceBean)row).mergeWidth(defaultBean) ;
							}
							((SourceBean)row).setName("ROW") ;
						}
					}
				}
		
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::getConfigRow::Error",e);
			//e.printStackTrace();
		}
		return dataRow ;
    }
    
    private void setShortMessage(final SourceBean statement, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
    	String message = "" ;
    	String pageKey = (String)statement.getAttribute("page") ; 
    	String moduleKey = (String)statement.getAttribute("module") ; 
    	String fieldsetKey = (String)statement.getAttribute("filedset") ; 
    	String key        = (String)statement.getAttribute("key") ;
    	SourceBean filterParameterValue = (SourceBean)statement.getAttribute("FILTER_PARAMETER_VALUE") ;

    	if (pageKey != null && pageKey.trim().length() >0) {
			String pageName = (String)getParameter((SourceBean)statement.getFilteredSourceBeanAttribute("PARAMETER", "NAME", pageKey), requestContainer, responseContainer) ;
    		message += pageName.trim() + " ";
    	}
    	if (moduleKey != null && moduleKey.trim().length() >0) {
			String moduleName = (String)getParameter((SourceBean)statement.getFilteredSourceBeanAttribute("PARAMETER", "NAME", moduleKey), requestContainer, responseContainer) ;
    		message += moduleName.trim() + " ";
    	}
    	if (fieldsetKey != null && fieldsetKey.trim().length() >0) {
			String fieldsetName = (String)getParameter((SourceBean)statement.getFilteredSourceBeanAttribute("PARAMETER", "NAME", fieldsetKey), requestContainer, responseContainer) ;
    		message += fieldsetName.trim() + " ";
    	}
    	if (key != null && key.trim().length() >0)
    		message += key.trim() + " ";
    	if (filterParameterValue != null) {
    		message += "("+getParameter(filterParameterValue, requestContainer, responseContainer)+")" ;
    	}
    	SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer();
    	permanentContainer.delAttribute("SHORT_MESS") ;
    	permanentContainer.setAttribute("SHORT_MESS", message);
    }
    
    /**
     * Delete a row from configuration
     * @param statement
     * @return
     */
    public boolean deleteConfigRow(SourceBean statement, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
    	boolean ret =true ;
		try {
			SourceBean genericBean = getBeanForSpecialAttributes(statement, requestContainer, responseContainer) ;
			
			String key        = (String)statement.getAttribute("key") ;
			String paramName  = (String)getParameter((SourceBean)statement.getAttribute("FILTER_PARAMETER_NAME"), requestContainer, responseContainer) ;
			String paramValue = (String)getParameter((SourceBean)statement.getAttribute("FILTER_PARAMETER_VALUE"), requestContainer, responseContainer) ;
			
		
			genericBean.delAttribute(key, paramName, paramValue) ;
			
			if(key.equalsIgnoreCase("MODULES.MODULE")) {
				key="VALIDATIONS.SERVICE";
				
				genericBean.delAttribute(key, paramName, paramValue) ;
			}
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::deleteConfigRow::Error",e);
			//e.printStackTrace();
			ret = false ;
		}
		return ret ;
    }
    
    /**
     * Insert a row into configuration
     * @param statement
     * @param requestContainer
     * @param responseContainer
     * @return
     */
    public boolean insertConfigRow(SourceBean statement, SourceBean newBean, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
    	boolean ret =true ;
		try {
			// Pulisce il bean dagli attributi nulli
			for(SourceBeanAttribute attributo : (List<SourceBeanAttribute>)newBean.getContainedAttributes()) {
				String key = attributo.getKey();
				if(attributo.getValue()!=null && (attributo.getValue().toString().length()==0)) {
					newBean.delAttribute(key);
				}
			}
			SourceBean genericBean = getBeanForSpecialAttributes(statement, requestContainer, responseContainer) ;
	    	
				
			String key = (String)statement.getAttribute("key") ;
	    	SourceBean filterParameterName = (SourceBean)statement.getAttribute("FILTER_PARAMETER_NAME") ;
	    	SourceBean filterParameterValue = (SourceBean)statement.getAttribute("FILTER_PARAMETER_VALUE") ;
	    	if (filterParameterName != null && filterParameterValue != null) {
	    		String paramName  = (String)getParameter(filterParameterName, requestContainer, responseContainer) ;
	    		String paramValue = (String)getParameter(filterParameterValue, requestContainer, responseContainer) ;
	    		genericBean = (SourceBean)genericBean.getFilteredSourceBeanAttribute(key, paramName, paramValue) ;
	    	} else if (key != null && key.trim().length() > 0){
	    		
	    	    if (genericBean.getAttribute(key)==null) {
	    			SourceBean bean = new SourceBean(key);
	    			genericBean.setAttribute(bean);
	    		}
	    	    genericBean = (SourceBean)genericBean.getAttribute(key) ;	    

	    	}
	    	
//	    	if (key.equalsIgnoreCase("MODULES")) {
//	    		String moduleName = (String) newBean.getAttribute("NAME");
//	    		/*
//	    		if (moduleName.indexOf("Dettaglio") >= 0)
//	    			responseContainer.getServiceResponse().setAttribute("IS_DETAIL_MODULE","TRUE");
//	    		*/
//	    	}

			
			genericBean.setAttribute(newBean) ;
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::insertConfigRow::Error",e);
			//e.printStackTrace();
			ret = false ;
		}
		return ret ;
    }

    /**
     * Update a row into configuration
     * @param statement
     * @param requestContainer
     * @param responseContainer
     * @return
     */
    public boolean updateConfigRow(SourceBean statement, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
    	boolean ret =true ;
		try {
			SourceBean genericBean = getBeanForSpecialAttributes(statement, requestContainer, responseContainer) ;
			
			if (statement.getAttribute("node")==null) {
		    	String key        = (String)statement.getAttribute("key") ;
		    	String paramName  = (String)getParameter((SourceBean)statement.getAttribute("FILTER_PARAMETER_NAME"), requestContainer, responseContainer) ;
				String paramValue = (String)getParameter((SourceBean)statement.getAttribute("FILTER_PARAMETER_VALUE"), requestContainer, responseContainer) ;
				genericBean = (SourceBean)genericBean.getFilteredSourceBeanAttribute(key,paramName,paramValue);
			}
			
			List<SourceBean> parameters = (List<SourceBean>)statement.getAttributeAsList("PARAMETER");
			for(SourceBean parameter : parameters) {
		        String parameterName = (String) parameter.getAttribute("NAME");
		        String parameterScope = (String) parameter.getAttribute("SCOPE");
		        Object parameterValue = ContextScooping.getScopedParameter(requestContainer,responseContainer, parameterName,parameterScope, parameter);
		        if(parameterValue!=null && parameterValue.toString().length()>0) {
		        	genericBean.updAttribute(parameterName,parameterValue);
		        }
			}
		} catch (SourceBeanException e) {
			TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::updateConfigRow::Error",e);
			//e.printStackTrace();
			ret = false ;
		}
		return ret ;
    }

    /**
     * Query Special Attribute management method.
     * @param statement
     * @param requestContainer
     * @param responseContainer
     * @return
     */
    private SourceBean getBeanForSpecialAttributes(SourceBean statement, final RequestContainer requestContainer, final ResponseContainer responseContainer) throws SourceBeanException {
		SourceBean genericBean = ConfigSingleton.getInstance() ;

		if (statement.getAttribute("node")!=null) {
			
			
			//genericBean =  (SourceBean)getConfigRow(statement, requestContainer, responseContainer);
			
			String query = (String)statement.getAttribute("node");
			//String xparameters = (String)statement.getAttribute("parameters");
			
			List<SourceBean> xparameters = statement.getAttributeAsList("SELECT_PARAMETER");
    
			int count = 0;
			for (int i = 0; i < query.length(); i++) {
				if (query.charAt(i) == '?') {
					count++;
				}
			}
			
			if (count!=xparameters.size()) {
				responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, "Numero di parametri errato"));
			}
			
			String elab = query;
			
			SourceBean bean = ConfigSingleton.getInstance();
			
			Iterator<SourceBean> parameters = xparameters.iterator();
			
			while (elab.indexOf("[?]")>=0) {
				
		    	SourceBean parameter = parameters.next();
		    	String parName = (String)parameter.getAttribute("NAME");
		    	String parValue = (String)getParameter(parameter, requestContainer, responseContainer);
		    	bean = (SourceBean)bean.getFilteredSourceBeanAttribute(elab.substring(0,elab.indexOf("[?]")), parName, parValue) ;
				elab = elab.substring(elab.indexOf("[?]")+3);
			}
			
			genericBean = bean;	
			
				
		} else {
		
			String pageKey        = (String)statement.getAttribute("page") ;
			if (pageKey != null && pageKey.trim().length() > 0) {
				String pageName = (String)getParameter((SourceBean)statement.getFilteredSourceBeanAttribute("PARAMETER", "NAME", pageKey), requestContainer, responseContainer) ;
				if (pageName != null) {
					SourceBean pageBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("PAGES.PAGE", "NAME", pageName) ;
					String key = "MODULES.MODULE" ;
					if (pageBean.containsAttribute("GRAPH")) {
						key = "GRAPH."+key ;
					}
					// List<SourceBean> moduleList = new ArrayList<SourceBean>() ;
					String moduleName = null ;
					SourceBean moduleBean = null ;
					genericBean = new SourceBean("ROOT") ;
					SourceBean newModuleBean = new SourceBean("MODULES") ;
					for (Object modulePage : pageBean.getAttributeAsList(key)) {
						moduleName =(String)((SourceBean)modulePage).getAttribute("NAME") ;
						moduleBean =  (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", moduleName) ;
						if (moduleBean != null)
							newModuleBean.setAttribute(moduleBean) ;
					}
					genericBean.setAttribute(newModuleBean) ;
				}
			}
	
			String moduleKey        = (String)statement.getAttribute("module") ;
			if (moduleKey != null && moduleKey.trim().length() > 0) {
				String moduleName = (String)getParameter((SourceBean)statement.getFilteredSourceBeanAttribute("PARAMETER", "NAME", moduleKey), requestContainer, responseContainer) ;
				if (moduleName != null) {
					genericBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", moduleName) ;
				}
			}
			
			String fieldsetKey        = (String)statement.getAttribute("fieldset") ;
			if (fieldsetKey != null && fieldsetKey.trim().length() > 0) {
				String fieldsetName = (String)getParameter((SourceBean)statement.getFilteredSourceBeanAttribute("PARAMETER", "NAME", fieldsetKey), requestContainer, responseContainer) ;
				if (fieldsetName != null) {
					genericBean = (SourceBean)genericBean.getFilteredSourceBeanAttribute("CONFIG.FIELDS.FIELDSET", "NAME", fieldsetName) ;
				}
			}
			
		}
		return genericBean ;
    }
    
    /**
     * Ritorna risultati nel formato ritornato dal QueryExecutor
     * @param statement
     * @param requestContainer
     * @param responseContainer
     * @return
     * @throws SourceBeanException
     */
    public SourceBean getConfigForQuery(SourceBean statement, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
    	SourceBean rowsSourceBean = null ;

    	// Verifica se deve estrarre l'elenco dei moduli
		String pageKey   = (String)statement.getAttribute("modulepage") ;
		if (pageKey != null && pageKey.trim().length() > 0) {
			String pageName = (String)getParameter((SourceBean)statement.getFilteredSourceBeanAttribute("PARAMETER", "NAME", pageKey), requestContainer, responseContainer) ;
			if (pageName != null) {
				SourceBean pageBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("PAGES.PAGE", "NAME", pageName) ;
				String key = "MODULES.MODULE" ;
				if (pageBean.containsAttribute("GRAPH")) {
					key = "GRAPH."+key ;
				}
				try {
					rowsSourceBean = new SourceBean("ROWS") ;
					SourceBean rowBean = null ;
					for (Object modulePage : pageBean.getAttributeAsList(key)) {
						String name = (String)((SourceBean)modulePage).getAttribute("NAME") ;
						rowBean = new SourceBean("ROW") ;
						rowBean.setAttribute("DISPLAY", name) ;
						/*
						if (name.indexOf("Mod") == 0)
							name = name.substring(3) ;
						if (name.indexOf("Lista") > 0) 
							name = name.substring(0, name.length() - 5) ;
						else if (name.indexOf("Combo") > 0) 
							name = name.substring(0, name.length() - 5) ;
						else if (name.indexOf("Dettaglio") > 0)
							name = name .substring(0, name.length() - 9) ;
						*/
						rowBean.setAttribute("VALUE", name) ;
						rowsSourceBean.setAttribute(rowBean) ;
					}
				} catch (SourceBeanException e) {
					TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::getConfigForQuery::Error",e);
					//e.printStackTrace();
				}
			}
		}
		else {
			// Sample configuration
			//	<FIELD name="Publisher" widget="combo" size="1">
			//		<INSERT is_readonly="FALSE" is_mandatory="TRUE" is_visible="TRUE" > 
			//	   	      <SELECT_QUERY key="PUBLISHERS.PUBLISHER" displayKey="NAME" valueKey="NAME" />
			String keyName = (String)statement.getAttribute("key") ;
			String displayKey = (String)statement.getAttribute("displayKey") ;
			String valueKey = (String)statement.getAttribute("valueKey") ;
			if (displayKey == null)
				displayKey = valueKey ;
			try {
				List<Object> elementi = null;
				rowsSourceBean = new SourceBean("ROWS") ;
				SourceBean rowBean = null ;
				// Se si desiderano i fieldset prepara la selezione con i filtri
				if (keyName.equalsIgnoreCase("CONFIG.FIELDS.FIELDSET")) {
					elementi = new ArrayList(0);
					List<SourceBean> parametri= (List<SourceBean>) statement.getAttributeAsList("PARAMETER");
					String parameterName = (String) (parametri.get(0)).getAttribute("NAME");
					String parameterScope = (String) (parametri.get(0)).getAttribute("SCOPE");
					String subKey = (String) statement.getAttribute("SUBKEY");
					String moduleName = (String) ContextScooping.getScopedParameter(requestContainer,responseContainer,parameterName,parameterScope);
					SourceBean modulo = (SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE","NAME",moduleName); 
					List<SourceBean> elementiSuperiori = (List<SourceBean>)modulo.getAttributeAsList(keyName);
					for(SourceBean elem : elementiSuperiori) {
						
						if (subKey!=null && subKey.trim().length()>0) {
							for(SourceBean elemento : (List<SourceBean>)elem.getAttributeAsList(subKey)) {
								elementi.add(elemento);
							}	
						} else {
							elementi.add(elem);
						}
						
					}
				} else if (keyName.equalsIgnoreCase("CONFIG.COLUMNS.COLUMN")) {
					elementi = new ArrayList(0);
					List<SourceBean> parametri= (List<SourceBean>) statement.getAttributeAsList("PARAMETER");
					String parameterName = (String) (parametri.get(0)).getAttribute("NAME");
					String parameterScope = (String) (parametri.get(0)).getAttribute("SCOPE");
					String actionName = (String) ContextScooping.getScopedParameter(requestContainer,responseContainer,parameterName,parameterScope);
					SourceBean modulo = (SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("ACTIONS.ACTION","NAME",actionName); 
					List<SourceBean> elementiSuperiori = (List<SourceBean>)modulo.getAttributeAsList(keyName);
					for(SourceBean elem : elementiSuperiori) {
						elementi.add(elem);
					}
				} else if (keyName.equalsIgnoreCase("CONFIG.FIELD")) {
					elementi = new ArrayList(0);
					List<SourceBean> parametri= (List<SourceBean>) statement.getAttributeAsList("PARAMETER");
					String parameterName = (String) (parametri.get(0)).getAttribute("NAME");
					String parameterScope = (String) (parametri.get(0)).getAttribute("SCOPE");
					String lookupName = (String) ContextScooping.getScopedParameter(requestContainer,responseContainer,parameterName,parameterScope);
					SourceBean lookup = (SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("LOOKUPS.LOOKUP","NAME",lookupName);
					if(lookup!=null) {
						List<SourceBean> elementiSuperiori = (List<SourceBean>)lookup.getAttributeAsList(keyName);
						for(SourceBean elem : elementiSuperiori) {
							elementi.add(elem);
						}
					}
				}
				else elementi = ConfigSingleton.getInstance().getAttributeAsList(keyName);
				for (Object bean : elementi) {
					String display = (String)((SourceBean)bean).getAttribute(displayKey);
					String value = (String)((SourceBean)bean).getAttribute(valueKey);
					rowBean = new SourceBean("ROW") ;
					rowBean.setAttribute("DISPLAY",display);
					rowBean.setAttribute("VALUE",value);
					rowsSourceBean.setAttribute(rowBean);
				}
			} catch (SourceBeanException e) {
				TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::getConfigForQuery::Error",e);
				//e.printStackTrace();
			}
		}
    	return rowsSourceBean ;
    }
    
    /**
     * This metohod returns a list of SourceBean from configuration
     *
     * @param statement beanStatement:
     *        <SELECT_QUERY key="PAGES.PAGE" page="var name">
     *            <FILTER_PARAMETER_NAME  type="ABSOLUTE" value="NAME" scope="" />
     *            <FILTER_PARAMETER_VALUE type="RELATIVE" value="NAME" scope="SERVICE_REQUEST" />
     *        </SELECT_QUERY>
     *
     * @param requestContainer
     * @param responseContainer
     * @return
     */
    public List getConfigList(SourceBean statement, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
    	List listBean = null ;
    	/*
		listBean = handleConfigSpecialsKey(statement, requestContainer, responseContainer) ;
		if (listBean == null) {
			String key        = (String)statement.getAttribute("key") ;
	    	String paramName  = (String)ConfigRowHandler.getParameter((SourceBean)statement.getAttribute("FILTER_PARAMETER_NAME"), requestContainer, responseContainer) ;
			String paramValue = (String)ConfigRowHandler.getParameter((SourceBean)statement.getAttribute("FILTER_PARAMETER_VALUE"), requestContainer, responseContainer) ;
			if ((paramName != null && paramName.trim().length() > 0) &&
				(paramValue != null && paramValue.trim().length() > 0))
				listBean = (List)ConfigSingleton.getInstance().getFilteredSourceBeanAttributeAsList(key, paramName, paramValue) ;
		else
			listBean = (List)ConfigSingleton.getInstance().getAttributeAsList(key) ;
		}
*/
    	
    	try {
		if (statement.getAttribute("xpath")!=null) {
			
			String xquery = (String)statement.getAttribute("xpath");
			//String xparameters = (String)statement.getAttribute("parameters");
			
			List<SourceBean> xparameters = statement.getAttributeAsList("PARAMETER");
	    
			int count = 0;
			for (int i = 0; i < xquery.length(); i++) {
				if (xquery.charAt(i) == '?') {
					count++;
				}
			}
			
			if (count!=xparameters.size()) {
				responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, "Numero di parametri errato"));
			}
			
			String elab = xquery;
			
			SourceBean bean = ConfigSingleton.getInstance();
			

			bean.toXML();
			
			XPathFactory factory = XPathFactory.newInstance();
			XPath xpath = factory.newXPath();
			
			Iterator<SourceBean> parameters = xparameters.iterator();
			
			StringTokenizer tokenizer = new StringTokenizer(elab,"?");
			Iterator<SourceBean> paramsIterator = xparameters.iterator();
			StringBuffer sbuff = new StringBuffer();
			
			while (tokenizer.hasMoreTokens() && paramsIterator.hasNext()) {
				String element = tokenizer.nextToken();
				sbuff.append(element);
				
		    	SourceBean parameter = paramsIterator.next();
		    	//String parName = (String)parameter.getAttribute("NAME");
		    	String parValue = (String)getParameter(parameter, requestContainer, responseContainer);
		    	//sbuff.append(parName+"='");
		    	sbuff.append(parValue);
		    	//sbuff.append("'");
		    	
			}
			
			if (tokenizer.hasMoreTokens()) {
				sbuff.append(tokenizer.nextToken());
			}
			
			try {
				
				
				XPathExpression expr = xpath.compile(sbuff.toString());
				NodeList result = (NodeList)expr.evaluate(bean.toDocument(),XPathConstants.NODESET);
				
//				String out =  expr.evaluate(bean.toDocument());
				
				//String set = expr.evaluate(bean.toDocument());
				List<SourceBean> elencoOggetti = new ArrayList<SourceBean>();
				for (int i = 0; i < result.getLength(); i++) {
					Node node = result.item(i);
					elencoOggetti.add(sourceBeanFromNode(node));
				}
//				System.out.println(out);
//				set.length();
//				dataRow = SourceBean.fromXMLString(result.toString());
				listBean = elencoOggetti;
			} catch (XPathExpressionException e) {
				TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::getConfigList::Error",e);
				//e.printStackTrace();
			}

//			while (elab.indexOf("[?]")>=0) {
//				
//		    	SourceBean parameter = parameters.next();
//		    	String parName = (String)parameter.getAttribute("NAME");
//		    	String parValue = (String)getParameter(parameter, requestContainer, responseContainer);
//		    	bean = (SourceBean)bean.getFilteredSourceBeanAttribute(elab.substring(0,elab.indexOf("[?]")), parName, parValue) ;
//				elab = elab.substring(elab.indexOf("[?]")+3);
//			}
//			
//			if (elab.length()>0) {
//				listBean = bean.getAttributeAsList(elab.substring(1));	
//			}
			
				
		} else {
			
			SourceBean genericBean;
			
			
				genericBean = getBeanForSpecialAttributes(statement, requestContainer, responseContainer);
		    	String key        = (String)statement.getAttribute("key") ;
		    	SourceBean filterParameterName = (SourceBean)statement.getAttribute("FILTER_PARAMETER_NAME") ;
		    	SourceBean filterParameterValue = (SourceBean)statement.getAttribute("FILTER_PARAMETER_VALUE") ;
		    	String paramName = null, paramValue = null ;
		    	if (filterParameterName != null && filterParameterValue != null) {
		    		paramName  = (String)getParameter(filterParameterName, requestContainer, responseContainer) ;
		    		paramValue = (String)getParameter(filterParameterValue, requestContainer, responseContainer) ;
		    	}
		    	if (paramName != null && paramValue != null) {
		    		listBean = genericBean.getFilteredSourceBeanAttributeAsList(key, paramName, paramValue) ;
		    	} else {
		    		listBean = genericBean.getAttributeAsList(key) ;
		    	}
	
			}
			for (SourceBean filterParameter : (List<SourceBean>)statement.getAttributeAsList("FILTER_PARAMETER")) {
		    		if (filterParameter != null) {
			    		listBean = filterByParameter(listBean, filterParameter, requestContainer, responseContainer) ;
		    		}
		    	}
		    	
				String defaultKey = (String)statement.getAttribute("default") ;
				SourceBean defaultBean = null ;
				if (defaultKey != null && defaultKey.trim().length() > 0) {
					defaultBean = (SourceBean)ConfigSingleton.getInstance().getAttribute(defaultKey) ;
	
					for (Object row : ((List)listBean)) {
						if (row != null && row instanceof SourceBean) {
							((SourceBean)row).mergeWidth(defaultBean) ;
							// Se  un FIELD allora fisso alcune eccezioni
							if (((SourceBean)row).containsAttribute("WIDGET")) {
								String widget = (String)((SourceBean)row).getAttribute("WIDGET") ;
								if (! widget.equalsIgnoreCase(FieldSupport.WIDGET_CALENDAR)) {
									((SourceBean)row).delAttribute("CALSEP") ;
									((SourceBean)row).delAttribute("CALFMT") ;
								}
								if (! widget.equalsIgnoreCase(FieldSupport.WIDGET_TEXT_AREA)) {
									((SourceBean)row).delAttribute("INSERT.TEXT_AREA") ;
									((SourceBean)row).delAttribute("UPDATE.TEXT_AREA") ;
								}
							}
						}
					}
				}
			} catch (SourceBeanException e) {
				TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::getConfigList::Error",e);
				//e.printStackTrace();
			}
		
		return listBean ;
		
	}

    private List  filterByParameter(List listBean, SourceBean parameter,  final RequestContainer requestContainer, final ResponseContainer responseContainer) {
    	if (parameter != null && listBean != null && listBean.size() > 0) {
	    	// Lettura attributi di "tipo" del parametro
		    String parameterType = (String) parameter.getAttribute("TYPE");
		    String parameterValue = (String) parameter.getAttribute("VALUE");
		    String parameterScope = (String) parameter.getAttribute("SCOPE");
		    boolean uppercase = Boolean.parseBoolean((String)parameter.getAttribute("UPPERCASE", "FALSE"));
	
	        Object parameterValueObject = null ; 
	        if (parameterType.equalsIgnoreCase("ABSOLUTE")) {
	        	parameterValueObject = parameterValue;
	        } else {
	            parameterValueObject = 
	            	ContextScooping.getScopedParameter(requestContainer, responseContainer,
										parameterValue, parameterScope, parameter);
	        }
	        
	        if (uppercase && parameterValueObject instanceof String) {
	        	parameterValueObject = ((String)parameterValueObject).toUpperCase() ;
			}
	        
	        if (parameterValueObject != null && parameterValueObject.toString().trim().length() > 0) {
	        	String key = (""+parameter.getAttribute("KEY")).trim() ;
	        	ArrayList<SourceBean> returnBean = new ArrayList<SourceBean>() ;
	        	String rowAttr = null ;
	        	int parLen = (""+parameterValueObject).length() ;
	        	for (SourceBean row : (List<SourceBean>)listBean) {
	        		rowAttr = ("" + row.getAttribute(key)).trim().toUpperCase() ;
	        		if (rowAttr.length() >= parLen && rowAttr.indexOf(""+parameterValueObject) != -1) {
	        			returnBean.add(row) ;
	        		}
	        	}
	        	listBean = returnBean ;
	        }
        }
    	return listBean ;
    }
    
    /**
     * Returns generic Attributes in specify module
     * 
     * @param moduleName String with name of module
     * @param key        String in dot format for specific needs attribute
     * @param requestContainer
     * @param responseContainer
     * @return
     */
    private List getModulesAttributes(String moduleName, String key, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
		List listBean = null ;
    	SourceBean moduleBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", moduleName) ;
    	if (key != null && moduleBean != null) {
    		if (key.equalsIgnoreCase("QUERIES")) {
    			listBean = getModulesQueries(moduleBean, key, requestContainer, responseContainer) ;
    		} else {
    			listBean = moduleBean.getAttributeAsList(key) ;
    		}
    	}
    	
		return listBean ;
    }
	
    /**
     * Returns queries for module
     * 
     * @param moduleBean es:
     * 
     * Sample (1)
     * <CONFIG title="Elenco Moduli/Azioni">
     * 	<ROW_PROVIDER class="it.eng.spago.paginator.smart.impl.DBRowHandler">
     * 		<CONFIG pool="as400masterpool">
     * 			<LIST_QUERY statement="SELEZIONA_MODULO">
     * 				<SORT_ACTION name="sortByCode" sql="code" />
     * 				<SORT_ACTION name="sortByDescription" sql="description" />
     * 				<FILTER_PARAMETER scope="SERVICE_REQUEST" type="RELATIVE" value="FILTER_code" sql="ANNRNM = ?" />
     * 				<FILTER_PARAMETER scope="SERVICE_REQUEST" type="RELATIVE" value="FILTER_description" sql="UPPER(ANQPTT) like ?" />
     * 			</LIST_QUERY>
     * 			<DELETE_QUERY statement="DELETE_MODULO">
     * 				<PARAMETER type="RELATIVE" value="code" scope="SERVICE_REQUEST" />
     * 			</DELETE_QUERY> 
     * 		</CONFIG>
     * 	</ROW_PROVIDER>
     * . . .
     * </CONFIG>
     * 
     * Sample (2)
     * 	<QUERIES>
     * 		<SELECT_QUERY statement="DETTAGLIO_MODULO">
     * 			<PARAMETER type="RELATIVE" value="code" scope="SERVICE_REQUEST" />
     * 		</SELECT_QUERY>
     * 		<INSERT_QUERY statement="INSERT_MODULO">
     * 		  	<PARAMETER type="RELATIVE" value="code" scope="SERVICE_REQUEST" /> 
     * 		  	<PARAMETER type="RELATIVE" value="description" scope="SERVICE_REQUEST" /> 
     * 		</INSERT_QUERY>
     * 		<UPDATE_QUERY statement="UPDATE_MODULO">
     * 			<PARAMETER type="RELATIVE" value="description" scope="SERVICE_REQUEST" />
     * 			<PARAMETER type="RELATIVE" value="code" scope="SERVICE_REQUEST" />
     * 		</UPDATE_QUERY>
     * 	</QUERIES>
     * 
     * @param key
     * @param requestContainer
     * @param responseContainer
     * @return
     */
    private List getModulesQueries(SourceBean moduleBean, String key, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
		List listBean = new ArrayList() ;
    	boolean hasRowProvider = moduleBean.containsAttribute("CONFIG.ROW_PROVIDER") ;
    	String listQueryPath = (hasRowProvider) ? "CONFIG.ROW_PROVIDER.CONFIG.LIST_QUERY" : "CONFIG.LIST_QUERY" ;
    	addModuleQuery(listBean, moduleBean, listQueryPath, requestContainer, responseContainer) ;
    	String deleteQueryPath = (hasRowProvider) ? "CONFIG.ROW_PROVIDER.CONFIG.DELETE_QUERY" : "CONFIG.DELETE_QUERY" ;
    	addModuleQuery(listBean, moduleBean, deleteQueryPath, requestContainer, responseContainer) ;
    	addModuleQuery(listBean, moduleBean, "CONFIG.QUERIES.SELECT_QUERY", requestContainer, responseContainer) ;
    	addModuleQuery(listBean, moduleBean, "CONFIG.QUERIES.INSERT_QUERY", requestContainer, responseContainer) ;
    	addModuleQuery(listBean, moduleBean, "CONFIG.QUERIES.UPDATE_QUERY", requestContainer, responseContainer) ;
    	
		return listBean ;
    }
    
    /**
     * Adds query bean to input list
     * @param listBean
     * @param moduleBean
     * @param key
     * @param requestContainer
     * @param responseContainer
     */
    private void addModuleQuery(List listBean, SourceBean moduleBean, String key, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
    	SourceBean queryBean = (SourceBean)moduleBean.getAttribute(key) ;
    	if (queryBean != null) {
    		try {
				queryBean.setAttribute("XML", normalizeAttribute(queryBean.toXML(false, true))) ;
				listBean.add(queryBean) ;
			} catch (Exception ex) {
				responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, ex));
			}
    	}
    }
		
    /**
     * Returns a list of MODULE for the giving page Name
     * 
     * @param pageName String with name of page
     * @param requestContainer
     * @param responseContainer
     * @return
     */
    private List getModules(String pageName, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
		List listBean = null ;
		if (pageName != null) {
			SourceBean pageBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("PAGES.PAGE", "NAME", pageName) ;
			if (pageBean != null) {
				boolean hasGraphDefinition = pageBean.containsAttribute("class") ;
				String keyName = (hasGraphDefinition) ? "GRAPH.MODULES.MODULE" : "MODULES.MODULE" ;
				List moduleList = pageBean.getAttributeAsList(keyName) ;
				listBean = getModules(moduleList, requestContainer, responseContainer) ;
			} else {
				responseContainer.getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.WARNING, "Configurazione per la pagina '" + pageName + "' NON trovata"));
			}
		}
    	
		return listBean ;
    }
    
    /**
     * Returns a list with requested MODULE
     * 
     * @param moduleList      List<SourceBean>
     *          es: <MODULE id="lista" name="ModModulesLista" />
     *          
     * @param requestContainer
     * @param responseContainer
     * @return
     */
    private List getModules(List moduleList, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
		List listBean = null ;
    	for (Object mod : moduleList) {
    		if (mod != null) {
    			String modName = (String)((SourceBean)mod).getAttribute("NAME") ;
    			SourceBean moduleBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("MODULES.MODULE", "NAME", modName) ;
    			if (listBean == null) {
    				listBean = new ArrayList() ;
    			}
	    		try {
	    			boolean hasRowProvider = moduleBean.containsAttribute("CONFIG.ROW_PROVIDER") ;
	    			if (hasRowProvider) {
    	    			// Adds attributes for generalization
    	    			String pool = (String)moduleBean.getAttribute("CONFIG.ROW_PROVIDER.CONFIG.POOL") ;
						((SourceBean)moduleBean.getAttribute("CONFIG")).setAttribute("POOL", pool) ;
	    	    	}
				} catch (Exception ex) {
					responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, ex));
				}

    			listBean.add(moduleBean) ;
    		}
    	}
		return listBean ;
    }
    
    /**
     * Recupera il valore (String) dei tag <PARAMETER />
     * @param parameterBean
     * @return
     */
    public Object getParameter(final SourceBean parameterBean, final RequestContainer requestContainer, final ResponseContainer responseContainer) {
        Object parameterValueObject = null ; 
		
	    // Lettura attributi di "tipo" del parametro
	    String parameterType = (String) parameterBean.getAttribute("TYPE");
	    String parameterValue = (String) parameterBean.getAttribute("VALUE");
	    String parameterScope = (String) parameterBean.getAttribute("SCOPE");
	    boolean uppercase = Boolean.parseBoolean((String)parameterBean.getAttribute("UPPERCASE", "FALSE"));

        if (parameterType.equalsIgnoreCase("ABSOLUTE")) {
        	parameterValueObject = parameterValue;
        } else {
            parameterValueObject = 
            	ContextScooping.getScopedParameter(requestContainer,
									responseContainer, parameterValue, parameterScope, parameterBean);
        }
        
        // Aggiunta per Spago 2.0
        if (parameterValueObject instanceof NullValue) {
        	parameterValueObject = null ;
        }
        if (uppercase && parameterValueObject instanceof String) {
        	parameterValueObject = ((String)parameterValueObject).toUpperCase() ;
		}
        
        return parameterValueObject ;
	}
    
    public String normalizeAttribute(String attribute) {
        if (attribute == null)
            return null;
        char[] attributeArray = attribute.toCharArray();
        StringBuffer normalizedAttribute = new StringBuffer();
        normalizedAttribute.append("<table id=\"titletb\">") ;
        boolean opentd = false ;
        for (int i = 0; i < attributeArray.length; i++) {
        	if (! opentd) {
        		opentd = true ;
        		normalizedAttribute.append("<tr><td id=\"titletb\">") ;
        	}
            char c = attributeArray[i];
            switch (c) {
                case '<' :
                    {
                        normalizedAttribute.append("&lt;");
                        break;
                    } // case '<'
                case '>' :
                    {
                        normalizedAttribute.append("&gt;");
                        break;
                    }
                case '&' :
                    {
                        normalizedAttribute.append("&amp;");
                        break;
                    }
                case '"' :
                    {
                        normalizedAttribute.append("&quot;");
                        break;
                    }
                case '\r':
                		break ;
                case '\n': {
                    normalizedAttribute.append("</td></tr>");
                    opentd = false ;
                    break;
                }

                default :
                    {
                        normalizedAttribute.append(c);
                    }
            } // switch (c)
        } // for (int i = 0; i < attributeArray.length; i++)
        if (opentd) {
            normalizedAttribute.append("</td></tr>");
        }
        normalizedAttribute.append("</table>") ;
        return normalizedAttribute.toString();
    } // public static String normalizeAttribute(String attribute)
    
    
    /**
     * 
     * @param node
     * @return
     */
    public SourceBean sourceBeanFromNode(Node node) {
    	
		SourceBean det = null;
		
		if (node!=null) {
			try {
				det = new SourceBean(node.getNodeName());
				
				for (int j = 0; j < node.getAttributes().getLength(); j++) {
					Node attr = node.getAttributes().item(j);
					det.setAttribute(attr.getNodeName(), attr.getNodeValue());
				}
	
				for (int i = 0; i < node.getChildNodes().getLength(); i++) {
					Node childNode = node.getChildNodes().item(i);
					det.setAttribute(sourceBeanFromNode(childNode));
				}
			} catch (DOMException e) {
				TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::sourceBeanFromNode::Error",e);
				//e.printStackTrace();
			} catch (SourceBeanException e) {
				TracerSingleton.log(Constants.NOME_MODULO,TracerSingleton.CRITICAL,"ConfigRowHandler::sourceBeanFromNode::Error",e);
				//e.printStackTrace();
			}
		}
		return det;
		
    }
    
    
}