/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import java.io.Serializable;

public class QueryParameter implements Serializable {
	private static final long serialVersionUID = 40882808291141527L;
	
	/**
	 * Questo metodo ritorna in formato "queryString" (parametro1=vaolore1&parametro2=valore2& ...)
	 * I parametri vengono estratti attraverso uno statement sql che ritorna delle righe con il formato:
	 *   name, type, value, scope  (Esattamete come i <PARAMETER>)
	 * I parametri vengono poi valutati dal ContextScooping
	 * 
	 * @param requestContainer
	 * @param responseContainer
	 * @param queryParameterBean
	 * @return
	 * @throws Exception
	 */
	public static String getParametersAsQueryString(final RequestContainer requestContainer, final ResponseContainer responseContainer, final SourceBean rowBean, final SourceBean queryParameterBean) throws Exception {
		String queryString="";
		SourceBean rows = (SourceBean)QueryExecutor.executeQuery(requestContainer, responseContainer,
				(String)queryParameterBean.getAttribute("pool"), queryParameterBean, QueryExecutor.READ) ;
		for (Object row : rows.getAttributeAsList("ROW")) {
			String name = ((String)((SourceBean)row).getAttribute("NAME", "")).trim() ;
			queryString = queryString + name + "=" +  getParameterValue(requestContainer, responseContainer, rowBean, (SourceBean)row) + "&" ;
		}
		return queryString ;
	}
	
	/**
	 * Ritorna il valore di uno specifico parametro
	 * es: <PARAMETER name="LISTA_VUOTA" type="RELATIVE", scope="SERVICE_RESPONSE" /> 
	 * @param bean
	 * @return
	 */
	protected static String getParameterValue(final RequestContainer requestContainer, final ResponseContainer responseContainer, SourceBean rowBean, SourceBean row) {
		String ret = "" ;
		if (row != null) {
            String parameterType = ((String)row.getAttribute("TYPE", "")).trim();
            if (parameterType != null && parameterType.equalsIgnoreCase("ABSOLUTE")) {
            	ret = ((String) row.getAttribute("VALUE")).trim();
            } else if (parameterType != null && rowBean != null && parameterType.equalsIgnoreCase("LOCAL")) {
            	ret = (String)rowBean.getAttribute(((String)row.getAttribute("NAME")).trim()) ;
            } else {
			ret = (String) ContextScooping.getScopedParameter(requestContainer,
					responseContainer,
					(String)row.getAttribute("NAME"),
					(String)row.getAttribute("SCOPE"),
					null);
            }
			if (ret == null)
				ret = "" ;
		}
		
		return ret ;
	}
}