/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.sql.TimestampDecorator;
import it.eng.spago.tracing.TracerSingleton;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.StringTokenizer;

/**
 * La classe <code>SourceBeanComparator</code> permette di ordinare collezioni di SourceBean
 * attraverso i parametri richiesti
 *  
 * <code>ListFilterValidator</code>
 * <p>
 * 
 * @version 1.0, 01/01/2008
 * @author Gabriele Rigamonti
 * @see SourceBean
 */
public class SourceBeanComparator implements Comparator {
	public static final String ASCENDANT_ORDER = "ASC";
	public static final String DESCENDANT_ORDER = "DESC";
	private String _compareAttributeListName = "";
	private String _orderType = ASCENDANT_ORDER;

	public SourceBeanComparator(String compareAttributeListName, String orderType) {
		_compareAttributeListName = compareAttributeListName;
		_orderType = checkOrderType(orderType);
	}

	public int compare(Object o1, Object o2) {
		try {
			SourceBean left = (SourceBean) o1;
			SourceBean right = (SourceBean) o2;

			StringTokenizer stk = new StringTokenizer(_compareAttributeListName, ",");
			int test = 0;
			for (; stk.hasMoreTokens();) {
				String compareAttributeName = stk.nextToken();
				test = compare(left, right, compareAttributeName);
				if (_orderType.equals(DESCENDANT_ORDER))
					test *= -1; // inverto l'ordine
				if (test != 0)
					return test; // Ho capito chi  il maggiore e quindi
									// ritorno indicazioni di conseguenza
			}
			return test;
		} catch (ClassCastException ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "InstructionComparator::compare:", ex);
			throw ex;
		}
	}

	/**
	 * Controlla il tipo di ordinamento
	 * 
	 * @param orderType
	 * @return
	 */
	private String checkOrderType(String orderType) {
		String order = orderType;
		if (_orderType == null && ((!_orderType.equalsIgnoreCase(ASCENDANT_ORDER) || (!_orderType.equalsIgnoreCase(DESCENDANT_ORDER)))))
			_orderType = ASCENDANT_ORDER;
		return order;
	}

	/**
	 * Effettua il compare di una key
	 * 
	 * @param leftBean
	 * @param rightBean
	 * @param compareAttributeName
	 * @return
	 */
	private int compare(SourceBean leftBean, SourceBean rightBean, String compareAttributeName) {
		Object left = leftBean.getAttribute(compareAttributeName);
		Object right = rightBean.getAttribute(compareAttributeName);

		int ret = 0;
		if (left instanceof Double) {
			ret = ((Double) left).compareTo((Double) right);
		} else if (left instanceof TimestampDecorator) {
			ret = ((TimestampDecorator) left).compareTo((TimestampDecorator) right);
		} else if (left instanceof Integer) {
			ret = ((Integer) left).compareTo((Integer) right);
		} else if (left instanceof BigDecimal) {
			ret = ((BigDecimal) left).compareTo((BigDecimal) right);
		} else {
			if (isNumericString(left.toString()) && isNumericString(right.toString())) {
				if (Double.parseDouble(left.toString()) > Double.parseDouble(right.toString())) {
					ret = 1;
				}
				if (Double.parseDouble(left.toString()) < Double.parseDouble(right.toString())) {
					ret = -1;
				}
				if (Double.parseDouble(left.toString()) == Double.parseDouble(right.toString())) {
					ret = 0;
				}
			} else
				ret = (left.toString().compareTo(right.toString())); // String
		}
		return ret;
	}

	/**
	 * Controlla se la string contiene una rappresentazione di un valore
	 * numerico
	 * 
	 * @param parameter
	 * @return
	 */
	private static boolean isNumericString(String parameter) {
		boolean numeric = false;

		if (parameter != null) {
			try {
				Double.parseDouble(parameter);
				numeric = true;
			} catch (Exception ex) {
				// non gestita, rimane impostata a false, quindi non e' un
				// numerico
			}
		}
		return numeric;
	}
}