/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util.pcmlField;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;
import it.eng.spago.util.sqlField.SqlFieldType;

import java.io.Serializable;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

/**
 * @author scarciello
 *
 */
public class PcmlFieldCreator implements Serializable {

	private static final long serialVersionUID = 8694288518614269514L;
	
	private SourceBean _typesList;
	private SourceBean _campiLog;
	ArrayList<SqlFieldType> _fieldTypeList = null;

	public PcmlFieldCreator(SourceBean campiLog, SourceBean typesList) throws SourceBeanException {
		_typesList = typesList;
		setCampiLog(campiLog);
	}

	@SuppressWarnings("unchecked")
	private void setCampiLog(SourceBean campiLog) throws SourceBeanException {
		if (campiLog != null) {
			List<SourceBean> rows = campiLog.getAttributeAsList("ROW");
			for (SourceBean row : rows) {
				if (row != null) {
					String codAmbiente = (String) row.getAttribute("nomeCampo");
					if (codAmbiente != null)
						row.updAttribute("nomeCampo", codAmbiente.trim());
				}
			}
		}
		_campiLog = campiLog;
	}

	private boolean isLogField(final String nomeCampo) {
		String fieldToSearch = nomeCampo;
		return (_campiLog.getFilteredSourceBeanAttribute("ROW", "nomeCampo", fieldToSearch) != null);
	}

	public List<SqlFieldType> getFieldTypeList(SourceBean pcmlData) throws Exception {
		if (_fieldTypeList != null)
			return _fieldTypeList;
		_fieldTypeList = new ArrayList<SqlFieldType>();
		int i=0;
		for(SourceBean dataRow : (ArrayList<SourceBean>)pcmlData.getAttributeAsList("PROGRAM.DATA")) {
			i++;
			String name = (String)dataRow.getAttribute("name");
			String type = (String)dataRow.getAttribute("type");
			String length = (String)dataRow.getAttribute("length");
			String precision = (String)dataRow.getAttribute("precision");
			String usage = (String)dataRow.getAttribute("usage");

			if(!isLogField(name)) {
				SqlFieldType aField = new SqlFieldType();
				aField.setNomeCampo(name);
				aField.setProgressivoCampo(i * 10); // Es. 10, 20, 30, ...
				aField.setTipoCampo(_typesList, checkType(type));
				aField.setDescrizioneCampo(name);
				aField.setCampoWeb(name.replaceAll(" ", "").trim());
				aField.setLunghezzaCampo(Integer.parseInt(length));
				if(precision!=null && precision.length()>0) {
					aField.setDecimaliCampo(Integer.parseInt(precision));
				}
				aField.setCampoNull(true);
				aField.setCampoChiave(false);
				if(usage.equalsIgnoreCase("INPUT"))
					usage = "I";
				if(usage.equalsIgnoreCase("OUTPUT"))
					usage = "O";
				if(usage.equalsIgnoreCase("INPUTOUTPUT"))
					usage = "E";
				aField.setUsage(usage);
				_fieldTypeList.add(aField);
			}
		}
		return _fieldTypeList;
	}

	protected int checkType(String parameterType) {
		int returnValue = 0;
		if (parameterType.equalsIgnoreCase("char")) {
			returnValue = Types.VARCHAR;
		}
		if (parameterType.equalsIgnoreCase("packed")) {
			returnValue = Types.DOUBLE;
		}
		return returnValue;
	}
	
	/*
	public List<SqlFieldType> getFieldTypeList(SourceBean serviceRequest) throws Exception {
		DataConnection dataConnection = null;
		Statement stmt = null;
		ResultSet rs = null;
		try {
			dataConnection = QueryExecutor.openConnection(_poolName, null); // TODO: Usato dal DAOGEN
			stmt = dataConnection.getInternalConnection().createStatement();
			stmt.setMaxRows(1) ;
			rs = stmt.executeQuery(makeSqlStatement(_codAmbiente, _fileFisico));
			// Retrieve the number, type and properties of the resultset's
			// columns
			ResultSetMetaData rsMetaData = rs.getMetaData();
			int colCount = rsMetaData.getColumnCount();
			String nomeCampo;
			for (int i = 1; i <= colCount; i++) {
				nomeCampo = rsMetaData.getColumnName(i);
				if (!isLogField(nomeCampo)) {
					boolean campoChiave = false;
					try {
						campoChiave = isPrimaryKey(rsMetaData.getColumnLabel(i), dataConnection, rsMetaData.getCatalogName(i), _codAmbiente, _fileFisico);
					}
					catch (Exception ex) {
						
					}
				}
			}
		} // try
		catch (EMFInternalError ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "SqlFieldCreator::getFieldTypeList:", (Exception) ex);
			throw ex;
		} // catch (Exception ex) try
		finally {
			if (rs != null)
				rs.close();
			if (stmt != null)
				stmt.close();
			Utils.releaseResources(dataConnection, null, null);
		} // finally try
	}
	*/
}
