/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util.sqlField;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.std.DefaultScrollableDataResult;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;
import java.io.Serializable;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

/**
 * @author grassi
 * 
 */
public class SqlFieldCreator implements Serializable {
	private String _poolName;
	private String _fileFisico;
	private String _codAmbiente;
	private String _prefissoCampi;
	private SourceBean _typesList;
	private SourceBean _campiLog;
	ArrayList<SqlFieldType> _fieldTypeList = null;

	/**
	 * Makes a list of SqlFieldType using jdbc feature
	 * 
	 * @param poolName
	 *            pool defined in dataConnection
	 * @param fileFisico
	 *            tableName of fileName for AS400 like environments
	 * @param codAmbiente
	 *            filePrefix or tablePrefix (schema)
	 * @param typesList
	 *            List (ROWS.ROW format) contains common definition for mapping
	 *            sql Type (es. widget, validator, ...)
	 * @throws SourceBeanException
	 */
	public SqlFieldCreator(String poolName, String fileFisico, String codAmbiente, SourceBean campiLog, String prefissoCampi, SourceBean typesList) throws SourceBeanException {
		_poolName = poolName;
		_fileFisico = fileFisico;
		_codAmbiente = codAmbiente;
		_typesList = typesList;
		setCampiLog(campiLog);
		_prefissoCampi = prefissoCampi;
	}

	/**
	 * Returns fields definition and property for requested table/schema/pool
	 * 
	 * @param serviceRequest
	 * @return
	 * @throws Exception
	 */
	public List<SqlFieldType> getFieldTypeList(SourceBean serviceRequest) throws Exception {
		if (_fieldTypeList != null)
			return _fieldTypeList;
		_fieldTypeList = new ArrayList<SqlFieldType>();
		DataConnection dataConnection = null;
		Statement stmt = null;
		ResultSet rs = null;
		try {
			dataConnection = QueryExecutor.openConnection(_poolName, null); // TODO: Usato dal DAOGEN
			stmt = dataConnection.getInternalConnection().createStatement();
			stmt.setMaxRows(1) ;
			rs = stmt.executeQuery(makeSqlStatement(_codAmbiente, _fileFisico));
			// Retrieve the number, type and properties of the resultset's
			// columns
			ResultSetMetaData rsMetaData = rs.getMetaData();
			int colCount = rsMetaData.getColumnCount();
			String nomeCampo;
			for (int i = 1; i <= colCount; i++) {
				nomeCampo = rsMetaData.getColumnName(i);
				if (!isLogField(nomeCampo)) {
					boolean campoChiave = false;
					SqlFieldType aField = new SqlFieldType();
					aField.setNomeCampo(nomeCampo);
					aField.setProgressivoCampo(i * 10); // Es. 10, 20, 30, ...
					aField.setTipoCampo(_typesList, rsMetaData.getColumnType(i));
					aField.setDescrizioneCampo(rsMetaData.getColumnLabel(i));
					aField.setCampoWeb(rsMetaData.getColumnLabel(i).replaceAll(" ", "").trim());
					aField.setLunghezzaCampo(rsMetaData.getPrecision(i));
					aField.setDecimaliCampo(rsMetaData.getScale(i));
					aField.setCampoNull((rsMetaData.isNullable(i) == ResultSetMetaData.columnNullable));
					try {
						campoChiave = isPrimaryKey(rsMetaData.getColumnLabel(i), dataConnection, rsMetaData.getCatalogName(i), _codAmbiente, _fileFisico);
					}
					catch (Exception ex) {
						
					}
					aField.setCampoChiave(campoChiave);
					// 03/01/2008 - R.S. - Inserito per il settaggio del campo Null - se il campo non  chiave
					//aField.setCampoNull(!campoChiave);
					_fieldTypeList.add(aField);
				}
			}
		} // try
		catch (EMFInternalError ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "SqlFieldCreator::getFieldTypeList:", (Exception) ex);
			throw ex;
		} // catch (Exception ex) try
		finally {
			if (rs != null)
				rs.close();
			if (stmt != null)
				stmt.close();
			Utils.releaseResources(dataConnection, null, null);
		} // finally try
		return _fieldTypeList;
	}

	/**
	 * Verifica se il campo appartiene al log
	 * 
	 * @param nomeCampo
	 * @return
	 */
	private boolean isLogField(final String nomeCampo) {
		String fieldToSearch = nomeCampo;
		if (_prefissoCampi != null)
			fieldToSearch = fieldToSearch.substring(_prefissoCampi.length());
		return (_campiLog.getFilteredSourceBeanAttribute("ROW", "nomeCampo", fieldToSearch) != null);
	}

	private boolean isPrimaryKey(String colName, DataConnection dataConnection, String catalog, String schema, String table) throws Exception {
		DatabaseMetaData dbmd = dataConnection.getInternalConnection().getMetaData();
		ResultSet rs = null;
		boolean ret = false;
		// int numcol=0;
		try {
			rs = dbmd.getPrimaryKeys(catalog, null, table);
			while ((!ret) && rs.next()) {
				int cols = rs.getMetaData().getColumnCount();
				for (int i = 1; ret == false && i <= cols; i++) {
					if ((rs.getString(i) != null) && (rs.getString(i).equals(colName)))
						ret = true; // Yes it's a key !!
				}
			}
		} // try
		catch (Exception ex) {
			;
		} finally {
			if (rs != null)
				rs.close();
		}
		return ret;
	}

	/**
	 * Esegue il trim del campo codAmbiente per tutti i record contenuti
	 * 
	 * @param SourceBean
	 *            campiLog
	 */
	@SuppressWarnings("unchecked")
	private void setCampiLog(SourceBean campiLog) throws SourceBeanException {
		if (campiLog != null) {
			List<SourceBean> rows = campiLog.getAttributeAsList("ROW");
			for (SourceBean row : rows) {
				if (row != null) {
					String codAmbiente = (String) row.getAttribute("nomeCampo");
					if (codAmbiente != null)
						row.updAttribute("nomeCampo", codAmbiente.trim());
				}
			}
		}
		_campiLog = campiLog;
	}

	/**
	 * Make statement for retrive columns attribute
	 * 
	 * @param serviceRequest
	 * @return
	 * @throws Exception
	 */
	private String makeSqlStatement(final String codAmbiente, final String fileFisico) throws Exception {
		String ret = fileFisico;
		if (codAmbiente != null && codAmbiente.trim().length() > 0) {
			ret = codAmbiente.trim() + "." + fileFisico;
		}
		String statement = "SELECT * FROM ";
		return statement + ret;
	}
}