/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util.sqlField;

import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;
import java.io.Serializable;
import java.util.List;

public class SqlFieldType implements Serializable {
	private int      _progressivoCampo ;               // es. 10,20,30, ...
	private int      _tipoCampo ;                      // Codice numerico
	private int      _sqlType ;                        // Codice sql Types
	private int      _decimaliCampo ;                  // Per i tipi decimal, float
	private int      _lunghezzaCampo ;                 //
	private int      _internalLength ;                 //
	private int      _externalLength ;                 //
	private String   _nomeCampo ;                      // 
	private String   _descrizioneCampo ;               // Textual description only for documentation pourpose
	private String   _widgetCampo ;                    // widget name for detail module (es. TEXT, COMBO, CALENDAR, ...)
	private boolean  _campoChiave ;                    // true if is key
	private boolean  _campoNull ;                      // true if accept null value
	private String   _webName ;                        // Field name over the web
	private String   _useInInsert;					   // Use field in insert statement
	private String   _useInUpdate;					   // Use field in update statement
	private String   _usage;                           // Field usage form PCML fields
	
	public void setWidgetCampo(String template) {
		_widgetCampo = template;
	}
	
	public void setCampoWeb(String name) {
		if (name != null && name.length()>0) 
			_webName = name;
	}
	
	public void setTipoCampo(SourceBean sqlTypeList, int sqlType) throws Exception {
		try {
			SourceBean findType = null;
			List<SourceBean> sqlTypeListFiltered = (List<SourceBean>)sqlTypeList.getFilteredSourceBeanAttributeAsList("ROW", "CODTIPO", new Integer(sqlType));
			if(sqlTypeListFiltered.size()>0)
				findType = (SourceBean)(sqlTypeListFiltered.get(0)) ;
			_sqlType = sqlType ;
			if(findType!=null) {
				_tipoCampo = ((Integer)findType.getAttribute("codTipo")).intValue() ;
				_widgetCampo = (String)findType.getAttribute("widgetTipo") ;
				_internalLength = (Integer) (findType.getAttribute("internalLength") == null ? 0 : findType.getAttribute("internalLength"));
				_externalLength = (Integer) (findType.getAttribute("externalLength") == null ? 0 : findType.getAttribute("externalLength"));
			}
		} catch (Exception ex) {
			TracerSingleton.log("Spago", TracerSingleton.CRITICAL,
					"FieldType::setTipoCampo - Errore durante la decodifica del tipo : " + sqlType, ex);
			throw ex;
		}
	}

	public void setTipoCampo(int tipo) {
		_tipoCampo = tipo;
	}
	
	/**
	 * @return Restituisce il valore _campoChiave.
	 */
	public boolean isCampoChiave() {
		return _campoChiave;
	}

	/**
	 * @return Restituisce il valore _campoNull.
	 */
	public boolean isCampoNull() {
		return _campoNull;
	}

	/**
	 * @return Restituisce il valore _decimaliCampo.
	 */
	public int getDecimaliCampo() {
		return _decimaliCampo;
	}

	/**
	 * @return Restituisce il valore _descrizioneCampo.
	 */
	public String getDescrizioneCampo() {
		return _descrizioneCampo;
	}

	/**
	 * @return Restituisce il valore _lunghezzaCampo.
	 */
	public int getLunghezzaCampo() {
		return _lunghezzaCampo;
	}
	
	/**
	 * @return Restituisce il valore _internalLength.
	 */
	public int getInternalLength() {
		return _internalLength;
	}

	/**
	 * @return Restituisce il valore _externalLength.
	 */
	public int getExternalLength() {
		return _externalLength;
	}

	/**
	 * 
	 * @return Restituisce il valore di _webname
	 */
	public String getCampoWeb() {
		return _webName ;
	}
	
	/**
	 * @return Restituisce il valore _nomeCampo.
	 */
	public String getNomeCampo() {
		return _nomeCampo;
	}

	/**
	 * @return Restituisce il valore _progressivoCampo.
	 */
	public int getProgressivoCampo() {
		return _progressivoCampo;
	}

	/**
	 * @return Restituisce il valore _sqlType.
	 */
	public int getSqlType() {
		return _sqlType;
	}

	/**
	 * @return Restituisce il valore _tipoCampo.
	 */
	public int getTipoCampo() {
		return _tipoCampo;
	}

	/**
	 * @return Restituisce il valore _widgetCampo.
	 */
	public String getWidgetCampo() {
		return _widgetCampo;
	}

	public String getUseInInsert() {
		return _useInInsert;
	}

	public String getUseInUpdate() {
		return _useInUpdate;
	}
	/**
	 * @param chiave Il valore _campoChiave da impostare.
	 */
	public void setCampoChiave(boolean chiave) {
		_campoChiave = chiave;
	}

	/**
	 * @param null1 Il valore _campoNull da impostare.
	 */
	public void setCampoNull(boolean null1) {
		_campoNull = null1;
	}

	/**
	 * @param campo Il valore _decimaliCampo da impostare.
	 */
	public void setDecimaliCampo(int campo) {
		_decimaliCampo = campo;
	}

	/**
	 * @param campo Il valore _descrizioneCampo da impostare.
	 */
	public void setDescrizioneCampo(String campo) {
		_descrizioneCampo = campo;
	}

	/**
	 * @param campo Il valore _lunghezzaCampo da impostare.
	 */
	public void setLunghezzaCampo(int campo) {
		_lunghezzaCampo = campo;
	}

	/**
	 * @param campo Il valore _nomeCampo da impostare.
	 */
	public void setNomeCampo(String campo) {
		_nomeCampo = campo;
	}

	/**
	 * @param campo Il valore _progressivoCampo da impostare.
	 */
	public void setProgressivoCampo(int campo) {
		_progressivoCampo = campo;
	}

	public void setUseInInsert(String useField) {
		_useInInsert = useField;
	}

	public void setUseInUpdate(String useField) {
		_useInUpdate = useField;
	}
	
	public void setUsage(String usageField) {
		_usage = usageField;
	}

	public String getUsage() {
		return _usage;
	}

}