/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util.sqlField;

import it.eng.spago.base.Constants;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;
import java.io.Serializable;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author grassi
 *
 */
public class SqlTableList implements Serializable {
	private static final long serialVersionUID = 1136885173162418436L;

	/**
     * Search defined tables in DB
	 * @param poolName
	 * @param catalog a catalog name; must match the catalog name as it
     *        is stored in the database; "" retrieves those without a catalog;
     *        <code>null</code> means that the catalog name should not be used to narrow
     *        the search
     * @param schemaPattern a schema name pattern; must match the schema name
     *        as it is stored in the database; "" retrieves those without a schema;
     *        <code>null</code> means that the schema name should not be used to narrow
     *        the search
     * @param tableNamePattern a table name pattern; must match the
     *        table name as it is stored in the database 
	 * @return List of SqlTableType
	 * @throws Exception
	 */
	public static List<SqlTableType> getDefinedTables(String poolName, String catalog,  String schemaPattern, String tableNamePattern) throws Exception {
		DataConnection dataConnection = null;
		ResultSet rs = null;
		List<SqlTableType> listSqlTableType = null ;
		try {
			dataConnection = QueryExecutor.openConnection(poolName, null);
			DatabaseMetaData dbmd = dataConnection.getInternalConnection().getMetaData();
			rs = dbmd.getTables(catalog, schemaPattern, tableNamePattern, null) ;
			SqlTableType sqlTableType = null ;
			while (rs.next()) {
				sqlTableType = new SqlTableType(rs) ;
				if (listSqlTableType == null)
					listSqlTableType = new ArrayList<SqlTableType>() ;
				listSqlTableType.add(sqlTableType) ;
			}
			
			if (listSqlTableType != null) {
				SqlTableTypeComparator comp = new SqlTableTypeComparator() ;
				Collections.sort(listSqlTableType, comp);
			}
			
		} // try
		catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "SqlTableList::getDefinedTables:", (Exception) ex);
			throw ex;
		} // catch (Exception ex) try
		finally {
			if (rs != null)
				rs.close();
			Utils.releaseResources(dataConnection, null, null);
		} // finally try
		return listSqlTableType;
	}
}
