/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util.sqlField;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import java.io.Serializable;
import java.sql.ResultSet;

/**
 * @author grassi
 *
 */
public class SqlTableType implements Serializable {
	private static final long serialVersionUID = 1L;

	public final static String TABLE_TYPE = "TABLE";
	public final static String VIEW_TYPE = "TABLE";
	public final static String SYSTEM_TABLE_TYPE = "SYSTEM TABLE";
	public final static String GLOBAL_TEMPORARY_TYPE = "GLOBAL TEMPORARY";
	public final static String LOCAL_TEMPORARY_TYPE = "LOCAL TEMPORARY";
	public final static String ALIAS_TYPE = "ALIAS";
	public final static String SYNONYM_TYPE = "SYNONYM";
	private String tableCatalog;
	private String tableSchema;
	private String tableName;
	private String tableType;
	private String explanatoryComment;

	public SqlTableType(ResultSet rs) throws Exception {
		tableCatalog = rs.getString(1) ; // "TABLE_CAT" 
		tableSchema = rs.getString(2) ; // "TABLE_SCHEM"
		tableName = rs.getString(3); // "TABLE_NAME")
		tableType = rs.getString(4) ; // "TABLE_TYPE")
		explanatoryComment = rs.getString(5) ;; // "REMARKS" ;
	}
	
	/**
	 * @return Restituisce il valore explanatoryComment.
	 */
	public String getExplanatoryComment() {
		return explanatoryComment;
	}


	/**
	 * @return Restituisce il valore tableCatalog.
	 */
	public String getTableCatalog() {
		return tableCatalog;
	}

	/**
	 * @return Restituisce il valore tableName.
	 */
	public String getTableName() {
		return tableName;
	}

	/**
	 * @return Restituisce il valore tableSchema.
	 */
	public String getTableSchema() {
		return tableSchema;
	}

	/**
	 * @return Restituisce il valore tableType.
	 *         Typical types are "TABLE", "VIEW",	"SYSTEM TABLE", "GLOBAL TEMPORARY", 
     *			"LOCAL TEMPORARY", "ALIAS", "SYNONYM".
	 */
	public String getTableType() {
		return tableType;
	}

	/**
	 * @param explanatoryComment Il valore explanatoryComment da impostare.
	 */
	public void setExplanatoryComment(String explanatoryComment) {
		this.explanatoryComment = explanatoryComment;
	}

	/**
	 * @param tableCatalog Il valore tableCatalog da impostare.
	 */
	public void setTableCatalog(String tableCatalog) {
		this.tableCatalog = tableCatalog;
	}

	/**
	 * @param tableName Il valore tableName da impostare.
	 */
	public void setTableName(String tableName) {
		this.tableName = tableName;
	}

	/**
	 * @param tableSchema Il valore tableSchema da impostare.
	 */
	public void setTableSchema(String tableSchema) {
		this.tableSchema = tableSchema;
	}

	/**
	 * @param tableType Il valore tableType da impostare.
	 */
	public void setTableType(String tableType) {
		this.tableType = tableType;
	}

	/**
	 * Converte la classe in un SourceBean
	 * @param sourceBeanName
	 * @return
	 * @throws SourceBeanException
	 */
	public SourceBean toSourceBean(String sourceBeanName) throws SourceBeanException {
		SourceBean sb = new SourceBean(sourceBeanName) ;
		if (tableCatalog != null) sb.setAttribute("tableCatalog", tableCatalog) ;
		if (tableSchema != null) sb.setAttribute("tableSchema", tableSchema) ;
		if (tableName != null) sb.setAttribute("tableName", tableName) ;
		if (tableType != null) sb.setAttribute("tableType", tableType) ;
		if (explanatoryComment != null) sb.setAttribute("explanatoryComment", explanatoryComment) ;
		
		return sb ;
	}
}