/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.validation;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

public class FieldValidatorFactory {

	public static FieldValidatorIFace getFieldValidator(String fieldType){
		
		TracerSingleton.log(Constants.NOME_MODULO,
				TracerSingleton.INFORMATION, "FieldValidatorFactory:: getFieldValidator:: Start Getting validator for type ["+fieldType+"]");
		
		SourceBean fieldValidatorSourceBean = (SourceBean)ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("FIELD-VALIDATORS.FIELD-VALIDATOR", "fieldType", fieldType);
		
		String fieldValidatorImplementingClass = (String)fieldValidatorSourceBean.getAttribute("fieldValidatorClass");
		
		TracerSingleton.log(Constants.NOME_MODULO,
				TracerSingleton.INFORMATION, "FieldValidatorFactory:: getFieldValidator:: fieldValidatorImplementingClass ["+fieldValidatorImplementingClass+"]");
		
		FieldValidatorIFace fieldValidator = null;
		try{
			 fieldValidator = (FieldValidatorIFace)Class.forName(fieldValidatorImplementingClass).newInstance(); 
		}catch (Exception e) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "Exception instantiating validator for type " +fieldType + "" + e);
			return null;
		}
		
		TracerSingleton.log(Constants.NOME_MODULO,
				TracerSingleton.INFORMATION, "FieldValidatorFactory:: getFieldValidator:: fieldValidator Created");
		
		SourceBean config = (SourceBean)fieldValidatorSourceBean.getAttribute("CONFIG"); 
        fieldValidator.init(config);
		
		if (config != null){
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.DEBUG, "FieldValidatorFactory:: getFieldValidator:: fieldValidator Initialized with configuration", config);
		}
		
		return fieldValidator;
	}
	
}
