/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.validation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.apache.axis.server.ParamList;
import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;
import it.eng.spago.base.Constants;
import it.eng.spago.base.NullValue;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.QueryExecutor;
import it.eng.spago.util.StatementProvider;

/**
 * Gestisce le validazioni SQL
 * 
 * @author A. Grassi
 * @version 0.1
 */
public class SqlValidator extends AbstractValidator {
        private static final long serialVersionUID = -7285721648603416911L;
        private String _businessType = "" ;
        private String _businessName = "" ;
        
        public void init(SourceBean config) {
                // TODO Stub di metodo generato automaticamente
                super.init(config);
                if (config != null && getResponseContainer() != null) {
                        _businessType = getResponseContainer().getBusinessType() ;
                        _businessName = getResponseContainer().getBusinessName() ;
                }
        }

        /**
         * Effettua i controlli formali sui dati del form.
         * Normalmente si implementano in questo metodo i controlli di validità sintattica dei campi.
         * Deve restituire true se la verifica deve proseguire con gli altri validatori, false altrimenti
         * 
         * @param request
         * @param errorHandler
         */
        public boolean check(SourceBean request, EMFErrorHandler errorHandler) {
                return true;
        }

        /**
         * Effettua i controlli di dominio sui dati della sezione.
         * Normalmente si implementano in questo metodo i controlli incrociati sui campi.
         * Questo metodo viene invocato solo se i controlli formali non hanno prodotto errori.
         * Deve restituire true se la verifica deve proseguire con gli altri validatori, false altrimenti.
         * 
         * @param request
         * @param errorHandler
         */
        public boolean validate(SourceBean request, EMFErrorHandler errorHandler) {
        Monitor monitor = null;
        
        try {
            monitor = MonitorFactory.start("sqlValidator." + _businessType.toLowerCase() + "." + _businessName.toLowerCase());

                        if (! needsValidation(request))
                                return false ; // Non ci sono le condizioni per proseguire la validazione
            
            Iterator queries =  _config.getAttributeAsList("SELECT_QUERY").iterator() ;
            if(!queries.hasNext())
                queries =  _config.getAttributeAsList("LIST_QUERY").iterator() ;
                        while (queries.hasNext()) {
                                if (! sqlValidate(request, errorHandler, (SourceBean)queries.next()))
                                        return false ;
                        }
                } catch (NullPointerException ex) {
                        ; // Intenzionalmente vuoto
                } finally {
            if (monitor != null)
                monitor.stop();
        }

                if (_config == null) return true ;

                return true;
        }

        
        /**
         * Esegue il query passato, valuta i risultati, eventualmente inserisce gli errori
         * Ritorna false se il controllo di validazione non è stato superato con esito positivo
         * 
         * @param request
         * @param errorHandler
         * @param query
         * @return
         */
        protected boolean sqlValidate(SourceBean request, EMFErrorHandler errorHandler, SourceBean query) {
                String pool = (String)_config.getAttribute("POOL") ;
                if (query != null && pool != null) {
                        boolean mandatory = Boolean.parseBoolean((String)_config.getAttribute("mandatory", "true")) ;
                        // Verifica che i parametri siano almeno parzialmente compilati
                        //  .. se i parametri sono vuoti la query non va eseguita
                        if (areFilledParameters(request, query)) {
                                // List parameters = 
                                SourceBean rowsSourceBean = ((SourceBean)QueryExecutor.executeQuery(_requestContainer, getResponseContainer(), pool, query, "SELECT"));
                                if (rowsSourceBean != null) {
                                        List rowsList = rowsSourceBean.getAttributeAsList("ROW");
                                        if (rowsList == null || rowsList.size() != 1) {
                                                List fieldsName = getFieldsName(query) ;
                                                //errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, 10121, fieldsName)) ;
                                                for(String field : (List<String>)fieldsName) {
                                                        ArrayList fields = new ArrayList();
                                                        fields.add(field);
                                                        errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR, 10121, fields)) ;
                                                }
                                                return false ;
                                        }
                                }
                        }
                } else {
                        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "SqlValidator::validate: Query or Pool not founded");
                        errorHandler.addError(new EMFInternalError(EMFErrorSeverity.WARNING, "Query o Pool non specificato in configurazione del validators"));
                }
                
                return true ;
        }
        
        /**
         * Test compilazione parametri
         * @param request
         * @param query
         * @return true se trova almeno un PARAMETER con un valore
         */
        protected boolean areFilledParameters(SourceBean request, SourceBean query) {
                boolean ret = false ;
        List filterParameters = query.getAttributeAsList("FILTER_PARAMETER");
        for (int i = 0; !ret && i < filterParameters.size(); i++) {
                ret = isFilled((SourceBean)filterParameters.get(i)) ;
        }
        
        List parameters = query.getAttributeAsList("PARAMETER");
        for (int i = 0; !ret && i < parameters.size(); i++) {
                ret = isFilled((SourceBean)parameters.get(i)) ;
        }
        
                return ret ;
        }
        
        protected boolean isFilled(final SourceBean parameter) {
            // Lettura attributi di "tipo" del parametro
            String parameterType = (String) parameter.getAttribute("TYPE");
            String parameterValue = (String) parameter.getAttribute("VALUE");
            String parameterScope = (String) parameter.getAttribute("SCOPE");

            Object parameterValueObject = null ; 
        if (parameterType.equalsIgnoreCase("ABSOLUTE")) {
                return true;
        }
        parameterValueObject = ContextScooping.getScopedParameter(_requestContainer, getResponseContainer(), 
                                parameterValue, parameterScope, parameter);

        // Aggiunta per Spago 2.0
        if (parameterValueObject instanceof NullValue) {
                parameterValueObject = null ;
        }
        if (parameterValueObject != null) {
                if ((""+parameterValueObject).trim().length() == 0)
                        parameterValueObject = null ;
        }
        
        return (parameterValueObject != null) ;
        }
        
        protected List getFieldsName(SourceBean query) {
                ArrayList paramsName = new ArrayList() ;
                Iterator params = query.getAttributeAsList("FILTER_PARAMETER").iterator() ;
                while (params.hasNext()) {
                        paramsName.add( MessageBundle.getMessage((String)((SourceBean)params.next()).getAttribute("VALUE"))) ;
                }
                params = query.getAttributeAsList("PARAMETER").iterator() ;
                while (params.hasNext()) {
                        paramsName.add( MessageBundle.getMessage((String)((SourceBean)params.next()).getAttribute("VALUE"))) ;
                }

                return paramsName ;
        }
}
