/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.impl.SyntaxChecker;
import it.eng.spago.validation.impl.ValidationServices;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.validator.GenericValidator;

public class NumericEnumerationValidator extends NumericValidator {

    private static final long serialVersionUID = 1L;
    
    public static final String ERROR_ENUM_VALUE_IDENTIFIER = "ERROR_CODE_ENUM_VALUE";
    public static final String ERRORE_ENUM_VALUE = "10121"; 

    public void validateField(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams) throws EMFValidationError {
        // Make base checks
        super.validateField(serviceRequest, fieldName, value, fieldValidationParams);
        
        // Making the check as number is better than making with Strings: for example
        // "+123" is different from "123", but the represented number is the same
        List enumeration = new ArrayList();
        
        String enumValue = getParameter(SyntaxChecker.ENUM_ATTRIBUTE, null, fieldValidationParams);
        NumberFormat nf = getNumberFormat();
                
        try {
            StringTokenizer stk = new StringTokenizer(enumValue, ",");
            while(stk.hasMoreTokens()) {
                String token = stk.nextToken();
                if (!GenericValidator.isBlankOrNull(token)){
                    Number enumerationNumber = nf.parse(token);
                    Double convertedEnumValue = new Double(enumerationNumber.doubleValue());
                    enumeration.add(convertedEnumValue);
                }
            }

            Object typedValue = ValidationServices.getTypedServiceRequest(serviceRequest).getAttribute(getFieldAlias());
            if (!enumeration.contains(typedValue)) {
                // Makes the list available in the messages
                getMessagesParams().add(enumValue);
                throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(ERROR_ENUM_VALUE_IDENTIFIER, ERRORE_ENUM_VALUE, fieldValidationParams), getMessagesParams());
            }
        } catch (SourceBeanException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR, "NumericEnumerationValidator::validateField", ex);
        } catch (ParseException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR, "NumericEnumerationValidator::validateField", ex);
        }
    }

}
