/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.impl.SyntaxChecker;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.validator.GenericValidator;

/**
 * @author Cattiv
 *
 */
public class NumericStringValidator extends AbstractFieldValidator {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7914265592931936280L;

	public static final String ERRORE_CAMPO_NUMERICO = "10106"; 
    private List messagesParams = null;

	private Locale locale = null;
    private String fieldAlias = null;

    public void init(SourceBean config) {
        super.init(config);

        String language = "it";
        String country = "IT";
        
        if (config != null){
            language = (String)config.getAttribute("LANGUAGE.language","it");
            country = (String)config.getAttribute("COUNTRY.country","IT");
        }
        
        // Create all the objects locale dependent
        locale = new Locale(language, country);
        messagesParams = new ArrayList();
	}
    

	/* (non Javadoc)
	 * @see it.eng.spago.validation.FieldValidatorIFace#validateField(it.eng.spago.base.SourceBean, java.lang.String, java.lang.String, java.util.Map)
	 */
	public void validateField(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams) throws EMFValidationError {
		String regexp = "^[-+]?\\d*$";

		List params = getMessagesParams();
		params.add(fieldName);
        params.add(getUserFieldName(fieldName));

        if (!GenericValidator.matchRegexp(value, regexp)){
			throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,getParameter(GENERIC_ERROR_IDENTIFIER, ERRORE_CAMPO_NUMERICO, fieldValidationParams),params);
		}
        else {
    		String alias = getParameter(SyntaxChecker.PARAM_FIELD_ALIAS, null, fieldValidationParams);
    		if (alias == null){
    			alias = fieldName;
    		}
            setFieldAlias(alias);
            String convertedValue = value;
	        saveTypedValue(serviceRequest, alias, convertedValue);
        }
	}

		
    public List getMessagesParams() {
        return messagesParams;
    }

    public void setMessagesParams(List messagesParams) {
        this.messagesParams = messagesParams;
    }

    public String getFieldAlias() {
        return fieldAlias;
    }

    public void setFieldAlias(String fieldAlias) {
        this.fieldAlias = fieldAlias;
    }
		
}
